package com.example.epos_2021;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.MediaPlayer;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.provider.Settings;
import android.util.DisplayMetrics;
import android.util.Log;

import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.room.Room;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.example.epos_2021.callerid.CALLERID;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.MyExceptionHandler;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.comman.printer.WifiPrinter;
import com.example.epos_2021.comman.printer.ZoneRichPrinter;
import com.example.epos_2021.daos.AppDatabase;
import com.example.epos_2021.daos.relations.CategoryWithChildren;
import com.example.epos_2021.models.Addon;
import com.example.epos_2021.models.Category;
import com.example.epos_2021.models.EposUserPermission;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.OrderStatuses;
import com.example.epos_2021.models.OrderType;
import com.example.epos_2021.models.PaymentMethod;
import com.example.epos_2021.models.Printer;
import com.example.epos_2021.models.Product;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.models.TableStatus;
import com.example.epos_2021.models.User;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.network.NetworkInterceptor;
import com.example.epos_2021.online.services_receivers.TiffintomPartnerService;
import com.example.epos_2021.storageutils.MyPreferences;
import com.example.epos_2021.utils.LogUtils;
import com.google.firebase.FirebaseApp;
import com.google.firebase.crashlytics.FirebaseCrashlytics;
import com.google.firebase.messaging.FirebaseMessaging;
import com.google.gson.Gson;
import com.zj.wfsdk.WifiCommunication;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.URL;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.TimeUnit;

import okhttp3.OkHttpClient;

import static com.example.epos_2021.daos.AppDatabase.MIGRATION_10_11;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_11_12;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_12_13;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_13_14;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_14_15;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_1_2;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_2_3;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_3_4;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_4_5;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_5_6;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_6_7;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_7_8;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_8_9;
import static com.example.epos_2021.daos.AppDatabase.MIGRATION_9_10;

/**
 * Created by Amrish on 23-04-2021.
 */
public class MyApp extends Application {
    private static MyApp ourInstance;

    public static MyApp getInstance() {
        return ourInstance;
    }

    public static String currencySymbol = "£";
    public MyPreferences myPreferences;


    /**
     * To be saved as temp data, so we dont have to fetch everytime
     */
    public static ArrayList<OrderType> orderTypes = new ArrayList<>();
    public ArrayList<Category> categories = new ArrayList<>();
    public ArrayList<OrderItem> orderItems = new ArrayList<>();
    public ArrayList<PaymentMethod> paymentMethods = new ArrayList<>();
    public ArrayList<SiteSetting> siteSettings = new ArrayList<>();
    public ArrayList<TableStatus> tableStatuses = new ArrayList<>();
    public ArrayList<OrderStatuses> orderStatuses = new ArrayList<>();
    public AppDatabase appDatabase;
    public boolean isInternetAvailable = true;
    public String restaurant_id;
    public static int screenWidth = 0;
    public Bitmap businessLogo, tiffintomLogo;
    private MediaPlayer mp, delayedOrderMP, paymentSuccessMP, paymentFailedMP, newMessageMP;
    public boolean needToPrint = false;
    public boolean isCashDrawerOpen = false;
    public static EposUserPermission userPermission;
    public static int printerStatus = -1;
    public boolean shallWeRefreshOrders = true, isReservationOpen = false;
    public boolean showBanquetPorducts = false;
    public String current_day;
    public String banquetDay, banquetOrderTypes;
    public static String openedOrderTypeId;
    public WifiPrinter wifiPrinter;
    public WifiCommunication wifiCommunication;
    public Order openOrder;
    public boolean drawerCollapsed = true;

    @Override
    public void onCreate() {
        super.onCreate();

        ourInstance = this;
        restaurant_id = null;
        getRestaurantId();
        appDatabase = Room.databaseBuilder(getApplicationContext(),
                AppDatabase.class, "epos_2021-database")
                .addMigrations(MIGRATION_1_2)
                .addMigrations(MIGRATION_2_3)
                .addMigrations(MIGRATION_3_4)
                .addMigrations(MIGRATION_4_5)
                .addMigrations(MIGRATION_5_6)
                .addMigrations(MIGRATION_6_7)
                .addMigrations(MIGRATION_7_8)
                .addMigrations(MIGRATION_8_9)
                .addMigrations(MIGRATION_9_10)
                .addMigrations(MIGRATION_10_11)
                .addMigrations(MIGRATION_11_12)
                .addMigrations(MIGRATION_12_13)
                .addMigrations(MIGRATION_13_14)
                .addMigrations(MIGRATION_14_15)
                .fallbackToDestructiveMigration()
                .allowMainThreadQueries()
                .build();
        myPreferences = new MyPreferences(this);
        FirebaseCrashlytics.getInstance().setCrashlyticsCollectionEnabled(true);
        Thread.setDefaultUncaughtExceptionHandler(new MyExceptionHandler(this));
        getFirebaseToken();
        initNetwork();
        initOrderSoundMP();
        initPaymentFailedMP();
        initPaymentSuccessMP();
        initDelayedOrderSoundMP();
        initNewMessageSoundMP();
        setupData();
        CALLERID.getInstance().ConnectCallerId(this);
        current_day = CommonFunctions.getCurrentTimeFormatted("EEEE");
    }

    public void setupData() {
        new DataSetupAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private void initNetwork() {
        OkHttpClient okHttpClient = new OkHttpClient().newBuilder()
                .addNetworkInterceptor(new NetworkInterceptor())
                .readTimeout(1, TimeUnit.MINUTES)
                .writeTimeout(1, TimeUnit.MINUTES)
                .build();
        AndroidNetworking.initialize(getApplicationContext(), okHttpClient);
        new Timer().schedule(new CheckConnection(ourInstance), 0, 40000);
    }

    public static class df {
        public static String format(float value) {
            return String.format("%.2f", value);
        }

        public static String format(double value) {
            return String.format("%.2f", value);
        }
    }

    public static float calculatedAddonPrice(OrderType orderType, Addon addon) {
        if (orderType == null) return addon.price;
        if (orderType.type.equalsIgnoreCase("delivery")) return addon.delivery_price;
        if (orderType.type.equalsIgnoreCase("waiting")) return addon.waiting_price;
        if (orderType.type.equalsIgnoreCase("collection") || orderType.type.equalsIgnoreCase("takeaway") || orderType.type.equalsIgnoreCase("pickup"))
            return addon.takeaway_price;
        return addon.price;
    }

    public static float calculatedAddonPrice(String orderTypeId, Addon addon) {
        if (orderTypeId == null) return addon.price;
        if (orderTypeId.equalsIgnoreCase("3")) return addon.delivery_price;
        if (orderTypeId.equalsIgnoreCase("5")) return addon.takeaway_price;
        if (orderTypeId.equalsIgnoreCase("2")) return addon.takeaway_price;
        return addon.price;
    }

    public static float calculatedProductPrice(OrderType orderType, Product product) {
        if (orderType == null) return product.price;
        if (orderType.type.equalsIgnoreCase("delivery")) return product.delivery_price;
        if (orderType.type.equalsIgnoreCase("collection") || orderType.type.equalsIgnoreCase("waiting") || orderType.type.equalsIgnoreCase("takeaway") || orderType.type.equalsIgnoreCase("pickup"))
            return product.takeaway_price;
        return product.price;
    }

    public static float calculatedProductPrice(String orderTypeId, Product product) {
        MyApp myApp = MyApp.getInstance();
        if (!Validators.isNullOrEmpty(myApp.banquetDay) && !Validators.isNullOrEmpty(myApp.banquetOrderTypes) && product.is_banquet) {

            if (orderTypeId == null) return product.price;
            if (orderTypeId.equalsIgnoreCase("3")) {
                if (myApp.banquetOrderTypes.contains("delivery"))
                    return product.waiting_price;
                else return product.delivery_price;
            }
            if (orderTypeId.equalsIgnoreCase("5")) {
                if (myApp.banquetOrderTypes.contains("waiting"))
                    return product.waiting_price;
                else
                    return product.takeaway_price;
            }
            if (orderTypeId.equalsIgnoreCase("2")) {
                if (myApp.banquetOrderTypes.contains("collection"))
                    return product.waiting_price;
                else
                    return product.takeaway_price;
            }
            if (orderTypeId.equalsIgnoreCase("1")) {
                if (myApp.banquetOrderTypes.contains("dinein"))
                    return product.waiting_price;
                else
                    return product.price;
            }
            return product.price;
        } else {
            if (orderTypeId == null) return product.price;
            if (orderTypeId.equalsIgnoreCase("3")) return product.delivery_price;
            if (orderTypeId.equalsIgnoreCase("5")) return product.takeaway_price;
            if (orderTypeId.equalsIgnoreCase("2")) return product.takeaway_price;
            return product.price;
        }
    }

    public void setOrderItems(ArrayList<OrderItem> orderItems) {
        this.orderItems.clear();
        this.orderItems.addAll(orderItems);
    }

    public void setPaymentMethods(ArrayList<PaymentMethod> paymentMethods) {
        this.paymentMethods = paymentMethods;
    }

    public int getCartProductsCount(String productId) {
        int count = 0;
        for (OrderItem orderItem : orderItems) {
            if (!Validators.isNullOrEmpty(orderItem.product_id) && orderItem.product_id.equalsIgnoreCase(productId) && Validators.isNullOrEmpty(orderItem.order_split_id)) {
                count = count + orderItem.quantity;
            }

        }
        return count;

    }

    public void notifyCart(Context context, Intent intent) {
        LocalBroadcastManager.getInstance(context).sendBroadcast(intent);
    }

    public void notifyCallReceived(Context context, Intent intent) {
        if (context == null) context = this;
        LocalBroadcastManager.getInstance(context).sendBroadcast(intent);
    }

    public String getDeviceId() {
        return Settings.Secure.getString(getContentResolver(), Settings.Secure.ANDROID_ID);
    }

    public SiteSetting findSetting(String key) {
        SiteSetting siteSetting = new SiteSetting();
        siteSetting.key = key;
        int index = siteSettings.indexOf(siteSetting);
        if (index == -1) return null;
        if (siteSettings.size() == 0 || siteSettings.size() - 1 < index) return null;
        return siteSettings.get(index);
    }

    public TableStatus findStatus(String status) {
        TableStatus tableStatus = new TableStatus();
        tableStatus.status = status;
        int index = tableStatuses.indexOf(tableStatus);
        if (index == -1) return null;
        if (tableStatuses.size() == 0 || tableStatuses.size() - 1 < index) return null;
        return tableStatuses.get(index);
    }

    private class DataSetupAsync extends AsyncTask<Void, Void, String> {

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
        }

        @Override
        protected String doInBackground(Void... lists) {
            ArrayList<Category> tempCategories = new ArrayList<>();
            LogUtils.e("Settingup data");

            ArrayList<CategoryWithChildren> categoryWithChildren = (ArrayList<CategoryWithChildren>) appDatabase.categoryDao().listWithChild();
            for (CategoryWithChildren c : categoryWithChildren) {
                Category category = c.category;
                category.children = (ArrayList<Category>) c.children;
                ArrayList<Category> childs = new ArrayList<>();
                for (Category childCategory : category.children) {
                    if (!childCategory.disabled) {
                        childs.add(childCategory);
                    }
                }
                category.children = childs;
                tempCategories.add(category);
            }
            categories.clear();
            categories.addAll(tempCategories);

            generateBusinessLogoBitmap();
            generateTiffintomLogoBitmap();
            return "Executed";
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            LogUtils.e("Settingup data finished");
        }
    }


    private void performUserLogin() {
        if (myPreferences.getLoggedInUser() == null) return;
        LogUtils.e("Performing user login in background");
        String username = myPreferences.getLoggedInUser().username;
        String password = myPreferences.getLoggedInUser().password;
        AndroidNetworking.post(ApiEndPoints.user_login)
                .addBodyParameter("username", username)
                .addBodyParameter("password", password)
                .build()
                .getAsObject(User.class, new ParsedRequestListener<User>() {
                    @Override
                    public void onResponse(User response) {
                        LogUtils.e("User loging done in background");
                        myPreferences.saveEposUserPermission(new Gson().toJson(response.permissions));
                        MyApp.userPermission = response.permissions;
                        response.permissions = null;
                        response.password = password;
                        myPreferences.saveLoggedInUser(response);
                        myPreferences.userLoggedInWithoutInternet(false);
                    }

                    @Override
                    public void onError(ANError anError) {
                    }
                });
    }

    // Checking Internet connectivity
    public boolean isConnected(Context context) {
        return isInternetAvailable;
    }

    private boolean hasNetworkAvailable(Context context) {
        ConnectivityManager cm = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
//
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
        return activeNetwork != null && activeNetwork.isConnectedOrConnecting();
    }

    class CheckConnection extends TimerTask {
        private Context context;

        public CheckConnection(Context context) {
            this.context = context;
        }

        public void run() {
            isInternetAvailable = hasInternetConnected(context);
        }
    }

    private boolean hasInternetConnected(Context context) {
        if (hasNetworkAvailable(context)) {
            try {
                HttpURLConnection connection = (HttpURLConnection) new URL("https://www.google.com").openConnection();

                connection.setRequestProperty("User-Agent", "Test");
                connection.setRequestProperty("Connection", "close");
                connection.setConnectTimeout(1500);
                connection.connect();

                boolean internetAvailable = (connection.getResponseCode() == 200);
                if (internetAvailable && myPreferences.isUserLoginWithoutInternet()) {
                    performUserLogin();
                }
                return internetAvailable;
            } catch (MalformedURLException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return false;
    }

    public String getRestaurantId() {
        if (!Validators.isNullOrEmpty(restaurant_id) && !restaurant_id.equalsIgnoreCase("null")) {
            return restaurant_id;
        }
        SiteSetting resIdSetting = findSetting("res_id");
        if (resIdSetting == null || Validators.isNullOrEmpty(resIdSetting.value) || resIdSetting.value.equals("null")) {
            restaurant_id = "";
            return "";
        } else {
            restaurant_id = resIdSetting.value;
            return restaurant_id;
        }
    }

    private void getFirebaseToken() {
        FirebaseApp.initializeApp(ourInstance);
        FirebaseMessaging.getInstance().getToken().addOnCompleteListener(task -> {
            if (!task.isSuccessful()) {
                return;
            }
            String token = task.getResult();
            myPreferences.setUserFCMToken(token);
        });
    }

    public void generateTiffintomLogoBitmap() {
        new Thread(() -> {
            tiffintomLogo = BitmapFactory.decodeResource(this.getResources(), R.drawable.printer_tiffinotom_logo);
        }).start();
    }

    public void generateBusinessLogoBitmap() {
        new Thread(() -> {
            SiteSetting logoSetting = findSetting("logo");
            if (logoSetting == null || Validators.isNullOrEmpty(logoSetting.value))
                businessLogo = null;
            else {
                try {
                    URL url = new URL(logoSetting.value);
                    businessLogo = BitmapFactory.decodeStream(url.openConnection().getInputStream());
                    LogUtils.e("BITMAP::", "PRINT::", "URL::", url.toString());
                } catch (IOException e) {
                    e.printStackTrace();
                    LogUtils.e("BITMAP::", "PRINT::", Log.getStackTraceString(e));
                }
            }
        }).start();

    }

    public void noInternet(Activity activity) {
//        if (activity != null)
//            startActivity(new Intent(activity, NoInternetActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
    }

    public void initService() {
        startService(new Intent(getApplicationContext(), TiffintomPartnerService.class));
    }

    public void saveScreenWidth(Activity activity) {
        DisplayMetrics displayMetrics = new DisplayMetrics();
        activity.getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        screenWidth = displayMetrics.widthPixels;
    }

    public boolean productImageEnabled() {
        SiteSetting productImageSetting = findSetting("product_image_visible");
        if (productImageSetting == null) return false;
        if (productImageSetting.value.equalsIgnoreCase("no"))
            return false;
        return true;
    }

    private void initOrderSoundMP() {
        mp = MediaPlayer.create(this, R.raw.notification);
    }

    private void initDelayedOrderSoundMP() {
        delayedOrderMP = MediaPlayer.create(this, R.raw.delayed_order);
    }

    private void initPaymentSuccessMP() {
        paymentSuccessMP = MediaPlayer.create(this, R.raw.payment_approval_sound);
    }

    private void initNewMessageSoundMP() {
        newMessageMP = MediaPlayer.create(this, R.raw.new_message);
    }

    public void startOrderSound() {
        if (mp != null) {
            mp.start();
            mp.setLooping(false);
            mp.setVolume(100, 100);
        }
    }

    public void stopOrderSound() {
        if (mp != null && mp.isPlaying()) {
            mp.stop();
            mp.reset();
            mp.release();
            initOrderSoundMP();
        }
    }

    public void startDelayedOrderSound() {
        if (delayedOrderMP != null) {
            delayedOrderMP.start();
            delayedOrderMP.setLooping(true);
            delayedOrderMP.setVolume(100, 100);
        }
    }

    public void stopDelayedOrderSound() {
        if (delayedOrderMP != null && delayedOrderMP.isPlaying()) {
            delayedOrderMP.stop();
            delayedOrderMP.reset();
            delayedOrderMP.release();
            initDelayedOrderSoundMP();
        }
    }

    public void startPaymentSuccessSound() {
        if (paymentSuccessMP != null) {
            paymentSuccessMP.start();
            paymentSuccessMP.setVolume(100, 100);
        }
    }

    public void stopPaymentSuccessSound() {
        if (paymentSuccessMP != null && paymentSuccessMP.isPlaying()) {
            paymentSuccessMP.stop();
            paymentSuccessMP.reset();
            paymentSuccessMP.release();
            initPaymentSuccessMP();
        }
    }

    public void startNewMessageSound() {
        if (newMessageMP != null) {
            newMessageMP.start();
            newMessageMP.setVolume(100, 100);
        }
    }

    public void stopNewMessageSound() {
        if (newMessageMP != null && newMessageMP.isPlaying()) {
            newMessageMP.stop();
            newMessageMP.reset();
            newMessageMP.release();
            initNewMessageSoundMP();
        }
    }

    private void initPaymentFailedMP() {
        paymentFailedMP = MediaPlayer.create(this, R.raw.decline_sound);
    }

    public void startPaymentFailedSound() {
        if (paymentFailedMP != null) {
            paymentFailedMP.start();
            paymentFailedMP.setVolume(100, 100);
        }
    }

    public void stopPaymentFailedSound() {
        if (paymentFailedMP != null && paymentFailedMP.isPlaying()) {
            paymentFailedMP.stop();
            paymentFailedMP.reset();
            paymentFailedMP.release();
            initPaymentFailedMP();
        }
    }

    // Checking Printer connection
    public void checkPrinterIP(String ip, int port) {
        boolean exists = false;
        LogUtils.e("Cheking iP ");
        try {
            SocketAddress sockaddr = new InetSocketAddress(ip, port);
            // Create an unbound socket
            Socket sock = new Socket();
            LogUtils.e("Cheking iP inside");
            // This method will block no more than timeoutMs.
            // If the timeout occurs, SocketTimeoutException is thrown.
            int timeoutMs = 2000;   // 100 miliseconds
            sock.connect(sockaddr, timeoutMs);
            exists = true;
            LogUtils.e("IP Exists " + exists);
            printerStatus = 1;
//            ToastUtils.makeSnackToast(activity, "Printer connectable");
        } catch (IOException e) {
            printerStatus = 0;
            // Handle exception
            LogUtils.e("Cheking iP socket exception");
            e.printStackTrace();
            LogUtils.e(e.getMessage());
//            ToastUtils.makeSnackToast(activity, "Printer is not connected");
        }
    }

    public ZoneRichPrinter connectZonerich(ZoneRichPrinter zoneRichPrinter, Printer printer) {
//        if (printer != null && (printer.printer_model_name.contains("yoke4") || printer.printer_model_name.contains("pos 80") || printer.printer_model_name.contains("pos-80")))
//            zoneRichPrinter.connectPrinterForWifi(printer.ip);
        return zoneRichPrinter;
    }
}
