package com.example.epos_2021.activities;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.EditText;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.base.BaseActivity;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.models.Admin;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;

/**
 * Created by Amrish on 23-04-2021.
 */
public class AdminLoginActivity extends BaseActivity {

    EditText etUsername, etPassword;
    MaterialButton btnLogin;
    AlertDialog progressDialog;

    @Override
    protected void setXML() {
        super.setXML();
        setContentView(R.layout.activity_admin_login);
    }

    @Override
    protected void initViews() {
        super.initViews();
        progressDialog = CommonFunctions.customProgressDialog(this);
        etUsername = findViewById(R.id.input_username);
        etPassword = findViewById(R.id.input_password);
        btnLogin = findViewById(R.id.btn_login);
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setListener();
    }

    private void setListener() {
        btnLogin.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (isValid()) {
                    performAdminLogin();
                }
            }
        });
    }

    private boolean isValid() {
        if (Validators.isNullOrEmpty(etUsername.getText().toString())) {
            etUsername.setError("Please enter email address");
            etUsername.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etPassword.getText().toString())) {
            etPassword.setError("Please enter password");
            etPassword.requestFocus();
            return false;
        }
        return true;
    }

    private void performAdminLogin() {
        progressDialog.show();
        AndroidNetworking.post(ApiEndPoints.admin_login)
                .addQueryParameter("request_for", "admin")
                .addBodyParameter("username", etUsername.getText().toString())
                .addBodyParameter("password", etPassword.getText().toString())
                .build()
                .getAsObject(Admin.class, new ParsedRequestListener<Admin>() {
                    @Override
                    public void onResponse(Admin response) {
                        progressDialog.dismiss();
                        response.businesses = null;
                        myPreferences.saveEposAdminPassword(etPassword.getText().toString());
                        MyApp.getInstance().myPreferences.saveLoggedInAdmin(response);
                        startActivity(new Intent(AdminLoginActivity.this, SelectBusinessActivity.class));
                        finish();
                    }

                    @Override
                    public void onError(ANError anError) {
                        progressDialog.dismiss();
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(AdminLoginActivity.this, apiError.getMessage());
                        }
                    }
                });
    }
}
