package com.example.epos_2021.activities;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Paint;
import android.os.Bundle;
import android.os.Handler;
import android.os.PowerManager;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.bumptech.glide.Glide;
import com.example.epos_2021.BuildConfig;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.LogsAdapter;
import com.example.epos_2021.adapters.RecyclerMenuAdapter;
import com.example.epos_2021.base.BaseActivity;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.comman.printer.AidlUtil;
import com.example.epos_2021.comman.printer.SunmiPrinter;
import com.example.epos_2021.comman.printer.WifiPrinter;
import com.example.epos_2021.comman.printer.ZoneRichPrinter;
import com.example.epos_2021.fragment.ChatHomeFragment;
import com.example.epos_2021.fragment.ConfirmationDialogFragment;
import com.example.epos_2021.fragment.CustomersFragment;
import com.example.epos_2021.fragment.DineInFragment;
import com.example.epos_2021.fragment.EposOrdersHistoryFragment;
import com.example.epos_2021.fragment.HomeFragment;
import com.example.epos_2021.fragment.NewHomeFragment;
import com.example.epos_2021.fragment.NewOrderDialogFragment;
import com.example.epos_2021.fragment.NewReservationFragment;
import com.example.epos_2021.fragment.OrderTypeSelectionDialogFragment;
import com.example.epos_2021.fragment.OrdersAllFragment;
import com.example.epos_2021.fragment.ParkCallFragment;
import com.example.epos_2021.fragment.PaymentFragment;
import com.example.epos_2021.fragment.PushDialogFragment;
import com.example.epos_2021.fragment.ReservationListFragment;
import com.example.epos_2021.fragment.TablesLockingStatusFragment;
import com.example.epos_2021.fragment.VoucherHistoryFragment;
import com.example.epos_2021.merchant.fragments.MerchantSettingsFragment;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.models.CallLogs;
import com.example.epos_2021.models.Cashdraw;
import com.example.epos_2021.models.Customer;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.Printer;
import com.example.epos_2021.models.RecyclerMenuItem;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.models.User;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.online.fragments.OnlineOrderViewFragment;
import com.example.epos_2021.online.fragments.OnlineOrdersFragment;
import com.example.epos_2021.online.models.Counts;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.LogUtils;
import com.example.epos_2021.utils.OnSwipeTouchListener;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.material.card.MaterialCardView;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.firebase.messaging.FirebaseMessaging;
import com.google.gson.Gson;
import com.makeramen.roundedimageview.RoundedImageView;

import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;

import io.mpos.shared.helper.Log;

public class HomeActivity extends BaseActivity {

    private MaterialCardView cvCallerId;
    private LinearLayout llCallerId;
    private TextView tvCallerName, tvCallerNumber, tvCalleAddress;
    private ImageView imgCancel, imgAccept, imgHold;

    private TextView tvBusinessName, tvUsername, tvTime, tvRestaurantLogout;
    private LinearLayout llLogout;
    private RoundedImageView ivProfile;
    private TextView tvVersion;
    private boolean showLog = BuildConfig.DEBUG;
    private TextView tvDeviceId;
    Handler mHandler;
    FragmentManager manager;
    private CallLogs callLogs;
    private ZoneRichPrinter zoneRichPrinter;
    private WifiPrinter wifiPrinter;
    private SunmiPrinter sunmiPrinter;
    private Printer defaultPrinter;
    private SiteSetting webOrderSetting;
    private RecyclerView rvLogs, rvLeftMenu;
    private ArrayList<String> logs = new ArrayList<>();
    private LogsAdapter logsAdapter;
    private String destination;
    private ArrayList<RecyclerMenuItem> leftMenuItems = new ArrayList<>();
    private RecyclerMenuAdapter leftMenuAdapter;
    private String orderTypeId;
    private SiteSetting homeTheme;

    private MaterialCardView cvCollapse, cvDrawer, cvUserSection;
    private ImageView ivCollapse;
    private boolean isCollapsed = false;
    private TextView tvLogout;
    private FloatingActionButton fab;
    private LinearLayout llSideMain;


    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getIntent() != null) {
            destination = getIntent().getStringExtra("destination");
        }
    }

    @Override
    protected void setXML() {
        super.setXML();
        setContentView(R.layout.activity_home_new);
        manager = getSupportFragmentManager();

    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        myApp.initService();
        myApp.getRestaurantId();

        if (myApp.tiffintomLogo == null) {
            myApp.generateTiffintomLogoBitmap();
        }
        if (myApp.businessLogo == null) {
            myApp.generateBusinessLogoBitmap();
        }

        SiteSetting banquetDay = myApp.findSetting("banquet_day");
        SiteSetting banquetOrderType = myApp.findSetting("banquet_type");
        if (banquetDay != null && !Validators.isNullOrEmpty(banquetDay.value) && banquetDay.value.contains(myApp.current_day.toLowerCase())) {
            myApp.banquetDay = myApp.current_day;
            if (banquetOrderType != null && !Validators.isNullOrEmpty(banquetOrderType.value)) {
                myApp.banquetOrderTypes = banquetOrderType.value;
            }
            myApp.showBanquetPorducts = true;
        } else {
            myApp.banquetDay = null;
            myApp.showBanquetPorducts = false;
        }
        homeTheme = myApp.findSetting("home_theme");
        updateTime();
        setListeners();

        if (Validators.isNullOrEmpty(destination)) {
            changeHome();
            leftMenuAdapter.checkedId = "home";
        } else if (destination.equalsIgnoreCase("reservations")) {
            changeFragment(NewReservationFragment.getInstance(new Gson().fromJson(getIntent().getStringExtra("calllogs"), CallLogs.class)), true);
            leftMenuAdapter.checkedId = "reservations";
        } else {
            changeHome();
            leftMenuAdapter.checkedId = "home";
        }
        leftMenuAdapter.notifyDataSetChanged();
        wakeupScreen();
        fetchRestaurantDetails();
        changeHome();
    }

    private void fetchRestaurantDetails() {
        if (Validators.isNullOrEmpty(myApp.restaurant_id)) {
            LogUtils.e("HOME", "Restaurant ID not found");
            return;
        }
        LogUtils.e("HOME", "Fetching res details");
        AndroidNetworking.get(ApiEndPoints.restaurant_details + myApp.restaurant_id)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        myPreferences.saveRestaurant(response);
                        LogUtils.e("HOME", "Fetched res details");
                    }

                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e("HOME", "Error while fetching restaurant details in Splash");
                        anError.printStackTrace();
                    }
                });
    }

    private void changeHome() {
        //theme1 - home,theme2 - new home
        if (homeTheme != null && !Validators.isNullOrEmpty(homeTheme.value) && homeTheme.value.equalsIgnoreCase("theme2")) {
            changeFragment(NewHomeFragment.getInstance(), false);
        } else {
            changeFragment(HomeFragment.getInstance(), false);
        }
    }


    private void wakeupScreen() {
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        PowerManager powerManager = (PowerManager) getSystemService(POWER_SERVICE);
        PowerManager.WakeLock wakeLock = powerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK,
                "MyApp::MyWakelockTag");
        wakeLock.acquire();
    }

    @Override
    protected void initViews() {
        super.initViews();
        showLog = false;
        FirebaseMessaging.getInstance().subscribeToTopic("business_" + myPreferences.getLoggedInAdmin().selected_business.id).addOnCompleteListener(new OnCompleteListener<Void>() {
            @Override
            public void onComplete(@NonNull Task<Void> task) {
                Toast.makeText(HomeActivity.this, "Subscribe Success!", Toast.LENGTH_SHORT).show();
            }
        });
        webOrderSetting = myApp.findSetting("is_weborder");

        zoneRichPrinter = new ZoneRichPrinter(this);
        sunmiPrinter = new SunmiPrinter(this);
        wifiPrinter = WifiPrinter.getInstance();

        // Caller ID Related Starts
        cvCallerId = findViewById(R.id.cvCallerId);
        llCallerId = findViewById(R.id.llCallerId);

        tvCallerNumber = findViewById(R.id.tvCallerNumber);
        tvCallerName = findViewById(R.id.tvCallerName);
        tvCalleAddress = findViewById(R.id.tvCalleAddress);

        imgHold = findViewById(R.id.imgHold);
        imgCancel = findViewById(R.id.imgCancel);
        imgAccept = findViewById(R.id.imgAccept);

        // Caller ID Related Ends
        cvDrawer = findViewById(R.id.cvDrawer);
        cvCollapse = findViewById(R.id.cvCollapse);
        cvUserSection = findViewById(R.id.cvUserSection);
        ivCollapse = findViewById(R.id.ivCollapse);

        tvVersion = findViewById(R.id.tvVersion);
        ivProfile = findViewById(R.id.ivProfile);
        tvUsername = findViewById(R.id.tvUsername);
        tvDeviceId = findViewById(R.id.tvDeviceId);
        tvBusinessName = findViewById(R.id.tvBusinessName);
        tvTime = findViewById(R.id.tvTime);
        llLogout = findViewById(R.id.llLogout);
        tvLogout = findViewById(R.id.tvLogout);
        tvRestaurantLogout = findViewById(R.id.tvRestaurantLogout);
        fab = findViewById(R.id.fab);
        llSideMain = findViewById(R.id.llSideMain);
        tvRestaurantLogout.setPaintFlags(Paint.UNDERLINE_TEXT_FLAG);
        String currentTime = new SimpleDateFormat("h:mm a", Locale.getDefault()).format(new Date());
        tvTime.setText(currentTime);

        rvLeftMenu = findViewById(R.id.rvLeftMenu);
        rvLogs = findViewById(R.id.rvLogs);
        if (showLog)
            rvLogs.setVisibility(View.VISIBLE);
        else rvLogs.setVisibility(View.GONE);

        logsAdapter = new LogsAdapter(logs, (position, data) -> {
            ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Clear", null, 1, "Clear", "Cancel");
            confirmationDialogFragment.show(getSupportFragmentManager(), "confirm");
            confirmationDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String && ((String) o).equalsIgnoreCase("confirm")) {
                    logs.clear();
                    logsAdapter.notifyDataSetChanged();
                }
            });
        });
        rvLogs.setLayoutManager(new LinearLayoutManager(this, RecyclerView.VERTICAL, false));
        rvLogs.setAdapter(logsAdapter);

        leftMenuAdapter = new RecyclerMenuAdapter(leftMenuItems, true, (position, data) -> {
            RecyclerMenuItem model = (RecyclerMenuItem) data;
            handleMenuClick(model);

        });
        rvLeftMenu.setLayoutManager(new LinearLayoutManager(this, RecyclerView.VERTICAL, false));
        rvLeftMenu.setAdapter(leftMenuAdapter);

        isCollapsed = myApp.drawerCollapsed;
        collapseAction();
    }


    private void handleMenuClick(RecyclerMenuItem model) {
        if (model.id.equalsIgnoreCase("home")) {
            changeHome();
        }
        if (model.id.equalsIgnoreCase("new_order")) {
            createNewOrder();
        }
        if (model.id.equalsIgnoreCase("active_orders")) {
            changeFragment(OrdersAllFragment.getInstance(orderTypeId));
            orderTypeId = null;
        }
        if (model.id.equalsIgnoreCase("online_orders")) {
            changeFragment(OnlineOrdersFragment.getInstance("from_home"));
        }
        if (model.id.equalsIgnoreCase("tables")) {
            changeFragment(DineInFragment.getInstance());
        }
        if (model.id.equalsIgnoreCase("table_status")) {
            changeFragment(TablesLockingStatusFragment.getInstance());
        }
        if (model.id.equalsIgnoreCase("reservations")) {
            changeFragment(ReservationListFragment.getInstance());
        }
        if (model.id.equalsIgnoreCase("order_history")) {
            changeFragment(EposOrdersHistoryFragment.getInstance(null, false));
        }
        if (model.id.equalsIgnoreCase("cash_drawer")) {
            openCashDrawer();
        }
        if (model.id.equalsIgnoreCase("messages")) {
            changeFragment(new ChatHomeFragment());
        }
        if (model.id.equalsIgnoreCase("payments")) {
            changeFragment(PaymentFragment.getInstance());
        }
        if (model.id.equalsIgnoreCase("customers")) {
            changeFragment(CustomersFragment.getInstance());
        }
        if (model.id.equalsIgnoreCase("vouchers")) {
            changeFragment(VoucherHistoryFragment.getInstance());
        }
        if (model.id.equalsIgnoreCase("settings")) {
            changeFragment(MerchantSettingsFragment.getInstance());
        }
        if (model.id.equalsIgnoreCase("park_calls")) {
            changeFragment(ParkCallFragment.getInstance());
        }
        if (model.id.equalsIgnoreCase("push_data")) {
            startActivityForResult(new Intent(this, UploadOrdersActivity.class), Constants.CODE_REFRESH);
        }
        if (model.id.equalsIgnoreCase("pull_data")) {
            finishAffinity();
            startActivityForResult(new Intent(this, PullActivity.class).putExtra("destination", "home"), Constants.CODE_REFRESH);
        }
        if (model.id.equalsIgnoreCase("pull_customers")) {
            finishAffinity();
            startActivityForResult(new Intent(this, PullCustomersActivity.class).putExtra("destination", "home"), Constants.CODE_REFRESH);
        }
    }

    private void createNewOrder() {
        Fragment openedFragment = getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment);
        String orderTypeId = null;
        if (openedFragment != null && openedFragment instanceof OrdersAllFragment) {
            orderTypeId = MyApp.openedOrderTypeId;
            if (Validators.isNullOrEmpty(MyApp.openedOrderTypeId) ||
                    MyApp.openedOrderTypeId.equalsIgnoreCase("0") || MyApp.openedOrderTypeId.equalsIgnoreCase("6")) {
                openOrderTypeSelectionDialog(null);
            } else {
                if(orderTypeId!=null)
                     openNewOrderDialog(orderTypeId, null);
            }
        } else {
            openOrderTypeSelectionDialog(null);
        }
    }

    private void openOrderTypeSelectionDialog(CallLogs callLogs) {
        OrderTypeSelectionDialogFragment orderTypeSelectionDialog = OrderTypeSelectionDialogFragment.getInstance(callLogs);
        orderTypeSelectionDialog.show(getSupportFragmentManager(), "order_type");
        orderTypeSelectionDialog.setDialogDismissListener(o -> {
            if (o instanceof String) {
                String orderTypeId = (String) o;
                if(orderTypeId!=null)
                     openNewOrderDialog(orderTypeId, callLogs);

            } else if (o instanceof CallLogs) {
                changeFragment(NewReservationFragment.getInstance((CallLogs) o));
            }
        });
    }

    private void openNewOrderDialog(String orderTypeId, CallLogs callLogs) {
        try{
            NewOrderDialogFragment newOrderDialogFragment = NewOrderDialogFragment.getInstance(callLogs, orderTypeId);
            newOrderDialogFragment.show(getSupportFragmentManager(), "new_order");
            newOrderDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof Order) {
                    startActivity(new Intent(HomeActivity.this, NewOrderActivity.class)
                            .putExtra("order", new Gson().toJson(o))
                            .putExtra("fetch_delivery_charge", ((Order) o).order_type_id.equalsIgnoreCase("3")));
                } else if (o instanceof CallLogs) {
                    changeFragment(NewReservationFragment.getInstance((CallLogs) o));
                }
            });
        }
        catch (Exception e){ }
    }

    private void setListeners() {
        llLogout.setOnClickListener(v -> askForLogout(false));
        tvRestaurantLogout.setOnClickListener(v -> askForLogout(true));

        llCallerId.setOnTouchListener(new OnSwipeTouchListener(HomeActivity.this) {
            @Override
            public void onSwipeLeft() {
                super.onSwipeLeft();
                changeStatusCallerid("parked");
            }

            @Override
            public void onSwipeRight() {
                super.onSwipeRight();
                changeStatusCallerid("parked");
            }

            @Override
            public void onSwipeTop() {
                super.onSwipeTop();
                changeStatusCallerid("parked");
            }

            @Override
            public void onSwipeBottom() {
                super.onSwipeBottom();
                changeStatusCallerid("parked");
            }
        });

        imgAccept.setOnClickListener(v -> changeStatusCallerid("accepted"));
        imgCancel.setOnClickListener(v -> changeStatusCallerid("rejected"));
        imgHold.setOnClickListener(v -> changeStatusCallerid("parked"));

        cvCollapse.setOnClickListener(v -> {
            isCollapsed = !isCollapsed;
            collapseAction();
        });
        fab.setOnClickListener(v -> {
            isCollapsed = !isCollapsed;
            collapseAction();
        });
    }

    private void collapseAction() {
        leftMenuAdapter.collapsed = isCollapsed;
        myApp.drawerCollapsed = isCollapsed;
        leftMenuAdapter.notifyDataSetChanged();
        if (isCollapsed) {
            ivCollapse.setRotation(0);
            fab.setRotation(0);
            tvBusinessName.setVisibility(View.GONE);
            tvTime.setVisibility(View.GONE);
            tvDeviceId.setVisibility(View.GONE);
            tvVersion.setVisibility(View.GONE);
            tvLogout.setVisibility(View.GONE);
            tvUsername.setVisibility(View.GONE);
            tvRestaurantLogout.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);
            cvDrawer.setLayoutParams(new RelativeLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
            ViewGroup.MarginLayoutParams cardViewMarginParams = (ViewGroup.MarginLayoutParams) cvDrawer.getLayoutParams();
            cardViewMarginParams.setMargins(0, 0, 8, 0);
            cvDrawer.requestLayout();
            llSideMain.setLayoutParams(new RelativeLayout.LayoutParams(getResources().getDimensionPixelSize(R.dimen.width_110), ViewGroup.LayoutParams.MATCH_PARENT));

            RelativeLayout.LayoutParams layoutParams =
                    (RelativeLayout.LayoutParams) ivProfile.getLayoutParams();
            layoutParams.addRule(RelativeLayout.CENTER_HORIZONTAL, RelativeLayout.TRUE);
            ivProfile.setLayoutParams(layoutParams);
        } else {
            ivCollapse.setRotation(180);
            fab.setRotation(180);
            tvBusinessName.setVisibility(View.VISIBLE);
            tvTime.setVisibility(View.VISIBLE);
            tvDeviceId.setVisibility(View.VISIBLE);
            tvVersion.setVisibility(View.VISIBLE);
            tvLogout.setVisibility(View.VISIBLE);
            tvUsername.setVisibility(View.VISIBLE);
            tvRestaurantLogout.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_START);
            cvDrawer.setLayoutParams(new RelativeLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
            ViewGroup.MarginLayoutParams cardViewMarginParams = (ViewGroup.MarginLayoutParams) cvDrawer.getLayoutParams();
            cardViewMarginParams.setMargins(0, 0, 8, 0);
            cvDrawer.requestLayout();
            llSideMain.setLayoutParams(new RelativeLayout.LayoutParams(getResources().getDimensionPixelSize(R.dimen.width_180), ViewGroup.LayoutParams.MATCH_PARENT));

            RelativeLayout.LayoutParams layoutParams = (RelativeLayout.LayoutParams) ivProfile.getLayoutParams();
            layoutParams.removeRule(RelativeLayout.CENTER_HORIZONTAL);
            ivProfile.setLayoutParams(layoutParams);
        }
    }

    BroadcastReceiver callReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            callLogs = new Gson().fromJson(intent.getStringExtra("calllogs"), CallLogs.class);

            cvCallerId.setVisibility(View.VISIBLE);
            if (callLogs.customer != null) {
                Customer customer = callLogs.customer;
                if (!Validators.isNullOrEmpty(customer.name)) {
                    tvCallerName.setText(customer.name);
                } else {
                    tvCallerName.setText("");
                }
                StringBuilder sb = new StringBuilder();
                if (!Validators.isNullOrEmpty(customer.house_no)) {
                    sb.append(customer.house_no).append(", ");
                }
                if (!Validators.isNullOrEmpty(customer.street)) {
                    sb.append(customer.street).append("\n");
                }
                if (!Validators.isNullOrEmpty(customer.postcode)) {
                    sb.append(customer.postcode).append("\n");
                }

                if (!Validators.isNullOrEmpty(customer.city)) {
                    sb.append(customer.city).append("\n");
                }
                if (!Validators.isNullOrEmpty(customer.state)) {
                    sb.append(customer.state).append("\n");
                }
                if (!Validators.isNullOrEmpty(customer.distance)) {
                    sb.append(customer.distance).append(" Miles away");
                }

                if (!Validators.isNullOrEmpty(sb.toString())) {
                    tvCalleAddress.setText(sb.toString());
                } else {
                    tvCalleAddress.setText("");
                }

            } else {
                tvCallerName.setText("");
                tvCalleAddress.setText("");
            }
            tvCallerNumber.setText(callLogs.number);
        }
    };
    BroadcastReceiver logsReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (showLog) {
                logs.add(intent.getStringExtra("log"));
                logsAdapter.notifyDataSetChanged();
            }
        }
    };


    BroadcastReceiver onlineOrdersNotificationReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getBooleanExtra("refresh", false)) {
                Counts allCounts = new Gson().fromJson(intent.getStringExtra("counts"), Counts.class);
                setCounts(allCounts);
            }
        }
    };

    BroadcastReceiver fragmentChangeReciver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            orderTypeId = intent.getStringExtra("order_type");
            String destination = intent.getStringExtra("destination");

            if (destination.equalsIgnoreCase("active_orders")) {
                if (orderTypeId.equalsIgnoreCase("6")) {
                    leftMenuAdapter.checkedId = "online_orders";
                    handleMenuClick(leftMenuItems.get(3));
                } else {
                    leftMenuAdapter.checkedId = "active_orders";
                    handleMenuClick(leftMenuItems.get(2));
                }
            } else if (destination.equalsIgnoreCase("order_history")) {
                leftMenuAdapter.checkedId = "order_history";
            } else if (destination.equalsIgnoreCase("tables")) {
                leftMenuAdapter.checkedId = "tables";
            } else if (destination.equalsIgnoreCase("reservation_history")) {
                leftMenuAdapter.checkedId = "reservations";
            } else if (destination.equalsIgnoreCase("voucher_history")) {
                leftMenuAdapter.checkedId = "vouchers";
            }
            leftMenuAdapter.notifyDataSetChanged();
        }
    };

    private void notifyMenuItem(String s) {
        leftMenuAdapter.checkedId = s;
        leftMenuAdapter.notifyDataSetChanged();
    }

    private void askForLogout(boolean isAdmin) {
        ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Logout", "Are you sure you want to logout?", 1, "Confirm", "Cancel");
        confirmationDialogFragment.show(getSupportFragmentManager(), "logout");
        confirmationDialogFragment.setDialogDismissListener(o -> {
            if (o instanceof String) {
                if (((String) o).equalsIgnoreCase("confirm")) {
                    Log.e("isAdmin",""+isAdmin);
                    if (isAdmin) {
                        logout_from_app();
                    } else {
                        performUserLogout();
                    }
                }
            }
        });
    }

    private void performUserLogout() {

        String userId = myPreferences.getLoggedInUser().id;
        String server_URL = ApiEndPoints.users+userId+ApiEndPoints.user_logout;
        Log.e("Data",""+server_URL);

        AndroidNetworking.get(server_URL)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        Log.e("Data",""+response.toString());
                        doUserLogout();
                    }
                    @Override
                    public void onError(ANError anError) {
                        Log.e("Data",""+anError.getMessage()+""+anError.getErrorCode());
                        anError.printStackTrace();
                        doUserLogout();
                    }
                });
    }

    private void logout_from_app() {
        startActivityForResult(new Intent(this, UploadOrdersActivity.class).putExtra("admin_logout", true), Constants.CODE_ADMIN_LOGOUT);
        finish();
    }

    private void doUserLogout() {
        myPreferences.saveLoggedInUser(null);
        myPreferences.saveUserTokens(null, null);
        startActivity(new Intent(HomeActivity.this, SplashActivity.class));
        finish();
    }

    private void updateTime() {
        mHandler = new Handler();

        mHandler.postDelayed(new Runnable() {
            @Override
            public void run() {
                String currentTime = new SimpleDateFormat("hh:mm a", Locale.getDefault()).format(new Date());
                tvTime.setText(currentTime);
                mHandler.postDelayed(this, 1000);
            }
        }, 10);
    }

    private void setUpData() {
        leftMenuItems.clear();
        if (myPreferences.getLoggedInUser() != null) {
            tvUsername.setText(myPreferences.getLoggedInUser().username);
        }
        if (myPreferences.getLoggedInAdmin() != null) {
            tvBusinessName.setText(myPreferences.getLoggedInAdmin().selected_business.name);
        }
        tvDeviceId.setText("Device: " + myPreferences.getRegisteredDevice().name);
        tvVersion.setText("Version " + BuildConfig.VERSION_NAME);

        SiteSetting logoSetting = myApp.findSetting("logo");
        if (logoSetting != null && !Validators.isNullOrEmpty(logoSetting.value)) {
            Glide.with(this)
                    .load(logoSetting.value)
                    .placeholder(R.drawable.user_profile_image_placeholder)
                    .error(R.drawable.user_profile_image_placeholder)
                    .into(ivProfile);
        } else {
            Glide.with(this)
                    .load(R.drawable.user_profile_image_placeholder)
                    .placeholder(R.drawable.user_profile_image_placeholder)
                    .error(R.drawable.user_profile_image_placeholder)
                    .into(ivProfile);
        }

        RecyclerMenuItem model = new RecyclerMenuItem();
        model.title = "Home";
        model.id = "home";
        model.drawable = R.drawable.ic_baseline_dashboard_24;
        model.isCheckable = true;
        leftMenuItems.add(model);

        model = new RecyclerMenuItem();
        model.title = "New Order";
        model.id = "new_order";
        model.drawable = R.drawable.ic_baseline_add_circle_24;
        leftMenuItems.add(model);

        model = new RecyclerMenuItem();
        model.title = "Active orders";
        model.id = "active_orders";
        model.isCheckable = true;
        model.drawable = R.drawable.ic_baseline_receipt_24;
        leftMenuItems.add(model);

        if (!Validators.isNullOrEmpty(myApp.restaurant_id) && webOrderSetting != null && (webOrderSetting.value.equalsIgnoreCase("true") || webOrderSetting.value.equalsIgnoreCase("1")) && MyApp.userPermission.web_orders.actions.list) {
            model = new RecyclerMenuItem();
            model.title = "Online orders";
            model.id = "online_orders";
            model.isCheckable = true;
            model.drawable = R.drawable.icon_tiffinicon;
            leftMenuItems.add(model);
        }

        SiteSetting dineinSetting = myApp.findSetting("is_dinein");
        boolean isDinein = false;
        if (dineinSetting != null && (dineinSetting.value.equals("true") || dineinSetting.value.equals("1")) && (MyApp.userPermission != null && MyApp.userPermission.dinein.actions.list)) {
            isDinein = true;
            model = new RecyclerMenuItem();
            model.title = "Tables";
            model.id = "tables";
            model.isCheckable = true;
            model.drawable = R.drawable.icon_tables;
            leftMenuItems.add(model);
        }

        SiteSetting reservationSetting = myApp.findSetting("is_reservation");
        if (reservationSetting != null && (reservationSetting.value.equals("true") || reservationSetting.value.equals("1"))) {
            if (MyApp.userPermission != null && MyApp.userPermission.reservation.actions.list) {
                model = new RecyclerMenuItem();
                model.title = "Reservations";
                model.id = "reservations";
                model.isCheckable = true;
                model.drawable = R.drawable.reservation_icon;
                leftMenuItems.add(model);
            }
        }
        if (MyApp.userPermission != null && MyApp.userPermission.order_history.actions.list) {
            model = new RecyclerMenuItem();
            model.title = "Order history";
            model.id = "order_history";
            model.isCheckable = true;
            model.drawable = R.drawable.ic_baseline_history_24;
            leftMenuItems.add(model);
        }
        if ((MyApp.userPermission != null && MyApp.userPermission.cash_draw.actions.list)) {
            model = new RecyclerMenuItem();
            model.title = "Cash drawer";
            model.id = "cash_drawer";
            model.drawable = R.drawable.icon_pound_sign;
            leftMenuItems.add(model);
        }
        if (MyApp.userPermission != null && MyApp.userPermission.vouchers.actions.list) {
            model = new RecyclerMenuItem();
            model.title = "Vouchers";
            model.id = "vouchers";
            model.isCheckable = true;
            model.drawable = R.drawable.ic_baseline_payments_24;
            leftMenuItems.add(model);
        }
        if (!Validators.isNullOrEmpty(myApp.restaurant_id) && webOrderSetting != null && (webOrderSetting.value.equalsIgnoreCase("true") || webOrderSetting.value.equalsIgnoreCase("1")) && MyApp.userPermission != null && MyApp.userPermission.web_orders.actions.list && MyApp.userPermission.send_message.actions.list) {
            model = new RecyclerMenuItem();
            model.title = "Messages";
            model.id = "messages";
            model.isCheckable = true;
            model.drawable = R.drawable.ic_baseline_forum_24;
            leftMenuItems.add(model);
        }
        model = new RecyclerMenuItem();
        model.title = "Payments";
        model.id = "payments";
        model.isCheckable = true;
        model.drawable = R.drawable.ic_baseline_payment_24;
        leftMenuItems.add(model);
        if (myPreferences.getCallerIdEnableStatus()) {
            model = new RecyclerMenuItem();
            model.title = "Park Calls";
            model.id = "park_calls";
            model.isCheckable = true;
            model.drawable = R.drawable.ic_baseline_call_24;
            leftMenuItems.add(model);
        }
        if (MyApp.userPermission != null && MyApp.userPermission.customers.actions.list) {
            model = new RecyclerMenuItem();
            model.title = "Customers";
            model.id = "customers";
            model.isCheckable = true;
            model.drawable = R.drawable.user_profile_image_placeholder;
            leftMenuItems.add(model);
        }
        if (MyApp.userPermission != null && MyApp.userPermission.dinein.actions.list && isDinein) {
            model = new RecyclerMenuItem();
            model.title = "Table status";
            model.id = "table_status";
            model.isCheckable = true;
            model.drawable = R.drawable.icon_tables;
            leftMenuItems.add(model);
        }
        if (MyApp.userPermission != null && MyApp.userPermission.setting.actions.list) {
            model = new RecyclerMenuItem();
            model.title = "Settings";
            model.id = "settings";
            model.isCheckable = true;
            model.drawable = R.drawable.settings;
            leftMenuItems.add(model);
        }

        model = new RecyclerMenuItem();
        model.title = "Pull data";
        model.id = "pull_data";
        model.drawable = R.drawable.ic_baseline_arrow_downward_24;
        leftMenuItems.add(model);

        model = new RecyclerMenuItem();
        model.title = "Pull customers";
        model.id = "pull_customers";
        model.drawable = R.drawable.ic_baseline_arrow_downward_24;
        leftMenuItems.add(model);
        model = new RecyclerMenuItem();
        model.title = "Push data";
        model.id = "push_data";
        model.drawable = R.drawable.ic_baseline_arrow_upward_24;
        leftMenuItems.add(model);

        leftMenuAdapter.notifyDataSetChanged();
    }

    private void changeFragment(Fragment fragment) {
        FragmentManager manager = getSupportFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.nav_host_fragment, fragment);
        transaction.addToBackStack("can_go_back");
        transaction.commit();
        invalidateOptionsMenu();
    }

    private void changeFragment(Fragment fragment, boolean enableBackPress) {
        FragmentManager manager = getSupportFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.nav_host_fragment, fragment);
        if (enableBackPress) {
            transaction.addToBackStack("can_go_back");
        }
        transaction.commit();
        invalidateOptionsMenu();
    }

    private void openCashDrawer() {

        Cashdraw cashdraw = new Cashdraw();
        cashdraw.user_id = myApp.myPreferences.getLoggedInUser().id;
        cashdraw.device_id = myApp.myPreferences.getRegisteredDevice().id;
        cashdraw.value = "1";
        defaultPrinter = myPreferences.getDefaultPrinter();
        if (defaultPrinter != null && defaultPrinter.printer_model_name != null) {

            if (defaultPrinter.printer_model_name.toLowerCase().contains("88 h")) {
                zoneRichPrinter.connectPrinter(defaultPrinter.ip);
                zoneRichPrinter.openCashDrawer();
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("sunmi") || defaultPrinter.printer_model_name.toLowerCase().contains("t2")) {
                sunmiPrinter.openCashDrawer();
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("pos-80") || defaultPrinter.printer_model_name.toLowerCase().contains("pos 80") || defaultPrinter.printer_model_name.toLowerCase().contains("yoke4")) {
                wifiPrinter.connect(defaultPrinter.ip);
                CommonFunctions.functionThatDelay(200);
                wifiPrinter.openCashDrawer();
            }
        } else {
            ToastUtils.makeLongToast(HomeActivity.this, "No printer model found");
        }

        AndroidNetworking.post(ApiEndPoints.cashdraw_log)
                .addApplicationJsonBody(cashdraw)
                .build()
                .getAsObject(Cashdraw.class, new ParsedRequestListener<Cashdraw>() {
                    @Override
                    public void onResponse(Cashdraw response) {
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            Log.e("apiError", anError.getErrorCode() + " " + anError.getMessage());
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(HomeActivity.this, apiError.getMessage());
                        } else {
                            anError.printStackTrace();
                            ToastUtils.makeSnackToast(HomeActivity.this, "Something went wrong!");
                        }
                    }
                });

    }

    @Override
    public void onBackPressed() {
        Fragment openedFragment = getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment);

        if (openedFragment instanceof NewHomeFragment || openedFragment instanceof HomeFragment) {
            finishAffinity();
        } else {
            if (manager.getBackStackEntryCount() > 0) {
                manager.popBackStack();
                manager.addOnBackStackChangedListener(() -> {

                    Fragment latestChangedFragment = getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment);
                    if (latestChangedFragment instanceof OrdersAllFragment) {
                        notifyMenuItem("active_orders");
                    } else if (latestChangedFragment instanceof PaymentFragment) {
                        notifyMenuItem("payments");
                    } else if (latestChangedFragment instanceof OnlineOrdersFragment) {
                        notifyMenuItem("online_orders");
                    } else if (latestChangedFragment instanceof TablesLockingStatusFragment) {
                        notifyMenuItem("table_status");
                    } else if (latestChangedFragment instanceof DineInFragment) {
                        notifyMenuItem("tables");
                    } else if (latestChangedFragment instanceof VoucherHistoryFragment) {
                        notifyMenuItem("vouchers");
                    } else if (latestChangedFragment instanceof ReservationListFragment) {
                        notifyMenuItem("reservations");
                    } else if (latestChangedFragment instanceof CustomersFragment) {
                        notifyMenuItem("customers");
                    } else if (latestChangedFragment instanceof HomeFragment || latestChangedFragment instanceof NewHomeFragment) {
                        notifyMenuItem("home");
                    }

                });
            } else {
                finishAffinity();
            }
        }
    }


    private void changeStatusCallerid(String status) {
        if (callLogs == null) {
            cvCallerId.setVisibility(View.GONE);
            return;
        }
        callLogs.status = status;
        cvCallerId.setVisibility(View.GONE);

        if (callLogs.status.equalsIgnoreCase("accepted")) {
            openOrderTypeSelectionDialog(callLogs);
        }
        new Thread(() -> {
            callLogs.uploadable = true;
            appDatabase.calllogDao().insert(callLogs);
            callLogs = null;
        }).start();
    }

    private void setCounts(Counts allCounts) {
        for (RecyclerMenuItem menuItem : leftMenuItems) {
            if (menuItem.id.equalsIgnoreCase("reservations")) {
                menuItem.count = allCounts.pending_bookings;
            }
            if (menuItem.id.equalsIgnoreCase("messages")) {
                menuItem.count = allCounts.unread_chat;
            }
            if (menuItem.id.equalsIgnoreCase("online_orders")) {
                menuItem.count = allCounts.pending_orders;
            }
        }
        leftMenuAdapter.notifyDataSetChanged();

    }

    @Override
    protected void onResume() {
        super.onResume();
        MyApp.userPermission = myPreferences.getEposUserPermission();
        setUpData();
        checkPrinterConnectivity();
        LocalBroadcastManager.getInstance(HomeActivity.this).registerReceiver(push_receiver, new IntentFilter(Constants.PUSH_NOTIFICATION));
        LocalBroadcastManager.getInstance(this).registerReceiver(callReceiver, new IntentFilter("call_received"));
        LocalBroadcastManager.getInstance(this).registerReceiver(logsReceiver, new IntentFilter("log"));
        LocalBroadcastManager.getInstance(this).registerReceiver(onlineOrdersNotificationReceiver, new IntentFilter(Constants.NOTIFICATION));
        LocalBroadcastManager.getInstance(this).registerReceiver(fragmentChangeReciver, new IntentFilter("order_fragment_change"));

    }

    private void checkPrinterConnectivity() {
        defaultPrinter = myPreferences.getDefaultPrinter();
        if (defaultPrinter != null) {
            new Thread(() -> {
                if (!Validators.isNullOrEmpty(defaultPrinter.printer_model_name) && (defaultPrinter.printer_model_name.toLowerCase().contains("88 h"))) {
                    myApp.checkPrinterIP(defaultPrinter.ip, 9100);
                } else if (!Validators.isNullOrEmpty(defaultPrinter.printer_model_name) && defaultPrinter.printer_model_name.toLowerCase().contains("sunmi") || defaultPrinter.printer_model_name.toLowerCase().contains("t2")) {
                    if (!AidlUtil.getInstance().isConnect())
                        AidlUtil.getInstance().connectPrinterService(this);
                } else if (!Validators.isNullOrEmpty(defaultPrinter.printer_model_name) && defaultPrinter.printer_model_name.toLowerCase().contains("pos-80") || defaultPrinter.printer_model_name.toLowerCase().contains("pos 80") || defaultPrinter.printer_model_name.toLowerCase().contains("yoke4")) {
//                    myApp.checkPrinterIP(HomeActivity.this, defaultPrinter.ip, 9100, defaultPrinter.printer_model_name.toLowerCase());
                }
            }).start();
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
//        ceBridge.Disconnect();
//        LogUtils.e("ON PAUSE");
        try {
            if (push_receiver != null) {
                LocalBroadcastManager.getInstance(HomeActivity.this).unregisterReceiver(push_receiver);
            }
            if (callReceiver != null) {
                LocalBroadcastManager.getInstance(this).unregisterReceiver(callReceiver);
            }
            if (logsReceiver != null)
                LocalBroadcastManager.getInstance(this).unregisterReceiver(logsReceiver);

            if (onlineOrdersNotificationReceiver != null) {
                LocalBroadcastManager.getInstance(this).unregisterReceiver(onlineOrdersNotificationReceiver);
            }
            if (fragmentChangeReciver != null) {
                LocalBroadcastManager.getInstance(this).unregisterReceiver(fragmentChangeReciver);
            }
        } catch (Exception e) {

        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        LogUtils.e("ON DESTROY");
    }

    BroadcastReceiver push_receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getBooleanExtra("push", true)) {
                String title = intent.getStringExtra("title");
                String message = intent.getStringExtra("message");
                String order_id = intent.getStringExtra("order_id");
                String order_type = intent.getStringExtra("order_type");
                String payment_status = intent.getStringExtra("payment_status");
                PushDialogFragment pushDialogFragment = PushDialogFragment.getInstance(title, message, order_id, payment_status, order_type);
                pushDialogFragment.show(getSupportFragmentManager(), "push");
                pushDialogFragment.setDialogDismissListener(o -> {
                    if (o instanceof String) {
                        int orderType = -1;
                        if (Validators.isNullOrEmpty(order_type)) {
                            orderType = 0;
                        } else if (order_type.equalsIgnoreCase("delivery")) {
                            orderType = 1;
                        } else if (order_type.equalsIgnoreCase("pickup")) {
                            orderType = 2;
                        }
                        changeFragment(OnlineOrderViewFragment.getInstance(order_id, orderType), true);
                    }
                });
            }
        }
    };


}
