package com.example.epos_2021.activities;

import android.content.Intent;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.appcompat.app.ActionBarDrawerToggle;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.AppCompatButton;
import androidx.appcompat.widget.SwitchCompat;
import androidx.core.view.GravityCompat;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.bumptech.glide.Glide;
import com.example.epos_2021.BuildConfig;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.MiniUserAdapter;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.daos.AppDatabase;
import com.example.epos_2021.fragment.ConfirmationDialogFragment;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.models.EposUserPermission;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.models.User;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.online.fragments.EnterPasswordDialogFragment;
import com.example.epos_2021.online.fragments.RejectReasonDialogFragment;
import com.example.epos_2021.storageutils.MyPreferences;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;
import com.google.android.material.chip.Chip;
import com.google.firebase.messaging.FirebaseMessaging;
import com.google.gson.Gson;
import com.makeramen.roundedimageview.RoundedImageView;

import java.io.File;
import java.util.ArrayList;

public class LoginActivity extends AppCompatActivity {
    private RecyclerView rvUsers;
    private EditText etUsername, etPassword;
    private MaterialButton btn1, btn2, btn3, btn4, btn5, btn6, btn7, btn8, btn9, btn0, btnDot, btnClear;

    private AppCompatButton btnLogin;
    private LinearLayout llSettings;
    private TextView tvOrderSyncMode, tvCallerId,tvRemoteDevice;
    private Chip chipBack;
    private RoundedImageView ivProfile;
    private TextView tvBusinessName, tvResLogout, tvVersion;

    private ArrayList<User> users = new ArrayList<>();
    private MiniUserAdapter userAdapter;
    AlertDialog progressDialog;
    MyApp myApp = MyApp.getInstance();
    MyPreferences myPreferences = myApp.myPreferences;
    AppDatabase appDatabase = myApp.appDatabase;
    DrawerLayout drawerLayout;
    ActionBarDrawerToggle actionBarDrawerToggle;
    SwitchCompat switchCallerId;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login_copy);
        initViews();
        fetchMiniUsers();
        setListeners();

    }

    private void initViews() {
        progressDialog = CommonFunctions.customProgressDialog(this);
        rvUsers = findViewById(R.id.rvUsers);
        etUsername = findViewById(R.id.input_username);
        etPassword = findViewById(R.id.input_password);
        llSettings = findViewById(R.id.llSettings);
        btn1 = findViewById(R.id.btn1);
        btn2 = findViewById(R.id.btn2);
        btn3 = findViewById(R.id.btn3);
        btn4 = findViewById(R.id.btn4);
        btn5 = findViewById(R.id.btn5);
        btn6 = findViewById(R.id.btn6);
        btn7 = findViewById(R.id.btn7);
        btn8 = findViewById(R.id.btn8);
        btn9 = findViewById(R.id.btn9);
        btn0 = findViewById(R.id.btn0);
        btnDot = findViewById(R.id.btnDot);
        btnLogin = findViewById(R.id.btn_login);
        btnClear = findViewById(R.id.btnClear);
        drawerLayout = findViewById(R.id.drawer_layout);
        tvOrderSyncMode = findViewById(R.id.tvOrderSyncMode);
        tvRemoteDevice = findViewById(R.id.tvRemoteDevice);
        chipBack = findViewById(R.id.chipBack);
        switchCallerId = findViewById(R.id.switchCallerId);
        tvCallerId = findViewById(R.id.tvCallerId);
        ivProfile = findViewById(R.id.ivProfile);
        tvBusinessName = findViewById(R.id.tvBusinessName);
        tvResLogout = findViewById(R.id.tvRestaurantLogout);
        tvVersion = findViewById(R.id.tvVersion);

        tvResLogout.setPaintFlags(Paint.UNDERLINE_TEXT_FLAG);
        tvVersion.setText("Version " + BuildConfig.VERSION_NAME);
        userAdapter = new MiniUserAdapter(this, users, (position, data) -> {
            User model = (User) data;
            etUsername.setText(model.username);
            etUsername.setSelection(etUsername.length());
            etPassword.setText("");
        });
        rvUsers.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));
        rvUsers.setAdapter(userAdapter);
        switchCallerId.setChecked(myPreferences.getCallerIdEnableStatus());

        if (myPreferences.getLoggedInAdmin() != null) {
            tvBusinessName.setText(myPreferences.getLoggedInAdmin().selected_business.name);
        }
        SiteSetting logoSetting = myApp.findSetting("logo");
        if (logoSetting != null && !Validators.isNullOrEmpty(logoSetting.value)) {
            Glide.with(this)
                    .load(logoSetting.value)
                    .placeholder(R.drawable.user_profile_image_placeholder)
                    .error(R.drawable.user_profile_image_placeholder)
                    .into(ivProfile);
        } else {
            Glide.with(this)
                    .load(R.drawable.user_profile_image_placeholder)
                    .placeholder(R.drawable.user_profile_image_placeholder)
                    .error(R.drawable.user_profile_image_placeholder)
                    .into(ivProfile);
        }

    }

    private void setListeners() {
        etPassword.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (editable.toString().length() != 0) {
                    etPassword.setSelection(etPassword.length());
                }
            }
        });

        btn1.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "1"));
        btn2.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "2"));
        btn3.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "3"));
        btn4.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "4"));
        btn5.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "5"));
        btn6.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "6"));
        btn7.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "7"));
        btn8.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "8"));
        btn9.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "9"));
        btn0.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "0"));
        btnDot.setOnClickListener(v -> etPassword.setText(etPassword.getText() + "."));

        btnClear.setOnClickListener(v -> {
            etPassword.setText("");
        });

        switchCallerId.setOnCheckedChangeListener((compoundButton, b) -> {
            myPreferences.saveCallerIdEnableStatus(b);
        });

        btnLogin.setOnClickListener(v -> {
            if (isValid()) {
                if (myApp.isConnected(this))
                    performUserLogin();
                else
                    offlineLogin();
            }
        });
        llSettings.setOnClickListener(v -> {
            if (!drawerLayout.isDrawerOpen(GravityCompat.START))
                drawerLayout.openDrawer(GravityCompat.START);
            else drawerLayout.closeDrawer(GravityCompat.END);
        });
        chipBack.setOnClickListener(view -> {
            drawerLayout.closeDrawers();
        });
        tvOrderSyncMode.setOnClickListener(view -> {
            drawerLayout.closeDrawers();
            RejectReasonDialogFragment rejectReasonDialogFragment = RejectReasonDialogFragment.getInstance("Order Sync Mode", "Please select sync mode for orders", 0, "Auto,Manual", myPreferences.getOrderSyncMode());
            rejectReasonDialogFragment.show(getSupportFragmentManager(), "reason");
            rejectReasonDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String) {
                    String mode = (String) o;
                    EnterPasswordDialogFragment enterPasswordDialogFragment = EnterPasswordDialogFragment.getInstance(null, "Enter password", "Please enter password change order sync mode");
                    enterPasswordDialogFragment.show(getSupportFragmentManager(), "password");
                    enterPasswordDialogFragment.setDialogDismissListener(p -> {
                        if (p instanceof String) {
                            String password = (String) p;
                            if (password.equalsIgnoreCase(myPreferences.getEposAdminPassword())) {
                                myPreferences.saveOrderSyncMode(mode);
                                ToastUtils.makeSnackToast(LoginActivity.this, "Order sync mode changed");
                            } else {
                                ToastUtils.makeSnackToast(LoginActivity.this, "Invalid password");
                            }
                        }
                    });

                }
            });
        });
        tvRemoteDevice.setOnClickListener(v -> {
            drawerLayout.closeDrawers();
            RejectReasonDialogFragment rejectReasonDialogFragment = RejectReasonDialogFragment.getInstance("Remote Device", "Is this Remote Device?", 0, "On,Off", myPreferences.getRemoteDeviceEnableStatus() ? "On" : "Off");
            rejectReasonDialogFragment.show(getSupportFragmentManager(), "reason");
            rejectReasonDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String) {
                    String mode = (String) o;
                    myPreferences.saveRemoteDeviceEnableStatus(mode.equalsIgnoreCase("on"));

                }
            });
        });
        tvCallerId.setOnClickListener(view -> {
            drawerLayout.closeDrawers();
            RejectReasonDialogFragment rejectReasonDialogFragment = RejectReasonDialogFragment.getInstance("Caller ID", "Turn on or off Caller ID", 0, "On,Off", myPreferences.getCallerIdEnableStatus() ? "On" : "Off");
            rejectReasonDialogFragment.show(getSupportFragmentManager(), "reason");
            rejectReasonDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String) {
                    String mode = (String) o;
                    myPreferences.saveCallerIdEnableStatus(mode.equalsIgnoreCase("on"));

                }
            });
        });
        tvResLogout.setOnClickListener(v -> {
            askForLogout(true);
        });
    }

    private void askForLogout(boolean isAdmin) {
        ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Logout", "Are you sure you want to logout?", 1, "Confirm", "Cancel");
        confirmationDialogFragment.show(getSupportFragmentManager(), "logout");
        confirmationDialogFragment.setDialogDismissListener(o -> {
            if (o instanceof String) {
                if (((String) o).equalsIgnoreCase("confirm")) {
                    doAdminLogout();
                }
            }
        });
    }

    private void doAdminLogout() {
        ToastUtils.makeToast(LoginActivity.this, "Logout successfully!!");
        FirebaseMessaging.getInstance().unsubscribeFromTopic("business_" + myPreferences.getLoggedInAdmin().selected_business.id);
        myPreferences.saveLoggedInAdmin(null);
        myPreferences.saveLoggedInUser(null);
        myPreferences.saveUserTokens(null, null);
        myPreferences.saveAdminTokens(null, null);
        myPreferences.userLoggedInWithoutInternet(false);
        myPreferences.saveBusinessId(null);
        new Thread(() -> {
            appDatabase.emptyPulledDao().nukeAddon();
            appDatabase.emptyPulledDao().nukeCategory();
            appDatabase.emptyPulledDao().nukeCustomer();
            appDatabase.emptyPulledDao().nukePrepLocation();
            appDatabase.emptyPulledDao().nukeProduct();
            appDatabase.emptyPulledDao().nukeProductAddon();
            appDatabase.emptyPulledDao().nukeProductIngredient();
            appDatabase.emptyPulledDao().nukeTable();
            appDatabase.emptyPulledDao().nukeFloor();
            appDatabase.emptyPulledDao().nukeOrderItemIngredient();
            appDatabase.emptyPulledDao().nukeOrderItemAddon();
            appDatabase.emptyPulledDao().nukeOrderPayment();
            appDatabase.emptyPulledDao().nukeOrderSplit();
            appDatabase.emptyPulledDao().nukeOrder();
            appDatabase.emptyPulledDao().nukeUsers();
            appDatabase.emptyPulledDao().nukeReservations();
            appDatabase.emptyPulledDao().nukeVouchers();
            appDatabase.orderDao().resetPrimaryKey();
            try {
                deleteImages();
            } catch (Exception e) {
                e.printStackTrace();
            }

        }).start();
        myPreferences.savePullStatus(false);
        startActivity(new Intent(LoginActivity.this, SplashActivity.class));
        finish();
    }

    private void deleteImages() {
        File dir = new File(getApplicationContext().getExternalFilesDir("epos2021").getAbsolutePath());
        if (dir.isDirectory()) {
            String[] children = dir.list();
            if (children != null) {
                for (String child : children) {
                    try {
                        new File(dir, child).delete();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

            }
        }
    }

    private boolean isValid() {
        if (Validators.isNullOrEmpty(etUsername.getText().toString())) {
            ToastUtils.makeToast(LoginActivity.this, "Please enter username");
            return false;
        }
        if (Validators.isNullOrEmpty(etPassword.getText().toString())) {
            ToastUtils.makeToast(LoginActivity.this, "Please enter password");
            return false;
        }

        return true;
    }

    private void fetchMiniUsers() {
        progressDialog.show();
        new Thread(() -> {
            users.clear();
            users.addAll(appDatabase.userDao().list());
            runOnUiThread(() -> {
                userAdapter.notifyDataSetChanged();
                progressDialog.dismiss();
            });
        }).start();
    }

    private void performUserLogin() {
        progressDialog.show();
        AndroidNetworking.post(ApiEndPoints.user_login)
                .addBodyParameter("username", etUsername.getText().toString())
                .addBodyParameter("password", etPassword.getText().toString())
                .build()
                .getAsObject(User.class, new ParsedRequestListener<User>() {
                    @Override
                    public void onResponse(User response) {
                        myPreferences.saveEposUserPermission(new Gson().toJson(response.permissions));
                        MyApp.userPermission = (EposUserPermission) response.permissions;
                        response.permissions = null;
                        response.password = etPassword.getText().toString();
                        myPreferences.saveLoggedInUser(response);
                        progressDialog.dismiss();
                        ToastUtils.makeToast(LoginActivity.this, "Login successfully");
                        startActivity(new Intent(LoginActivity.this, HomeActivity.class));
                        finish();
                    }
                    @Override
                    public void onError(ANError anError) {
                        progressDialog.dismiss();
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(LoginActivity.this, apiError.getMessage());
                        } else if (anError.getErrorCode() >= 500) {
                            ToastUtils.makeSnackToast(LoginActivity.this, "Error occurred while login");
                        } else {
                            offlineLogin();
                        }
                    }
                });
    }

    private void offlineLogin() {
        String userName = etUsername.getText().toString();
        String password = etPassword.getText().toString();
        new Thread(() -> {
            User user = appDatabase.userDao().view(userName);
            if (user != null) {
                if (password.equals(user.password)) {
                    myPreferences.saveLoggedInUser(user);
                    myPreferences.saveEposUserPermission(user.string_permission);
                    myPreferences.userLoggedInWithoutInternet(true);
                    ToastUtils.makeToast(LoginActivity.this, "Login successfully");
                    runOnUiThread(() -> {
                        startActivity(new Intent(LoginActivity.this, HomeActivity.class));
                        finish();
                    });
                } else {
                    ToastUtils.makeSnackToast(LoginActivity.this, "Invalid password");
                }
            } else {
                ToastUtils.makeSnackToast(LoginActivity.this, "Invalid username");
            }
        }).start();
    }

    protected void changeStatusBarColor(int color) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Window window = getWindow();
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
            window.setStatusBarColor(color);
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (!myApp.isConnected(this)) {
            changeStatusBarColor(Color.RED);
        } else {
            changeStatusBarColor(Color.BLACK);
        }
    }
}