package com.example.epos_2021.activities;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.view.GravityCompat;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.Links;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.OrderReviewItemsAdapter;
import com.example.epos_2021.adapters.SplitGroupAdapter;
import com.example.epos_2021.base.BaseActivity;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.comman.printer.SunmiPrinter;
import com.example.epos_2021.comman.printer.WifiPrinter;
import com.example.epos_2021.comman.printer.ZoneRichPrinter;
import com.example.epos_2021.daos.relations.OrderItemWithAddonsIngredients;
import com.example.epos_2021.daos.relations.OrderWithItems;
import com.example.epos_2021.fragment.AmountSelectionFragment;
import com.example.epos_2021.fragment.OrderItemViewDialogFragment;
import com.example.epos_2021.fragment.PaymentMethodSelectionFragment;
import com.example.epos_2021.fragment.PrintOptionsDialogFragment;
import com.example.epos_2021.fragment.SendOrderDialogFragment;
import com.example.epos_2021.interfaces.RecyclerviewItemClickListener;
import com.example.epos_2021.models.Discount;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.OrderItemAddon;
import com.example.epos_2021.models.OrderItemIngredient;
import com.example.epos_2021.models.OrderPayment;
import com.example.epos_2021.models.OrderSplit;
import com.example.epos_2021.models.PaymentMethod;
import com.example.epos_2021.models.PrepLocation;
import com.example.epos_2021.models.Printer;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.LogUtils;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.card.MaterialCardView;
import com.google.android.material.chip.Chip;
import com.google.firebase.crashlytics.FirebaseCrashlytics;
import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;


public class OrderReviewActivity extends BaseActivity {

    private AlertDialog progressBarDialog;
    private Chip ivBack;
    private TextView tvPreviousScreen, tvCurrentScreen, tvEditQty;
    private RecyclerView rvOrderItems;
    private LinearLayout llDone;
    private MaterialCardView cvEditOrder, cvSendOrder, cvPrintOrder;
    private DrawerLayout drawerLayout;
    private Chip ivDrawerBack;
    private RecyclerView rvSplitGroup;

    private ZoneRichPrinter zoneRichPrinter;
    private SunmiPrinter sunmiPrinter;
    private WifiPrinter wifiPrinter;
    private Printer defaultPrinter;
    private Printer forceFullyDefaultPrinter;
    private Order order;
    private OrderReviewItemsAdapter orderItemsAdapter;
    private SiteSetting printBillMainSetting, sendOrderMainSetting, sendOrderUpdateMainSetting, sendDupOrderMainSetting, sendDupOrderUpdateMainSetting;
    private OrderSplit selectedSplit;
    int _order_id, _order_split_id;
    String payment_method;
    float total, total_paid;
    private String footerA, footerB;
    private String dateTimeModeInPrint;
    FragmentManager manager;
    private int headerAlignment = 0;
    private float cartGrandTotal = 0, cartSubTotal = 0, cartDiscount = 0, cartGratuity = 0, cartServiceCharge = 0, cartDeliveryCharge = 0;
    private SiteSetting gratuityMainSetting, printAllMainSetting, buzzerSetting, totalVisibilitySetting, serviceChargeSetting, serviceChargeTypeSetting, serviceChargeEnableSetting, serviceChargeOptionSetting, ticketHeaderSetting, ticketHeaderTypeSetting, footerASetting, footerBSetting, orderDateTimeSetting, tableOrOrderIdSetting, sendOrderActionSetting, payBillActionSetting, payBillActionModeSetting;
    private boolean shouldFetchDeliveryCharge = false, serviceChargeAutoMode = true, serviceChargeEnabled = false, serviceChargeVisible = false, voucherPaymentsEnabled = false, discountEnabled = true, gratuityEnabled = true, autoDiscountApplied = false, printTotalInPrint = true, needToUpload = true, voucherPaymentDone = true, printAction = false;
    private ArrayList<OrderItem> orderItems = new ArrayList<>();
    private ArrayList<Printer> printers = new ArrayList<>();
    boolean splitByPersonPrint = false, canEditOrder;
    float printableTotal = 0;
    private SplitGroupAdapter splitGroupAdapter;
    private final ArrayList<OrderSplit> orderSplits = new ArrayList<>();
    private TextView tvText;

    @Override
    protected void setXML() {
        super.setXML();
        setContentView(R.layout.activity_order_review);
        manager = getSupportFragmentManager();
    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getIntent() != null) {
            _order_id = getIntent().getIntExtra("_order_id", 0);
            _order_split_id = getIntent().getIntExtra("_order_split_id", 0);
            total = getIntent().getFloatExtra("order_total", 0);
            payment_method = getIntent().getStringExtra("payment_method");
            total_paid = getIntent().getFloatExtra("paid_amount", 0);
        }
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        fetchOffline();
        LocalBroadcastManager.getInstance(OrderReviewActivity.this).registerReceiver(fragmentChangeReceiver, new IntentFilter(Constants.FRAGMENT_CHANGE));
        if (Validators.isNullOrEmpty(payment_method)) {
            changeFragment(new PaymentMethodSelectionFragment());
        } else {
            changeFragment(new AmountSelectionFragment());
        }
    }

    @Override
    protected void initViews() {
        super.initViews();
        defaultPrinter = myPreferences.getDefaultPrinter();
        zoneRichPrinter = new ZoneRichPrinter(this);
        sunmiPrinter = new SunmiPrinter(this);
        wifiPrinter = WifiPrinter.getInstance();
        forceFullyDefaultPrinter = myPreferences.getDefaultPrinter(true);

        progressBarDialog = CommonFunctions.customProgressDialog(OrderReviewActivity.this);
        ivBack = findViewById(R.id.chipBack);
        tvPreviousScreen = findViewById(R.id.tvPreviousScreen);
        tvCurrentScreen = findViewById(R.id.tvCurrentScreen);
        tvEditQty = findViewById(R.id.tvEditQty);
        rvOrderItems = findViewById(R.id.rvSelectedProducts);
        cvEditOrder = findViewById(R.id.cvEditOrder);
        cvSendOrder = findViewById(R.id.cvSendOrder);
        cvPrintOrder = findViewById(R.id.cvPrintOrder);
        llDone = findViewById(R.id.llDone);
        drawerLayout = findViewById(R.id.drawer_layout);
        ivDrawerBack = findViewById(R.id.chipDrawerBack);
        rvSplitGroup = findViewById(R.id.rvSplitGroups);
        tvText = findViewById(R.id.tvText);

        tvCurrentScreen.setText("> Checkout");
        drawerLayout.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED);
        loadSettings();
        setAdapters();
        setButtonVisibilties();
        setListeners();
        printers = selectedBusiness.printers;
    }

    private void fetchOffline() {
        new FetchOrderOfflineAsyncTask(() -> {
            orderItemsAdapter.notifyDataSetChanged();
            myApp.notifyCart(OrderReviewActivity.this, new Intent(Constants.EPOS_ORDER_UPDATE).putExtra("refresh", true).putExtra("_order_split_id", _order_split_id));
            return null;
        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private void setButtonVisibilties() {
        if (MyApp.userPermission != null && MyApp.userPermission.send_order.actions.list && showSendOrderButton())
            cvSendOrder.setVisibility(View.VISIBLE);
        else
            cvSendOrder.setVisibility(View.GONE);
        if (showPrintButton()) {
            cvPrintOrder.setVisibility(View.VISIBLE);
        } else
            cvPrintOrder.setVisibility(View.GONE);
    }

    private boolean showPrintButton() {
        if (MyApp.userPermission != null && (MyApp.userPermission.print_all.actions.list || MyApp.userPermission.print_bill.actions.list)) {
            if (printBillMainSetting == null && printAllMainSetting == null) {
                return false;
            } else if ((printBillMainSetting != null && (printBillMainSetting.value.equalsIgnoreCase("false") || printBillMainSetting.value.equalsIgnoreCase("0"))) && (printAllMainSetting != null && (printAllMainSetting.value.equalsIgnoreCase("false") || printAllMainSetting.value.equalsIgnoreCase("0")))) {
                return false;
            } else {
                return true;
            }
        } else return false;
    }

    private boolean showSendOrderButton() {
        if (sendOrderMainSetting == null && sendOrderUpdateMainSetting == null && sendDupOrderMainSetting == null && sendDupOrderUpdateMainSetting == null) {
            return false;
        }
        return (sendOrderMainSetting == null || (!sendOrderMainSetting.value.equalsIgnoreCase("false") && !sendOrderMainSetting.value.equalsIgnoreCase("0"))) || (sendOrderUpdateMainSetting == null || (!sendOrderUpdateMainSetting.value.equalsIgnoreCase("false") && !sendOrderUpdateMainSetting.value.equalsIgnoreCase("0"))) || (sendDupOrderMainSetting == null || (!sendDupOrderMainSetting.value.equalsIgnoreCase("false") && !sendDupOrderMainSetting.value.equalsIgnoreCase("0"))) || (sendDupOrderUpdateMainSetting == null || (!sendDupOrderUpdateMainSetting.value.equalsIgnoreCase("false") && !sendDupOrderUpdateMainSetting.value.equalsIgnoreCase("0")));
    }

    private void setListeners() {
        ivBack.setOnClickListener(v -> onBackPressed());

        ivDrawerBack.setOnClickListener(v -> drawerLayout.closeDrawer(GravityCompat.END));
        tvEditQty.setOnClickListener(v -> {
            //Edit Qty
            toggleEditOption();
        });
        llDone.setOnClickListener(v -> {
            //Edit Qty
            toggleEditOption();
        });

        cvEditOrder.setOnClickListener(v -> {
            finish();
        });
        cvSendOrder.setOnClickListener(v -> {
            SendOrderDialogFragment sendOrderDialogFragment = SendOrderDialogFragment.getInstance(sendOrderMainSetting, sendOrderUpdateMainSetting, sendDupOrderMainSetting, sendDupOrderUpdateMainSetting);
            sendOrderDialogFragment.show(getSupportFragmentManager(), "send_order");
            sendOrderDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String) {
                    order.order_action_id = (String) o;
                    new UpdateOrderAsyncTask(() -> {
                        String action = null;
                        if (sendOrderActionSetting != null && sendOrderActionSetting.value.equalsIgnoreCase("prep_ticket")) {
                            if (order.order_action_id.equalsIgnoreCase("1")) {
//                                createPrint("Full Order", true, true, false);
                                action = "full_order";
                            } else if (order.order_action_id.equalsIgnoreCase("2")) {
//                                createPrint("Duplicate Order", true, true, false);
                                action = "duplicate_order";
                            } else if (order.order_action_id.equalsIgnoreCase("3")) {
//                                createPrint("Updated Order", true, true, true);
                                action = "updated_order";
                            } else if (order.order_action_id.equalsIgnoreCase("4")) {
//                                createPrint("Duplicate Updated Order", true, true, true);
                                action = "duplicate_updated_order";
                            } else {
                                onBackPressed();
                                action = "";
                            }
                        } else {
//                            onBackPressed();
                            action = "";
                        }
                        myApp.needToPrint = true;
                        Intent intent = new Intent();
                        intent.putExtra("order_action", action);
                        setResult(RESULT_OK, intent);
                        finish();
                        return null;
                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                }
            });
        });

        cvPrintOrder.setOnClickListener(v -> {
//            LocalBroadcastManager.getInstance(OrderReviewActivity.this).sendBroadcast(new Intent("order_button_click").putExtra("print_order", true));
            PrintOptionsDialogFragment printOptionsDialogFragment = PrintOptionsDialogFragment.getInstance(printBillMainSetting, printAllMainSetting);
            printOptionsDialogFragment.show(getSupportFragmentManager(), "print_options");
            printOptionsDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String) {
//                    if (((String) o).equalsIgnoreCase("print_bill")) {
//                        createPrint("", false, true, false);
//                    } else if (((String) o).equalsIgnoreCase("print_all")) {
//                        createPrint("", true, false, false);
//                        createPrint("", false, true, false);
//                    }
                    myApp.needToPrint = true;
                    Intent intent = new Intent();
                    intent.putExtra("order_action", ((String) o));
                    setResult(RESULT_OK, intent);
                    finish();
                }
            });
        });

    }

    private void toggleEditOption() {
        orderItemsAdapter.shouldEdit = !orderItemsAdapter.shouldEdit;
        if (orderItemsAdapter.shouldEdit) {
            llDone.setVisibility(View.VISIBLE);
            tvEditQty.setVisibility(View.GONE);
        } else {
            llDone.setVisibility(View.GONE);
            tvEditQty.setVisibility(View.VISIBLE);
        }
        orderItemsAdapter.notifyDataSetChanged();
    }

    private void setAdapters() {
        orderItemsAdapter = new OrderReviewItemsAdapter(orderItems, new RecyclerviewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {
                OrderItem orderItem = (OrderItem) data;
                editProduct(orderItem);
            }
        }, (position, data) -> {
            OrderItem orderItem = (OrderItem) data;
            orderItem.quantity = 0;
            editProduct(orderItem);
        }, (position, data) -> {
            openOrderItemDetails((OrderItem) data);
        });
        rvOrderItems.setLayoutManager(new GridLayoutManager(this, getResources().getInteger(R.integer.order_review_items_span)));
        rvOrderItems.setAdapter(orderItemsAdapter);

        splitGroupAdapter = new SplitGroupAdapter(orderSplits, (position, data) -> {
            OrderSplit orderSplit = (OrderSplit) data;
            if (_order_split_id != orderSplit._id) {
                _order_split_id = orderSplit._id;
                selectedSplit = orderSplit;
                changeFragment(new PaymentMethodSelectionFragment());
                fetchOffline();
            }
        });
        rvSplitGroup.setLayoutManager(new LinearLayoutManager(this, RecyclerView.HORIZONTAL, false));
        rvSplitGroup.setAdapter(splitGroupAdapter);
    }

    private void openOrderItemDetails(OrderItem data) {
        OrderItemViewDialogFragment orderItemViewDialogFragment = OrderItemViewDialogFragment.getInstance(data);
        orderItemViewDialogFragment.show(getSupportFragmentManager(), "show");
        orderItemViewDialogFragment.setCancelable(true);
    }

    private void loadSettings() {
        gratuityMainSetting = myApp.findSetting("gratuity");
        printAllMainSetting = myApp.findSetting("is_printall");
        printBillMainSetting = myApp.findSetting("is_printbill");
        buzzerSetting = myApp.findSetting("open_cash_drawer_after_send_to_kitchen");
        sendOrderMainSetting = myApp.findSetting("is_sendorder");
        sendOrderUpdateMainSetting = myApp.findSetting("is_sendorderupdate");
        sendDupOrderMainSetting = myApp.findSetting("is_sendduplicateorder");
        sendDupOrderUpdateMainSetting = myApp.findSetting("is_sendorderupdate");

        sendOrderActionSetting = myApp.findSetting("send_order_action");
        payBillActionSetting = myApp.findSetting("pay_bill_action");
        payBillActionModeSetting = myApp.findSetting("pay_bill_action_type");
        totalVisibilitySetting = myApp.findSetting("kitchen_total_visible");
        footerASetting = myApp.findSetting("footer_a");
        footerBSetting = myApp.findSetting("footer_b");
        orderDateTimeSetting = myApp.findSetting("date_time_in_receipt_printing");
        tableOrOrderIdSetting = myApp.findSetting("use_table_number_or_order_number");
        serviceChargeEnableSetting = myApp.findSetting("is_service_charge");
        serviceChargeSetting = myApp.findSetting("service_charge");
        serviceChargeOptionSetting = myApp.findSetting("service_charge_type_option");
        serviceChargeTypeSetting = myApp.findSetting("service_charge_type");

        ticketHeaderTypeSetting = myApp.findSetting("ticket_header_type");
        if (ticketHeaderTypeSetting != null && ticketHeaderTypeSetting.value.equalsIgnoreCase("right")) {
            headerAlignment = 2;
        }
        if (ticketHeaderTypeSetting != null && ticketHeaderTypeSetting.value.equalsIgnoreCase("center")) {
            headerAlignment = 1;
        }
        ticketHeaderSetting = myApp.findSetting("ticket_header");
        tvText.setVisibility(View.GONE);
        if (serviceChargeEnableSetting != null && (serviceChargeEnableSetting.value.equalsIgnoreCase("true") || serviceChargeEnableSetting.value.equalsIgnoreCase("1"))) {
            serviceChargeEnabled = true;
            serviceChargeVisible = true;
        }
        if (serviceChargeEnabled && serviceChargeOptionSetting != null && serviceChargeOptionSetting.value.equalsIgnoreCase("manual")) {
            serviceChargeAutoMode = false;
        }

     /*   if (Links.service_charges_mode == 0) {
            if (!Links.service_charges)
                serviceChargeEnabled = false;
        }*/

        if (footerASetting != null) {
            footerA = footerASetting.value;
        }
        if (footerBSetting != null) {
            footerB = footerBSetting.value;
        }
        if (orderDateTimeSetting != null) {
            dateTimeModeInPrint = orderDateTimeSetting.value;
        }
        if (totalVisibilitySetting != null && (totalVisibilitySetting.value.equalsIgnoreCase("no") || totalVisibilitySetting.value.equalsIgnoreCase("false") || totalVisibilitySetting.value.equalsIgnoreCase("0"))) {
            printTotalInPrint = false;
        }
    }

    private class UpdateOrderAsyncTask extends AsyncTask<Object, Object, Object> {
        Callable<Void> nextMethod;

        public UpdateOrderAsyncTask(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected Object doInBackground(Object... objects) {
            order.sub_total = myApp.appDatabase.orderDao().getItemSubTotal(order._id);

            cartSubTotal = order.sub_total;
            if (serviceChargeEnabled) {
                if (serviceChargeAutoMode) {
                    float serviceChargeValue = Float.parseFloat(serviceChargeSetting.value);
                    if (serviceChargeTypeSetting.value.equalsIgnoreCase("flat")) {
                        cartServiceCharge = serviceChargeValue;
                    } else {
                        if (serviceChargeValue > 0) {
                            cartServiceCharge = cartSubTotal * serviceChargeValue / 100;
                        } else {
                            cartServiceCharge = 0;
                        }
                    }
                }
            } else {
                cartServiceCharge = 0;
            }
            cartDiscount = order.discount;
            if (discountEnabled) {
                List<Discount> discounts = appDatabase.discountDao().eligibleDiscounts("%" + myApp.current_day + "%", "%" + order.order_type_id + "%", order.sub_total, CommonFunctions.getCurrentTimeFormatted("yyyy-MM-dd"), CommonFunctions.getCurrentTimeFormatted("yyyy-MM-dd"));
                if (discounts.size() > 0) {
                    Discount appliedDiscount = discounts.get(0);
                    autoDiscountApplied = false;
                    if (!Validators.isNullOrEmpty(appliedDiscount.discount_type) && appliedDiscount.discount_type.equalsIgnoreCase("flat")) {
                        if (cartSubTotal > 0)
                            cartDiscount = appliedDiscount.discount;
                        else cartDiscount = 0;
                        autoDiscountApplied = true;
                    }
                    if (!Validators.isNullOrEmpty(appliedDiscount.discount_type) && appliedDiscount.discount_type.equalsIgnoreCase("percentage")) {
                        if (cartSubTotal > 0)
                            cartDiscount = cartSubTotal * appliedDiscount.discount / 100;
                        else
                            cartDiscount = 0;

                        autoDiscountApplied = true;
                    }
                } else {
                    if (autoDiscountApplied)
                        cartDiscount = 0;
                }
            }

            order.gratuity = cartGratuity;
            order.discount = cartDiscount;
            order.service_charge = cartServiceCharge;
            cartGrandTotal = ((cartSubTotal + cartGratuity + cartServiceCharge + cartDeliveryCharge)  - cartDiscount);
            order.total = cartGrandTotal;
            if (!order.order_status_id.equalsIgnoreCase("5") && !order.order_status_id.equalsIgnoreCase("10")) {
                order.updated_at = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
            }
            appDatabase.orderDao().update(order);
            return null;
        }

        @Override
        protected void onPostExecute(Object s) {
            super.onPostExecute(s);
            if (nextMethod != null) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private class FetchOrderOfflineAsyncTask extends AsyncTask<Object, Object, Object> {
        Callable<Void> nextMethod;

        public FetchOrderOfflineAsyncTask(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected Object doInBackground(Object... objects) {
            OrderWithItems orderWithItems = myApp.appDatabase.orderDao().orderWithItems(_order_id);
            if (orderWithItems != null) {
                order = orderWithItems.order;
                boolean splitException = false;
                if (order.split_type != null) {
                    try {
                        appDatabase.orderSplitDao().updatePaidAmount(_order_id);
                    } catch (Exception e) {
                        e.printStackTrace();
                        splitException = true;
                    }
                }
                order.total_paid = appDatabase.orderPaymentDao().getTotalPaidAmount(_order_id);
                if (order.order_type_id != null && order.order_type_id.equalsIgnoreCase("1")) {
                    order.order_type = "Dine in";
                }
                if (order.order_type_id != null && order.order_type_id.equalsIgnoreCase("2")) {
                    order.order_type = "Collection";
                }
                if (order.order_type_id != null && order.order_type_id.equalsIgnoreCase("3")) {
                    order.order_type = "Delivery";
                }
                if (order.order_type_id != null && order.order_type_id.equalsIgnoreCase("4")) {
                    order.order_type = "Bar";
                }
                if (order.order_type_id != null && order.order_type_id.equalsIgnoreCase("5")) {
                    order.order_type = "Waiting";
                }
                if (shouldFetchDeliveryCharge) {
                    order.delivery_charge = cartDeliveryCharge;
                }

                Log.e("Order_type", " " + order.order_type);
                if (serviceChargeVisible) {
                    SiteSetting services_charges = myApp.findSetting("service_charge_order_type");
                    List<String> split_funcation = Links.split_funcation(services_charges.value);
                    for (int i = 0; i < split_funcation.size(); i++) {
                        if (order.order_type.equalsIgnoreCase(split_funcation.get(i).toString())) {
                            serviceChargeEnabled = true;
                            break;
                        } else {
                            serviceChargeEnabled = false;
                        }
                    }
                }

                if (!Validators.isNullOrEmpty(order.table_id))
                    order.table_number = appDatabase.tableDao().view(order.table_id).number;
                order.order_payments = (ArrayList<OrderPayment>) orderWithItems.orderPayments;
                order.order_splits = (ArrayList<OrderSplit>) orderWithItems.orderSplits;
                if (order.order_splits.size() > 0 && _order_split_id == 0) {
                    _order_split_id = order.order_splits.get(0)._id;
                    selectedSplit = order.order_splits.get(0);
                }
                if (splitException) {
                    for (OrderSplit orderSplit : order.order_splits) {
                        orderSplit.paid_amount = appDatabase.orderPaymentDao().getSplitTotalPaidAmount(_order_id, orderSplit._id);
                        appDatabase.orderSplitDao().update(orderSplit);
                    }
                }
                order.order_items = new ArrayList<>();
                orderItems.clear();
                for (OrderItemWithAddonsIngredients item : orderWithItems.orderItems) {
                    OrderItem orderItem = item.orderItem;
                    orderItem.order_item_addons = (ArrayList<OrderItemAddon>) item.orderItemAddons;
                    orderItem.order_item_ingredients = (ArrayList<OrderItemIngredient>) item.orderItemIngredients;
                    order.order_items.add(orderItem);
                    if (_order_split_id == orderItem._order_split_id || orderItem._order_split_id == 0) {
                        orderItems.add(orderItem);
                    }
                }
                order.customer = myApp.appDatabase.customerDao().view(order._customer_id);
                if (order.customer == null) {
                    order.customer = appDatabase.customerDao().view(order.customer_id);
                }

                if (order.service_charge == 0) {
                    if (serviceChargeEnabled) {
                        if (serviceChargeAutoMode) {
                            float serviceChargeValue = Float.parseFloat(serviceChargeSetting.value);
                            if (serviceChargeTypeSetting.value.equalsIgnoreCase("flat")) {
                                cartServiceCharge = serviceChargeValue;
                            } else {
                                if (serviceChargeValue > 0) {
                                    cartServiceCharge = cartSubTotal * serviceChargeValue / 100;
                                } else {
                                    cartServiceCharge = 0;
                                }
                            }
                        }
                    } else {
                        cartServiceCharge = 0;
                    }
                } else {
                    cartServiceCharge = order.service_charge;
                }
                if (order.split_type != null && order.split_type.equalsIgnoreCase("order_item")) {
                    cartServiceCharge = 0;
                    serviceChargeEnabled = false;
                    discountEnabled = false;
                    gratuityEnabled = false;
                }
                order.service_charge = cartServiceCharge;
                cartSubTotal = order.sub_total;
                cartDiscount = order.discount;

                if (discountEnabled) {
                    List<Discount> discounts = appDatabase.discountDao().eligibleDiscounts("%" + myApp.current_day + "%", "%" + order.order_type_id + "%", order.sub_total, CommonFunctions.getCurrentTimeFormatted("yyyy-MM-dd"), CommonFunctions.getCurrentTimeFormatted("yyyy-MM-dd"));
                    if (discounts.size() > 0) {
                        Discount appliedDiscount = discounts.get(0);
                        autoDiscountApplied = false;
                        if (!Validators.isNullOrEmpty(appliedDiscount.discount_type) && appliedDiscount.discount_type.equalsIgnoreCase("flat")) {
                            if (cartSubTotal > 0)
                                cartDiscount = appliedDiscount.discount;
                            else cartDiscount = 0;
                            autoDiscountApplied = true;
                        }
                        if (!Validators.isNullOrEmpty(appliedDiscount.discount_type) && appliedDiscount.discount_type.equalsIgnoreCase("percentage")) {
                            if (cartSubTotal > 0)
                                cartDiscount = cartSubTotal * appliedDiscount.discount / 100;
                            else
                                cartDiscount = 0;
                            autoDiscountApplied = true;
                        }
                    } else {
                        if (autoDiscountApplied) cartDiscount = 0;
                    }
                }
                order.discount = cartDiscount;
                order.total = ((order.sub_total + order.gratuity + order.service_charge + order.delivery_charge) - order.discount);

               // order.total = ((cartSubTotal + cartGratuity + cartServiceCharge + cartDeliveryCharge)  - cartDiscount);

                if (!order.order_status_id.equalsIgnoreCase("5") && !order.order_status_id.equalsIgnoreCase("10")) {
                    if (order.total == 0) {
                        order.order_status_id = "2";
                        order.order_status = "Order Taken";
                    } else if (order.total == order.total_paid) {
                        if (!order.order_status_id.equalsIgnoreCase("6")) {
                            order.order_status_id = "4";
                            order.order_status = "Order Paid";
                        }
                    } else if (order.total_paid > 0) {
                        order.order_status_id = "7";
                        order.order_status = "Part Payment Received";
                    } else {
                        if (!order.order_status_id.equalsIgnoreCase("3")) {
                            order.order_status_id = "2";
                            order.order_status = "Order Taken";
                        }
                    }
                    order.updated_at = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
                }
                myApp.appDatabase.orderDao().update(order);

                myApp.notifyCart(OrderReviewActivity.this, new Intent("cart_update").putExtra("notify_products", true));

                PaymentMethod voucherPaymentMethod = appDatabase.paymentMethodDao().view("5");
                if (order.split_type == null && voucherPaymentMethod != null && !voucherPaymentMethod.disabled) {
                    OrderPayment voucherPayment = appDatabase.orderPaymentDao().findByMethod(_order_id, "5");
                    voucherPaymentsEnabled = voucherPayment == null;
                    voucherPaymentDone = voucherPayment != null;
                } else {
                    voucherPaymentsEnabled = false;
                }
                if (_order_split_id > 0) {
                    selectedSplit = appDatabase.orderSplitDao().view(_order_split_id);
                    total_paid = appDatabase.orderPaymentDao().getSplitTotalPaidAmount(_order_id, _order_split_id);
                }
                if (!Validators.isNullOrEmpty(order.split_type) && order.split_type.equalsIgnoreCase("person")) {
                    canEditOrder = false;
                } else {
                    if (CommonFunctions.orderPaymentStatus(order.total, order.total_paid) == 1) {
                        canEditOrder = false;
                    } else {
                        canEditOrder = true;
                    }
                }
            }
            return null;
        }

        @Override
        protected void onPostExecute(Object s) {
            super.onPostExecute(s);
            if (order != null)
                if (nextMethod != null) {
                    try {
                        tvPreviousScreen.setText(order.order_type);
                        updateViews();
                        nextMethod.call();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
        }
    }

    private void updateViews() {
        if (order.order_splits != null && order.order_splits.size() > 0) {
            rvSplitGroup.setVisibility(View.VISIBLE);
            orderSplits.clear();
            orderSplits.addAll(order.order_splits);
            orderItemsAdapter.shouldEdit = canEditOrder;
            if (!canEditOrder) {
                tvEditQty.setVisibility(View.GONE);
                llDone.setVisibility(View.GONE);
            } else {
                tvEditQty.setVisibility(View.VISIBLE);
            }
            orderItemsAdapter.notifyDataSetChanged();
            splitGroupAdapter.selectedSplitId = _order_split_id;
            splitGroupAdapter.notifyDataSetChanged();
            changeFragment(new PaymentMethodSelectionFragment());
        } else {
            rvSplitGroup.setVisibility(View.GONE);
        }
    }

    private void editProduct(OrderItem orderItem) {
        ArrayList<OrderItem> tempItems = new ArrayList<>();
        tempItems.add(orderItem);
        new AddOrUpdateOrderItemAsyncTask(() -> {
            setResult(Activity.RESULT_OK);
            fetchOffline();
            return null;
        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, tempItems);
    }

    private class AddOrUpdateOrderItemAsyncTask extends AsyncTask<Object, Object, Object> {
        Callable<Void> nextMethod;

        public AddOrUpdateOrderItemAsyncTask(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected Object doInBackground(Object... objects) {
            if (objects[0] instanceof List) {
                ArrayList<OrderItem> orderItems = (ArrayList<OrderItem>) objects[0];
                for (OrderItem orderItem : orderItems) {
                    OrderItem dbOrderItem = appDatabase.orderItemDao().view(orderItem._id);
                    if (orderItem._order_split_id > 0) {
                        OrderSplit dbSplit = null;
                        if (selectedSplit != null) {
                            if (orderItem._order_split_id == selectedSplit._id) {
                                dbSplit = selectedSplit;
                            } else {
                                dbSplit = appDatabase.orderSplitDao().view(orderItem._order_split_id);
                            }
                        } else {
                            dbSplit = appDatabase.orderSplitDao().view(orderItem._order_split_id);
                        }
                        if (dbSplit.paid_amount > 0) {
                            break;
                        }
                    }
                    int dbOrderItemId, quantity;
                    if (orderItem.quantity <= 0) {
                        if (dbOrderItem != null) {
                            appDatabase.orderItemDao().delete(dbOrderItem);
                            if (orderItem._order_split_id > 0) {
                                appDatabase.orderSplitDao().updateTotalAmount(orderItem._order_id, orderItem._order_split_id);
                            }
                            continue;
                        }
                    }
                    if (dbOrderItem != null) {
                        dbOrderItem.quantity = orderItem.quantity;
                        quantity = dbOrderItem.quantity;
                        dbOrderItem.sub_total = dbOrderItem.quantity * orderItem.price;
                        dbOrderItem.instruction_price = orderItem.instruction_price;
                        dbOrderItem.special_instruction = orderItem.special_instruction;
                        dbOrderItem.total = dbOrderItem.sub_total + dbOrderItem.addons_price + dbOrderItem.ingredients_price + (dbOrderItem.instruction_price * dbOrderItem.quantity);
                        myApp.appDatabase.orderItemDao().update(dbOrderItem);
                        dbOrderItemId = dbOrderItem._id;
                    } else {
                        quantity = orderItem.quantity;
                        orderItem.order_id = order.id;
                        orderItem._order_id = _order_id;
                        orderItem.sub_total = orderItem.quantity * orderItem.price;
                        orderItem.total = orderItem.sub_total + orderItem.addons_price + orderItem.ingredients_price + (orderItem.instruction_price * orderItem.quantity);
                        dbOrderItemId = (int) myApp.appDatabase.orderItemDao().insert(orderItem);
                    }
                    float addonTotal = 0;
                    if (orderItem.order_item_addons != null) {
                        for (OrderItemAddon itemAddon : orderItem.order_item_addons) {
                            itemAddon._order_item_id = dbOrderItemId;
                            itemAddon.total = itemAddon.quantity * itemAddon.price;
                            addonTotal = addonTotal + itemAddon.total;
                        }
                        myApp.appDatabase.orderItemAddonDao().insertAll(orderItem.order_item_addons);
                    }

                    float ingredientTotal = 0;
                    if (orderItem.order_item_ingredients != null) {
                        appDatabase.orderItemIngredientDao().deleteAll(dbOrderItemId);
                        for (OrderItemIngredient itemIngredient : orderItem.order_item_ingredients) {
                            itemIngredient._order_item_id = dbOrderItemId;
                            itemIngredient.total = itemIngredient.quantity * itemIngredient.price;
                            ingredientTotal = ingredientTotal + itemIngredient.total;
                        }
                        appDatabase.orderItemIngredientDao().insertAll(orderItem.order_item_ingredients);
                    }

                    ingredientTotal = ingredientTotal * quantity;
                    addonTotal = addonTotal * quantity;
                    myApp.appDatabase.orderItemDao().updateOrderItemTotal(dbOrderItemId, addonTotal, ingredientTotal);
                    if (orderItem._order_split_id > 0) {
                        appDatabase.orderSplitDao().updateTotalAmount(orderItem._order_id, orderItem._order_split_id);
                    }
                }

                order.sub_total = myApp.appDatabase.orderDao().getItemSubTotal(_order_id);
                order.updated_at = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
                appDatabase.orderDao().update(order);
            }

            return null;
        }

        @Override
        protected void onPostExecute(Object s) {
            super.onPostExecute(s);
            if (nextMethod != null) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private void changeFragment(Fragment fragment) {
        if (getIntent() != null) {
            getIntent().putExtra("_order_split_id", _order_split_id);
        }
        if (manager.getBackStackEntryCount() > 0)
            manager.popBackStack("can_go_back", FragmentManager.POP_BACK_STACK_INCLUSIVE);
        if (getIntent() != null) {
            fragment.setArguments(getIntent().getExtras());
        }
        FragmentManager manager = getSupportFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.flContainer, fragment);
        transaction.commit();

    }

    private void createPrint(String title, boolean kitchenCopy, boolean goBack, boolean updateOrder) {
//        print(title, kitchenCopy, updateOrder, goBack);
//        if (true) return;
        LogUtils.e("OnActivityResult::Review activity: create print");
        if (defaultPrinter != null && defaultPrinter.printer_model_name != null) {
            if (kitchenCopy) {
                new PrepLocationSenderAsync(title, updateOrder, () -> {
                    if (goBack)
                        onBackPressed();
                    return null;
                }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
            } else {
                new OrderItemsPrintBlockAsync(null, () -> {
                    if (goBack) onBackPressed();
                    return null;
                }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
            }
        } else {
            ToastUtils.makeLongToast(this, "No printer model found");
            if (goBack) {
                onBackPressed();
            }
        }

    }


    class PrepLocationSenderAsync extends AsyncTask<String, String, String> {
        HashMap<PrepLocation, List<OrderItem>> prepOrderItems = new HashMap<>();
        String title;
        boolean updatedOrder;
        Callable<Void> nextMethod;

        public PrepLocationSenderAsync(String title, boolean updatedOrder, Callable<Void> nextMethod) {
            this.title = title;
            this.updatedOrder = updatedOrder;
            this.nextMethod = nextMethod;
        }

        @Override
        protected String doInBackground(String... strings) {

            ArrayList<PrepLocation> prepLocations = (ArrayList<PrepLocation>) appDatabase.prepLocationDao().list();
            for (PrepLocation prepLocation : prepLocations) {
                List<OrderItem> miscPrepOits = new ArrayList<>();
                List<OrderItem> prepOits = new ArrayList<>();
                List<OrderItem> allOits = new ArrayList<>();
                /*if (updatedOrder) {
                    miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocations(order._id, prepLocation.id, false);
                    prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocations(order._id, prepLocation.id, false);
                } else {
                    miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocations(order._id, prepLocation.id);
                    prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocations(order._id, prepLocation.id);
                }*/
                if (updatedOrder) {
                    if (order.order_type.equalsIgnoreCase("Collection")) {
                        miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocationsCollection(order._id, prepLocation.id, false);
                        prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocationsCollection(order._id, prepLocation.id, false);
                    } else if (order.order_type.equalsIgnoreCase("Delivery")) {
                        miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocationsDelivery(order._id, prepLocation.id, false);
                        prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocationsDelivery(order._id, prepLocation.id, false);
                    } else if (order.order_type.equalsIgnoreCase("Waiting")) {
                        miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocationsBanquet(order._id, prepLocation.id, false);
                        prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocationsBanquet(order._id, prepLocation.id, false);
                    } else {
                        miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocations(order._id, prepLocation.id, false);
                        prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocations(order._id, prepLocation.id, false);
                    }
                } else {
                    if (order.order_type.equalsIgnoreCase("Collection")) {
                        miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocationsCollection(order._id, prepLocation.id);
                        prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocationsCollection(order._id, prepLocation.id);
                    } else if (order.order_type.equalsIgnoreCase("Delivery")) {
                        miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocationsDelivery(order._id, prepLocation.id);
                        prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocationsDelivery(order._id, prepLocation.id);
                    } else if (order.order_type.equalsIgnoreCase("Waiting")) {
                        miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocationsBanquet(order._id, prepLocation.id);
                        prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocationsBanquet(order._id, prepLocation.id);
                    } else {
                        miscPrepOits = appDatabase.orderItemDao().orderItemsForPrepLocations(order._id, prepLocation.id);
                        prepOits = appDatabase.orderItemDao().orderItemProductForPrepLocations(order._id, prepLocation.id);
                    }
                }

                if (miscPrepOits != null && miscPrepOits.size() > 0) {
                    allOits.addAll(miscPrepOits);
                }

                if (prepOits != null && prepOits.size() > 0) {
                    allOits.addAll(prepOits);
                }
                if (allOits.size() > 0) {
                    prepOrderItems.put(prepLocation, allOits);
                    for (OrderItem oit : allOits) {
                        oit.sent_to_kitchen = true;
                        appDatabase.orderItemDao().update(oit);
                    }
                }

                for (Map.Entry<PrepLocation, List<OrderItem>> entry : prepOrderItems.entrySet()) {
                    for (OrderItem orderItem : (ArrayList<OrderItem>) entry.getValue()) {
                        orderItem.order_item_ingredients = (ArrayList<OrderItemIngredient>) appDatabase.orderItemIngredientDao().list(orderItem._id);
                        orderItem.order_item_addons = (ArrayList<OrderItemAddon>) appDatabase.orderItemAddonDao().list(orderItem._id);
                    }
                }

            }
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            int position = 0;
            for (Map.Entry<PrepLocation, List<OrderItem>> entry : prepOrderItems.entrySet()) {
                position++;
                PrepLocation prepLocation = entry.getKey();
                if (printers == null) {
                    ToastUtils.makeToast(OrderReviewActivity.this, "Business printers are null");
                    continue;
                }
                Printer prepPrinter = null;
                for (Printer pp : printers) {
                    if (prepLocation.printer_id.equalsIgnoreCase(pp.id)) {
                        prepPrinter = pp;
                    }
                }
                if (prepPrinter == null) {
                    FirebaseCrashlytics.getInstance().log("Preplocation printer not found,: " + new Gson().toJson(prepLocation));
                    ToastUtils.makeToast(OrderReviewActivity.this, "Preplocation printer not found,: " + prepLocation.printer_name);
                    continue;
                }
                if (defaultPrinter == null && forceFullyDefaultPrinter != null && prepPrinter.ip.equalsIgnoreCase(forceFullyDefaultPrinter.ip)) {
                    continue;
                }
                Order tempOrder = new Gson().fromJson(new Gson().toJson(order), Order.class);
                tempOrder.order_items = (ArrayList<OrderItem>) entry.getValue();

                if (tempOrder.order_items.size() > 0) {
                    if (prepPrinter.printer_model_name.toLowerCase().contains("88 h")) {
                        zoneRichPrinter.connectPrinter(prepPrinter.ip);
                        CommonFunctions.functionThatDelay(200);
                        zoneRichPrinter.printOrderEpos(myApp.businessLogo, title, (ticketHeaderSetting != null) ? ticketHeaderSetting.value : null, headerAlignment, tableOrOrderIdSetting != null ? tableOrOrderIdSetting.value : null, dateTimeModeInPrint, tempOrder, printTotalInPrint, footerA, footerB, true, false);
                        if (buzzerSetting != null && !Validators.isNullOrEmpty(buzzerSetting.value) && buzzerSetting.value.toLowerCase().equalsIgnoreCase("yes") && prepLocation.cash_draw) {
                            zoneRichPrinter.connectPrinter(prepPrinter.ip);
                            CommonFunctions.functionThatDelay(200);
                            zoneRichPrinter.openCashDrawer();
                        }
                    } else if (prepPrinter.printer_model_name.toLowerCase().contains("t2") || prepPrinter.printer_model_name.toLowerCase().contains("sunmi")) {
                        sunmiPrinter.printOrderEpos(myApp.businessLogo, title, (ticketHeaderSetting != null) ? ticketHeaderSetting.value : null, headerAlignment, tableOrOrderIdSetting != null ? tableOrOrderIdSetting.value : null, dateTimeModeInPrint, tempOrder, printTotalInPrint, footerA, footerB, true, false);
                        if (buzzerSetting != null && !Validators.isNullOrEmpty(buzzerSetting.value) && buzzerSetting.value.toLowerCase().equalsIgnoreCase("yes") && prepLocation.cash_draw) {
                            sunmiPrinter.openCashDrawer();
                        }
                    } else if (prepPrinter.printer_model_name.toLowerCase().contains("pos-80") || prepPrinter.printer_model_name.toLowerCase().contains("pos 80") || prepPrinter.printer_model_name.toLowerCase().contains("yoke4")) {
//                        ZoneRichPrinter prepZonerich = new ZoneRichPrinter(OrderReviewActivity.this);
//                        prepZonerich.connectPrinter(prepPrinter.ip);
//                        try {
//                            Thread.sleep(100);
//                        } catch (InterruptedException e) {
//                            e.printStackTrace();
//                        }
//                        prepZonerich.printOrderEpos(myApp.businessLogo, title, (ticketHeaderSetting != null) ? ticketHeaderSetting.value : null, headerAlignment, tableOrOrderIdSetting != null ? tableOrOrderIdSetting.value : null, dateTimeModeInPrint, tempOrder, printTotalInPrint, footerA, footerB, true, false);
//                        if (buzzerSetting != null && !Validators.isNullOrEmpty(buzzerSetting.value) && buzzerSetting.value.toLowerCase().equalsIgnoreCase("yes") && prepLocation.cash_draw) {
//                            prepZonerich.connectPrinter(prepPrinter.ip);
//                            try {
//                                Thread.sleep(100);
//                            } catch (InterruptedException e) {
//                                e.printStackTrace();
//                            }
//                            prepZonerich.openCashDrawer();
//                        }

                        wifiPrinter.disconnect();
                        CommonFunctions.functionThatDelay(100);
                        wifiPrinter.connect(prepPrinter.ip);
                        CommonFunctions.functionThatDelay(100);
                        if (buzzerSetting != null && !Validators.isNullOrEmpty(buzzerSetting.value) && buzzerSetting.value.toLowerCase().equalsIgnoreCase("yes") && prepLocation.cash_draw) {
                            wifiPrinter.openCashDrawer(false);
                        }
                        wifiPrinter.printOrderEpos(myApp.businessLogo, title, (ticketHeaderSetting != null) ? ticketHeaderSetting.value : null, headerAlignment, tableOrOrderIdSetting != null ? tableOrOrderIdSetting.value : null, dateTimeModeInPrint, tempOrder, printTotalInPrint, footerA, footerB, true, false);
                    }
                }
            }
            if (position >= prepOrderItems.size()) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            myApp.isCashDrawerOpen = false;
        }
    }

    class OrderItemsPrintBlockAsync extends AsyncTask<Integer, Integer, Integer> {
        String title;
        ArrayList<OrderItem> groupedOrderItems = new ArrayList<>();
        Callable<Void> nextMethod;

        public OrderItemsPrintBlockAsync(String title, Callable<Void> nextMethod) {
            this.title = title;
            this.nextMethod = nextMethod;
        }

        @Override
        protected Integer doInBackground(Integer... integers) {
            List<OrderItem> miscOrderItems;
            List<OrderItem> productOrderItems;
            int _split_id = 0;
            if (integers.length > 0) {
                _split_id = integers[0];
                miscOrderItems = appDatabase.orderItemDao().orderItemsMiscForPrint(order._id, _split_id);
                productOrderItems = appDatabase.orderItemDao().orderItemProductForPrint(order._id, _split_id);
            } else {
                miscOrderItems = appDatabase.orderItemDao().orderItemsMiscForPrint(order._id);
                productOrderItems = appDatabase.orderItemDao().orderItemProductForPrint(order._id);
            }
            if (miscOrderItems != null && miscOrderItems.size() > 0) {
                groupedOrderItems.addAll(miscOrderItems);
            }

            if (productOrderItems != null && productOrderItems.size() > 0) {
                groupedOrderItems.addAll(productOrderItems);
            }
            if (_split_id > 0) {
                OrderSplit orderSplit = appDatabase.orderSplitDao().view(_split_id);
                order.total = orderSplit.total;
            }

            return 0;
        }

        @Override
        protected void onPostExecute(Integer integer) {
            super.onPostExecute(integer);
            Order tempOrder = new Gson().fromJson(new Gson().toJson(order), Order.class);
            if (splitByPersonPrint) {
                tempOrder.total = printableTotal;
            }
            tempOrder.order_items = groupedOrderItems;
            if (defaultPrinter.printer_model_name.toLowerCase().contains("88 h")) {
                zoneRichPrinter.connectPrinter(defaultPrinter.ip);
                CommonFunctions.functionThatDelay(200);
                zoneRichPrinter.printOrderEpos(myApp.businessLogo, title, (ticketHeaderSetting != null) ? ticketHeaderSetting.value : null, headerAlignment, tableOrOrderIdSetting != null ? tableOrOrderIdSetting.value : null, dateTimeModeInPrint, tempOrder, true, footerA, footerB, false, false);
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("t2") || defaultPrinter.printer_model_name.toLowerCase().contains("sunmi")) {
                sunmiPrinter.printOrderEpos(myApp.businessLogo, title, (ticketHeaderSetting != null) ? ticketHeaderSetting.value : null, headerAlignment, tableOrOrderIdSetting != null ? tableOrOrderIdSetting.value : null, dateTimeModeInPrint, tempOrder, true, footerA, footerB, false, false);
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("pos-80") || defaultPrinter.printer_model_name.toLowerCase().contains("pos 80") || defaultPrinter.printer_model_name.toLowerCase().contains("yoke4")) {
//                zoneRichPrinter.connectPrinter(defaultPrinter.ip);
//                try {
//                    Thread.sleep(100);
//                } catch (InterruptedException e) {
//                    e.printStackTrace();
//                }
//                ToastUtils.makeToast(OrderReviewActivity.this,defaultPrinter.ip);
//                zoneRichPrinter.printOrderEpos(myApp.businessLogo, title, (ticketHeaderSetting != null) ? ticketHeaderSetting.value : null, headerAlignment, tableOrOrderIdSetting != null ? tableOrOrderIdSetting.value : null, dateTimeModeInPrint, tempOrder, true, footerA, footerB, false, false);
                wifiPrinter.disconnect();
                CommonFunctions.functionThatDelay(100);
                wifiPrinter.connect(defaultPrinter.ip);
                CommonFunctions.functionThatDelay(100);
                wifiPrinter.printOrderEpos(myApp.businessLogo, title, (ticketHeaderSetting != null) ? ticketHeaderSetting.value : null, headerAlignment, tableOrOrderIdSetting != null ? tableOrOrderIdSetting.value : null, dateTimeModeInPrint, tempOrder, printTotalInPrint, footerA, footerB, false, false);

            }
            myApp.isCashDrawerOpen = false;
            try {
                nextMethod.call();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

    }


    BroadcastReceiver fragmentChangeReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getExtras() != null) {
                boolean canEdit = intent.getBooleanExtra("can_edit", false);
                if (canEdit && canEditOrder) {
                    tvEditQty.setVisibility(View.VISIBLE);
                    llDone.setVisibility(View.GONE);
                    orderItemsAdapter.shouldEdit = false;
                    orderItemsAdapter.shouldClick = true;
                } else {
                    tvEditQty.setVisibility(View.GONE);
                    llDone.setVisibility(View.GONE);
                    orderItemsAdapter.shouldClick = false;
                    orderItemsAdapter.shouldEdit = false;
                }
                orderItemsAdapter.notifyDataSetChanged();
            }
        }
    };

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (fragmentChangeReceiver != null)
            LocalBroadcastManager.getInstance(OrderReviewActivity.this).unregisterReceiver(fragmentChangeReceiver);
    }

    @Override
    public void onBackPressed() {
        setResult(RESULT_OK);
        Fragment openedFragment = getSupportFragmentManager().findFragmentById(R.id.flContainer);

        if (drawerLayout.isDrawerOpen(GravityCompat.START)) {
            drawerLayout.closeDrawer(GravityCompat.END);
        } else {
            if (openedFragment instanceof PaymentMethodSelectionFragment) {
                finish();
            } else {
                if (manager.getBackStackEntryCount() > 0) {
                    manager.popBackStack();
                } else if (openedFragment instanceof AmountSelectionFragment) {
                    changeFragment(new PaymentMethodSelectionFragment());
                } else {
                    finish();
                }
            }
        }

    }
}
