package com.example.epos_2021.activities;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.ProductAddonAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.daos.AppDatabase;
import com.example.epos_2021.fragment.ConfirmationDialogFragment;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.Addon;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.OrderItemAddon;
import com.example.epos_2021.models.Product;
import com.example.epos_2021.models.ProductAddon;
import com.example.epos_2021.storageutils.MyPreferences;
import com.example.epos_2021.utils.LogUtils;
import com.google.android.material.button.MaterialButton;
import com.google.android.material.chip.Chip;
import com.google.gson.Gson;

import java.io.File;
import java.util.ArrayList;

/**
 * Created by Amrish on 29-05-2021.
 */
public class ProductAddonDialogFragment extends BaseFragment {

    private Chip chipBack;
    private RelativeLayout rlPlusQty, rlMinusQty;
    private TextView tvProductName, tvShortName, tvProductAmount, tvModifiedQty;
    private ImageView ivProduct;
    private RecyclerView rvProductAddons, rvSubAddonOptions;
    private ProductAddonAdapter productAddonAdapter;
    private Product product;
    private String orderTypeId, productId;
    private ArrayList<ProductAddon> productAddons = new ArrayList<>();
    private AlertDialog progressBarDialog;
    private MaterialButton btnConfirm, btnCancel;
    private Toolbar toolbar;
    private int modifiedQuantity = 1;
    public MyApp myApp = MyApp.getInstance();
    public MyPreferences myPreferences = myApp.myPreferences;
    public AppDatabase appDatabase = myApp.appDatabase;

    private DialogDismissListener dialogDismissListener;
    private int total_ingredients;

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    public static ProductAddonDialogFragment getInstance(String order_id, Product product, int total_ingredients) {
        ProductAddonDialogFragment addonFragment = new ProductAddonDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("order_type_id", order_id);
        bundle.putString("product", new Gson().toJson(product));
        bundle.putInt("total_ingredients", total_ingredients);
        addonFragment.setArguments(bundle);
        return addonFragment;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            product = new Gson().fromJson(getArguments().getString("product"), Product.class);
            orderTypeId = getArguments().getString("order_type_id");
            total_ingredients = getArguments().getInt("total_ingredients");

            LogUtils.e("product", new Gson().toJson(product));
        }

        return inflater.inflate(R.layout.fragment_product_addons, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);

        new FetchProductDetailsAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, product.id);
        setListeners();
    }

    private void initViews(View view) {
        progressBarDialog = CommonFunctions.customProgressDialog(getActivity());
        rvProductAddons = view.findViewById(R.id.rvProductAddons);
        btnCancel = view.findViewById(R.id.btnCancel);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        tvProductName = view.findViewById(R.id.tvAddonName);
        tvShortName = view.findViewById(R.id.tvSize);
        tvProductAmount = view.findViewById(R.id.tvAmount);
        ivProduct = view.findViewById(R.id.ivProduct);
        rlPlusQty = view.findViewById(R.id.rlPlusModified);
        rlMinusQty = view.findViewById(R.id.rlMinusModified);
        tvModifiedQty = view.findViewById(R.id.tvModiefiedQuantity);
        chipBack = view.findViewById(R.id.chipBack);

        productAddonAdapter = new ProductAddonAdapter(productAddons, orderTypeId, (position, data) -> {

        }, (position, data) -> {

        });
        rvProductAddons.setAdapter(productAddonAdapter);
        rvProductAddons.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));

    }

    private void updateViews() {
        tvProductName.setText(product.name);
        tvShortName.setText(product.short_name);
        tvProductAmount.setText(MyApp.currencySymbol + " " + MyApp.df.format(product.selected_price));
        if (myApp.productImageEnabled() && !Validators.isNullOrEmpty(product.image)) {
            ivProduct.setVisibility(View.VISIBLE);
            Glide.with(getActivity())
                    .load(new File(getActivity().getExternalFilesDir("epos2021").getAbsolutePath()) + "/" + product.id + ".png")
                    .into(ivProduct);
        } else
            ivProduct.setVisibility(View.GONE);
    }

    private void setListeners() {
        chipBack.setOnClickListener(v -> getActivity().onBackPressed());
        btnConfirm.setOnClickListener(view -> {
            createOrderItem();
        });
        btnCancel.setOnClickListener(view -> {
            getActivity().onBackPressed();
        });

        tvModifiedQty.setText("1");
        rlPlusQty.setOnClickListener(v -> {
            modifiedQuantity++;
            tvModifiedQty.setText(String.valueOf(modifiedQuantity));
        });
        rlMinusQty.setOnClickListener(v -> {
            if (modifiedQuantity > 1) {
                modifiedQuantity--;
                tvModifiedQty.setText(String.valueOf(modifiedQuantity));
            }
        });
    }

    private void createOrderItem() {
        int position = 0;
        ArrayList<OrderItemAddon> allSelectedAddons = new ArrayList<>();
        for (ProductAddon productAddon : productAddons) {
            if (productAddon.selectedQuantity == 0 && productAddon.min_quantity == 0) continue;
            if (productAddon.selectedQuantity >= productAddon.min_quantity && productAddon.selectedQuantity <= productAddon.max_quantity) {
                OrderItemAddon orderItemAddon = new OrderItemAddon();
                orderItemAddon.addon_id = productAddon.addon.id;
                orderItemAddon.addon_name = productAddon.addon.name;
                orderItemAddon.price = productAddon.addon.selected_price;
                orderItemAddon.updater_id = MyApp.getInstance().myPreferences.getLoggedInUser().id;
                orderItemAddon.quantity = 1;
                orderItemAddon.total = orderItemAddon.price * orderItemAddon.quantity;
                orderItemAddon.addon = productAddon.addon;
                allSelectedAddons.add(orderItemAddon);
                allSelectedAddons.addAll(productAddon.selectedOrderItemAddon);
            } else {
                productAddonAdapter.expandedPosition = position;
                String message = "";
                if (productAddon.addon != null)
                    message = "Please select " + productAddon.addon.name;
                else message = "Please select addon";
                productAddonAdapter.notifyDataSetChanged();
                ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Addon required", message, 2, "Okay", null);
                confirmationDialogFragment.show(getActivity().getSupportFragmentManager(), "action");
                return;
            }
            position++;
        }

        OrderItem orderItem = new OrderItem();
        orderItem.product_name = product.name;
        orderItem.product_id = product.id;
        orderItem.quantity = modifiedQuantity;
        orderItem.order_item_addons = allSelectedAddons;
        orderItem.price = MyApp.calculatedProductPrice(orderTypeId, product);
        orderItem.sub_total = (orderItem.quantity * orderItem.price);
        orderItem.addons_price = getAddonPrice(allSelectedAddons);
        orderItem.total = orderItem.sub_total + orderItem.addons_price + orderItem.ingredients_price;
        orderItem.updater_id = myPreferences.getLoggedInUser().id;
        if (dialogDismissListener != null)
            dialogDismissListener.onDialogDismiss(orderItem);

        if (total_ingredients == 0)
            myApp.notifyCart(getActivity(), new Intent("cart_update").putExtra("order_item", new Gson().toJson(orderItem)).putExtra("notify_products", true));
    }

    private class FetchProductDetailsAsync extends AsyncTask<String, String, String> {
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            progressBarDialog.show();
        }

        @Override
        protected String doInBackground(String... strings) {
            String productId = strings[0];
            productAddons.addAll(appDatabase.productAddonDao().listByProduct(productId));
            for (ProductAddon productAddon : productAddons) {
                productAddon.addon = appDatabase.addonDao().view(productAddon.addon_id);
                productAddon.addon.children = new ArrayList<>();
                productAddon.addon.children.addAll(appDatabase.addonDao().list(productAddon.addon.id));
                if (productAddon.addon.children != null && productAddon.addon.children.size() > 0) {
                    for (Addon addonL1 : productAddon.addon.children) {
                        addonL1.children = new ArrayList<>();
                        addonL1.children.addAll(appDatabase.addonDao().list(addonL1.id));
                        if (addonL1.children != null && addonL1.children.size() > 0) {
                            for (Addon addonL2 : addonL1.children) {
                                addonL2.children = new ArrayList<>();
                                addonL2.children.addAll(appDatabase.addonDao().list(addonL2.id));
                            }
                        }
                    }
                }
            }

            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            progressBarDialog.dismiss();
            if (productAddons.size() > 0) {
                productAddonAdapter.expandedPosition = 0;
            }
            productAddonAdapter.notifyDataSetChanged();
            updateViews();
        }
    }

    private float getAddonPrice(ArrayList<OrderItemAddon> orderItemAddons) {
        float addonPrice = 0;
        for (OrderItemAddon orderItemAdd : orderItemAddons) {
            addonPrice = addonPrice + orderItemAdd.total;
        }
        return addonPrice;
    }
}
