package com.example.epos_2021.activities;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.airbnb.lottie.LottieAnimationView;
import com.example.epos_2021.R;
import com.example.epos_2021.base.BaseActivity;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.daos.relations.OrderWithItems;
import com.example.epos_2021.models.Customer;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.Reservation;
import com.example.epos_2021.services.SingleCustomerUploadService;
import com.example.epos_2021.services.SingleOrderUploaderService;
import com.example.epos_2021.services.SingleReservationUploadService;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.LogUtils;
import com.example.epos_2021.utils.ToastUtils;
import com.google.firebase.messaging.FirebaseMessaging;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 01-06-2021.
 */
public class UploadOrdersActivity extends BaseActivity {
    @Override
    protected void setXML() {
        super.setXML();
        setContentView(R.layout.activity_upload_orders);
    }

    private Toolbar toolbar;
    private TextView tvUploadStatus, tvMessage;
    private ArrayList<Order> orders = new ArrayList<>();
    private ArrayList<Reservation> reservations = new ArrayList<>();
    private ArrayList<Customer> customers = new ArrayList<>();
    private LottieAnimationView lottieAnimationView;
    private boolean admin_logout = false;
    boolean canGoBack = false, finish = false;

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getIntent() != null) {
            admin_logout = getIntent().getBooleanExtra("admin_logout", false);
            finish = getIntent().getBooleanExtra("finish", false);
        }
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        new FetchAllDataAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    @Override
    protected void initViews() {
        super.initViews();
        lottieAnimationView = findViewById(R.id.loading);
        toolbar = findViewById(R.id.toolbar);
        tvUploadStatus = findViewById(R.id.tvUploadCount);
        tvMessage = findViewById(R.id.tvMessage);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setTitle("Push data");
        toolbar.setNavigationIcon(R.drawable.ic_baseline_arrow_back_24);

    }

    class FetchAllDataAsync extends AsyncTask<String, String, String> {

        @Override
        protected String doInBackground(String... strings) {
            orders.clear();
            List<OrderWithItems> orderWithItemsList = appDatabase.orderDao().ordersWithItems();
            for (OrderWithItems orderWithItems : orderWithItemsList) {
                Order order = orderWithItems.order;

                orders.add(order);
            }
            reservations.clear();
            reservations.addAll(appDatabase.reservationDao().uploadableList());

            customers.clear();
            customers.addAll(appDatabase.customerDao().uploadableList());

            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            tvUploadStatus.setVisibility(View.VISIBLE);
            startUploading();
        }
    }

    private void startUploading() {
        if (orders.size() > 0) {
            tvUploadStatus.setText(String.format("Uploading orders %d of %d", 1, orders.size()));
            startService(new Intent(UploadOrdersActivity.this, SingleOrderUploaderService.class).putExtra("forcefully", true).putExtra("_order_id", orders.get(0)._id).putExtra("position", 0));
        } else if (reservations.size() > 0) {
            startReservations();
        } else if (customers.size() > 0) {
            startCustomers();
        } else {
            goBack();
        }
    }

    private void startCustomers() {
        tvUploadStatus.setText(String.format("Uploading customers %d of %d", 1, customers.size()));
        startService(new Intent(UploadOrdersActivity.this, SingleCustomerUploadService.class).putExtra("forcefully", true).putExtra("_customer_id", customers.get(0)._id).putExtra("position", 0));
    }

    private void startReservations() {
        tvUploadStatus.setText(String.format("Uploading reservations %d of %d", 1, reservations.size()));
        startService(new Intent(UploadOrdersActivity.this, SingleReservationUploadService.class).putExtra("forcefully", true).putExtra("_reservation_id", reservations.get(0)._id).putExtra("position", 0));
    }

    private void goBack() {
        if (admin_logout) {
            doAdminLogout();
        } else if (finish) {
            setResult(RESULT_OK);
            finish();
        } else {
            startActivity(new Intent(UploadOrdersActivity.this, HomeActivity.class));
            finish();
        }
    }


    BroadcastReceiver orderUpdateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int position = intent.getIntExtra("position", -1);
            if (position == orders.size() - 1) {
                if (reservations.size() > 0)
                    startReservations();
                else if (customers.size() > 0)
                    startCustomers();
                else goBack();
            } else {
                tvUploadStatus.setText(String.format("Uploading orders %d of %d", position + 2, orders.size()));
                startService(new Intent(UploadOrdersActivity.this, SingleOrderUploaderService.class).putExtra("forcefully", true).putExtra("_order_id", orders.get(position + 1)._id).putExtra("position", position + 1));
            }

            if (intent.getStringExtra("message") != null) {
                String message = "\n" + intent.getStringExtra("message");
                tvMessage.append(message);
            }

        }
    };
    BroadcastReceiver reservationUpdateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int position = intent.getIntExtra("position", -1);
            if (position == reservations.size() - 1) {
                startCustomers();
            } else {
                tvUploadStatus.setText(String.format("Uploading reservations %d of %d", position + 2, reservations.size()));
                startService(new Intent(UploadOrdersActivity.this, SingleReservationUploadService.class).putExtra("forcefully", true).putExtra("_reservation_id", reservations.get(position + 1)._id).putExtra("position", position + 1));
            }

            if (intent.getStringExtra("message") != null) {
                String message = "\n" + intent.getStringExtra("message");
                tvMessage.append(message);
            }

        }
    };
    BroadcastReceiver customerUpdateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int position = intent.getIntExtra("position", -1);
            LogUtils.e("position:" + position);
            if (position == customers.size() - 1) {
                goBack();
            } else {
                tvUploadStatus.setText(String.format("Uploading customers %d of %d", position + 2, customers.size()));
                startService(new Intent(UploadOrdersActivity.this, SingleCustomerUploadService.class).putExtra("forcefully", true).putExtra("_customer_id", customers.get(position + 1)._id).putExtra("position", position + 1));
            }

            if (intent.getStringExtra("message") != null) {
                String message = "\n" + CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), "dd-MM hh:mm:ss a") + " " + intent.getStringExtra("message");
                tvMessage.append(message);
            }

        }
    };

    private void doAdminLogout() {
        ToastUtils.makeToast(UploadOrdersActivity.this, "Logout successfully!!");
        FirebaseMessaging.getInstance().unsubscribeFromTopic("business_" + myPreferences.getLoggedInAdmin().selected_business.id);
        myPreferences.saveLoggedInAdmin(null);
        myPreferences.saveLoggedInUser(null);
        myPreferences.saveUserTokens(null, null);
        myPreferences.saveAdminTokens(null, null);
        myPreferences.userLoggedInWithoutInternet(false);
        myPreferences.saveBusinessId(null);
        new Thread(() -> {
            appDatabase.emptyPulledDao().nukeAddon();
            appDatabase.emptyPulledDao().nukeCategory();
            appDatabase.emptyPulledDao().nukeCustomer();
            appDatabase.emptyPulledDao().nukePrepLocation();
            appDatabase.emptyPulledDao().nukeProduct();
            appDatabase.emptyPulledDao().nukeProductAddon();
            appDatabase.emptyPulledDao().nukeProductIngredient();
            appDatabase.emptyPulledDao().nukeTable();
            appDatabase.emptyPulledDao().nukeFloor();
            appDatabase.emptyPulledDao().nukeOrderItemIngredient();
            appDatabase.emptyPulledDao().nukeOrderItemAddon();
            appDatabase.emptyPulledDao().nukeOrderPayment();
            appDatabase.emptyPulledDao().nukeOrderSplit();
            appDatabase.emptyPulledDao().nukeOrder();
            appDatabase.emptyPulledDao().nukeUsers();
            appDatabase.emptyPulledDao().nukeReservations();
            appDatabase.emptyPulledDao().nukeVouchers();
            appDatabase.orderDao().resetPrimaryKey();
            try {
                deleteImages();
            } catch (Exception e) {
                e.printStackTrace();
            }

        }).start();
        myPreferences.savePullStatus(false);
        startActivity(new Intent(UploadOrdersActivity.this, SplashActivity.class));
        finish();
    }

    private void deleteImages() {
        File dir = new File(getApplicationContext().getExternalFilesDir("epos2021").getAbsolutePath());
        if (dir.isDirectory()) {
            String[] children = dir.list();
            if (children != null) {
                for (String child : children) {
                    try {
                        new File(dir, child).delete();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

            }
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        if (orderUpdateReceiver != null)
            LocalBroadcastManager.getInstance(this).registerReceiver(orderUpdateReceiver, new IntentFilter(
                    Constants.EPOS_ORDER_UPDATE));
        if (customerUpdateReceiver != null)
            LocalBroadcastManager.getInstance(this).registerReceiver(customerUpdateReceiver, new IntentFilter(
                    "single_customer_upload"));
        if (reservationUpdateReceiver != null)
            LocalBroadcastManager.getInstance(this).registerReceiver(reservationUpdateReceiver, new IntentFilter(
                    "single_reservation_upload"));
    }

    @Override
    public void onBackPressed() {
        if (canGoBack) super.onBackPressed();
        else ToastUtils.makeLongToast(this, "Please wait while uploading data");
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (orderUpdateReceiver != null) {
            LocalBroadcastManager.getInstance(this).unregisterReceiver(orderUpdateReceiver);
        }
        if (customerUpdateReceiver != null) {
            LocalBroadcastManager.getInstance(this).unregisterReceiver(customerUpdateReceiver);
        }
        if (reservationUpdateReceiver != null) {
            LocalBroadcastManager.getInstance(this).unregisterReceiver(reservationUpdateReceiver);
        }
    }
}
