package com.example.epos_2021.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Filter;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.example.epos_2021.R;
import com.example.epos_2021.models.Customer;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 07-05-2021.
 */
public class CustomerAutoSuggestAdapter extends ArrayAdapter<Customer> {

    private ArrayList<Customer> customers;
    private ArrayList<Customer> tempCustomers;
    private ArrayList<Customer> filteredCustomers;
    List<Customer> items, tempItems, suggestions;
    private Activity activity;

    public CustomerAutoSuggestAdapter(@NonNull Activity context, List<Customer> items) {
        super(context, 0, items);
        this.activity = context;
        this.items = items;
        tempItems = new ArrayList<Customer>(items); // this makes the difference.
        suggestions = new ArrayList<Customer>();
    }

    public void updateData(List<Customer> items) {
        this.items = items;
        tempItems = new ArrayList<Customer>(items); // this makes the difference.
        if (activity != null) {
            activity.runOnUiThread(this::notifyDataSetChanged);
        }
    }

    @NonNull
    @Override
    public View getView(int position, @Nullable View convertView, @NonNull ViewGroup parent) {
        // Check if an existing view is being reused, otherwise inflate the view
        if (convertView == null) {
            convertView = LayoutInflater.from(getContext()).inflate(R.layout.item_customer, parent, false);
        }
        TextView tvCustomer = convertView.findViewById(R.id.tvCustomer);

        Customer customer = items.get(position);
        tvCustomer.setText(customer.name + " (" + customer.mobile + ")");
        return convertView;
    }

    @Override
    public Filter getFilter() {
        return nameFilter;
    }

    /**
     * Custom Filter implementation for custom suggestions we provide.
     */
    Filter nameFilter = new Filter() {
        @Override
        public CharSequence convertResultToString(Object resultValue) {
            String str = ((Customer) resultValue).name;
            return str;
        }

        @Override
        protected FilterResults performFiltering(CharSequence constraint) {
            if (activity != null) {
                activity.runOnUiThread(() -> {

                });
            }
            if (constraint != null) {
                suggestions.clear();
                int count = 0;
                for (Customer people : tempItems) {
                    if ((people.name != null && people.name.toLowerCase().contains(constraint.toString().toLowerCase())) || (people.mobile != null && people.mobile.contains(constraint.toString()))) {
                        suggestions.add(people);
                        count++;
                        if (count == 25) {
                            break;
                        }
                    }
                }
                FilterResults filterResults = new FilterResults();
                filterResults.values = suggestions;
                filterResults.count = suggestions.size();
                return filterResults;
            } else {
                return new FilterResults();
            }
        }

        @Override
        protected void publishResults(CharSequence constraint, FilterResults results) {
            if (activity != null) {
                activity.runOnUiThread(() -> {
                    List<Customer> filterList = (ArrayList<Customer>) results.values;
                    if (results != null && results.count > 0) {
                        clear();
                        addAll(filterList);
                        notifyDataSetChanged();
                    }

                });
            }
        }
    };
}
