package com.example.epos_2021.adapters;

import android.app.Activity;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.R;
import com.example.epos_2021.interfaces.RecyclerviewItemClickListener;
import com.example.epos_2021.models.User;
import com.google.android.material.card.MaterialCardView;

import java.util.ArrayList;

public class MiniUserAdapter extends RecyclerView.Adapter<MiniUserAdapter.ViewHolder> {
    private Activity activity;
    private ArrayList<User> users;
    private RecyclerviewItemClickListener itemClickListener;
    private String selectedUsername = "";

    public MiniUserAdapter(Activity activity, ArrayList<User> users, RecyclerviewItemClickListener itemClickListener) {
        this.users = users;
        this.activity = activity;
        this.itemClickListener = itemClickListener;
    }

    @NonNull
    @Override
    public MiniUserAdapter.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_login_users, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull MiniUserAdapter.ViewHolder holder, int position) {
        User userModel = users.get(position);
        holder.tvUserName.setText(userModel.username);
        holder.cvUser.setOnClickListener(v -> {
            if (itemClickListener != null) {
                selectedUsername = userModel.username;
                itemClickListener.onItemClick(position, userModel);
                notifyDataSetChanged();
            }

        });
        Context context = holder.itemView.getContext();
        if (selectedUsername.equalsIgnoreCase(userModel.username)) {
            holder.cvUser.setCardBackgroundColor(ContextCompat.getColor(context, R.color.persian_green));
            holder.tvUserName.setTextColor(ContextCompat.getColor(context, R.color.white));
        } else {
            holder.cvUser.setCardBackgroundColor(ContextCompat.getColor(context, R.color.white));
            holder.tvUserName.setTextColor(ContextCompat.getColor(context, R.color.black));
        }
    }

    @Override
    public int getItemCount() {
        return users.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        private TextView tvUserName;
        private MaterialCardView cvUser;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            tvUserName = itemView.findViewById(R.id.tvUserName);
            cvUser = itemView.findViewById(R.id.cvUser);
        }
    }
}
