package com.example.epos_2021.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.interfaces.RecyclerviewItemClickListener;
import com.example.epos_2021.models.Addon;
import com.google.android.material.card.MaterialCardView;

import java.util.ArrayList;

public class SubAddonAdapter extends RecyclerView.Adapter<SubAddonAdapter.ViewHolder> {

    private final ArrayList<Addon> subAddons;
    private final RecyclerviewItemClickListener itemClickListener;

    public SubAddonAdapter(ArrayList<Addon> subAddons, RecyclerviewItemClickListener itemClickListener) {
        this.subAddons = subAddons;
        this.itemClickListener = itemClickListener;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_subaddon, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        Addon addon = subAddons.get(position);
        Context context = holder.itemView.getContext();
        holder.tvSubAddon.setText(addon.name);
        holder.tvPrice.setText(MyApp.currencySymbol + MyApp.df.format(addon.price));

        if (addon.selected) {
            holder.cvSubAddon.setCardBackgroundColor(ContextCompat.getColor(context, R.color.persian_green));
            holder.tvSubAddon.setTextColor(ContextCompat.getColor(context, R.color.white));
            holder.tvPrice.setTextColor(ContextCompat.getColor(context, R.color.white));
        } else {
            holder.cvSubAddon.setCardBackgroundColor(ContextCompat.getColor(holder.itemView.getContext(), R.color.white));
            holder.tvSubAddon.setTextColor(ContextCompat.getColor(context, R.color.black));
            holder.tvPrice.setTextColor(ContextCompat.getColor(context, R.color.black));
        }

        holder.cvSubAddon.setOnClickListener(v -> {
            if (itemClickListener != null) {
                itemClickListener.onItemClick(position, addon);
            }
        });
    }

    @Override
    public int getItemCount() {
        return subAddons.size();
    }

    public static class ViewHolder extends RecyclerView.ViewHolder {

        private TextView tvSubAddon, tvPrice;
        private MaterialCardView cvSubAddon;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            tvSubAddon = itemView.findViewById(R.id.tvSubAddon);
            tvPrice = itemView.findViewById(R.id.tvPrice);
            cvSubAddon = itemView.findViewById(R.id.cvSubAddon);
        }
    }
}
