package com.example.epos_2021.adapters;

import android.graphics.Color;
import android.graphics.drawable.GradientDrawable;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.interfaces.RecyclerviewItemClickListener;
import com.example.epos_2021.interfaces.RecyclerviewItemClickListenerWithTag;
import com.example.epos_2021.models.Table;
import com.example.epos_2021.models.TableStatus;
import com.example.epos_2021.utils.Constants;
import com.google.android.material.card.MaterialCardView;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class TablesAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private ArrayList<Table> tables;
    private boolean fromTheme2;
    private RecyclerviewItemClickListener itemClickListener, actionClick;
    private RecyclerviewItemClickListenerWithTag homeActionClickListener;
    private List<ViewHolder> lstHolders;
    private List<HomeViewHolder> lstHomeHolders;
    private Handler handler = new Handler();
    MyApp myApp = MyApp.getInstance();

    TableStatus vacantStatus = myApp.findStatus("Vacant");
    TableStatus takingOrderStatus = myApp.findStatus("Taking Order");
    TableStatus orderTakenStatus = myApp.findStatus("Order Taken");
    TableStatus servedStatus = myApp.findStatus("Served");
    TableStatus servedPaidStatus = myApp.findStatus("Served and paid");

    private Runnable updateTime = new Runnable() {
        @Override
        public void run() {
            synchronized (lstHolders) {
                long currentTime = System.currentTimeMillis();
                for (ViewHolder holder : lstHolders) {
                    holder.updateTimeRemaining(currentTime);
                }
            }
            synchronized (lstHomeHolders) {
                long currentTime = System.currentTimeMillis();
                for (HomeViewHolder holder : lstHomeHolders) {
                    holder.updateTimeRemaining(currentTime);
                }
            }

        }
    };

    private void startUpdateTimer() {
        Timer tmr = new Timer();
        tmr.schedule(new TimerTask() {
            @Override
            public void run() {
                handler.post(updateTime);
            }
        }, 1000, 1000);
    }

    public TablesAdapter(ArrayList<Table> tables, boolean fromTheme2, RecyclerviewItemClickListener itemClickListener, RecyclerviewItemClickListenerWithTag actionClick) {
        this.tables = tables;
        this.itemClickListener = itemClickListener;
        this.homeActionClickListener = actionClick;
        this.fromTheme2 = fromTheme2;
        lstHolders = new ArrayList<>();
        lstHomeHolders = new ArrayList<>();
        startUpdateTimer();
    }


    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        if (viewType == R.layout.item_table) {
            return new ViewHolder(LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false));
        } else {
            return new HomeViewHolder(LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false));
        }
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder viewHolder, int position) {
        if (getItemViewType(position) == R.layout.item_table) {
            ViewHolder holder = (ViewHolder) viewHolder;
            Table table = tables.get(position);

            synchronized (lstHolders) {
                lstHolders.add(holder);
            }
            holder.setData(table);

            String action = "";
            if (table.status != null && (vacantStatus != null && takingOrderStatus != null && orderTakenStatus != null && servedStatus != null && servedPaidStatus != null)) {

                if (table.status.equalsIgnoreCase(vacantStatus.status)) {
                    holder.tvTotal.setVisibility(View.INVISIBLE);
                    holder.tvTimeDiff.setVisibility(View.INVISIBLE);
                    holder.cvAction.setVisibility(View.INVISIBLE);
                } else if (table.status.equalsIgnoreCase(takingOrderStatus.status)) {
                    //Can cancel order only if permitted.
                    action = "cancel_order";
                    holder.tvAction.setText("Cancel Order");
                    holder.cvAction.setVisibility(View.VISIBLE);
                } else if (table.status.equalsIgnoreCase(orderTakenStatus.status)) {
                    // Now we can perform served order status
                    action = "serve_order";
                    holder.tvAction.setText("Serve Order");
                    holder.cvAction.setVisibility(View.VISIBLE);
                } else if (table.status.equalsIgnoreCase(servedStatus.status)) {
                    // No we need pay order button
                    action = "pay_order";
                    holder.tvAction.setText("Pay order");
                    holder.cvAction.setVisibility(View.VISIBLE);
                } else if (table.status.equalsIgnoreCase(servedPaidStatus.status)) {
                    //Now we can complete or vacant the table
                    action = "complete_order";
                    holder.tvAction.setText("Mark as Vacant");
                    holder.cvAction.setVisibility(View.VISIBLE);
                }
                if (!table.status.equalsIgnoreCase(vacantStatus.status.toLowerCase())) {
                    if (table.last_order_total > 0) {
                        holder.tvTotal.setVisibility(View.VISIBLE);
                        holder.tvTotal.setText("Total: " + MyApp.currencySymbol + MyApp.df.format(table.last_order_total));
                    } else {
                        holder.tvTotal.setVisibility(View.INVISIBLE);
                    }
                } else {
                    holder.tvTotal.setVisibility(View.INVISIBLE);
                }
            } else {
                holder.tvTotal.setVisibility(View.INVISIBLE);
                holder.tvTimeDiff.setVisibility(View.INVISIBLE);
                holder.cvAction.setVisibility(View.INVISIBLE);
            }

            holder.cvTable.setOnClickListener(v -> {
                if (itemClickListener != null) {
                    itemClickListener.onItemClick(position, table);
                }
            });
            String finalAction = action;
            holder.tvAction.setOnClickListener(view -> {
                if (homeActionClickListener != null)
                    homeActionClickListener.onItemClick(position, finalAction, table);
            });
        } else if (getItemViewType(position) == R.layout.item_home_tables) {
            HomeViewHolder holder = (HomeViewHolder) viewHolder;
            Table table = tables.get(position);

            synchronized (lstHomeHolders) {
                lstHomeHolders.add(holder);
            }
            holder.setData(table);
            holder.cvTable.setOnClickListener(v -> {
                if (itemClickListener != null) {
                    itemClickListener.onItemClick(position, table);
                }
            });
            String action = "";
            if (table.status != null) {
                if (table.status.equalsIgnoreCase(vacantStatus.status)) {
                    holder.ivAction.setVisibility(View.GONE);
                } else if (table.status.equalsIgnoreCase(takingOrderStatus.status)) {
                    //Can cancel order only if permitted.
                    action = "cancel_order";
                    holder.ivAction.setImageResource(R.drawable.ic_baseline_close_24);
                    holder.ivAction.setVisibility(View.VISIBLE);
                } else if (table.status.equalsIgnoreCase(orderTakenStatus.status)) {
                    // Now we can perform served order status
                    action = "serve_order";
                    holder.ivAction.setImageResource(R.drawable.icon_serving);
                    holder.ivAction.setVisibility(View.VISIBLE);
                } else if (table.status.equalsIgnoreCase(servedStatus.status)) {
                    // No we need pay order button
                    action = "pay_order";
                    holder.ivAction.setImageResource(R.drawable.icon_pound_sign);
                    holder.ivAction.setVisibility(View.VISIBLE);
                } else if (table.status.equalsIgnoreCase(servedPaidStatus.status)) {
                    //Now we can complete or vacant the table
                    action = "complete_order";
                    holder.ivAction.setImageResource(R.drawable.ic_baseline_black_done_24);
                    holder.ivAction.setVisibility(View.VISIBLE);
                }
            }

            String finalAction = action;
            holder.ivAction.setOnClickListener(view -> {
                if (homeActionClickListener != null)
                    homeActionClickListener.onItemClick(position, finalAction, table);
            });
        }
    }

    @Override
    public int getItemViewType(int position) {
        if (fromTheme2) {
            return R.layout.item_home_tables;
        } else {
            return R.layout.item_table;
        }
    }

    @Override
    public int getItemCount() {
        return tables.size();
    }

    public static class ViewHolder extends RecyclerView.ViewHolder {
        LinearLayout llTable;
        TextView tvTableStatus, tvTable, tvAction, tvTotal, tvTimeDiff;
        MaterialCardView cvTable, cvAction;
        Table table;
        MyApp myApp = MyApp.getInstance();
        public TableStatus vacantStatus = myApp.findStatus("Vacant");
        public TableStatus takingOrderStatus = myApp.findStatus("Taking Order");
        public TableStatus orderTakenStatus = myApp.findStatus("Order Taken");
        public TableStatus servedStatus = myApp.findStatus("Served");
        public TableStatus servedPaidStatus = myApp.findStatus("Served and paid");

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTotal = itemView.findViewById(R.id.tvTotal);
            llTable = itemView.findViewById(R.id.llTable);
            tvTableStatus = itemView.findViewById(R.id.tvTableStatus);
            cvTable = itemView.findViewById(R.id.cvTables);
            tvTable = itemView.findViewById(R.id.tvTable);
            tvAction = itemView.findViewById(R.id.tvAction);
            tvTimeDiff = itemView.findViewById(R.id.tvTimer);
            cvAction = itemView.findViewById(R.id.cvAction);
        }

        public void setData(Table data) {
            table = data;
            tvTable.setText(table.number);
            cvAction.setVisibility(View.INVISIBLE);
            tvTotal.setVisibility(View.INVISIBLE);
            tvTimeDiff.setVisibility(View.VISIBLE);


            if (table.table_status != null) {
                table.table_status_id = table.table_status.id;
                table.status = table.table_status.status;
                tvTableStatus.setText(table.table_status.status);

                GradientDrawable gd = new GradientDrawable(GradientDrawable.Orientation.TOP_BOTTOM,
                        new int[]{Color.parseColor(table.table_status.top_color),
                                Color.parseColor(table.table_status.bottom_color)});

                gd.setCornerRadius(4f);
                llTable.setBackground(gd);
                if (table.status.toLowerCase().equalsIgnoreCase("vacant")) {
                    tvTimeDiff.setVisibility(View.INVISIBLE);
                } else {
                    tvTimeDiff.setVisibility(View.VISIBLE);
                }
            } else {
                tvTimeDiff.setVisibility(View.INVISIBLE);
            }
            updateTimeRemaining(System.currentTimeMillis());
        }

        public void updateTimeRemaining(long currentTime) {
            if (table._last_order_id > 0 && !Validators.isNullOrEmpty(table.last_order_created_at)) {
                tvTimeDiff.setText("Ongoing - " + calculatedTime(CommonFunctions.convertToDate(table.last_order_created_at, Constants.PHP_DATE_TIME_FORMAT_ZULU), new Date()));
            } else {
                tvTimeDiff.setText("");
            }
        }
    }

    private static String calculatedTime(Date created, Date current) {

        if (created == null) return "";
        long diff = current.getTime() - created.getTime();

        long diffSeconds = diff / 1000 % 60;
        long diffMinutes = diff / (60 * 1000) % 60;
        long diffHours = diff / (60 * 60 * 1000);
        int diffInDays = (int) diff / (1000 * 60 * 60 * 24);


        String formate = String.valueOf((diffHours > 9 ? diffHours : "0" + diffHours) + ":" + (diffMinutes > 9 ? diffMinutes : "0" + diffMinutes) + ":" + (diffSeconds > 9 ? diffSeconds : "0" + diffSeconds));
        return formate;
    }

    public static class HomeViewHolder extends RecyclerView.ViewHolder {

        private MaterialCardView cvTable;
        private TextView tvTable, tvTimeDiff, tvTableStatus;
        private RelativeLayout rlAction;
        private ImageView ivAction;
        private LinearLayout llTable, llTimeDiff;
        Table table;

        public HomeViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTableStatus = itemView.findViewById(R.id.tvTableStatus);
            llTable = itemView.findViewById(R.id.llTable);
            cvTable = itemView.findViewById(R.id.cvTables);
            tvTable = itemView.findViewById(R.id.tvTable);
            tvTimeDiff = itemView.findViewById(R.id.tvTimeDiff);
            llTimeDiff = itemView.findViewById(R.id.llTimeDiff);
            rlAction = itemView.findViewById(R.id.rlAction);
            ivAction = itemView.findViewById(R.id.ivAction);
        }

        public void setData(Table data) {
            table = data;
            tvTable.setText(table.number);

            llTimeDiff.setVisibility(View.VISIBLE);
            tvTableStatus.setVisibility(View.VISIBLE);


            if (table.table_status != null) {
                tvTableStatus.setText(table.table_status.status);
                table.status = table.table_status.status;
                table.table_status_id = table.table_status.id;

                GradientDrawable gd = new GradientDrawable(
                        GradientDrawable.Orientation.TOP_BOTTOM,
                        new int[]{Color.parseColor(table.table_status.top_color), Color.parseColor(table.table_status.bottom_color)});
                gd.setCornerRadius(4f);
                llTable.setBackground(gd);
                if (table.status.toLowerCase().equalsIgnoreCase("vacant")) {
                    llTimeDiff.setVisibility(View.INVISIBLE);
                } else {
                    llTimeDiff.setVisibility(View.VISIBLE);
                }
            } else {
                llTimeDiff.setVisibility(View.INVISIBLE);
            }
            updateTimeRemaining(System.currentTimeMillis());
        }

        public void updateTimeRemaining(long currentTime) {
            if (table.status != null && !table.status.equalsIgnoreCase("vacant") && table._last_order_id > 0 && !Validators.isNullOrEmpty(table.last_order_created_at)) {
                llTimeDiff.setVisibility(View.VISIBLE);
                tvTimeDiff.setText(calculatedTime(CommonFunctions.convertToDate(table.last_order_created_at, Constants.PHP_DATE_TIME_FORMAT_ZULU), new Date()));
            } else {
                tvTimeDiff.setText("");
                llTimeDiff.setVisibility(View.INVISIBLE);
            }
        }

    }

    @Override
    public void onDetachedFromRecyclerView(@NonNull RecyclerView recyclerView) {
        super.onDetachedFromRecyclerView(recyclerView);
        handler.removeCallbacks(updateTime);
    }

}
