/**
 * UsbDataBinder.Java
 * CID Eays Bridge 2.1 Sample
 * Kybele Softwares
 * http://www.cideasy.com
 * Release : 2020.10.10
 **/

package com.example.epos_2021.callerid;

import android.content.Context;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.util.Log;

public class UsbDataBinder {

    private static class AsyncReader extends AsyncTask<String, String, String> {

        public String doInBackground(String... params) {
            try {
                int r = mConnection.bulkTransfer(rEndpoint, bytes, bytes.length, TIMEOUT);
                if (r >= 0) {
                    int i = 0;
                    for (byte b : bytes) {
                        trancatedBytes[i] = bytes[i];
                        i++;
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            return null;
        }

        public void onPostExecute(String result) {
            analyzePackage(trancatedBytes);
            _standby = false;
            _sleept = 50;
        }
    }

    private static class AsyncWriter extends AsyncTask<String, String, String> {

        protected String doInBackground(String... Answer) {
            byte[] bytes = Answer[0].getBytes();
            if (mConnection.claimInterface(mIntf, forceClaim)) {
                mConnection.bulkTransfer(wEndpoint, bytes, bytes.length, TIMEOUT); //do in another thread
            }
            return null;
        }

        protected void onPostExecute() {
        }
    }

    private static final boolean disposeCorrupted = false;
    private static final boolean cdCorrectionEnable = false;

    private static final String ackAnswer = "ACK\r\n";
    private static final String dckAnswer = "DCK\r\n";
    private static final String nakAnswer = "NAK\r\n";

    private static byte[] bytes = new byte[64];
    private static byte[] trancatedBytes = new byte[64];
    private static int TIMEOUT = 0;
    private static boolean forceClaim = true;
    private static Boolean _standby = false;
    private static int _sleept = 1000;

    private UsbDevice mDevice;
    private UsbManager mUsbManager;
    private static UsbDeviceConnection mConnection;
    private static UsbInterface mIntf;
    private static UsbEndpoint rEndpoint;
    private static UsbEndpoint wEndpoint;

    private Thread _readingThread = null;
    private boolean runReadThread = false;

    private Context ctx;

    private static char sPort = 0;
    private static String sDateTime = "";
    private static String sNumber = "";
    private static String sOther = "";

    private static char thePort = 0;
    private static String theDateTime = "";
    private static String theCallerId = "";


    public interface binderListener {
        public void onBinderLineEvent(char thePort, String theEvent);

        public void onBinderCliReceived(char thePort, String theDateTime, String theCallerId);
    }

    private static binderListener listener;

    public void setBinderListener(binderListener listener) {
        this.listener = listener;
    }

    public UsbDataBinder(UsbManager manager, UsbDevice device) {
        // TODO Auto-generated constructor stub
        mUsbManager = manager;
        mDevice = device;
        mIntf = mDevice.getInterface(0);
        rEndpoint = mIntf.getEndpoint(0);
        wEndpoint = mIntf.getEndpoint(1);
        mConnection = mUsbManager.openDevice(mDevice);
    }

    public void onDestroy() {
        if (mConnection != null) {
            runReadThread = false;
            mConnection.releaseInterface(mIntf);
            mConnection.close();
            mConnection = null;
        }
    }

    public void StartCidEasyThread(Context _ctx) {
        ctx = _ctx;
        this.listener = null;
        _standby = false;

        if (_readingThread == null) {
            _readingThread = new Thread(readerReceiver);
            runReadThread = true;
            _readingThread.start();
        }
    }

    private Runnable readerReceiver = new Runnable() {
        public void run() {
            while (runReadThread) {
                try {
                    if (!_standby) {
                        _standby = true;
                        if (mConnection.claimInterface(mIntf, forceClaim))
                            new AsyncReader().execute(new String[]{""});
                    }
                } catch (Exception e) {
                    Log.d("CEBridge - readReceiver", Log.getStackTraceString(e));
                }
            }

            Sleep(_sleept);
        }
    };

    private static void sendData(final String sendString) {
        try {
            new AsyncWriter().execute(new String[]{sendString});
        } catch (Exception e) {
            Log.d("CEBridge - sendData", Log.getStackTraceString(e));
        }
    }

    private static void analyzePackage(byte[] bytes) {
        try {
            final String strPackage = composeString(bytes);

            if (strPackage.contains("ENQ")) {
                sendData(ackAnswer);
            } else {
                if (strPackage.contains("ETB")) {
                    sendData(ackAnswer);
                } else {
                    if (strPackage.contains("STA")) {
                        echoLineEvent(strPackage);
                    } else {
                        if (testCliPackage(bytes)) {
                            sendData(dckAnswer);
                        } else {
                            sendData(nakAnswer);
                        }
                    }
                }
            }
        } catch (Exception e) {
            Log.d("CEBridge - analyzePackage", Log.getStackTraceString(e));
        }
    }

    private static void echoLineEvent(String strPackage) {
        try {
            char thePort = strPackage.substring(4, 5).charAt(0);
            String theEvent = strPackage.substring(6, strPackage.length() - 2);

            if (listener != null) listener.onBinderLineEvent(thePort, theEvent);
        } catch (Exception e) {
            Log.d("CEBridge - echoLineEvent", Log.getStackTraceString(e));
        }
    }

    private static boolean testCliPackage(byte[] Package) {
        try {
            char pPort = (char) Package[0];
            int pType = Math.abs((int) Package[1]);
            int pLen = Math.abs((int) Package[2]);

            int crrCharNdx = 0;
            byte loopAgain = 2;

            if ((pLen < 65) && (pPort >= 'A') && (pPort <= 'D')) {
                do {
                    loopAgain--;
                    if (pType == 0x80) {
                        crrCharNdx = parseMDMF(Package);
                    }
                    if (pType == 0x04) {
                        crrCharNdx = parseSDMF(Package);
                    }
                    if (cdCorrectionEnable) tryFixPackage(crrCharNdx, Package);
                } while ((crrCharNdx != 0) && (loopAgain > 0));

                if (testCheckDigit(Package) || !disposeCorrupted)
                    cliReceived((char) sPort, sDateTime, sNumber);
            }
        } catch (Exception e) {
            Log.d("CEBridge - testCliPackage", Log.getStackTraceString(e));
        }

        return true;
    }

    private static void tryFixPackage(int crrCharNdx, byte[] inputReport) {
        byte theChar = 48;

        try {
            if ((crrCharNdx > 0) && (crrCharNdx < 64)) {
                while (theChar < 58) {
                    inputReport[crrCharNdx] = theChar;
                    if (testCheckDigit(inputReport)) theChar = 99;
                    theChar++;
                }
            }
        } catch (Exception e) {
            Log.d("CEBridge - tryFixPackage", Log.getStackTraceString(e));
        }
    }


    private static boolean testCheckDigit(byte[] inputReport) {
        byte pDigit = 0;
        byte sayaca = 0;
        int pckLen = 0;

        try {
            pckLen = inputReport[2] + 4;
            for (sayaca = 1; sayaca < pckLen; sayaca++) {
                pDigit = (byte) (pDigit + inputReport[sayaca]);
            }

            pDigit = (byte) (0x100 - pDigit);
        } catch (Exception e) {
            Log.d("CEBridge - testCheckDigit", Log.getStackTraceString(e));
        }

        return pDigit == inputReport[pckLen];
    }

    private static int parseMDMF(byte[] Package) {
        int packlength = 0;
        int datalength = 0;
        int Sayaca;

        int invalidChar = 0;
        int icCount = 0;
        char theChar = 0;

        try {
            packlength = Package[2] + 4;
            sPort = (char) Package[0];
            sNumber = "";
            sDateTime = "";
            sOther = "";

            if (packlength > Package.length) {
                packlength = Package.length;
            }

            if (packlength > 0) {
                Sayaca = 3;
                datalength = 0;
                while (Sayaca < Package.length) {
                    if (Package[Sayaca] == 1) // Date Field
                    {
                        Sayaca++;
                        if (Sayaca < packlength - 1) {
                            datalength = Package[Sayaca];
                            if (datalength != 8) datalength = 8;
                            if (datalength == 8) {
                                while ((datalength > 0) && (datalength < packlength)) {
                                    Sayaca++;
                                    theChar = (char) Package[Sayaca];
                                    if ((theChar > '9') || (theChar < '0')) {
                                        invalidChar = (icCount * 10000) + Sayaca;
                                        icCount++;
                                    }
                                    sDateTime = sDateTime + theChar;
                                    datalength--;
                                }
                            }
                        }
                    }

                    if (Package[Sayaca] == 2)  // Number Field
                    {
                        Sayaca++;
                        if (Sayaca < packlength - 1) {
                            datalength = Package[Sayaca];
                            if (datalength > (packlength - Sayaca - 1))
                                datalength = (packlength - Sayaca - 1);
                            while ((datalength > 0) && (datalength < packlength)) {
                                Sayaca++;
                                theChar = (char) Package[Sayaca];
                                if ((theChar > '9') || (theChar < '0')) {
                                    invalidChar = (icCount * 10000) + Sayaca;
                                    icCount++;
                                }
                                sNumber = sNumber + theChar;
                                datalength--;
                            }
                        }
                    }

                    if (Package[Sayaca] == 3) {
                        // Other Fields
                        Sayaca++;
                        if (Sayaca < packlength - 1) {
                            datalength = Package[Sayaca];
                            while ((datalength > 0) && (datalength < packlength)) {
                                Sayaca++;
                                theChar = (char) Package[Sayaca];
                                if ((theChar > '9') || (theChar < '0')) {
                                    invalidChar = (icCount * 10000) + Sayaca;
                                    icCount++;
                                }
                                sOther = sOther + theChar;
                                datalength--;
                            }
                        }
                    }
                    Sayaca++;
                }
            }
        } catch (Exception e) {
            Log.d("CEBridge - parseMDMF", Log.getStackTraceString(e));
        }

        return invalidChar;
    }

    private static int parseSDMF(byte[] Package) {
        int packlength = 0;
        int Sayaca;

        int invalidChar = 0;
        int icCount = 0;
        char theChar = 0;

        try {
            packlength = Package[2] + 4;
            sPort = (char) Package[0];
            sNumber = "";
            sDateTime = "";
            sOther = "";

            for (Sayaca = 3; Sayaca <= packlength - 2; Sayaca++) {
                if (Sayaca < Package.length) {
                    theChar = (char) Package[Sayaca];
                    if ((theChar > '9') || (theChar < '0')) {
                        invalidChar = (icCount * 10000) + Sayaca;
                        icCount++;
                    }
                    if (Sayaca < 11) {
                        sDateTime = sDateTime + (char) Package[Sayaca];
                    } else {
                        sNumber = sNumber + theChar;
                    }
                }
            }
        } catch (Exception e) {
            Log.d("CEBridge - parsesDMF", Log.getStackTraceString(e));
        }

        return invalidChar;
    }

    private static void cliReceived(char Port, String DateTime, String CallerId) {
        try {
            if (listener != null) listener.onBinderCliReceived(Port, DateTime, CallerId);
        } catch (Exception e) {
            Log.d("CEBridge - cliReceived", Log.getStackTraceString(e));
        }
    }


    private void Sleep(int milliseconds) {
        try {
            Thread.sleep(milliseconds);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    private static String composeString(byte[] bytes) {
        String strPackage = "";

        try {
            StringBuilder builder = new StringBuilder();
            for (byte b : bytes) {
                if (b > 0) {
                    char c = (char) b;
                    builder.append(c);
                }
            }
            strPackage = builder.toString();
        } catch (Exception e) {
            Log.d("CEBridge - composeString", Log.getStackTraceString(e));
        }

        return strPackage;
    }


}
