package com.example.epos_2021.comman.printer;

import static com.example.epos_2021.comman.printer.AidlUtil.Alignment.CENTER;
import static com.example.epos_2021.comman.printer.AidlUtil.Alignment.LEFT;
import static com.example.epos_2021.comman.printer.AidlUtil.Alignment.RIGHT;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.os.IBinder;
import android.text.Html;
import android.util.Log;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.OrderItemAddon;
import com.example.epos_2021.models.OrderItemIngredient;
import com.example.epos_2021.models.OrderPayment;
import com.example.epos_2021.models.ReportTextModel;
import com.example.epos_2021.models.Reservation;
import com.example.epos_2021.models.SalesReport;
import com.example.epos_2021.models.Voucher;
import com.example.epos_2021.online.models.OnlineReservation;
import com.example.epos_2021.online.models.OrderDetail;
import com.example.epos_2021.online.models.OrderProductDetail;
import com.example.epos_2021.utils.Constants;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;

import woyou.aidlservice.jiuiv5.IWoyouService;

/**
 * Created by Amrish on 04-12-2020.
 */
public class SunmiPrinter {
    private Context context;
    public IWoyouService woyouService;
    private MyApp myApp = MyApp.getInstance();

    ServiceConnection connService = new ServiceConnection() {

        @Override
        public void onServiceDisconnected(ComponentName name) {
            woyouService = null;
        }

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            woyouService = IWoyouService.Stub.asInterface(service);
        }
    };

    public SunmiPrinter(Context context) {
        this.context = context;
        startService(context);

    }

    private void startService(Context context) {
        Intent intent = new Intent();
        intent.setPackage("woyou.aidlservice.jiuiv5");
        intent.setAction("woyou.aidlservice.jiuiv5.IWoyouService");
        context.startService(intent);//Start printer service
        context.bindService(intent, connService, Context.BIND_AUTO_CREATE);
    }

    public void openCashDrawer() {
        AidlUtil.getInstance().openCashDrawer();
    }

    public void printCardReaderReceipt(Bitmap bm, String title, String header, int headerAlignment, String actual_amount, String tip_amount, String grand_total, String main_content, String please, String final_date, String footer, String footer1) {
        if (bm != null)
            AidlUtil.getInstance().printBitmap(bm);
        AidlUtil.getInstance().printText(header, 32, true, false, headerAlignment);
        AidlUtil.getInstance().printText(title, 40, true, false, CENTER);
        AidlUtil.getInstance().printText("--------------------------------", 24, true, false, CENTER);
        if (!Validators.isNullOrEmpty(tip_amount)) {
            AidlUtil.getInstance().printText(actual_amount, 36, true, false, CENTER);
            AidlUtil.getInstance().printText(tip_amount, 36, true, false, CENTER);
        }
        AidlUtil.getInstance().printText(grand_total, 52, true, false, CENTER);
        AidlUtil.getInstance().printText(main_content, 26, true, false, AidlUtil.Alignment.LEFT);
        AidlUtil.getInstance().printText(please, 26, true, false, CENTER);
        AidlUtil.getInstance().printText(final_date, 26, true, false, AidlUtil.Alignment.LEFT);
        AidlUtil.getInstance().printText("--------------------------------", 24, true, false, CENTER);
        AidlUtil.getInstance().printText(footer, 32, true, false, CENTER);
        AidlUtil.getInstance().printText(footer1, 32, true, false, CENTER);
        AidlUtil.getInstance().CutPaper();

    }

    public void printMotoReceipt(Bitmap bm, String title, String header, int headerAlignment, String amount, String main_content, String date_final, String please, String footer, String footer1) {
        try {
            if (bm != null) {
                AidlUtil.getInstance().printBitmap(bm);
            }
            AidlUtil.getInstance().printText(header, 32, true, false, headerAlignment);
            AidlUtil.getInstance().printText(title, 40, true, false, CENTER);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);

            amount = amount.replace("£", "");
            amount = amount.replace("\n", "");
            amount = amount + " GBP";
            AidlUtil.getInstance().printText(amount, 52, true, false, CENTER);
            AidlUtil.getInstance().printText(main_content.replace(" :", ": "), 22, true, false, AidlUtil.Alignment.LEFT);

            AidlUtil.getInstance().printText(date_final, 26, true, false, CENTER);
            AidlUtil.getInstance().printText(please, 26, true, false, CENTER);
            AidlUtil.getInstance().printText("--------------------------------", 24, true, false, CENTER);
            AidlUtil.getInstance().printText(footer, 32, true, false, CENTER);
            AidlUtil.getInstance().printText(footer1, 32, true, false, CENTER);
            AidlUtil.getInstance().CutPaper();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public void printOrder(Bitmap tiffintomBitmap, OrderDetail orderDetail, boolean kitchenCopy) {
        int orderType = -1;
        if (Validators.isNullOrEmpty(orderDetail.order_type)) {
            orderType = 0;
        } else if (orderDetail.order_type.equalsIgnoreCase("delivery")) {
            orderType = 1;
        } else if (orderDetail.order_type.equalsIgnoreCase("pickup")) {
            orderType = 2;
        }

        try {
            woyouService.sendRAWData(ESCUtil.boldOn(), null);
            if (tiffintomBitmap != null) {
                woyouService.setAlignment(1, null);
                woyouService.printBitmap(tiffintomBitmap, null);
                woyouService.printText("\n", null);
            }

            String todayDate = CommonFunctions.getCurrentTimeFormatted("dd-MM-yyyy");
            String orderDateString = CommonFunctions.formatUnknownDateTime(orderDetail.created, Constants.PHP_DATE_TIME_FORMAT_ZULU, "dd-MM-yyyy");
            if (!todayDate.equalsIgnoreCase(orderDateString)) {
                woyouService.setFontSize(40, null);
                woyouService.setAlignment(1, null);//Right
                woyouService.printText("This Order Is Not For Today\n\n", null);
            }

            //woyouService.sendRAWData(ESCUtil.boldOn(), null);
            woyouService.setAlignment(1, null);//Right
            woyouService.setFontSize(26, null);
            woyouService.printText("\n\n", null);

            //woyouService.sendRAWData(ESCUtil.boldOn(), null);
            woyouService.printText("Tiffintom.com sent you order\n\n", null);

            woyouService.setAlignment(1, null);//Right
            woyouService.setFontSize(36, null);
            woyouService.printText(CommonFunctions.getOnlineOrderType(orderType) + "\n", null);

            woyouService.setAlignment(1, null);//Right
            woyouService.setFontSize(30, null);
            woyouService.printText("" + Html.fromHtml("------------------------------------<br>"), null);

            woyouService.setFontSize(36, null);
            woyouService.printText("ORDER NO: " + orderDetail.order_number + "\n", null);

            if (orderType > 0) {
                if (orderDetail.assoonas.equalsIgnoreCase("now")) {
                    woyouService.printText("ASAP" + "\n", null);
                } else {
                    woyouService.printText(orderDetail.delivery_time + " " + CommonFunctions.formatUnknownDateTime(orderDetail.delivery_date, "yyyy-MM-dd", "dd-MM-yyyy") + "\n", null);
                }
            }
            woyouService.setFontSize(30, null);
            woyouService.printText("" + Html.fromHtml("------------------------------------<br>"), null);

            if (kitchenCopy)
                woyouService.setFontSize(40, null);
            else woyouService.setFontSize(30, null);
            woyouService.setAlignment(LEFT, null);
            String itemName = "", price = "", qty = "";
            for (OrderProductDetail productDetail : orderDetail.cart_view) {
                String productName = (kitchenCopy && !Validators.isNullOrEmpty(productDetail.short_name)) ? productDetail.short_name : productDetail.menu_name;
                String productQty = String.valueOf(productDetail.quantity);
                String totalPrice = String.valueOf(productDetail.total_price);
                // Here we need 16 characters in product name so we send 16 in
                // @params charactersInLine
                price = (!kitchenCopy) ? MyApp.df.format(productDetail.total_price) : "";
                itemName = getProductNameWithPad(productName, price, kitchenCopy ? 20 : 28);
                qty = productQty;
                if (productQty.length() <= 1) {
                    qty = " " + productQty + " ";
                } else if (productQty.length() <= 2) {
                    qty = productQty + " ";

                } else if (productQty.length() <= 3) {
                    qty = productQty + " ";

                } else if (productQty.length() <= 4) {
                    qty = productQty + "";
                }
                String printItem;
                printItem = qty + itemName + "\n";
                int fontSize = 30;
                if (kitchenCopy) fontSize = 36;
                woyouService.setAlignment(LEFT, null);
                woyouService.setFontSize(fontSize, null);
                woyouService.printText(printItem, null);

                if (!Validators.isNullOrEmpty(productDetail.subaddons_name)) {
                    try {
                        //String[] parts = menus.get(position).getSubaddonsName().split(",");
                        String[] parts = productDetail.subaddons_name.split(",");

                        for (String subAddonPart : parts) {
                            int fontSizeSub = 30;
                            int charactersInLine = 20;
                            if (kitchenCopy) {
                                fontSizeSub = 36;
                                charactersInLine = 16;
                            }
                            String printedItem = getSubaddonWithPadOnline(subAddonPart, charactersInLine);
                            woyouService.setAlignment(LEFT, null);
                            woyouService.setFontSize(fontSizeSub, null);
                            woyouService.printText(printedItem, null);

                        }
                    } catch (Exception e) {
                        Log.e("Addon_Error", e.getMessage());
                    }
                }
            }
            woyouService.setAlignment(CENTER, null);
            woyouService.setFontSize(30, null);
            woyouService.printText("" + Html.fromHtml("------------------------------------<br>"), null);

            woyouService.setAlignment(LEFT, null);
            woyouService.setFontSize(32, null);
            woyouService.printText(padRightSpaces("SubTotal:", 25) + MyApp.currencySymbol + MyApp.df.format(orderDetail.order_sub_total) + "\n", null);

            if (orderDetail.delivery_charge > 0) {
                woyouService.printText(padRightSpaces("Delivery Charge:", 25) + MyApp.currencySymbol + MyApp.df.format(orderDetail.delivery_charge) + "\n", null);
            }
            if (orderDetail.offer_amount > 0) {
                woyouService.printText(padRightSpaces("Offer:", 25) + MyApp.currencySymbol + MyApp.df.format(orderDetail.offer_amount) + "\n", null);
            }
            if (orderDetail.service_charge > 0) {
                if (!"no".equalsIgnoreCase(orderDetail.service_charge_status)) {
                    woyouService.printText(padRightSpaces("Service Charge:", 25) + MyApp.currencySymbol + MyApp.df.format(orderDetail.service_charge) + "\n", null);
                }
            }
            if (orderDetail.driver_tip > 0) {
                woyouService.printText(padRightSpaces("Driver Tip:", 25) + MyApp.currencySymbol + MyApp.df.format(orderDetail.driver_tip) + "\n", null);
            }
            if (orderDetail.reward_offer > 0) {
                woyouService.printText(padRightSpaces("Loyalty Point Discount:", 25) + MyApp.currencySymbol + MyApp.df.format(orderDetail.reward_offer) + "\n", null);
            }
            woyouService.setFontSize(36, null);
            woyouService.printText(padRightSpaces("Grand Total:  ", 25) + MyApp.currencySymbol + MyApp.df.format(orderDetail.order_grand_total) + "\n", null);

            woyouService.setFontSize(30, null);
            woyouService.setAlignment(1, null);
            woyouService.printText("" + Html.fromHtml("------------------------------------<br>"), null);
            woyouService.setAlignment(0, null);
            if (!Validators.isNullOrEmpty(orderDetail.order_description)) {

                woyouService.sendRAWData(ESCUtil.boldOn(), null);
                woyouService.setFontSize(kitchenCopy ? 40 : 32, null);
                woyouService.printText("Comments: " + orderDetail.order_description + "\n", null);
                woyouService.sendRAWData(ESCUtil.boldOff(), null);

                woyouService.setAlignment(1, null);
                woyouService.setFontSize(26, null);
                woyouService.printText("" + Html.fromHtml("------------------------------------<br>"), null);

            }
            if (!Validators.isNullOrEmpty(orderDetail.delivery_instruction)) {

                woyouService.sendRAWData(ESCUtil.boldOn(), null);
                woyouService.setFontSize(32, null);
                woyouService.printText("Delivery Instructions: " + orderDetail.delivery_instruction + "\n", null);
                woyouService.sendRAWData(ESCUtil.boldOff(), null);
                woyouService.setAlignment(1, null);
                woyouService.setFontSize(26, null);
                woyouService.printText("" + Html.fromHtml("--------------------------------------<br>"), null);

            }
            woyouService.setAlignment(0, null);
            woyouService.setFontSize(32, null);
            woyouService.sendRAWData(ESCUtil.boldOn(), null);
            woyouService.printText("Cust Name: " + orderDetail.customer_name + "\n", null);

            if (!Validators.isNullOrEmpty(orderDetail.customer_phone)) {
                woyouService.printText("Cust No: " + orderDetail.customer_phone + "\n", null);
            }
            woyouService.sendRAWData(ESCUtil.boldOff(), null);
            woyouService.setAlignment(1, null);
            woyouService.setFontSize(30, null);
            woyouService.printText("" + Html.fromHtml("--------------------------------------<br>"), null);

            if (orderType == 1) {

                String address = orderDetail.address + "\n";
                String[] parts = address.split("[\\\\s,]+");
                StringBuilder str = new StringBuilder();

                for (String name : parts) {
                    str.append(name).append("\n");
                }
                woyouService.setFontSize(32, null);
                woyouService.sendRAWData(ESCUtil.boldOn(), null);
                woyouService.setAlignment(LEFT, null);
                woyouService.printText("Del. address: " + address, null);
                woyouService.sendRAWData(ESCUtil.boldOff(), null);

                woyouService.setFontSize(26, null);
                woyouService.setAlignment(1, null);
                woyouService.printText("" + Html.fromHtml("--------------------------------------<br>"), null);
            }

            if (orderType >= 0) {
                woyouService.setAlignment(0, null);
                if (orderDetail.status.equalsIgnoreCase("accepted") && !Validators.isNullOrEmpty(orderDetail.preparation)) {
                    woyouService.setFontSize(36, null);
                    woyouService.setAlignment(LEFT, null);
                    woyouService.printText("Confirmed for: ", null);
                    woyouService.printText("" + orderDetail.preparation + "\n", null);
                    woyouService.setFontSize(26, null);
                    woyouService.setAlignment(1, null);
                    woyouService.printText("" + Html.fromHtml("--------------------------------------<br>"), null);
                }
            }
            woyouService.setAlignment(0, null);
            woyouService.setFontSize(36, null);
            woyouService.sendRAWData(ESCUtil.boldOn(), null);
            woyouService.printText("Order " + (orderDetail.payment_status.equalsIgnoreCase("np") ? "Unpaid" : "Paid") + "\n", null);
            woyouService.sendRAWData(ESCUtil.boldOff(), null);


            woyouService.setFontSize(26, null);
            String paymentName = "";
            if (orderDetail.payment_method.equalsIgnoreCase("stripe")) paymentName = "ONLINE";
            if (orderDetail.payment_method.equalsIgnoreCase("cod")) paymentName = "COD";
            if (orderDetail.payment_method.equalsIgnoreCase("wallet") || orderDetail.payment_method.equalsIgnoreCase("credit"))
                paymentName = "WALLET";
            if (orderDetail.payment_method.equalsIgnoreCase("paypal")) paymentName = "PAYPAL";
            woyouService.printText(" Payment Type : " + paymentName + "\n", null);

            woyouService.setFontSize(30, null);
            woyouService.setAlignment(1, null);
            woyouService.printText("" + Html.fromHtml("--------------------------------------<br>"), null);

            woyouService.sendRAWData(ESCUtil.boldOn(), null);
            woyouService.setAlignment(0, null);
            woyouService.setFontSize(32, null);
            woyouService.printText("Receipt Time:\n", null);
            woyouService.printText(CommonFunctions.getCurrentTimeFormatted("dd/MM/yyyy hh:mm a") + "\n", null);
            woyouService.sendRAWData(ESCUtil.boldOff(), null);

            woyouService.printText("Order status: " + orderDetail.status + "\n", null);
            if (orderDetail.status.equalsIgnoreCase("failed")) {
                woyouService.printText("Reason: " + orderDetail.failed_reason + "\n", null);
            }

            woyouService.sendRAWData(ESCUtil.boldOn(), null);
            woyouService.setFontSize(28, null);

            woyouService.printText(MyApp.getInstance().myPreferences.getLoggedInAdmin().selected_business.name + ", " + MyApp.getInstance().myPreferences.getLoggedInAdmin().selected_business.address + "\n", null);
            woyouService.printText("Thanks!\n", null);

            woyouService.lineWrap(3, null);
            woyouService.sendRAWData(ESCUtil.cutPaper(), null);

        } catch (Exception e) {
            System.out.println(e);
        }

    }

    public void printOrderEpos(Bitmap tiffintomLogo, String title, String ticket_header, int headerAlignment, String tableNumberOrderIdMode, String dateTimeMode, Order orderDetail, boolean printTotalInPrint, String footerA, String footerB, boolean kitchenCopy, boolean isCashDrawerOpen) {
        int orderType = -1;
        if (Validators.isNullOrEmpty(orderDetail.order_type_id)) {
            orderType = 0;
        } else {
            orderType = Integer.parseInt(orderDetail.order_type_id);
        }

        if (!kitchenCopy && tiffintomLogo != null) {
            AidlUtil.getInstance().printBitmap(tiffintomLogo);
        }
        if (!kitchenCopy) {
            AidlUtil.getInstance().printText(ticket_header, 32, true, false, headerAlignment);
        }
        if (kitchenCopy) {
            AidlUtil.getInstance().printText("** " + orderDetail.order_type + " **", 40, true, false, CENTER);

            if (orderDetail.order_type_id.equalsIgnoreCase("2") && !Validators.isNullOrEmpty(orderDetail.customer_name)) {
                AidlUtil.getInstance().printText(orderDetail.customer_name, 40, false, false, CENTER);
            }
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
        }

        if (!Validators.isNullOrEmpty(title)) {
            AidlUtil.getInstance().printText(title, 40, false, false, CENTER);
        }
        String tableNumberOrderIdText = "";
        if (tableNumberOrderIdMode != null) {
            if (tableNumberOrderIdMode.equalsIgnoreCase("order_number") || tableNumberOrderIdMode.equalsIgnoreCase("both")) {
                if (Validators.isNullOrEmpty(orderDetail.id)) {
                    tableNumberOrderIdText = "Order ID: " + orderDetail._id;
                } else
                    tableNumberOrderIdText = "Order ID: " + orderDetail.id;

            }
            if (orderType == 1 && (tableNumberOrderIdMode.equalsIgnoreCase("table_number") || tableNumberOrderIdMode.equalsIgnoreCase("both"))) {
                if (tableNumberOrderIdText.length() > 1) {
                    tableNumberOrderIdText = tableNumberOrderIdText + "\n";
                }
                tableNumberOrderIdText = tableNumberOrderIdText + "Table No: " + orderDetail.table_number + "  No of Diners:  " + orderDetail.no_guest;

            }
            if (tableNumberOrderIdMode.equalsIgnoreCase("none")) {
                tableNumberOrderIdText = "";
            }
            if (tableNumberOrderIdText.length() > 0) {
                if (!Validators.isNullOrEmpty(title))
                    AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
                AidlUtil.getInstance().printText(tableNumberOrderIdText, 32, false, false, LEFT);
                AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
            }
        }

        if (!Validators.isNullOrEmpty(dateTimeMode)) {
            String dateFormat = "dd-MM-yyyy hh:mm a";
            String dateFormatTitle = "Order Date Time: ";
            if (dateTimeMode.equalsIgnoreCase("date_time")) {
                dateFormat = "dd-MM-yyyy hh:mm a";
                dateFormatTitle = "Order Date Time: ";
            }
            if (dateTimeMode.equalsIgnoreCase("time")) {
                dateFormat = "hh:mm a";
                dateFormatTitle = "Order Time: ";
            }
            if (dateTimeMode.equalsIgnoreCase("date")) {
                dateFormat = "dd-MM-yyyy";
                dateFormatTitle = "Order Date: ";
            }
            if (orderDetail.created_at.contains("Z")) {
                AidlUtil.getInstance().printText(dateFormatTitle + CommonFunctions.formatUnknownDateTime(orderDetail.created_at, Constants.PHP_DATE_TIME_FORMAT_ZULU, dateFormat), 32, false, false, LEFT);
            } else {
                AidlUtil.getInstance().printText(dateFormatTitle + CommonFunctions.formatUnknownDateTime(orderDetail.created_at, Constants.PHP_DATE_TIME_FORMAT, dateFormat), 32, false, false, LEFT);
            }

        }
        if (orderType > 1 && !kitchenCopy) {
            AidlUtil.getInstance().printText("Order Type: " + orderDetail.order_type, 32, false, false, LEFT);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
        }

        AidlUtil.getInstance().printText("Qty Name", 32, false, false, LEFT);
        String itemName = "", price = "", qty = "";

        String lastBlockId = null;
        boolean lastItemWasMisc = false;

        for (OrderItem productDetail : orderDetail.order_items) {
            String productName = (kitchenCopy && !Validators.isNullOrEmpty(productDetail.product_short_name)) ? productDetail.product_short_name : productDetail.product_name;
            String productQty = String.valueOf(productDetail.quantity);
            // Here we need 16 characters in product name so we send 16 in
            // @params charactersInLine
            price = (!kitchenCopy) ? MyApp.df.format(productDetail.total) : "";
            if (kitchenCopy)
                itemName = getProductNameWithPad(productName, price, 25);
            else
                itemName = getProductNameWithPad(productName, price, 28);
            qty = productQty;
            if (productQty.length() <= 1) {
                qty = " " + productQty + " ";
            } else if (productQty.length() <= 2) {
                qty = productQty + " ";

            } else if (productQty.length() <= 3) {
                qty = productQty + " ";

            } else if (productQty.length() <= 4) {
                qty = productQty + "";
            }
            if (lastBlockId != null) {
                if (!lastBlockId.equalsIgnoreCase(productDetail.block_id)) {
                    if (kitchenCopy)
                        AidlUtil.getInstance().printText("-----------------------------", 30, false, false, CENTER);
                }
            } else {
                if (lastItemWasMisc && !productDetail.misc) {
                    if (kitchenCopy)
                        AidlUtil.getInstance().printText("-----------------------------", 30, false, false, CENTER);
                }
            }

            String printItem;
            printItem = qty + itemName.trim();
            if (!kitchenCopy) {
                AidlUtil.getInstance().printText(printItem, 30, true, false, LEFT);
                if (!Validators.isNullOrEmpty(productDetail.special_instruction))
                    AidlUtil.getInstance().printText("   Instruction: " + productDetail.special_instruction, 30, true, false, LEFT);
            } else {
                AidlUtil.getInstance().printText(printItem, 40, true, false, LEFT);
                if (!Validators.isNullOrEmpty(productDetail.special_instruction))
                    AidlUtil.getInstance().printText("   Instruction: " + productDetail.special_instruction, 40, true, false, LEFT);
            }

            for (OrderItemAddon orderItemAddon : productDetail.order_item_addons) {
                if (!kitchenCopy) {
                    String subdomainPrintText = getSubaddonWithPad(orderItemAddon.addon_name, 30);
                    AidlUtil.getInstance().printText(subdomainPrintText, 30, true, false, LEFT);
                } else {
                    String subdomainPrintText = getSubaddonWithPad(orderItemAddon.addon_name, 30);
                    AidlUtil.getInstance().printText(subdomainPrintText, 30, true, false, LEFT);
                }
            }

            for (OrderItemIngredient orderItemIngredient : productDetail.order_item_ingredients) {
                String ingredientName = orderItemIngredient.ingredient_name;
                if (orderItemIngredient.with) {
                    ingredientName = "  " + orderItemIngredient.quantity + " Extra " + ingredientName;
                }
                if (orderItemIngredient.without) {
                    ingredientName = "No " + ingredientName;
                }
                if (!kitchenCopy) {
                    String subdomainPrintText = getSubaddonWithPad(ingredientName, 30);
                    AidlUtil.getInstance().printText(subdomainPrintText, 30, true, false, LEFT);
                } else {
                    String subdomainPrintText = getSubaddonWithPad(ingredientName, 30);
                    AidlUtil.getInstance().printText(subdomainPrintText, 30, true, false, LEFT);
                }
            }

            lastItemWasMisc = productDetail.misc;
            lastBlockId = productDetail.block_id;
        }

        if (printTotalInPrint) {
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
            AidlUtil.getInstance().printText(padRightSpaces("SubTotal :", 1) + MyApp.df.format(orderDetail.sub_total), 32, false, false, RIGHT);
            if (orderDetail.delivery_charge > 0) {
                AidlUtil.getInstance().printText(padRightSpaces("Delivery charge :", 1) + MyApp.df.format(orderDetail.delivery_charge), 32, false, false, RIGHT);
            }
            if (orderDetail.gratuity > 0) {
                AidlUtil.getInstance().printText(padRightSpaces("Gratuity :", 1) + MyApp.df.format(orderDetail.gratuity), 32, false, false, RIGHT);
            }
            if (orderDetail.service_charge > 0) {
                AidlUtil.getInstance().printText(padRightSpaces("Service charge :", 1) + MyApp.df.format(orderDetail.service_charge), 32, false, false, RIGHT);
            }
            if (orderDetail.discount > 0) {
                AidlUtil.getInstance().printText(padRightSpaces("Discount :", 1) + MyApp.df.format(orderDetail.discount), 32, false, false, RIGHT);
            }

            AidlUtil.getInstance().printText("Grand Total : " + MyApp.df.format(orderDetail.total), 32, true, false, RIGHT);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
        }

        if (orderDetail.order_payments != null && orderDetail.order_payments.size() > 0) {

            AidlUtil.getInstance().printText("Payment Details:", 32, false, false, LEFT);

            for (OrderPayment orderPayment : orderDetail.order_payments) {
                String paymentDetailToPrint = orderPayment.payment_method_name + " Payment : " + orderPayment.amount;
                AidlUtil.getInstance().printText(paymentDetailToPrint, 32, false, false, LEFT);

            }
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
        }

        String toprint = "Served By: " + MyApp.getInstance().myPreferences.getLoggedInUser().username;
        AidlUtil.getInstance().printText(toprint, 32, false, false, LEFT);

        AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
        if (!Validators.isNullOrEmpty(orderDetail.customer_name)) {
            AidlUtil.getInstance().printText("Cust Name:" + orderDetail.customer_name, 32, false, false, LEFT);
        }

        if (!Validators.isNullOrEmpty(orderDetail.customer.mobile)) {
            AidlUtil.getInstance().printText("Cust No:" + orderDetail.customer.mobile, 32, false, false, LEFT);
        }
        if (orderType == 3) {
            String fulladdress = "Address : " + orderDetail.customer.house_no + ","
                    + orderDetail.customer.street + ", " + orderDetail.customer.state + ", " +
                    orderDetail.customer.country + ", " + orderDetail.customer.postcode;
            AidlUtil.getInstance().printText(fulladdress, 32, false, false, LEFT);
        }
        if (CommonFunctions.orderPaymentStatus(orderDetail.total, orderDetail.total_paid) == -1) {
            AidlUtil.getInstance().printText("Payment Status: UnPaid", 32, false, false, LEFT);
        } else if (CommonFunctions.orderPaymentStatus(orderDetail.total, orderDetail.total_paid) == 1) {
            AidlUtil.getInstance().printText("Payment Status: Paid", 32, false, false, LEFT);
        } else if (CommonFunctions.orderPaymentStatus(orderDetail.total, orderDetail.total_paid) == 0) {
            AidlUtil.getInstance().printText("Payment Status: Partially Paid", 32, false, false, LEFT);
        }
        AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
        if (!Validators.isNullOrEmpty(orderDetail.comment)) {
            AidlUtil.getInstance().printText("Comment: " + orderDetail.comment, kitchenCopy ? 40 : 32, false, false, LEFT);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
        }
        if (kitchenCopy) {
            AidlUtil.getInstance().printText("** " + orderDetail.order_type + " **", 40, true, false, CENTER);
        }

        if (!Validators.isNullOrEmpty(footerA)) {
            AidlUtil.getInstance().printText(footerA, 32, false, false, CENTER);
        }
        if (!Validators.isNullOrEmpty(footerB)) {
            AidlUtil.getInstance().printText(footerB, 32, false, false, CENTER);
        }

        AidlUtil.getInstance().CutPaper();
    }

    public void printReservationList(Bitmap tiffintomLogo, String title, String ticket_header, int headerAlignment, ArrayList<Object> allReservations, String footerA, String footerB, String fromdate, String todate) {
        try {

            if (tiffintomLogo != null) {
                AidlUtil.getInstance().printBitmap(tiffintomLogo);
            }
            AidlUtil.getInstance().printText(ticket_header, 32, true, false, headerAlignment);
            if (!Validators.isNullOrEmpty(title))
                AidlUtil.getInstance().printText(title, 40, true, false, CENTER);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);

            String date = "";
            if (fromdate != null) {
                if (fromdate.equalsIgnoreCase("0") && todate.equalsIgnoreCase("0")) {
                    date = "Show All";
                } else {
                    date = fromdate + " - " + todate;
                }
            } else {
                date = "All";
            }

            AidlUtil.getInstance().printText(date, 32, true, false, LEFT);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);

            for (Object obj : allReservations) {
                StringBuilder sb = new StringBuilder();
                if (obj instanceof Reservation) {
                    sb = new StringBuilder();
                    Reservation reservation = (Reservation) obj;
                    sb.append(padRightSpaces("Name: " + reservation.customer_name, 22))
                            .append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "dd/MM/yyyy"));

                    if (!Validators.isNullOrEmpty(reservation.telephone)) {
                        if (reservation.reservation_date_time.toLowerCase().contains("z")) {
                            sb.append("\n").append(padRightSpaces("Number: " + reservation.telephone, 22)).append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, Constants.PHP_DATE_TIME_FORMAT_ZULU, "hh:mm a"));
                        } else {
                            sb.append("\n").append(padRightSpaces("Number: " + reservation.telephone, 22)).append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "hh:mm a"));
                        }
                    } else {
                        if (reservation.reservation_date_time.toLowerCase().contains("z")) {
                            sb.append("\n").append(padRightSpaces("", 22)).append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, Constants.PHP_DATE_TIME_FORMAT_ZULU, "hh:mm a"));
                        } else {
                            sb.append("\n").append(padRightSpaces("", 22)).append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "hh:mm a"));
                        }
                    }

                    if (!Validators.isNullOrEmpty(reservation.table_number)) {
                        sb.append("\n").append("Table: ").append(reservation.table_number);
                    }
                    sb.append("\n").append(padRightSpaces("Dinner: " + reservation.diners, 18)).append("Status: ").append(reservation.reservation_status);

                    if (!Validators.isNullOrEmpty(reservation.special_instruction)) {
                        sb.append("\n").append("Special Instruction: ").append(reservation.special_instruction);
                    }
                }
                if (obj instanceof OnlineReservation) {

                    sb = new StringBuilder();
                    OnlineReservation reservation = (OnlineReservation) obj;
                    sb.append(padRightSpaces("Name: " + reservation.customer_name, 22))
                            .append(CommonFunctions.formatUnknownDateTime(reservation.booking_date, "yyyy-MM-dd", "dd/MM/yyyy"));

                    if (!Validators.isNullOrEmpty(reservation.booking_phone)) {
                        sb.append("\n").append(padRightSpaces("Number: " + reservation.booking_phone, 22)).append(reservation.booking_time);
                    } else {
                        sb.append("\n").append(padRightSpaces("", 22)).append(reservation.booking_time).append("\n");
                    }
                    sb.append("\n").append(padRightSpaces("Dinner: " + reservation.guest_count, 18)).append("Status: ").append(reservation.status);
                    if (reservation.status.equalsIgnoreCase("cancel") && !Validators.isNullOrEmpty(reservation.cancel_reason)) {
                        sb.append("\n").append("Reason: ").append(reservation.cancel_reason);
                    }
                    if (!Validators.isNullOrEmpty(reservation.booking_instruction) && !reservation.status.equalsIgnoreCase("cancel")) {
                        sb.append("\n").append("Booking Instruction: ").append(reservation.booking_instruction);
                    }

                }
                if (sb.toString().length() > 0) {
                    AidlUtil.getInstance().printText(sb.toString(), 32, true, false, LEFT);
                    AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
                }

            }

            AidlUtil.getInstance().printText(footerA, 32, true, false, CENTER);
            AidlUtil.getInstance().printText(footerB, 32, true, false, CENTER);
            AidlUtil.getInstance().CutPaper();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void printReservation(Bitmap tiffintomLogo, String title, String ticket_header, int headerAlignment, Reservation reservation, String footerA, String footerB) {
        try {

            if (tiffintomLogo != null) {
                AidlUtil.getInstance().printBitmap(tiffintomLogo);
            }
            AidlUtil.getInstance().printText(ticket_header, 32, true, false, headerAlignment);
            if (!Validators.isNullOrEmpty(title))
                AidlUtil.getInstance().printText(title, 40, true, false, CENTER);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);

            AidlUtil.getInstance().printText("Customer Name: " + reservation.customer_name, 32, false, false, LEFT);

            if (!Validators.isNullOrEmpty(reservation.telephone)) {
                AidlUtil.getInstance().printText("Customer Number  : " + reservation.telephone, 32, false, false, LEFT);
            }

            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
            if (reservation.reservation_date_time.toLowerCase().contains("z")) {
                AidlUtil.getInstance().printText("Reservation Date : " + CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, Constants.PHP_DATE_TIME_FORMAT_ZULU, "dd/MM/yyyy"), 32, false, false, LEFT);
                AidlUtil.getInstance().printText("Reservation Time : " + CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, Constants.PHP_DATE_TIME_FORMAT_ZULU, "hh:mm a"), 32, false, false, LEFT);
            } else {
                AidlUtil.getInstance().printText("Reservation Date : " + CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "dd/MM/yyyy"), 32, false, false, LEFT);
                AidlUtil.getInstance().printText("Reservation Time : " + CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "hh:mm a"), 32, false, false, LEFT);
            }
            AidlUtil.getInstance().printText("No of Diners     : " + reservation.diners, 32, false, false, LEFT);

            if (!reservation.online_reservation) {
                if (!Validators.isNullOrEmpty(reservation.deposit_type))
                    AidlUtil.getInstance().printText("Deposit Type     : " + reservation.deposit_type, 32, false, false, LEFT);
                AidlUtil.getInstance().printText("Deposit Amount   : " + reservation.deposit_amount, 32, false, false, LEFT);
            }
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);

            if (!Validators.isNullOrEmpty(reservation.special_instruction)) {
                AidlUtil.getInstance().printText("Special Instructions : " + reservation.special_instruction, 32, false, false, LEFT);
                AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
            }

            AidlUtil.getInstance().printText(footerA, 32, true, false, CENTER);
            AidlUtil.getInstance().printText(footerB, 32, true, false, CENTER);
            AidlUtil.getInstance().CutPaper();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void printVoucher(Bitmap tiffintomLogo, String title, String ticket_header, int headerAlignment, Voucher voucher, String footerA, String footerB) {
        try {

            if (tiffintomLogo != null) {
                AidlUtil.getInstance().printBitmap(tiffintomLogo);
            }
            AidlUtil.getInstance().printText(ticket_header, 32, true, false, headerAlignment);
            if (!Validators.isNullOrEmpty(title))
                AidlUtil.getInstance().printText(title, 40, true, false, CENTER);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);

            AidlUtil.getInstance().printText(voucher.voucher_code, 40, true, false, CENTER);

            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
            AidlUtil.getInstance().printText(MyApp.df.format(voucher.voucher_amount), 40, true, false, CENTER);

            StringBuilder sb = new StringBuilder();

            sb.append("Name: ").append(voucher.customer_name).append("\n");

//            if (voucher.customer != null) {
//                sb.append("Number: " + voucher.customer.mobile + "\n");
//                sb.append("Email: " + voucher.customer.email + "\n");
//            }

            sb.append("VALIDITY \n");

            sb.append("VALID FROM : ").append(CommonFunctions.formatUnknownDateTime(voucher.start_date, Constants.PHP_DATE_TIME_FORMAT, "dd/MM/yyyy")).append("\n");

            if (Validators.isNullOrEmpty(voucher.end_date) || voucher.end_date.contains("203")) {
                sb.append("VALID Till : No Expiry\n");
            } else {
                sb.append("VALID Till : ").append(CommonFunctions.formatUnknownDateTime(voucher.start_date, Constants.PHP_DATE_TIME_FORMAT, "dd/MM/yyyy")).append("\n");
            }

            sb.append("NOT VALID \n");

            if (!Validators.isNullOrEmpty(voucher.week_day)) {
                String[] daystList = voucher.week_day.split(",");
                StringBuilder daysBuilder = new StringBuilder();
                for (String str : daystList) {
                    if (Validators.isNullOrEmpty(str)) continue;
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Monday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Tuesday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Wednesday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Thursday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Friday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Saturday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Sunday");
                    }
                    if (daysBuilder.length() > 0) {
                        daysBuilder.append(", ");
                    }

                }
                sb.append("Not Valid on: ").append(daysBuilder.toString()).append("\n");
            }

            if (!Validators.isNullOrEmpty(voucher.order_type_id)) {
                String[] orderList = voucher.order_type_id.split(",");
                StringBuilder ordersBuilder = new StringBuilder();
                for (String str : orderList) {
                    if (Validators.isNullOrEmpty(str)) continue;
                    if (str.equalsIgnoreCase("1"))
                        ordersBuilder.append("Dine In");

                    if (str.equalsIgnoreCase("2"))
                        ordersBuilder.append("Collection");

                    if (str.equalsIgnoreCase("3"))
                        ordersBuilder.append("Delivery");

                    if (str.equalsIgnoreCase("4"))
                        ordersBuilder.append("Bar");

                    if (str.equalsIgnoreCase("5"))
                        ordersBuilder.append("Waiting");

                    if (str.equalsIgnoreCase("6"))
                        ordersBuilder.append("Web Order");

                    if (ordersBuilder.length() > 0) {
                        ordersBuilder.append(", ");
                    }
                }
                sb.append("Not Valid on: ").append(ordersBuilder.toString()).append("\n");
            }
            AidlUtil.getInstance().printText(sb.toString(), 32, false, false, LEFT);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);

            AidlUtil.getInstance().printText(footerA, 32, true, false, CENTER);
            AidlUtil.getInstance().printText(footerB, 32, true, false, CENTER);
            AidlUtil.getInstance().CutPaper();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void printFullReport(Bitmap tiffintomLogo, String header, int headerAlignment, String title, String date, String user, ArrayList<Object> data, String footerA, String footerB) {
        if (tiffintomLogo != null) {
            AidlUtil.getInstance().printBitmap(tiffintomLogo);
        }
        AidlUtil.getInstance().printText(header, 32, true, false, headerAlignment);
        if (!Validators.isNullOrEmpty(title))
            AidlUtil.getInstance().printText(title, 40, true, false, CENTER);
        AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);

        if (!Validators.isNullOrEmpty(date)) {
            AidlUtil.getInstance().printText("Date: " + date, 32, true, false, CENTER);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
        }
        if (!Validators.isNullOrEmpty(user)) {
            AidlUtil.getInstance().printText("User name: " + user, 32, true, false, CENTER);
            AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);
        }
        for (Object objct : data) {
            if (objct instanceof ReportTextModel) {
                ReportTextModel textModel = (ReportTextModel) objct;

                String printText;
                if (!textModel.isHeader) {
                    if (!Validators.isNullOrEmpty(textModel.value)) {
                        if (textModel.value.contains(MyApp.currencySymbol)) {
                            printText = getTitleWithPad(textModel.title, textModel.value.replace(MyApp.currencySymbol, "") + " GBP", 24);
                        } else {
                            printText = getTitleWithPad(textModel.title, textModel.value, 28);
                        }
                    } else printText = getTitleWithPad(textModel.title, "", 28);
                } else {
                    printText = "\n" + textModel.title;
                }
                AidlUtil.getInstance().printText(printText, textModel.isHeader ? 34 : 32, textModel.isHeader, false, LEFT);

            }
            if (objct instanceof SalesReport) {
                SalesReport salesReport = (SalesReport) objct;
                String name = salesReport.product_name;
                if (Validators.isNullOrEmpty(name)) {
                    name = salesReport.category_name;
                }
                String printText;
                printText = getTitleWithPad(name + "x(" + salesReport.count + ")", salesReport.total, 28);
                AidlUtil.getInstance().printText(printText, 32, false, false, LEFT);

            }
        }

        AidlUtil.getInstance().printText("-------------------------------------", 30, false, false, CENTER);

        AidlUtil.getInstance().printText(footerA, 32, true, false, CENTER);
        AidlUtil.getInstance().printText(footerB, 32, true, false, CENTER);
        AidlUtil.getInstance().CutPaper();
    }

    private String getTitleWithPad(String productName, String price, int charactersInLine) {
        String itemName = "";
        boolean priceAdded = false;
        if (productName.length() > charactersInLine) {
            StringBuilder sb = new StringBuilder();
            for (String part : getParts(productName, charactersInLine)) {

                if (part.length() >= charactersInLine) {
                    sb.append(part);
                    if (!priceAdded) {
                        sb.append(" ").append(price);
                        priceAdded = true;
                    }
                    sb.append("\n");
                } else {
                    sb.append(padRightSpaces(part, charactersInLine));
                }
            }
            itemName = sb.toString();
        } else {
            itemName = padRightSpaces(productName, charactersInLine) + " " + price;
        }
        return itemName;
    }

    private String getProductNameWithPad(String productName, String price, int charactersInLine) {
        String itemName = "";
        boolean priceAdded = false;
        if (productName.length() > charactersInLine) {
            StringBuilder sb = new StringBuilder();
            for (String part : getParts(productName, charactersInLine)) {

                if (part.length() >= charactersInLine) {
                    sb.append(part);
                    if (!priceAdded) {
                        sb.append(" ").append(price);
                        priceAdded = true;
                    }
                    sb.append("\n");
                } else {
                    sb.append("   ").append(padRightSpaces(part, charactersInLine));
                }
            }
            itemName = sb.toString();
        } else {
            itemName = padRightSpaces(productName, charactersInLine) + " " + price;
        }
        return itemName;
    }

    private String getSubaddonWithPadOnline(String part, int charatersInLine) {
        String subAddonName = "";
        if (part.contains(":")) {
            String[] parts1 = part.split(":");

            if (parts1[1].length() > charatersInLine) {

                StringBuilder sb = new StringBuilder();
                for (String partSub : getParts(String.valueOf(parts1[1]), charatersInLine)) {

                    if (partSub.length() >= charatersInLine) {
                        sb.append("   ").append(partSub.trim()).append("\n");
                    } else {
                        sb.append("   ").append(padRightSpaces(partSub.trim(), charatersInLine)).append("\n");
                    }
                }
                subAddonName = sb.toString();

            } else {
                StringBuilder sb = new StringBuilder();
                sb.append("   ").append(padRightSpaces(parts1[1].trim(), charatersInLine)).append("\n");
                subAddonName = sb.toString();
            }

        } else {
            if (part.length() > charatersInLine) {

                StringBuilder sb = new StringBuilder();
                for (String partSub : getParts(part, charatersInLine)) {

                    if (partSub.length() >= charatersInLine) {
                        sb.append("   ").append(partSub.trim()).append("\n");
                    } else {
                        sb.append("   ").append(padRightSpaces(partSub.trim(), charatersInLine)).append("\n");
                    }
                }

                subAddonName = sb.toString();

            } else {
                StringBuilder sb = new StringBuilder();
                sb.append("   ").append(padRightSpaces(part.trim(), charatersInLine)).append("\n");
                subAddonName = sb.toString();

            }

        }
        return subAddonName;
    }

    private String getSubaddonWithPad(String part, int charatersInLine) {
        String subAddonName = "";
        if (part.contains(":")) {
            String[] parts1 = part.split(":");

            if (parts1[1].length() > charatersInLine) {

                StringBuilder sb = new StringBuilder();
                int index = 0;
                List<String> parts = getParts(String.valueOf(parts1[1]), charatersInLine);
                int appendTill = parts.size();
                for (String partSub : parts) {
                    if (partSub.length() >= charatersInLine) {
                        sb.append("   ").append(partSub.trim());
                    } else {
                        sb.append("   ").append(padRightSpaces(partSub.trim(), charatersInLine));
                    }
                    if (index < appendTill - 1) {
                        sb.append("\n");
                    }
                    index++;
                }
                subAddonName = sb.toString();

            } else {
                subAddonName = "   " + padRightSpaces(parts1[1].trim(), charatersInLine);
            }

        } else {
            if (part.length() > charatersInLine) {

                StringBuilder sb = new StringBuilder();
                int index = 0;
                List<String> parts = getParts(part, charatersInLine);
                int appendTill = parts.size();
                for (String partSub : parts) {

                    if (partSub.length() >= charatersInLine) {
                        sb.append("   ").append(partSub.trim());
                    } else {
                        sb.append("   ").append(padRightSpaces(partSub.trim(), charatersInLine));
                    }
                    if (index < appendTill - 1) {
                        sb.append("\n");
                    }
                    index++;
                }

                subAddonName = sb.toString();

            } else {
                subAddonName = "   " + padRightSpaces(part.trim(), charatersInLine);

            }

        }
        return subAddonName;
    }

    public static String padRightSpaces(String str, int n) {
        return String.format("%1$-" + n + "s", str);
    }

    public static String padLeftSpace(String str, int n) {
        return String.format("%1$" + n + "s", str);
    }


    private static List<String> getParts(String string, int partitionSize) {
        List<String> parts = new ArrayList<String>();
        int len = string.length();
        for (int i = 0; i < len; i += partitionSize) {
            parts.add(string.substring(i, Math.min(len, i + partitionSize)));
        }
        return parts;
    }

    private void writeToFile(String data, Context context) {
        try {
            OutputStreamWriter outputStreamWriter = new OutputStreamWriter(context.openFileOutput("print.txt", Context.MODE_PRIVATE));
            outputStreamWriter.write(data);
            outputStreamWriter.close();
        } catch (IOException e) {
            Log.e("Exception", "File write failed: " + e.toString());
        }
    }

    private StringBuffer getDays;
    private StringBuffer getOrderType;

    public void setDays(String string) {
        switch (string) {
            case "0":
                getDays.append("Monday, ");
                break;
            case "1":
                getDays.append("Tuesday, ");
                break;
            case "2":
                getDays.append("Wednesday, ");
                break;
            case "3":
                getDays.append("Thursday, ");
                break;
            case "4":
                getDays.append("Friday, ");
                break;
            case "5":
                getDays.append("Saturday, ");
                break;
            case "6":
                getDays.append("Sunday, ");
                break;
        }
    }

    public void setOrders(String string) {
        switch (string) {
            case "1":
                getOrderType.append("Dine In, ");
                break;
            case "2":
                getOrderType.append("Collection, ");
                break;
            case "3":
                getOrderType.append("Delivery, ");
                break;
            case "4":
                getOrderType.append("Bar, ");
                break;
            case "5":
                getOrderType.append("Waiting, ");
                break;
            case "6":
                getOrderType.append("Web Order, ");
                break;
        }
    }
}
