package com.example.epos_2021.comman.printer;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.util.Log;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.OrderItemAddon;
import com.example.epos_2021.models.OrderItemIngredient;
import com.example.epos_2021.models.OrderPayment;
import com.example.epos_2021.models.ReportTextModel;
import com.example.epos_2021.models.Reservation;
import com.example.epos_2021.models.SalesReport;
import com.example.epos_2021.models.Voucher;
import com.example.epos_2021.online.models.OnlineReservation;
import com.example.epos_2021.online.models.OrderDetail;
import com.example.epos_2021.online.models.OrderProductDetail;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.LogUtils;
import com.example.epos_2021.utils.ToastUtils;
import com.zqprintersdk.PrinterConst;
import com.zqprintersdk.ZQPrinterSDK;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class ZoneRichPrinter {

    public ZQPrinterSDK zonerich;
    public int nRet = -1;
    private Activity activity;
    Context context;
    private StringBuffer getDays;
    private StringBuffer getOrderType;

    public ZoneRichPrinter(Activity activity) {
        super();
        this.activity = activity;
        zonerich = new ZQPrinterSDK();
    }

    public void connectPrinter(String ip) {
        try {
            nRet = zonerich.Prn_Connect(ip + ":9100", activity);
        } catch (Exception e) {
            LogUtils.e("Zonerich Connection error", e.toString());
            ToastUtils.makeToast(activity, e.toString());
        }
    }

    private void disconnectPrinter() {
        zonerich.Prn_Disconnect();
    }

    public void openCashDrawer() {
        zonerich.Prn_OpenCashbox();
        disconnectPrinter();
    }

    public void printDemo() {
        zonerich.Prn_PrintText("Left Line", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
        zonerich.Prn_PrintText("Right Line", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
        zonerich.Prn_PrintText("Center Line", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
        zonerich.Prn_PrintText("Bold Left Line", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
        zonerich.Prn_PrintText("Bold Right Line", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
        zonerich.Prn_PrintText("Bold Center Line", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
        zonerich.Prn_CutPaper();
        disconnectPrinter();

    }

    public void printMotoReceipt(Bitmap bm, String title, String header, int headerAlignment, String amount, String main_content, String date_final, String please, String footer, String footer1) {
        try {
            if (bm != null) {
                zonerich.Prn_SetAlignment(PrinterConst.Alignment.CENTER);
                zonerich.Prn_PrintBitmap(bm, PrinterConst.BitmapSize.ZQSIZE0);
                zonerich.Prn_PrintText("\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            }

            if (headerAlignment < 2)
                zonerich.Prn_PrintText(header + "\n", headerAlignment, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            else {
                String[] headers = header.split("\n");
                for (String h : headers) {
                    zonerich.Prn_PrintText(padLeftSpace(h, 35) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
            }
            zonerich.Prn_PrintText(title + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);

            zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            amount = amount.replace("£", "");
            amount = amount.replace("\n", "");
            amount = amount + " GBP";

            zonerich.Prn_PrintText(amount + "\n\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);
            zonerich.Prn_PrintText(main_content + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText(date_final + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText(please + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText(footer + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText(footer1 + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_LineFeed(3);
            zonerich.Prn_CutPaper();
            disconnectPrinter();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void printCardReaderReceipt(Bitmap tiffintomLogo, String title, String header, int headerAlignment, String actual_amount, String tip_amount, String grand_total, String main_content, String please, String final_date, String footer, String footer1) {
        if (tiffintomLogo != null) {
            zonerich.Prn_SetAlignment(PrinterConst.Alignment.CENTER);
            zonerich.Prn_PrintBitmap(tiffintomLogo, PrinterConst.BitmapSize.ZQSIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
        }
        if (headerAlignment < 2)
            zonerich.Prn_PrintText(header + "\n", headerAlignment, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        else {
            String[] headers = header.split("\n");
            for (String h : headers) {
                zonerich.Prn_PrintText(padLeftSpace(h, 35) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
        }
        zonerich.Prn_PrintText(title + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);

        zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        if (!Validators.isNullOrEmpty(tip_amount)) {
            actual_amount = actual_amount.replace("£", "");
            actual_amount = actual_amount + " GBP";
            zonerich.Prn_PrintText(actual_amount + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE0);

            tip_amount = tip_amount.replace("£", "");
            tip_amount = tip_amount + " GBP";
            zonerich.Prn_PrintText(tip_amount + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE0);
        }
        grand_total = grand_total.replace("£", "");
        grand_total = grand_total + " GBP";

        zonerich.Prn_PrintText(grand_total + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);
        zonerich.Prn_PrintText(main_content + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText(please + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText(final_date + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText(footer + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText(footer1 + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_LineFeed(3);
        zonerich.Prn_CutPaper();
        disconnectPrinter();
    }

    DecimalFormat formatter = (DecimalFormat) NumberFormat.getNumberInstance(Locale.US);

    public void printOrder(Bitmap tiffintomLogo, OrderDetail orderDetail, boolean kitchenCopy) {
        int orderType = -1;
        if (Validators.isNullOrEmpty(orderDetail.order_type)) {
            orderType = 0;
        } else if (orderDetail.order_type.equalsIgnoreCase("delivery")) {
            orderType = 1;
        } else if (orderDetail.order_type.equalsIgnoreCase("pickup")) {
            orderType = 2;
        }
        formatter.applyPattern("0.00");
        try {
            if (tiffintomLogo != null) {
                LogUtils.e("PRINT::", "Tiffintom Logo printing");
                zonerich.Prn_SetAlignment(PrinterConst.Alignment.CENTER);
                zonerich.Prn_PrintBitmap(tiffintomLogo, PrinterConst.BitmapSize.ZQSIZE0);
                zonerich.Prn_PrintText("\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            } else {
                LogUtils.e("PRINT::", "Tiffintom Logo null");
            }

            String todayDate = CommonFunctions.getCurrentTimeFormatted("dd-MM-yyyy");
            String orderDateString = CommonFunctions.formatUnknownDateTime(orderDetail.created, Constants.PHP_DATE_TIME_FORMAT_ZULU, "dd-MM-yyyy");
            if (!todayDate.equalsIgnoreCase(orderDateString)) {
                zonerich.Prn_PrintText("This order is not for today" + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE1 | PrinterConst.WidthSize.SIZE1);
            }
            zonerich.Prn_PrintText("Tiffintom.com sent you order\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            zonerich.Prn_PrintText(CommonFunctions.getOnlineOrderType(orderType) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE1 | PrinterConst.WidthSize.SIZE1);
            zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("ORDER NO:" + orderDetail.order_number + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
            if (orderType > 0) {
                if (orderDetail.assoonas.equalsIgnoreCase("now")) {
                    zonerich.Prn_PrintText("ASAP" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
                } else {
                    zonerich.Prn_PrintText(orderDetail.delivery_time + " " + CommonFunctions.formatUnknownDateTime(orderDetail.delivery_date, "yyyy-MM-dd", "dd-MM-yyyy") + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
                }
                zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            }
            String itemName = "", price = "", qty = "";
            for (OrderProductDetail productDetail : orderDetail.cart_view) {
                String productName = (kitchenCopy && !Validators.isNullOrEmpty(productDetail.short_name)) ? productDetail.short_name : productDetail.menu_name;
                String productQty = String.valueOf(productDetail.quantity);
                // Here we need 16 characters in product name so we send 16 in
                // @params charactersInLine
                price = (!kitchenCopy) ? formatter.format(productDetail.total_price) : "";
                if (kitchenCopy)
                    itemName = productName;
                else
                    itemName = getProductNameWithPad(productName, price, 35);
                qty = productQty;
                if (productQty.length() <= 1) {
                    qty = " " + productQty + " ";
                } else if (productQty.length() <= 2) {
                    qty = productQty + " ";

                } else if (productQty.length() <= 3) {
                    qty = productQty + " ";

                } else if (productQty.length() <= 4) {
                    qty = productQty + "";
                }
                String printItem;
                printItem = qty + itemName + "\n";
                if (!kitchenCopy)
                    zonerich.Prn_PrintText(printItem, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
                else
                    zonerich.Prn_PrintText(printItem, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE1 | PrinterConst.WidthSize.SIZE1);

                if (!Validators.isNullOrEmpty(productDetail.subaddons_name)) {
                    try {
                        //String[] parts = menus.get(position).getSubaddonsName().split(",");
                        String[] parts = productDetail.subaddons_name.split(",");

                        for (String subAddonPart : parts) {
                            if (!kitchenCopy) {
                                String subdomainPrintText = getSubaddonWithPad(subAddonPart, 35);
                                zonerich.Prn_PrintText(subdomainPrintText, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
                            } else {
                                String subdomainPrintText = getSubaddonWithPad(subAddonPart, 20);
                                zonerich.Prn_PrintText(subdomainPrintText, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE1 | PrinterConst.WidthSize.SIZE1);
                            }
                        }
                    } catch (Exception e) {
                        Log.e("Addon_Error", e.getMessage());
                    }
                }
            }
            zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText(padRightSpaces("SubTotal:", 20) + MyApp.df.format(orderDetail.order_sub_total) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            if (orderDetail.delivery_charge > 0) {
                zonerich.Prn_PrintText(padRightSpaces("Delivery Charge:", 20) + MyApp.df.format(orderDetail.delivery_charge) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            if (orderDetail.offer_amount > 0) {
                zonerich.Prn_PrintText(padRightSpaces("Offer:", 20) + MyApp.df.format(orderDetail.offer_amount) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            if (orderDetail.service_charge > 0) {
                if (!"no".equalsIgnoreCase(orderDetail.service_charge_status)) {
                    zonerich.Prn_PrintText(padRightSpaces("Service Charge:", 20) + MyApp.df.format(orderDetail.service_charge) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
            }
            if (orderDetail.driver_tip > 0) {
                zonerich.Prn_PrintText(padRightSpaces("Driver Tip:", 20) + MyApp.df.format(orderDetail.driver_tip) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            if (orderDetail.reward_offer > 0) {
                zonerich.Prn_PrintText(padRightSpaces("Loyalty Point Discount:", 20) + MyApp.df.format(orderDetail.reward_offer) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            zonerich.Prn_PrintText("Grand Total: " + MyApp.df.format(orderDetail.order_grand_total) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
            zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            if (!Validators.isNullOrEmpty(orderDetail.order_description)) {
                if (kitchenCopy) {
                    zonerich.Prn_PrintText("Comments: " + orderDetail.order_description + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);
                } else {
                    zonerich.Prn_PrintText("Comments: " + orderDetail.order_description + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
                zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            if (!Validators.isNullOrEmpty(orderDetail.delivery_instruction)) {
                zonerich.Prn_PrintText("Delivery Instructions: " + orderDetail.delivery_instruction + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            zonerich.Prn_PrintText("Cust Name:" + orderDetail.customer_name + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            if (!Validators.isNullOrEmpty(orderDetail.customer_phone)) {
                zonerich.Prn_PrintText("Cust No:" + orderDetail.customer_phone + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            if (orderType == 1) {

                String address = orderDetail.address + "\n";
                String[] parts = address.split("[\\\\s,]+");
                StringBuilder str = new StringBuilder();

                for (String name : parts) {
                    str.append(name).append("\n");
                }
                zonerich.Prn_PrintText("Del. address:" + address + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
                zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }

            if (orderType >= 0) {

                if (orderDetail.status.equalsIgnoreCase("accepted") && !Validators.isNullOrEmpty(orderDetail.preparation)) {
                    zonerich.Prn_PrintText("Confirmed for:" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
                    zonerich.Prn_PrintText(orderDetail.preparation + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
                }
                zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            }
            zonerich.Prn_PrintText("Order " + (orderDetail.payment_status.equalsIgnoreCase("np") ? "Unpaid" : "Paid") + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);

            String paymentName = "";
            if (orderDetail.payment_method.equalsIgnoreCase("stripe")) paymentName = "ONLINE";
            if (orderDetail.payment_method.equalsIgnoreCase("cod")) paymentName = "COD";
            if (orderDetail.payment_method.equalsIgnoreCase("wallet") || orderDetail.payment_method.equalsIgnoreCase("credit"))
                paymentName = "WALLET";
            if (orderDetail.payment_method.equalsIgnoreCase("paypal")) paymentName = "PAYPAL";

            zonerich.Prn_PrintText("Payment Type:  " + paymentName + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
            zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("Receipt Time:\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
            zonerich.Prn_PrintText(CommonFunctions.getCurrentTimeFormatted("dd/MM/yyyy hh:mm a") + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
            zonerich.Prn_PrintText("Order status: " + orderDetail.status + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
            if (orderDetail.status.equalsIgnoreCase("failed")) {
                zonerich.Prn_PrintText("Reason: " + orderDetail.failed_reason + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
            }

            zonerich.Prn_PrintText(MyApp.getInstance().myPreferences.getLoggedInAdmin().selected_business.name + ", " + MyApp.getInstance().myPreferences.getLoggedInAdmin().selected_business.address + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);

            zonerich.Prn_PrintText("Thanks!" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_LineFeed(3);
            zonerich.Prn_CutPaper();
            zonerich.Prn_Disconnect();

        } catch (Exception e) {
            System.out.println(e);
        }
    }

    public void printOrderEpos(Bitmap tiffintomLogo, String title, String ticket_header, int headerAlignment, String tableNumberOrderIdMode, String dateTimeMode, Order orderDetail, boolean printTotalInPrint, String footerA, String footerB, boolean kitchenCopy, boolean isCashDrawerOpen) {
        int orderType = -1;
        if (Validators.isNullOrEmpty(orderDetail.order_type_id)) {
            orderType = 0;
        } else {
            orderType = Integer.parseInt(orderDetail.order_type_id);
        }

        formatter.applyPattern("0.00");
        try {
            if (!kitchenCopy && tiffintomLogo != null) {
                zonerich.Prn_SetAlignment(PrinterConst.Alignment.CENTER);
                zonerich.Prn_PrintBitmap(tiffintomLogo, PrinterConst.BitmapSize.ZQSIZE0);
                zonerich.Prn_PrintText("\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            }

            if (!kitchenCopy) {

                if (headerAlignment < 2)
                    zonerich.Prn_PrintText(ticket_header + "\n", headerAlignment, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                else {
                    String[] headers = ticket_header.split("\n");
                    for (String h : headers) {
                        zonerich.Prn_PrintText(padLeftSpace(h, 35) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                    }
                }
            }
            if (kitchenCopy) {
                zonerich.Prn_PrintText("** " + orderDetail.order_type + " **\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);
                if (orderDetail.order_type_id.equalsIgnoreCase("2") && !Validators.isNullOrEmpty(orderDetail.customer_name)) {
                    zonerich.Prn_PrintText(orderDetail.customer_name + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                            PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);
                }

                zonerich.Prn_PrintText("------------------------------------------------\n",
                        PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }

            if (!Validators.isNullOrEmpty(title)) {
                zonerich.Prn_PrintText(title + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);

            }

            String tableNumberOrderIdText = "";
            if (tableNumberOrderIdMode != null) {

                if (tableNumberOrderIdMode.equalsIgnoreCase("order_number") || tableNumberOrderIdMode.equalsIgnoreCase("both")) {
                    if (Validators.isNullOrEmpty(orderDetail.id)) {
                        tableNumberOrderIdText = "Order ID: " + orderDetail._id;
                    } else
                        tableNumberOrderIdText = "Order ID: " + orderDetail.id;

                }
                if (orderType == 1 && (tableNumberOrderIdMode.equalsIgnoreCase("table_number") || tableNumberOrderIdMode.equalsIgnoreCase("both"))) {
                    if (tableNumberOrderIdText.length() > 1) {
                        tableNumberOrderIdText = tableNumberOrderIdText + "\n";
                    }
                    tableNumberOrderIdText = tableNumberOrderIdText + "Table No: " + orderDetail.table_number + "  No of Diners:  " + orderDetail.no_guest;

                }
                if (tableNumberOrderIdMode.equalsIgnoreCase("none")) {
                    tableNumberOrderIdText = "";
                }
                if (tableNumberOrderIdMode.length() > 0) {
                    if (!Validators.isNullOrEmpty(title)) {
                        tableNumberOrderIdText = "\n------------------------------------------------\n" + tableNumberOrderIdText + "\n------------------------------------------------\n";
                    } else {
                        tableNumberOrderIdText = tableNumberOrderIdText + "\n------------------------------------------------\n";
                    }
                    zonerich.Prn_PrintText(tableNumberOrderIdText, PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
            }

            if (!Validators.isNullOrEmpty(dateTimeMode)) {
                String dateFormat = "dd-MM-yyyy hh:mm a";
                String dateFormatTitle = "Order Date Time: ";
                if (dateTimeMode.equalsIgnoreCase("date_time")) {
                    dateFormat = "dd-MM-yyyy hh:mm a";
                    dateFormatTitle = "Order Date Time: ";
                }
                if (dateTimeMode.equalsIgnoreCase("time")) {
                    dateFormat = "hh:mm a";
                    dateFormatTitle = "Order Time: ";
                }
                if (dateTimeMode.equalsIgnoreCase("date")) {
                    dateFormat = "dd-MM-yyyy";
                    dateFormatTitle = "Order Date: ";
                }
                if (orderDetail.created_at.contains("Z")) {
                    zonerich.Prn_PrintText(dateFormatTitle + CommonFunctions.formatUnknownDateTime(orderDetail.created_at, Constants.PHP_DATE_TIME_FORMAT_ZULU, dateFormat) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                            PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                } else {
                    zonerich.Prn_PrintText(dateFormatTitle + CommonFunctions.formatUnknownDateTime(orderDetail.created_at, Constants.PHP_DATE_TIME_FORMAT, dateFormat) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                            PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
            }
            if (orderType > 1 && !kitchenCopy) {
                zonerich.Prn_PrintText("Order Type: " + orderDetail.order_type + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                zonerich.Prn_PrintText("------------------------------------------------\n",
                        PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            zonerich.Prn_PrintText("Qty  Name" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            String itemName = "", price = "", qty = "";

            String lastBlockId = null;
            boolean lastItemWasMisc = false;

            for (OrderItem productDetail : orderDetail.order_items) {
                String productName = (kitchenCopy && !Validators.isNullOrEmpty(productDetail.product_short_name)) ? productDetail.product_short_name : productDetail.product_name;
                String productQty = String.valueOf(productDetail.quantity);
                // Here we need 16 characters in product name so we send 16 in
                // @params charactersInLine
                price = (!kitchenCopy) ? formatter.format(productDetail.total) : "";
                if (kitchenCopy)
                    itemName = productName;
                else
                    itemName = getProductNameWithPad(productName, price, 35);
                qty = productQty;
                if (productQty.length() <= 1) {
                    qty = " " + productQty + " ";
                } else if (productQty.length() <= 2) {
                    qty = productQty + " ";

                } else if (productQty.length() <= 3) {
                    qty = productQty + " ";

                } else if (productQty.length() <= 4) {
                    qty = productQty + "";
                }
                if (lastBlockId != null) {
                    if (!lastBlockId.equalsIgnoreCase(productDetail.block_id)) {
                        if (kitchenCopy)
                            zonerich.Prn_PrintText("----------------------------------\n",
                                    PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                    }
                } else {
                    if (lastItemWasMisc && !productDetail.misc) {
                        if (kitchenCopy)
                            zonerich.Prn_PrintText("----------------------------------\n",
                                    PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                    }
                }

                String printItem;
                printItem = qty + itemName + "\n";
                if (!kitchenCopy) {
                    zonerich.Prn_PrintText(printItem, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
                    if (!Validators.isNullOrEmpty(productDetail.special_instruction))
                        zonerich.Prn_PrintText("  " + "Instruction: " + productDetail.special_instruction + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
                } else {
                    zonerich.Prn_PrintText(printItem, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE1 | PrinterConst.WidthSize.SIZE1);
                    if (!Validators.isNullOrEmpty(productDetail.special_instruction))
                        zonerich.Prn_PrintText("  " + "Instruction: " + productDetail.special_instruction + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
                }

                for (OrderItemAddon orderItemAddon : productDetail.order_item_addons) {
                    if (!kitchenCopy) {
                        String subdomainPrintText = "  " + orderItemAddon.addon_name + "\n";
                        zonerich.Prn_PrintText(subdomainPrintText, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
                    } else {
                        String subdomainPrintText = "  " + orderItemAddon.addon_name + "\n";
                        zonerich.Prn_PrintText(subdomainPrintText, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
                    }
                }

                for (OrderItemIngredient orderItemIngredient : productDetail.order_item_ingredients) {
                    String ingredientName = orderItemIngredient.ingredient_name;
                    if (orderItemIngredient.with) {
                        ingredientName = "  " + orderItemIngredient.quantity + " Extra " + ingredientName;
                    }
                    if (orderItemIngredient.without) {
                        ingredientName = "  " + "No " + ingredientName;
                    }
                    String subdomainPrintText = ingredientName + "\n";
                    if (!kitchenCopy) {
                        zonerich.Prn_PrintText(subdomainPrintText, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
                    } else {
                        zonerich.Prn_PrintText(subdomainPrintText, PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
                    }
                }

                lastItemWasMisc = productDetail.misc;
                lastBlockId = productDetail.block_id;
            }
            if (printTotalInPrint) {
                zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                zonerich.Prn_PrintText(padRightSpaces("SubTotal:", 20) + MyApp.df.format(orderDetail.sub_total) + "\n", PrinterConst.Alignment.RIGHT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

                if (orderDetail.delivery_charge > 0) {
                    zonerich.Prn_PrintText(padRightSpaces("Delivery Charge:", 20) + MyApp.df.format(orderDetail.delivery_charge) + "\n", PrinterConst.Alignment.RIGHT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
                if (orderDetail.gratuity > 0) {
                    zonerich.Prn_PrintText(padRightSpaces("Gratuity :", 20) + MyApp.df.format(orderDetail.gratuity) + "\n", PrinterConst.Alignment.RIGHT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
                if (orderDetail.service_charge > 0) {
                    zonerich.Prn_PrintText(padRightSpaces("Service charge :", 20) + MyApp.df.format(orderDetail.service_charge) + "\n", PrinterConst.Alignment.RIGHT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
                if (orderDetail.discount > 0) {
                    zonerich.Prn_PrintText(padRightSpaces("Discount :", 20) + MyApp.df.format(orderDetail.discount) + "\n", PrinterConst.Alignment.RIGHT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }

                zonerich.Prn_PrintText("Grand Total: " + MyApp.df.format(orderDetail.total) + "\n", PrinterConst.Alignment.RIGHT, PrinterConst.Font.BOLD, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE1);
                zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }

            if (orderDetail.order_payments != null && orderDetail.order_payments.size() > 0) {

                zonerich.Prn_PrintText("Payment Details:" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                for (OrderPayment orderPayment : orderDetail.order_payments) {
                    String paymentDetailToPrint = orderPayment.payment_method_name + " Payment : " + orderPayment.amount + "\n";
                    zonerich.Prn_PrintText(paymentDetailToPrint, PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                            PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
                zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }

            String toprint = "Served By: " + MyApp.getInstance().myPreferences.getLoggedInUser().username;
            zonerich.Prn_PrintText(toprint + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            zonerich.Prn_PrintText("------------------------------------------------\n",
                    PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            if (!Validators.isNullOrEmpty(orderDetail.customer_name))
                zonerich.Prn_PrintText("Cust Name:" + orderDetail.customer_name + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            if (!Validators.isNullOrEmpty(orderDetail.customer.mobile)) {
                zonerich.Prn_PrintText("Cust No:" + orderDetail.customer.mobile + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            if (orderType == 3) {
                String fulladdress = "Address : " + orderDetail.customer.house_no + ","
                        + orderDetail.customer.street + ", " + orderDetail.customer.state + ", " +
                        orderDetail.customer.country + ", " + orderDetail.customer.postcode;
                zonerich.Prn_PrintText(fulladdress + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            if (orderDetail.total_paid == 0) {
                zonerich.Prn_PrintText("Payment Status: Unpaid" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            } else if (MyApp.df.format(orderDetail.total).equalsIgnoreCase(MyApp.df.format(orderDetail.total_paid))) {
                zonerich.Prn_PrintText("Payment Status: Paid" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            } else if (orderDetail.total_paid > 0) {
                zonerich.Prn_PrintText("Payment Status: Partially Paid" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            if (!Validators.isNullOrEmpty(orderDetail.comment)) {
                if (kitchenCopy) {
                    zonerich.Prn_PrintText("Comment: " + orderDetail.comment + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);
                } else {
                    zonerich.Prn_PrintText("Comment: " + orderDetail.comment + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
                zonerich.Prn_PrintText("------------------------------------------------\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
            if (kitchenCopy) {
                zonerich.Prn_PrintText("** " + orderDetail.order_type + " **\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);

            }

            if (!Validators.isNullOrEmpty(footerA)) {
                zonerich.Prn_PrintText(footerA + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            }
            if (!Validators.isNullOrEmpty(footerB)) {
                zonerich.Prn_PrintText(footerB + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            }

            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_LineFeed(3);

            if (isCashDrawerOpen) {
                zonerich.Prn_OpenCashbox();
            }

            zonerich.Prn_CutPaper();
            zonerich.Prn_Disconnect();

        } catch (Exception e) {
            LogUtils.e("PRINT::", Log.getStackTraceString(e));
            System.out.println(e);
        }
    }

    public void printReservationList(Bitmap tiffintomLogo, String title, String ticket_header, int headerAlignment, ArrayList<Object> allReservations, String footerA, String footerB, String fromdate, String todate) {
        try {

            if (tiffintomLogo != null) {
                zonerich.Prn_SetAlignment(PrinterConst.Alignment.CENTER);
                zonerich.Prn_PrintBitmap(tiffintomLogo, PrinterConst.BitmapSize.ZQSIZE0);
                zonerich.Prn_PrintText("\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            }

            if (headerAlignment < 2)
                zonerich.Prn_PrintText(ticket_header + "\n", headerAlignment, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            else {
                String[] headers = ticket_header.split("\n");
                for (String h : headers) {
                    zonerich.Prn_PrintText(padLeftSpace(h, 35) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
            }

            if (!Validators.isNullOrEmpty(title)) {
                zonerich.Prn_PrintText(title + "\n\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE2 | PrinterConst.HeightSize.SIZE2);
            }

            zonerich.Prn_PrintText("------------------------------------------------\n",
                    PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            String date = "";
            if (fromdate != null) {
                if (fromdate.equalsIgnoreCase("0") && todate.equalsIgnoreCase("0")) {
                    date = "Show All";
                } else {
                    date = fromdate + " - " + todate;
                }
            } else {
                date = "All";
            }


            zonerich.Prn_PrintText(date + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("------------------------------------------------\n",
                    PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            for (Object obj : allReservations) {
                StringBuilder sb = new StringBuilder();
                if (obj instanceof Reservation) {
                    sb = new StringBuilder();
                    Reservation reservation = (Reservation) obj;
                    sb.append(padRightSpaces("Name: " + reservation.customer_name, 30))
                            .append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "dd/MM/yyyy"));

                    if (!Validators.isNullOrEmpty(reservation.telephone)) {
                        if (reservation.reservation_date_time.toLowerCase().contains("z")) {
                            sb.append("\n").append(padRightSpaces("Number: " + reservation.telephone, 30)).append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, Constants.PHP_DATE_TIME_FORMAT_ZULU, "hh:mm a"));
                        } else {
                            sb.append("\n").append(padRightSpaces("Number: " + reservation.telephone, 30)).append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "hh:mm a"));
                        }
                    } else {
                        if (reservation.reservation_date_time.toLowerCase().contains("z")) {
                            sb.append("\n").append(padRightSpaces("", 20)).append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, Constants.PHP_DATE_TIME_FORMAT_ZULU, "hh:mm a"));
                        } else {
                            sb.append("\n").append(padRightSpaces("", 20)).append(CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "hh:mm a"));
                        }
                    }

                    if (!Validators.isNullOrEmpty(reservation.table_number)) {
                        sb.append("\n").append("Table: ").append(reservation.table_number);
                    }
                    sb.append("\n").append(padRightSpaces("Dinner: " + reservation.diners, 30)).append("Status: ").append(reservation.reservation_status);

                    if (!Validators.isNullOrEmpty(reservation.special_instruction)) {
                        sb.append("\n").append("Special Instruction: ").append(reservation.special_instruction);
                    }
                }
                if (obj instanceof OnlineReservation) {

                    sb = new StringBuilder();
                    OnlineReservation reservation = (OnlineReservation) obj;
                    sb.append(padRightSpaces("Name: " + reservation.customer_name, 30))
                            .append(CommonFunctions.formatUnknownDateTime(reservation.booking_date, "yyyy-MM-dd", "dd/MM/yyyy"));

                    if (!Validators.isNullOrEmpty(reservation.booking_phone)) {
                        sb.append("\n").append(padRightSpaces("Number: " + reservation.booking_phone, 30)).append(reservation.booking_time);
                    } else {
                        sb.append("\n").append(padRightSpaces("", 30)).append(reservation.booking_time).append("\n");
                    }
                    sb.append("\n").append(padRightSpaces("Dinner: " + reservation.guest_count, 30)).append("Status: ").append(reservation.status);
                    if (reservation.status.equalsIgnoreCase("cancel") && !Validators.isNullOrEmpty(reservation.cancel_reason)) {
                        sb.append("\n").append("Reason: ").append(reservation.cancel_reason);
                    }
                    if (!Validators.isNullOrEmpty(reservation.booking_instruction) && !reservation.status.equalsIgnoreCase("cancel")) {
                        sb.append("\n").append("Booking Instruction: ").append(reservation.booking_instruction);
                    }

                }
                if (sb.toString().length() > 0) {
                    zonerich.Prn_PrintText(sb.toString() + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                            PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                    zonerich.Prn_PrintText("----------------------------\n",
                            PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }

            }

            zonerich.Prn_PrintText(footerA + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText(footerB + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_LineFeed(3);
            zonerich.Prn_CutPaper();
            zonerich.Prn_Disconnect();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void printReservation(Bitmap tiffintomLogo, String title, String ticket_header, int headerAlignment, Reservation reservation, String footerA, String footerB) {
        try {

            if (tiffintomLogo != null) {
                zonerich.Prn_SetAlignment(PrinterConst.Alignment.CENTER);
                zonerich.Prn_PrintBitmap(tiffintomLogo, PrinterConst.BitmapSize.ZQSIZE0);
                zonerich.Prn_PrintText("\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            }

            if (headerAlignment < 2)
                zonerich.Prn_PrintText(ticket_header + "\n", headerAlignment, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            else {
                String[] headers = ticket_header.split("\n");
                for (String h : headers) {
                    zonerich.Prn_PrintText(padLeftSpace(h, 35) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
            }

            if (!Validators.isNullOrEmpty(title)) {
                zonerich.Prn_PrintText(title + "\n\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE2 | PrinterConst.HeightSize.SIZE2);
            }

            zonerich.Prn_PrintText("------------------------------------------------\n",
                    PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            String customername = "Customer Name: " + reservation.customer_name;
            zonerich.Prn_PrintText(customername + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            if (!Validators.isNullOrEmpty(reservation.telephone)) {
                String customerno = "Customer Number: " + reservation.telephone;
                zonerich.Prn_PrintText(customerno + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }

            zonerich.Prn_PrintText("------------------------------------------------\n",
                    PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            if (reservation.reservation_date_time.toLowerCase().contains("z")) {
                zonerich.Prn_PrintText("Reservation Date : " + CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, Constants.PHP_DATE_TIME_FORMAT_ZULU, "dd/MM/yyyy") + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                zonerich.Prn_PrintText("Reservation Time : " + CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, Constants.PHP_DATE_TIME_FORMAT_ZULU, "hh:mm a") + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            } else {
                zonerich.Prn_PrintText("Reservation Date : " + CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "dd/MM/yyyy") + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                zonerich.Prn_PrintText("Reservation Time : " + CommonFunctions.formatUnknownDateTime(reservation.reservation_date_time, "yyyy-MM-dd HH:mm:ss", "hh:mm a") + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }

            zonerich.Prn_PrintText("No of Diners     : " + reservation.diners + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            if (!reservation.online_reservation) {
                if (!Validators.isNullOrEmpty(reservation.deposit_type))
                    zonerich.Prn_PrintText("Deposit Type     : " + reservation.deposit_type + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                            PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                zonerich.Prn_PrintText("Deposit Amount   : " + reservation.deposit_amount + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }

            zonerich.Prn_PrintText("------------------------------------------------\n",
                    PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            if (!Validators.isNullOrEmpty(reservation.special_instruction)) {
                zonerich.Prn_PrintText("Special Instructions : " + "\n" + reservation.special_instruction + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

                zonerich.Prn_PrintText("------------------------------------------------\n",
                        PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }

            zonerich.Prn_PrintText(footerA + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText(footerB + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            zonerich.Prn_LineFeed(3);
            zonerich.Prn_CutPaper();
            zonerich.Prn_Disconnect();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void printVoucher(Bitmap tiffintomLogo, String title, String ticket_header, int headerAlignment, Voucher voucher, String footerA, String footerB) {
        try {

            if (tiffintomLogo != null) {
                zonerich.Prn_SetAlignment(PrinterConst.Alignment.CENTER);
                zonerich.Prn_PrintBitmap(tiffintomLogo, PrinterConst.BitmapSize.ZQSIZE0);
                zonerich.Prn_PrintText("\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
            }

            if (headerAlignment < 2)
                zonerich.Prn_PrintText(ticket_header + "\n", headerAlignment, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            else {
                String[] headers = ticket_header.split("\n");
                for (String h : headers) {
                    zonerich.Prn_PrintText(padLeftSpace(h, 35) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
            }

            if (!Validators.isNullOrEmpty(title)) {
                zonerich.Prn_PrintText(title + "\n\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                        PrinterConst.WidthSize.SIZE2 | PrinterConst.HeightSize.SIZE2);
            }

            zonerich.Prn_PrintText(voucher.voucher_code + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);

            zonerich.Prn_PrintText("------------------------------------------------\n",
                    PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);


            zonerich.Prn_PrintText(MyApp.df.format(voucher.voucher_amount) + " GBP\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE2 | PrinterConst.HeightSize.SIZE2);

            zonerich.Prn_PrintText("------------------------------------------------\n",
                    PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            StringBuilder sb = new StringBuilder();

            sb.append("Name: ").append(voucher.customer_name).append("\n");

//            if (voucher.customer != null) {
//                sb.append("Number: " + voucher.customer.mobile + "\n");
//                sb.append("Email: " + voucher.customer.email + "\n");
//            }

            sb.append("VALIDITY \n");

            sb.append("VALID FROM : ").append(CommonFunctions.formatUnknownDateTime(voucher.start_date, Constants.PHP_DATE_TIME_FORMAT, "dd/MM/yyyy")).append("\n");

            if (Validators.isNullOrEmpty(voucher.end_date) || voucher.end_date.contains("203")) {
                sb.append("VALID Till : No Expiry\n");
            } else {
                sb.append("VALID Till : ").append(CommonFunctions.formatUnknownDateTime(voucher.start_date, Constants.PHP_DATE_TIME_FORMAT, "dd/MM/yyyy")).append("\n");
            }

            if (!Validators.isNullOrEmpty(voucher.week_day)) {
                String[] daystList = voucher.week_day.split(",");
                StringBuilder daysBuilder = new StringBuilder();
                for (String str : daystList) {
                    if (Validators.isNullOrEmpty(str)) continue;
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Monday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Tuesday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Wednesday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Thursday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Friday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Saturday");
                    }
                    if (str.equalsIgnoreCase("0")) {
                        daysBuilder.append("Sunday");
                    }
                    if (daysBuilder.length() > 0) {
                        daysBuilder.append(", ");
                    }

                }
                sb.append("Not Valid on: ").append(daysBuilder.toString()).append("\n");
            }

            if (!Validators.isNullOrEmpty(voucher.order_type_id)) {
                String[] orderList = voucher.order_type_id.split(",");
                StringBuilder ordersBuilder = new StringBuilder();
                for (String str : orderList) {
                    if (Validators.isNullOrEmpty(str)) continue;
                    if (str.equalsIgnoreCase("1"))
                        ordersBuilder.append("Dine In");

                    if (str.equalsIgnoreCase("2"))
                        ordersBuilder.append("Collection");

                    if (str.equalsIgnoreCase("3"))
                        ordersBuilder.append("Delivery");

                    if (str.equalsIgnoreCase("4"))
                        ordersBuilder.append("Bar");

                    if (str.equalsIgnoreCase("5"))
                        ordersBuilder.append("Waiting");

                    if (str.equalsIgnoreCase("6"))
                        ordersBuilder.append("Web Order");

                    if (ordersBuilder.length() > 0) {
                        ordersBuilder.append(", ");
                    }
                }
                sb.append("Not Valid on: ").append(ordersBuilder.toString()).append("\n");
            }

            zonerich.Prn_PrintText(sb.toString(), PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            zonerich.Prn_PrintText("------------------------------------------------\n",
                    PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

            zonerich.Prn_PrintText(footerA + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText(footerB + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                    PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_LineFeed(3);
            zonerich.Prn_CutPaper();
            zonerich.Prn_Disconnect();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String getProductNameWithPad(String productName, String price, int charactersInLine) {
        String itemName = "";
        boolean priceAdded = false;
        if (productName.length() > charactersInLine) {
            StringBuilder sb = new StringBuilder();
            for (String part : getParts(productName, charactersInLine)) {

                if (part.length() >= charactersInLine) {
                    sb.append(part);
                    if (!priceAdded) {
                        sb.append(" ").append(price);
                        priceAdded = true;
                    }
                    sb.append("\n");
                } else {
                    sb.append("   ").append(padRightSpaces(part, charactersInLine));
                }
            }
            itemName = sb.toString();
        } else {
            itemName = padRightSpaces(productName, charactersInLine) + " " + price;
        }
        return itemName;
    }

    private String getSubaddonWithPad(String part, int charatersInLine) {
        String subAddonName = "";
        if (part.contains(":")) {
            String[] parts1 = part.split(":");

            if (parts1[1].length() > charatersInLine) {

                StringBuilder sb = new StringBuilder();
                for (String partSub : getParts(String.valueOf(parts1[1]), charatersInLine)) {

                    if (partSub.length() >= charatersInLine) {
                        sb.append("   ").append(partSub.trim()).append("\n");
                    } else {
                        sb.append("   ").append(padRightSpaces(partSub.trim(), charatersInLine)).append("\n");
                    }
                }
                subAddonName = sb.toString();

            } else {
                StringBuilder sb = new StringBuilder();
                sb.append("   ").append(padRightSpaces(parts1[1].trim(), charatersInLine)).append("\n");
                subAddonName = sb.toString();
            }

        } else {
            if (part.length() > charatersInLine) {

                StringBuilder sb = new StringBuilder();
                for (String partSub : getParts(part, charatersInLine)) {

                    if (partSub.length() >= charatersInLine) {
                        sb.append("   ").append(partSub.trim()).append("\n");
                    } else {
                        sb.append("   ").append(padRightSpaces(partSub.trim(), charatersInLine)).append("\n");
                    }
                }

                subAddonName = sb.toString();

            } else {
                StringBuilder sb = new StringBuilder();
                sb.append("   ").append(padRightSpaces(part.trim(), charatersInLine)).append("\n");
                subAddonName = sb.toString();

            }

        }
        return subAddonName;
    }

    public static String padRightSpaces(String str, int n) {
        return String.format("%1$-" + n + "s", str);
    }

    public static String padLeftSpace(String str, int n) {
        return String.format("%1$" + n + "s", str);
    }

    private static List<String> getParts(String string, int partitionSize) {
        List<String> parts = new ArrayList<String>();
        int len = string.length();
        for (int i = 0; i < len; i += partitionSize) {
            parts.add(string.substring(i, Math.min(len, i + partitionSize)));
        }
        return parts;
    }

    private static String fixedLengthString(String string, int length) {
        return String.format("%1$" + length + "s", string);
    }

    public void printFullReport(Bitmap bm, String header, int headerAlignment, String title, String date, String user, ArrayList<Object> data, String footerA, String footerB) {
        if (bm != null) {
            zonerich.Prn_SetAlignment(PrinterConst.Alignment.CENTER);
            zonerich.Prn_PrintBitmap(bm, PrinterConst.BitmapSize.ZQSIZE0);
            zonerich.Prn_PrintText("\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.HeightSize.SIZE0 | PrinterConst.WidthSize.SIZE0);
        }

        if (headerAlignment < 2)
            zonerich.Prn_PrintText(header + "\n", headerAlignment, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        else {
            String[] headers = header.split("\n");
            for (String h : headers) {
                zonerich.Prn_PrintText(padLeftSpace(h, 35) + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
        }
        if (!Validators.isNullOrEmpty(title))
            zonerich.Prn_PrintText(title + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);
        zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);

        if (!Validators.isNullOrEmpty(date)) {
            zonerich.Prn_PrintText("Date: " + date + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE1 | PrinterConst.HeightSize.SIZE1);
            zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        }
        if (!Validators.isNullOrEmpty(user)) {
            zonerich.Prn_PrintText("User name: " + user + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        }
        for (Object objct : data) {
            if (objct instanceof ReportTextModel) {
                ReportTextModel textModel = (ReportTextModel) objct;

                String printText;
                if (!textModel.isHeader) {
                    if (!Validators.isNullOrEmpty(textModel.value)) {
                        if (textModel.value.contains(MyApp.currencySymbol)) {
                            printText = getProductNameWithPad(textModel.title, textModel.value.replace(MyApp.currencySymbol, "") + " GBP", 35);
                        } else {
                            printText = getProductNameWithPad(textModel.title, textModel.value, 35);
                        }
                    } else printText = getProductNameWithPad(textModel.title, "", 35);
                } else {
                    printText = textModel.title;
                }
                if (textModel.isHeader)
                    zonerich.Prn_PrintText(printText + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.BOLD, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                else {
                    zonerich.Prn_PrintText(printText + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
                }
            }
            if (objct instanceof SalesReport) {
                SalesReport salesReport = (SalesReport) objct;
                String name = salesReport.product_name;
                if (Validators.isNullOrEmpty(name)) {
                    name = salesReport.category_name;
                }
                String printText;
                printText = getProductNameWithPad(name + "x(" + salesReport.count + ")", salesReport.total, 35);
                zonerich.Prn_PrintText(printText + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
            }
        }

        zonerich.Prn_PrintText("------------------------------------------------" + "\n", PrinterConst.Alignment.LEFT, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText(footerA + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText(footerB + "\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT, PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_PrintText("\n", PrinterConst.Alignment.CENTER, PrinterConst.Font.DEFAULT,
                PrinterConst.WidthSize.SIZE0 | PrinterConst.HeightSize.SIZE0);
        zonerich.Prn_LineFeed(3);
        zonerich.Prn_CutPaper();
        disconnectPrinter();
    }

    public void setDays(String string) {
        switch (string) {
            case "0":
                getDays.append("Monday, ");
                break;
            case "1":
                getDays.append("Tuesday, ");
                break;
            case "2":
                getDays.append("Wednesday, ");
                break;
            case "3":
                getDays.append("Thursday, ");
                break;
            case "4":
                getDays.append("Friday, ");
                break;
            case "5":
                getDays.append("Saturday, ");
                break;
            case "6":
                getDays.append("Sunday, ");
                break;
        }
    }

    public void setOrders(String string) {
        switch (string) {
            case "1":
                getOrderType.append("Dine In, ");
                break;
            case "2":
                getOrderType.append("Collection, ");
                break;
            case "3":
                getOrderType.append("Delivery, ");
                break;
            case "4":
                getOrderType.append("Bar, ");
                break;
            case "5":
                getOrderType.append("Waiting, ");
                break;
            case "6":
                getOrderType.append("Web Order, ");
                break;
        }
    }
}
