package com.example.epos_2021.daos;

import androidx.room.Database;
import androidx.room.RoomDatabase;
import androidx.room.migration.Migration;
import androidx.sqlite.db.SupportSQLiteDatabase;

import com.example.epos_2021.models.Addon;
import com.example.epos_2021.models.CallLogs;
import com.example.epos_2021.models.Category;
import com.example.epos_2021.models.Customer;
import com.example.epos_2021.models.Discount;
import com.example.epos_2021.models.Floor;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.OrderItemAddon;
import com.example.epos_2021.models.OrderItemIngredient;
import com.example.epos_2021.models.OrderPayment;
import com.example.epos_2021.models.OrderSplit;
import com.example.epos_2021.models.OrderStatuses;
import com.example.epos_2021.models.PaymentMethod;
import com.example.epos_2021.models.PrepLocation;
import com.example.epos_2021.models.PrintBlock;
import com.example.epos_2021.models.Product;
import com.example.epos_2021.models.ProductAddon;
import com.example.epos_2021.models.ProductIngredient;
import com.example.epos_2021.models.Reservation;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.models.Table;
import com.example.epos_2021.models.TableStatus;
import com.example.epos_2021.models.User;
import com.example.epos_2021.models.Voucher;

/**
 * Created by Amrish on 19-05-2021.
 */

@Database(entities =
        {
                User.class,
                Order.class,
                Customer.class,
                OrderItem.class,
                OrderItemAddon.class,
                OrderItemIngredient.class,
                OrderSplit.class,
                OrderPayment.class,
                SiteSetting.class,

                Addon.class,
                Product.class,
                ProductAddon.class,
                ProductIngredient.class,
                Category.class,
                Floor.class,
                Table.class,
                TableStatus.class,
                PrepLocation.class,
                PaymentMethod.class,
                PrintBlock.class,
                CallLogs.class,
                Reservation.class,
                Discount.class,
                Voucher.class,

                OrderStatuses.class,
        },
        version = 15, exportSchema = false)

public abstract class AppDatabase extends RoomDatabase {
    public abstract CustomerDao customerDao();

    public abstract OrderDao orderDao();

    public abstract OrderItemDao orderItemDao();

    public abstract OrderItemAddonDao orderItemAddonDao();

    public abstract OrderItemIngredientDao orderItemIngredientDao();

    public abstract OrderSplitDao orderSplitDao();

    public abstract OrderPaymentDao orderPaymentDao();

    public abstract SiteSettingDao siteSettingDao();

    public abstract AddonDao addonDao();

    public abstract ProductDao productDao();

    public abstract ProductAddonDao productAddonDao();

    public abstract ProductIngredientDao productIngredientDao();

    public abstract PrepLocationDao prepLocationDao();

    public abstract VoucherDao voucherDao();

    public abstract CategoryDao categoryDao();

    public abstract FloorDao floorDao();

    public abstract TableDao tableDao();

    public abstract TableStatusDao tableStatusDao();

    public abstract UserDao userDao();

    public abstract PaymentMethodDao paymentMethodDao();

    public abstract EmptyPulledDao emptyPulledDao();

    public abstract PrintBlockDao printBlockDao();

    public abstract CalllogDao calllogDao();

    public abstract ReservationDao reservationDao();

    public abstract DiscountDao discountDao();

    public abstract ReportDao reportDao();

    public abstract OrderStatuesDao orderStatuesDao();

    public static final Migration MIGRATION_1_2 = new Migration(1, 2) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE Floor ADD sequence INT NOT NULL DEFAULT 0");
        }
    };
    public static final Migration MIGRATION_2_3 = new Migration(2, 3) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("CREATE TABLE IF NOT EXISTS `Discount` (`id` TEXT NOT NULL, `order_type_id` TEXT, `week_day` TEXT, `start_date` TEXT, `end_date` TEXT, `discount` REAL NOT NULL, `minimum_value` REAL NOT NULL, `discount_type` TEXT, PRIMARY KEY(`id`))");
        }
    };
    public static final Migration MIGRATION_3_4 = new Migration(3, 4) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE `Order` ADD is_archived INT NOT NULL DEFAULT 0");
        }
    };
    public static final Migration MIGRATION_4_5 = new Migration(4, 5) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE `Table` ADD last_order_total REAL NOT NULL DEFAULT 0");
            database.execSQL("ALTER TABLE `Table` ADD last_order_created_at TEXT");
        }
    };
    public static final Migration MIGRATION_5_6 = new Migration(5, 6) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE `Order` ADD updated_at TEXT");
        }
    };
    public static final Migration MIGRATION_6_7 = new Migration(6, 7) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE `Product` ADD auto_addon INT NOT NULL DEFAULT 0");
            database.execSQL("ALTER TABLE `Table` ADD locked INT NOT NULL DEFAULT 0");
        }
    };
    public static final Migration MIGRATION_7_8 = new Migration(7, 8) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
//            database.execSQL("CREATE TABLE IF NOT EXISTS `Voucher` (`_id` INTEGER PRIMARY KEY AUTOINCREMENT,`_customer_id` INTEGER NOT NULL, `id` TEXT NOT NULL, `customer_id` TEXT, `order_type_id` TEXT, `week_day` TEXT,`voucher_code` TEXT,`customer_name` TEXT,`customer_mobile` TEXT, `customer_email` TEXT,`start_date` TEXT, `end_date` TEXT, `voucher_amount` REAL NOT NULL, `minimum_order_value` REAL NOT NULL,uploadable INT NOT NULL DEFAULT 0)");
            database.execSQL("CREATE TABLE IF NOT EXISTS `Voucher` (" +
                    "`_id` INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT," +
                    "`_customer_id` INTEGER NOT NULL, " +
                    "`id` TEXT, `customer_id` TEXT, " +
                    "`order_type_id` TEXT, `week_day` TEXT," +
                    "`voucher_code` TEXT,`customer_name` TEXT," +
                    "`customer_mobile` TEXT, " +
                    "`customer_email` TEXT," +
                    "`start_date` TEXT, " +
                    "`end_date` TEXT, " +
                    "`voucher_amount` REAL NOT NULL, " +
                    "`minimum_order_value` REAL NOT NULL" +
                    ")"
            );
        }
    };
    public static final Migration MIGRATION_8_9 = new Migration(8, 9) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
//            database.execSQL("ALTER TABLE `Voucher` ADD uploadable INTEGER NOT NULL DEFAULT 0");
        }
    };
    public static final Migration MIGRATION_9_10 = new Migration(9, 10) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE `Voucher` ADD uploadable INT NOT NULL DEFAULT 0");
        }
    };
    public static final Migration MIGRATION_10_11 = new Migration(10, 11) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE `Voucher` ADD created_at TEXT");
            database.execSQL("ALTER TABLE `Voucher` ADD updated_at TEXT");
        }
    };
    public static final Migration MIGRATION_11_12 = new Migration(11, 12) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE `Voucher` ADD disabled INT NOT NULL DEFAULT 0");
        }
    };
    public static final Migration MIGRATION_12_13 = new Migration(12, 13) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("CREATE TABLE IF NOT EXISTS `OrderStatuses` (" +
                    "`_id` INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT," +
                    "`id` TEXT, `status` TEXT, " +
                    "`top_color` TEXT, `bottom_color` TEXT," +
                    "`created_at` TEXT"+
                    ")"
                );
        }
    };
    public static final Migration MIGRATION_13_14 = new Migration(13, 14) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE `Product` ADD collection_preparation_location_id TEXT");
            database.execSQL("ALTER TABLE `Product` ADD delivery_preparation_location_id TEXT");
            database.execSQL("ALTER TABLE `Product` ADD banquet_preparation_location_id TEXT");
        }
    };
    public static final Migration MIGRATION_14_15 = new Migration(14, 15) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("ALTER TABLE `Order` ADD print_id TEXT");
        }
    };
}