package com.example.epos_2021.fragment;

import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import com.example.epos_2021.R;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.google.android.material.button.MaterialButton;

public class AddCommentDialogFragment extends DialogFragment {

    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private DialogDismissListener dialogDismissListener;

    public static AddCommentDialogFragment getInstance(String comment) {
        AddCommentDialogFragment fragment = new AddCommentDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("comment", comment);
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    private EditText etComment;
    private TextView tvWordCounter, tvTitle;
    private MaterialButton btnCancel, btnUpdate;
    String comment;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_add_comment_dialog, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null)
            comment = getArguments().getString("comment");
        initViews(view);
        setListeners();

    }


    private void initViews(View view) {

        tvTitle = view.findViewById(R.id.tvTitle);
        etComment = view.findViewById(R.id.etComment);
        btnCancel = view.findViewById(R.id.btnCancel);
        btnUpdate = view.findViewById(R.id.btnConfirm);
        tvWordCounter = view.findViewById(R.id.tvWordCounter);

        tvTitle.setText("Add Instructions");
        etComment.setHint("Enter Special Instructions");

        if (!Validators.isNullOrEmpty(comment)) {
            etComment.setText(comment);
            etComment.setSelection(etComment.getText().toString().length());
            btnUpdate.setText("Update");
        } else {

            btnUpdate.setText("Confirm");
        }
    }


    void setListeners() {
        btnCancel.setOnClickListener(view -> dismiss());

        btnUpdate.setOnClickListener(view -> {
            if (btnUpdate.getText().toString().equalsIgnoreCase("Confirm")) {
                if (Validators.isNullOrEmpty(etComment.getText().toString())) {
                    etComment.setError("Please enter comment");
                    etComment.requestFocus();
                } else {
                    if (dialogDismissListener != null)
                        dialogDismissListener.onDialogDismiss(etComment.getText().toString());
                    dismiss();
                }
            } else if (btnUpdate.getText().toString().equalsIgnoreCase("Update")) {
                if (dialogDismissListener != null)
                    dialogDismissListener.onDialogDismiss(etComment.getText().toString());
                dismiss();
            }
        });
        etComment.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }

            @Override
            public void afterTextChanged(Editable editable) {

                int length = etComment.length();
                tvWordCounter.setText(length + "/300");
                if (length > 300) {
                    tvWordCounter.setTextColor(getResources().getColor(R.color.red));
                } else {
                    tvWordCounter.setTextColor(getResources().getColor(R.color.black));
                }
            }
        });

    }


}