package com.example.epos_2021.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.DialogFragment;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.models.CallLogs;
import com.example.epos_2021.models.Customer;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.SwipeDismissTouchListener;
import com.example.epos_2021.utils.ToastUtils;
import com.google.gson.Gson;

import java.util.ArrayList;

public class CallerIdDialogFragment extends DialogFragment {

    private TextView tvCallerNumber;
    private ImageView imgCancel,imgAccept,imgHold;
    private CallLogs callLogs;
    private AlertDialog progressBarDialog;

    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private DialogDismissListener dialogDismissListener;

    public static CallerIdDialogFragment getInstance(CallLogs callLogs) {
        CallerIdDialogFragment fragment = new CallerIdDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("calllogs", new Gson().toJson(callLogs));
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_caller_id_dialog, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            callLogs = new Gson().fromJson(getArguments().getString("calllogs"), CallLogs.class);
        }
        initView(view);
        setListener();
    }

    private void initView(View view){
        progressBarDialog = CommonFunctions.customProgressDialog(getActivity());

        imgAccept = view.findViewById(R.id.imgAccept);
        imgCancel = view.findViewById(R.id.imgCancel);
        imgHold = view.findViewById(R.id.imgHold);
        tvCallerNumber = view.findViewById(R.id.tvCallerNumber);

        tvCallerNumber.setText(callLogs.number);
    }

    private void setListener(){
        imgAccept.setOnClickListener(v -> changeStatusCallerid("accepted"));
        imgCancel.setOnClickListener(v -> changeStatusCallerid("rejected"));
        imgHold.setOnClickListener(v -> changeStatusCallerid("parked"));
    }

    @Override
    public void onResume() {
        super.onResume();
        Window window = getDialog().getWindow();
        window.getDecorView().setOnTouchListener(new SwipeDismissTouchListener(window.getDecorView(), null, new SwipeDismissTouchListener.DismissCallbacks() {
            @Override
            public boolean canDismiss(Object token) {
                return true;
            }

            @Override
            public void onDismiss(View view, Object token) {
                dismiss();
            }
        }));
    }

    private void changeStatusCallerid(String status) {
        callLogs.status = status;

        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                progressBarDialog.show();
            });
        }

        AndroidNetworking.post(ApiEndPoints.calllogs+callLogs.id)
                .addApplicationJsonBody(callLogs)
                .build()
                .getAsObject(CallLogs.class, new ParsedRequestListener<CallLogs>() {
                    @Override
                    public void onResponse(CallLogs response) {
                        if (response.status.equalsIgnoreCase("accepted")){
                            createOrderOffline();
                        }else {
                            dismiss();
                        }
                    }
                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        } else {
                            anError.printStackTrace();
                            ToastUtils.makeSnackToast(getActivity(), "Something went wrong!");
                        }
                    }
                });

    }

    private void createOrderOffline() {
        Order order = new Order();
        Customer customer;
//        if (selectedCustomer != null) {
//            customer = selectedCustomer;
//        } else {
//        }
        customer = new Customer();
        customer.name = "";
        customer.mobile = callLogs.number;
        customer.email = "";

        //Sumeet changes...
        try{
            customer.street=callLogs.customer.street;
            customer.city=callLogs.customer.city;
        }catch (Exception e){
            e.printStackTrace();
        }

        order.customer_id = customer.id;
        order.customer = customer;
        order.updater_id = MyApp.getInstance().myPreferences.getLoggedInUser().id;
        order.order_status_id = "1";
        order.order_type_id = "5";
        order.order_items = new ArrayList<>();
        order.device_id = MyApp.getInstance().myPreferences.getRegisteredDevice().id;
        order.created_at = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
        order.delivery_date = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
        new Thread(() -> {
            // Save Customer
            Customer dbCustomer;
            if (customer._id == 0)
                dbCustomer = MyApp.getInstance().appDatabase.customerDao().view(customer.id);
            else dbCustomer = MyApp.getInstance().appDatabase.customerDao().view(customer._id);
            if (dbCustomer == null) {
                order._customer_id = (int) MyApp.getInstance().appDatabase.customerDao().insert(customer);
            } else {
                order._customer_id = dbCustomer._id;
            }
            // Save Order
            order._id = (int) MyApp.getInstance().appDatabase.orderDao().insert(order);
            if (MyApp.getInstance().isConnected(getActivity())) {
                createOrderOnline(order);
            } else {
                dialogDismissListener.onDialogDismiss(order);
                dismiss();
            }

        }).start();
    }

    private void createOrderOnline(Order order) {

        AndroidNetworking.post(ApiEndPoints.orders)
                .addApplicationJsonBody(order)
                .build()
                .getAsObject(Order.class, new ParsedRequestListener<Order>() {
                    @Override
                    public void onResponse(Order response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                progressBarDialog.dismiss();
                            });
                        }
                        MyApp.getInstance().setOrderItems(response.order_items);
                        new Thread(() -> {
                            // Save Customer
                            Customer dbCustomer;
                            if (order._customer_id != 0) {
                                dbCustomer = MyApp.getInstance().appDatabase.customerDao().view(order._customer_id);
                            } else {
                                dbCustomer = MyApp.getInstance().appDatabase.customerDao().view(response.customer_id);
                            }

                            if (dbCustomer == null) {
                                response._customer_id = (int) MyApp.getInstance().appDatabase.customerDao().insert(response.customer);
                            } else {
                                response._customer_id = dbCustomer._id;
                                response.customer._id = dbCustomer._id;
                                MyApp.getInstance().appDatabase.customerDao().update(response.customer);
                            }
                            // Save Order
                            response._id = (int) MyApp.getInstance().appDatabase.orderDao().insert(response);
                            dialogDismissListener.onDialogDismiss(response);
                            dismiss();
                        }).start();
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                progressBarDialog.show();
                            });
                        }
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        }
                    }
                });

    }

}