package com.example.epos_2021.fragment;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.RadioGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.daos.relations.OrderWithItems;
import com.example.epos_2021.models.Customer;
import com.example.epos_2021.models.Discount;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderSplit;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;

import java.util.List;
import java.util.concurrent.Callable;

public class CustomerDetailFragment extends BaseFragment {

    private TextView tvNameTitle, tvEmailTitle, tvAddressTitle, tvMobileTitle;
    private LinearLayout llMainLayout;
    private LinearLayout llCustomerDetails;
    private RadioGroup rgOrderType;
    private LinearLayout llSubTotal, llDiscount, llGratuity, llServiceCharge, llDelivery, llGrandTotal;
    private TextView tvGratuity, tvServiceCharge, tvDelivery, tvSubtotal, tvDiscount, tvGrandTotal, tvBalance;
    private MaterialButton btnSelectPayment;
    private TextView tvGratuityText, tvServiceChargeText, tvDiscountText;

    private TextView tvOrderId, tvCustomerName, tvEmail, tvAddress, tvMobile;
    private Order order;
    private int _order_id, _order_split_id;

    Float paidAmount, orderTotal, remainingBalance;
    private float cartGrandTotal = 0, cartSubTotal = 0, cartDiscount = 0, cartGratuity = 0, cartServiceCharge = 0;
    private boolean serviceChargeAutoMode = true, serviceChargeEnabled = false, discountEnabled = true, gratuityEnabled = true, autoDiscountApplied = false;
    private SiteSetting serviceChargeSetting, serviceChargeTypeSetting, serviceChargeEnableSetting, serviceChargeOptionSetting;

    public static CustomerDetailFragment getInstance(int _order_id) {
        CustomerDetailFragment detailFragment = new CustomerDetailFragment();
        Bundle bundle = new Bundle();
        bundle.putInt("_order_id", _order_id);
        detailFragment.setArguments(bundle);
        return detailFragment;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_customer_details, container, false);
    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getArguments() != null) {
            _order_id = getArguments().getInt("_order_id");
            cartDiscount = getArguments().getFloat("cart_discount");
            _order_split_id = getArguments().getInt("_order_split_id");
            paidAmount = getArguments().getFloat("paid_amount");
            orderTotal = getArguments().getFloat("order_total");
        }
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListerners();
        fetchOffline();
    }

    private void initViews(View view) {
        llMainLayout = view.findViewById(R.id.llMainLayout);
        llCustomerDetails = view.findViewById(R.id.llCustomerDetails);
        rgOrderType = view.findViewById(R.id.rgOrderType);
        tvOrderId = view.findViewById(R.id.tvOrderId);
        tvCustomerName = view.findViewById(R.id.tvCustomerName);
        tvEmail = view.findViewById(R.id.tvCustomerEmail);
        tvAddress = view.findViewById(R.id.tvAddress);
        tvMobile = view.findViewById(R.id.tvCustomerMobile);

        tvNameTitle = view.findViewById(R.id.tvNameTitle);
        tvEmailTitle = view.findViewById(R.id.tvEmailTitle);
        tvAddressTitle = view.findViewById(R.id.tvAddressTitle);
        tvMobileTitle = view.findViewById(R.id.tvMobileTitle);

        llSubTotal = view.findViewById(R.id.ll_SubTotal);
        llDelivery = view.findViewById(R.id.ll_Delivery);
        llGratuity = view.findViewById(R.id.ll_Gratuity);
        llServiceCharge = view.findViewById(R.id.ll_ServiceCharge);
        llDiscount = view.findViewById(R.id.ll_Discount);
        llGrandTotal = view.findViewById(R.id.llGrandTotal);

        tvSubtotal = view.findViewById(R.id.tvSubtotal);
        tvDelivery = view.findViewById(R.id.tvDelivery);
        tvGratuity = view.findViewById(R.id.tvGratuity);
        tvServiceCharge = view.findViewById(R.id.tvServiceFee);
        tvDiscount = view.findViewById(R.id.tvDiscount);
        tvGrandTotal = view.findViewById(R.id.tvGrandTotal);
        tvBalance = view.findViewById(R.id.tvBalance);
        btnSelectPayment = view.findViewById(R.id.btnSelectPayment);
        tvDiscountText = view.findViewById(R.id.tvDiscountText);
        tvServiceChargeText = view.findViewById(R.id.tvServiceText);
        tvGratuityText = view.findViewById(R.id.tvGratuityText);

        serviceChargeEnableSetting = myApp.findSetting("is_service_charge");
        serviceChargeSetting = myApp.findSetting("service_charge");
        serviceChargeOptionSetting = myApp.findSetting("service_charge_type_option");
        serviceChargeTypeSetting = myApp.findSetting("service_charge_type");

        if (serviceChargeEnableSetting != null && (serviceChargeEnableSetting.value.equalsIgnoreCase("true") || serviceChargeEnableSetting.value.equalsIgnoreCase("1"))) {
            serviceChargeEnabled = true;
        }
        if (serviceChargeEnabled && serviceChargeOptionSetting != null && serviceChargeOptionSetting.value.equalsIgnoreCase("manual")) {
            serviceChargeAutoMode = false;
        }
        tvNameTitle.setVisibility(View.GONE);
        tvCustomerName.setVisibility(View.GONE);

        tvEmailTitle.setVisibility(View.GONE);
        tvEmail.setVisibility(View.GONE);

        tvAddressTitle.setVisibility(View.GONE);
        tvAddress.setVisibility(View.GONE);

        tvMobileTitle.setVisibility(View.GONE);
        tvMobile.setVisibility(View.GONE);
    }

    private void updateViews() {
        if (order != null) {
            if (order.customer != null) {
                Customer customer = order.customer;
                llCustomerDetails.setVisibility(View.VISIBLE);

                tvOrderId.setText("ORD00" + order._id);
                if (!Validators.isNullOrEmpty(customer.name)) {
                    tvCustomerName.setText(customer.name);
                    tvCustomerName.setVisibility(View.VISIBLE);
                    tvNameTitle.setVisibility(View.VISIBLE);
                } else {
                    tvNameTitle.setVisibility(View.GONE);
                    tvCustomerName.setVisibility(View.GONE);
                }
                if (!Validators.isNullOrEmpty(customer.email)) {
                    tvEmail.setText(customer.email);
                    tvEmail.setVisibility(View.VISIBLE);
                    tvEmailTitle.setVisibility(View.VISIBLE);
                } else {
                    tvEmail.setVisibility(View.GONE);
                    tvEmailTitle.setVisibility(View.GONE);
                }
                if (!Validators.isNullOrEmpty(customer.mobile)) {
                    tvMobile.setText(customer.mobile);
                    tvMobile.setVisibility(View.VISIBLE);
                    tvMobileTitle.setVisibility(View.VISIBLE);
                } else {
                    tvMobile.setVisibility(View.GONE);
                    tvMobileTitle.setVisibility(View.GONE);
                }
                if (order.order_type_id.equalsIgnoreCase("3") || (order.order_type_id.equalsIgnoreCase("1") && order.table_number != null)) {
                    tvAddress.setVisibility(View.VISIBLE);
                    tvAddressTitle.setVisibility(View.VISIBLE);
                    StringBuilder customerDetails = new StringBuilder();
                    if (order.order_type_id.equalsIgnoreCase("3")) {
                        StringBuilder addressBuilder = new StringBuilder();
                        if (!Validators.isNullOrEmpty(customer.house_no)) {
                            addressBuilder.append(customer.house_no);
                        }
                        if (!Validators.isNullOrEmpty(customer.street)) {
                            if (addressBuilder.toString().length() > 0) {
                                addressBuilder.append(", ");
                            }
                            addressBuilder.append(customer.street);
                        }
                        if (!Validators.isNullOrEmpty(customer.city)) {
                            if (addressBuilder.toString().length() > 0) {
                                addressBuilder.append(", ");
                            }
                            addressBuilder.append(customer.city);
                        }
                        if (!Validators.isNullOrEmpty(customer.postcode)) {
                            if (addressBuilder.toString().length() > 0) {
                                addressBuilder.append(", ");
                            }
                            addressBuilder.append(customer.postcode);
                        }
                        customerDetails.append(addressBuilder.toString());
                    } else if (order.order_type_id.equalsIgnoreCase("1") && order.table_number != null) {
                        tvAddressTitle.setText("Table details");
                        customerDetails = new StringBuilder();
                        customerDetails.append("Table No: ");
                        customerDetails.append(order.table_number);
                        customerDetails.append("\n");
                        customerDetails.append("No of guests: ");
                        customerDetails.append(order.no_guest);
                    }
                    tvAddress.setText(customerDetails.toString());
                } else {
                    tvAddress.setVisibility(View.GONE);
                    tvAddressTitle.setVisibility(View.GONE);
                }
            }

            order.total = ((order.sub_total + order.gratuity + order.service_charge + order.delivery_charge) - order.discount);

            // cartGrandTotal = order.sub_total + order.gratuity - order.discount + order.service_charge + order.delivery_charge;

            order.total = cartGrandTotal;
            tvBalance.setText("Balance Remaining: " + MyApp.currencySymbol + MyApp.df.format(remainingBalance));
            tvSubtotal.setText(MyApp.currencySymbol + MyApp.df.format(order.sub_total));
            tvGrandTotal.setText(MyApp.currencySymbol + MyApp.df.format(cartGrandTotal));

            if (order.discount > 0) {
                llDiscount.setVisibility(View.VISIBLE);
                tvDiscount.setText(MyApp.currencySymbol + MyApp.df.format(order.discount));
                tvDiscountText.setCompoundDrawablesWithIntrinsicBounds(R.drawable.ic_baseline_green_close_18, 0, 0, 0);
            } else {
                llDiscount.setVisibility(View.GONE);
            }
            if (order.gratuity > 0) {
                llGratuity.setVisibility(View.VISIBLE);
                tvGratuity.setText(MyApp.currencySymbol + MyApp.df.format(order.gratuity));
                tvGratuityText.setCompoundDrawablesWithIntrinsicBounds(R.drawable.ic_baseline_green_close_18, 0, 0, 0);

            } else {
                llGratuity.setVisibility(View.GONE);
            }
            if (order.service_charge > 0) {
                llServiceCharge.setVisibility(View.VISIBLE);
                tvServiceCharge.setText(MyApp.currencySymbol + MyApp.df.format(order.service_charge));
                tvServiceChargeText.setCompoundDrawablesWithIntrinsicBounds(R.drawable.ic_baseline_green_close_18, 0, 0, 0);

            } else {
                llServiceCharge.setVisibility(View.GONE);
            }
            if (order.delivery_charge > 0) {
                llDelivery.setVisibility(View.VISIBLE);
                tvDelivery.setText(MyApp.currencySymbol + MyApp.df.format(order.delivery_charge));
            } else {
                llDelivery.setVisibility(View.GONE);
            }
            btnSelectPayment.setEnabled(remainingBalance > 0);
        }
    }

    private void setListerners() {
        btnSelectPayment.setOnClickListener(v -> {
            changeFragment(new PaymentMethodSelectionFragment());
        });
        tvDiscountText.setOnClickListener(v -> {
            cartDiscount = 0;
            new UpdateOrderAsyncTask(() -> {
                fetchOffline();
                ToastUtils.showSnackBar(getActivity(), llMainLayout, "Discount has been removed", "remove");
                return null;
            }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
        });
        tvServiceChargeText.setOnClickListener(v -> {
            cartServiceCharge = 0;
            new UpdateOrderAsyncTask(() -> {
                fetchOffline();
                ToastUtils.showSnackBar(getActivity(), llMainLayout, "Service charge has been removed", "remove");
                return null;
            }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
        });

        tvGratuityText.setOnClickListener(v -> {
            cartGratuity = 0;
            new UpdateOrderAsyncTask(() -> {
                fetchOffline();
                ToastUtils.showSnackBar(getActivity(), llMainLayout, "Gratuity has been removed", "remove");
                return null;
            }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
        });

    }

    private class UpdateOrderAsyncTask extends AsyncTask<Object, Object, Object> {
        Callable<Void> nextMethod;

        public UpdateOrderAsyncTask(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected Object doInBackground(Object... objects) {
            order.sub_total = myApp.appDatabase.orderDao().getItemSubTotal(order._id);

            cartSubTotal = order.sub_total;
            if (serviceChargeEnabled) {
                if (serviceChargeAutoMode) {
                    float serviceChargeValue = Float.parseFloat(serviceChargeSetting.value);
                    if (serviceChargeTypeSetting.value.equalsIgnoreCase("flat")) {
                        cartServiceCharge = serviceChargeValue;
                    } else {
                        if (serviceChargeValue > 0) {
                            cartServiceCharge = cartSubTotal * serviceChargeValue / 100;
                        } else {
                            cartServiceCharge = 0;
                        }
                    }
                }
            }

            if (discountEnabled) {
                List<Discount> discounts = appDatabase.discountDao().eligibleDiscounts("%" + myApp.current_day + "%", "%" + order.order_type_id + "%", order.sub_total, CommonFunctions.getCurrentTimeFormatted("yyyy-MM-dd"), CommonFunctions.getCurrentTimeFormatted("yyyy-MM-dd"));
                if (discounts.size() > 0) {
                    Discount appliedDiscount = discounts.get(0);
                    autoDiscountApplied = false;
                    if (!Validators.isNullOrEmpty(appliedDiscount.discount_type) && appliedDiscount.discount_type.equalsIgnoreCase("flat")) {
                        if (cartSubTotal > 0)
                            cartDiscount = appliedDiscount.discount;
                        else cartDiscount = 0;
                        autoDiscountApplied = true;
                    }
                    if (!Validators.isNullOrEmpty(appliedDiscount.discount_type) && appliedDiscount.discount_type.equalsIgnoreCase("percentage")) {
                        if (cartSubTotal > 0)
                            cartDiscount = cartSubTotal * appliedDiscount.discount / 100;
                        else cartDiscount = 0;
                        autoDiscountApplied = true;
                    }
                } else {
                    if (autoDiscountApplied) cartDiscount = 0;
                }
            }

            order.gratuity = cartGratuity;
            order.discount = cartDiscount;
            order.service_charge = cartServiceCharge;
            cartGrandTotal = cartSubTotal + cartGratuity - cartDiscount + cartServiceCharge + order.delivery_charge;
            order.total = cartGrandTotal;
            if (!order.order_status_id.equalsIgnoreCase("5") && !order.order_status_id.equalsIgnoreCase("10")) {
                order.updated_at = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
            }
            appDatabase.orderDao().update(order);
            return null;
        }

        @Override
        protected void onPostExecute(Object s) {
            super.onPostExecute(s);
            if (nextMethod != null) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private void fetchOffline() {
        new FetchOrderOfflineAsyncTask(() -> {
            updateViews();
            return null;
        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private class FetchOrderOfflineAsyncTask extends AsyncTask<Object, Object, Object> {
        Callable<Void> nextMethod;

        public FetchOrderOfflineAsyncTask(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected Object doInBackground(Object... objects) {
            OrderWithItems orderWithItems = myApp.appDatabase.orderDao().orderWithItems(_order_id);
            if (orderWithItems != null) {
                order = orderWithItems.order;
                order.customer = appDatabase.customerDao().view(order._customer_id);
                if (_order_split_id > 0) {
                    OrderSplit orderSplit = appDatabase.orderSplitDao().view(_order_split_id);
                    orderTotal = orderSplit.total;
                    paidAmount = appDatabase.orderPaymentDao().getSplitTotalPaidAmount(_order_id, _order_split_id);
                } else {
                    orderTotal = order.total;
                    paidAmount = order.total_paid;
                }
                remainingBalance = orderTotal - paidAmount;

                if (order.service_charge == 0) {
                    if (serviceChargeEnabled) {
                        if (serviceChargeAutoMode) {
                            float serviceChargeValue = Float.parseFloat(serviceChargeSetting.value);
                            if (serviceChargeTypeSetting.value.equalsIgnoreCase("flat")) {
                                cartServiceCharge = serviceChargeValue;
                            } else {
                                if (serviceChargeValue > 0) {
                                    cartServiceCharge = cartSubTotal * serviceChargeValue / 100;
                                } else {
                                    cartServiceCharge = 0;
                                }
                            }
                        }
                    } else {
                        cartServiceCharge = 0;
                    }
                } else {
                    cartServiceCharge = order.service_charge;
                }
                if (order.split_type != null && order.split_type.equalsIgnoreCase("order_item")) {
                    cartServiceCharge = 0;
                    serviceChargeEnabled = false;
                    discountEnabled = false;
                    gratuityEnabled = false;
                }

            }
            return null;
        }

        @Override
        protected void onPostExecute(Object s) {
            super.onPostExecute(s);
            if (order != null)
                if (nextMethod != null) {
                    try {
                        nextMethod.call();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
        }
    }

    private void changeFragment(Fragment fragment) {
        FragmentManager manager = getActivity().getSupportFragmentManager();
        if (getArguments() != null) {
            fragment.setArguments(getArguments());
        }
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.flContainer, fragment);
        transaction.addToBackStack("can_go_back");
        transaction.commitAllowingStateLoss();
    }


    @Override
    public void onResume() {
        super.onResume();
        LocalBroadcastManager.getInstance(getActivity()).registerReceiver(eposOrderUpdateReceiver, new IntentFilter(Constants.EPOS_ORDER_UPDATE));
    }

    @Override
    public void onStop() {
        super.onStop();
        if (eposOrderUpdateReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(eposOrderUpdateReceiver);
    }

    BroadcastReceiver eposOrderUpdateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getBooleanExtra("refresh", false)) {
                if (intent.hasExtra("_order_split_id")) {
                    _order_split_id = intent.getIntExtra("_order_split_id", 0);
                    getArguments().putInt("_order_split_id", _order_split_id);
                }
                fetchOffline();
            }
        }
    };
}
