package com.example.epos_2021.fragment;

import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.example.epos_2021.R;
import com.example.epos_2021.adapters.CustomersAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.models.Customer;
import com.google.android.material.button.MaterialButton;
import com.google.android.material.chip.Chip;

import java.util.ArrayList;


public class CustomersFragment extends BaseFragment {
    public static CustomersFragment getInstance() {
        return new CustomersFragment();
    }

    private RecyclerView rvCustomers;
    private SwipeRefreshLayout swipeRefreshLayout;
    private EditText etSearch;
    private MaterialButton tvSearch;
    private ArrayList<Customer> customers = new ArrayList<>();
    private CustomersAdapter customersAdapter;
    private Chip chipBack;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_customers, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        new FetchCustomersAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
        tvSearch.setOnClickListener(v -> {
            new FetchCustomersAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, etSearch.getText().toString());
        });
        swipeRefreshLayout.setOnRefreshListener(() -> {
            new FetchCustomersAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, etSearch.getText().toString());
        });

        etSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }

            @Override
            public void afterTextChanged(Editable editable) {
                new FetchCustomersAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, etSearch.getText().toString());
                CommonFunctions.hideKeyboard(getActivity(), etSearch);
            }
        });
    }

    private void initViews(View view) {
        chipBack = view.findViewById(R.id.chipBack);
        rvCustomers = view.findViewById(R.id.rvCustomers);
        etSearch = view.findViewById(R.id.etSearch);
        tvSearch = view.findViewById(R.id.btnSearch);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        customersAdapter = new CustomersAdapter(customers, (position, data) -> {
            CustomerInfoDialogFragment customerInfoDialogFragment = CustomerInfoDialogFragment.getInstance((Customer) data, true);
            customerInfoDialogFragment.show(getChildFragmentManager(), "edit");
            customerInfoDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof Customer) {
                    Customer c = (Customer) data;
                    Customer customer = (Customer) o;
                    customer._id = c._id;
                    customer.id = c.id;
                    etSearch.setText(null);
                    new Thread(() -> {
                        appDatabase.customerDao().update(customer);
                        new FetchCustomersAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);

                    }).start();
                }
            });
        }, (position, data) -> {
            ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Delete customer?", "Are you sure you want to delete customer", 2, "Confirm", "Cancel");
            confirmationDialogFragment.show(getChildFragmentManager(), "confirm");
            confirmationDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String && ((String) o).equalsIgnoreCase("confirm")) {
                    checkAndDeleteCustomer((Customer) data);
                }
            });
        });
        rvCustomers.setLayoutManager(new LinearLayoutManager(getActivity()));
        rvCustomers.setAdapter(customersAdapter);

        chipBack.setOnClickListener(view1 -> getActivity().onBackPressed());
    }

    private void checkAndDeleteCustomer(Customer customer) {

        new Thread(() -> {
            int totalOrders = appDatabase.orderDao().customersTotalOrder(customer._id);
            int totalReservations = appDatabase.reservationDao().customersTotalReservation(customer._id);
            if (totalOrders == 0 && totalReservations == 0) {
                appDatabase.customerDao().delete(customer._id);
            } else {
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Cannot Delete", "This customer have already orders/reservations", 2, "Okay", null);
                        confirmationDialogFragment.show(getChildFragmentManager(), "confirm");
                    });
                }
            }
        }).start();
    }

    class FetchCustomersAsync extends AsyncTask<String, String, String> {
        private ArrayList<Customer> customerArrayList = new ArrayList<>();

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            if (getActivity() != null) getActivity().runOnUiThread(() -> {
                swipeRefreshLayout.setRefreshing(true);

            });
        }

        @Override
        protected String doInBackground(String... strings) {
            customerArrayList.clear();
            if (strings.length > 0 && !Validators.isNullOrEmpty(strings[0]))
                customerArrayList.addAll(appDatabase.customerDao().list("%" + strings[0] + "%"));
            else customerArrayList.addAll(appDatabase.customerDao().list());
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            customers.clear();
            customers.addAll(customerArrayList);
            customersAdapter.notifyDataSetChanged();
            swipeRefreshLayout.setRefreshing(false);
        }
    }
}
