package com.example.epos_2021.fragment;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.viewpager.widget.ViewPager;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.activities.NewOrderActivity;
import com.example.epos_2021.adapters.FloorAdapter;
import com.example.epos_2021.adapters.ViewpagerTabAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.models.CallLogs;
import com.example.epos_2021.models.Floor;
import com.example.epos_2021.models.Order;
import com.google.android.material.button.MaterialButton;
import com.google.android.material.chip.Chip;
import com.google.gson.Gson;

import java.util.ArrayList;

public class DineInFragment extends BaseFragment {
    private ViewPager vpTables;
    private MaterialButton btnNewOrder;
    private ViewpagerTabAdapter viewpagerAdapter;
    private ArrayList<Floor> floors = new ArrayList<>();
    private RecyclerView rvFloors;
    private FloorAdapter floorAdapter;
    static AlertDialog progressDialog;
    private Chip chipBack;
    private LinearLayout llTitle;
    private boolean fromTheme2;

    public static DineInFragment getInstance() {
        return new DineInFragment();
    }

    public static DineInFragment getInstance(boolean fromTheme2) {
        DineInFragment dineInFragment = new DineInFragment();
        Bundle bundle = new Bundle();
        bundle.putBoolean("show_title", fromTheme2);
        dineInFragment.setArguments(bundle);
        return dineInFragment;
    }


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_dine_in, container, false);
    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getArguments() != null) {
            fromTheme2 = getArguments().getBoolean("show_title", true);
        }
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();
        new FloorsAsyncTask().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private class FloorsAsyncTask extends AsyncTask<String, String, String> {

        @Override
        protected String doInBackground(String... strings) {
            floors.clear();
            floors.addAll(appDatabase.floorDao().list());

            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            progressDialog.dismiss();
            if (floors != null && floors.size() > 0) {
                floorAdapter.selectedFloorId = floors.get(0).id;
                floorAdapter.notifyDataSetChanged();
                for (int i = 0; i < floors.size(); i++) {
                    viewpagerAdapter.addFragment(TablesFragment.getInstance(floors.get(i).id, fromTheme2), floors.get(i).name);
                }
                viewpagerAdapter.notifyDataSetChanged();
            }
        }
    }

    private void setListeners() {
        chipBack.setOnClickListener(v -> getActivity().onBackPressed());

        btnNewOrder.setOnClickListener(v -> {
            createNewOrder();
        });

        vpTables.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

            }

            @Override
            public void onPageSelected(int position) {
                // on page scroll change floor tables
                Floor selectedFloor = floors.get(position);
                floorAdapter.selectedFloorId = selectedFloor.id;
                floorAdapter.notifyDataSetChanged();

                rvFloors.scrollToPosition(position);
            }

            @Override
            public void onPageScrollStateChanged(int state) {

            }
        });

    }

    private void createNewOrder() {
        Fragment openedFragment = getActivity().getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment);
        String orderTypeId = null;
        if (openedFragment instanceof OrdersAllFragment) {
            orderTypeId = MyApp.openedOrderTypeId;
            if (Validators.isNullOrEmpty(MyApp.openedOrderTypeId) || MyApp.openedOrderTypeId.equalsIgnoreCase("0") || MyApp.openedOrderTypeId.equalsIgnoreCase("6")) {
                openOrderTypeSelectionDialog(null);
            } else {
                openNewOrderDialog(orderTypeId, null);
            }
        } else {
            openOrderTypeSelectionDialog(null);
        }
    }

    private void openOrderTypeSelectionDialog(CallLogs callLogs) {
        OrderTypeSelectionDialogFragment orderTypeSelectionDialog = OrderTypeSelectionDialogFragment.getInstance(callLogs);
        orderTypeSelectionDialog.show(getChildFragmentManager(), "order_type");
        orderTypeSelectionDialog.setDialogDismissListener(o -> {
            if (o instanceof String) {
                String orderTypeId = (String) o;
                openNewOrderDialog(orderTypeId, callLogs);
            } else if (o instanceof CallLogs) {
                changeFragment(NewReservationFragment.getInstance((CallLogs) o));
            }
        });
    }

    private void openNewOrderDialog(String orderTypeId, CallLogs callLogs) {
        NewOrderDialogFragment newOrderDialogFragment = NewOrderDialogFragment.getInstance(callLogs, orderTypeId);
        newOrderDialogFragment.show(getChildFragmentManager(), "new_order");
        newOrderDialogFragment.setDialogDismissListener(o -> {
            if (o instanceof Order) {
                startActivity(new Intent(getActivity(), NewOrderActivity.class).putExtra("order", new Gson().toJson(o)).putExtra("fetch_delivery_charge", ((Order) o).order_type_id.equalsIgnoreCase("3")));
            } else if (o instanceof CallLogs) {
                changeFragment(NewReservationFragment.getInstance((CallLogs) o));
            }
        });
    }

    private void changeFragment(Fragment fragment) {
        FragmentManager manager = getChildFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.nav_host_fragment, fragment);
        transaction.addToBackStack("can_go_back");
        transaction.commit();
    }

    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity());
        rvFloors = view.findViewById(R.id.rvFloors);
        vpTables = view.findViewById(R.id.vpTables);
        btnNewOrder = view.findViewById(R.id.btnNewOrder);
        chipBack = view.findViewById(R.id.chipBack);
        llTitle = view.findViewById(R.id.llTitle);

        if (fromTheme2) {
            llTitle.setVisibility(View.GONE);
            btnNewOrder.setVisibility(View.VISIBLE);
        } else {
            btnNewOrder.setVisibility(View.GONE);
            llTitle.setVisibility(View.VISIBLE);
        }
        floorAdapter = new FloorAdapter(floors, (position, data) -> {
            Floor selectedFloor = (Floor) data;
            vpTables.setCurrentItem(position);
            floorAdapter.selectedFloorId = selectedFloor.id;
            floorAdapter.notifyDataSetChanged();
        });
        rvFloors.setAdapter(floorAdapter);
        rvFloors.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false));
        viewpagerAdapter = new ViewpagerTabAdapter(getChildFragmentManager(), FragmentPagerAdapter.BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
        vpTables.setAdapter(viewpagerAdapter);


    }
}
