package com.example.epos_2021.fragment;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.example.epos_2021.R;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.MoneyTextWatcher;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.daos.relations.OrderWithItems;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.utils.Constants;
import com.google.android.material.button.MaterialButton;

import java.util.concurrent.Callable;

public class DiscountDialogFragment extends BaseFragment {

    private DialogDismissListener dialogDismissListener;

    public static DiscountDialogFragment getInstance(int _order_id, String title) {
        DiscountDialogFragment fragment = new DiscountDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putInt("_order_id", _order_id);
        bundle.putString("title", title);
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    private MaterialButton btn1, btn2, btn3, btn4, btn5, btn6, btn7, btn8, btn9, btn0, btnDot;
    private MaterialButton btnClear;
    private MaterialButton btnConfirm, btnCancel, btnFlat, btnPercentage;
    private EditText etAmount;
    private boolean flatAmountSelected = true;
    private boolean delPressed = false;
    private float orderTotal = 10.00f;
    private Order order;
    private int _order_id;
    private String title;
    private TextView tvTitle;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_discount_dialog, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            _order_id = getArguments().getInt("_order_id");
            title = getArguments().getString("title");

        }
        initViews(view);
        setListeners();
        fetchOffline();
    }

    private void initViews(View view) {
        tvTitle = view.findViewById(R.id.tvTitle);
        btnCancel = view.findViewById(R.id.btnCancel);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnClear = view.findViewById(R.id.btnClear);
        btn1 = view.findViewById(R.id.btn1);
        btn2 = view.findViewById(R.id.btn2);
        btn3 = view.findViewById(R.id.btn3);
        btn4 = view.findViewById(R.id.btn4);
        btn5 = view.findViewById(R.id.btn5);
        btn6 = view.findViewById(R.id.btn6);
        btn7 = view.findViewById(R.id.btn7);
        btn8 = view.findViewById(R.id.btn8);
        btn9 = view.findViewById(R.id.btn9);
        btn0 = view.findViewById(R.id.btn0);
        btnDot = view.findViewById(R.id.btnDot);
        etAmount = view.findViewById(R.id.etAmount);
        btnPercentage = view.findViewById(R.id.btnPercentage);
        btnFlat = view.findViewById(R.id.btnFlat);


        btnConfirm.setEnabled(false);
        btnDot.setVisibility(View.GONE);

        tvTitle.setText(title);



        if(title.equals("deliveryChargeType")){
            btnConfirm.setText("Apply Delivery Charge");

            btnPercentage.setVisibility(View.GONE);
            btnFlat.setVisibility(View.GONE);

            changeBtnColor(btnFlat);
            flatAmountSelected = true;
        }
        else{

            btnConfirm.setText("Apply " + title);

            changeBtnColor(btnPercentage);
            flatAmountSelected = false;
        }
    }

    private void setListeners() {

        btnPercentage.setOnClickListener(v -> {
            changeBtnColor(v);
            flatAmountSelected = false;
        });

        btnFlat.setOnClickListener(v -> {
            changeBtnColor(v);
            flatAmountSelected = true;
        });


        etAmount.addTextChangedListener(new MoneyTextWatcher(etAmount));

        etAmount.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                btnConfirm.setEnabled(!Validators.isNullOrEmpty(etAmount.getText().toString()) && !etAmount.getText().toString().equalsIgnoreCase("0.00"));
            }
        });
        btnCancel.setOnClickListener(view -> {
            getActivity().onBackPressed();
        });

        btnConfirm.setOnClickListener(view -> {
            float appliedAmount = Float.parseFloat(etAmount.getText().toString().replace(",", ""));

            if (!flatAmountSelected) {
                if (orderTotal > 0)
                    appliedAmount = appliedAmount * orderTotal / 100;
                else
                    appliedAmount = 0;
            }

            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss(appliedAmount);
            getActivity().onBackPressed();
        });

        btn1.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "1");
        });

        btn2.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "2");
        });

        btn3.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "3");
        });

        btn4.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "4");
        });

        btn5.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "5");
        });

        btn6.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "6");
        });

        btn7.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "7");
        });

        btn8.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "8");
        });

        btn9.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "9");
        });

        btn0.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + "0");
        });

        btnDot.setOnClickListener(v -> {
            etAmount.setText(etAmount.getText().toString() + ".");
        });

        btnClear.setOnClickListener(v -> {
            if (!flatAmountSelected)
                etAmount.setText("0.00");
            else etAmount.setText("0");
        });

    }

    private void changeBtnColor(View v) {
        btnPercentage.setBackgroundColor(getResources().getColor(R.color.orders_card_bg_grey));
        btnPercentage.setTextColor(getResources().getColor(R.color.reason_normal_text));

        btnFlat.setBackgroundColor(getResources().getColor(R.color.orders_card_bg_grey));
        btnFlat.setTextColor(getResources().getColor(R.color.reason_normal_text));

        if (v == btnPercentage) {
            btnPercentage.setBackgroundColor(getResources().getColor(R.color.persian_green));
            btnPercentage.setTextColor(getResources().getColor(R.color.white));
        }
        if (v == btnFlat) {
            btnFlat.setBackgroundColor(getResources().getColor(R.color.persian_green));
            btnFlat.setTextColor(getResources().getColor(R.color.white));
        }
    }

    private void fetchOffline() {
        new FetchOrderOfflineAsyncTask(() -> {
            orderTotal = order.sub_total;
            return null;
        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private class FetchOrderOfflineAsyncTask extends AsyncTask<Object, Object, Object> {
        Callable<Void> nextMethod;

        public FetchOrderOfflineAsyncTask(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected Object doInBackground(Object... objects) {
            OrderWithItems orderWithItems = myApp.appDatabase.orderDao().orderWithItems(_order_id);
            if (orderWithItems != null) {
                order = orderWithItems.order;
                order.customer = appDatabase.customerDao().view(order._customer_id);
            }
            return null;
        }

        @Override
        protected void onPostExecute(Object s) {
            super.onPostExecute(s);
            if (order != null)
                if (nextMethod != null) {
                    try {
                        nextMethod.call();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        LocalBroadcastManager.getInstance(getActivity()).registerReceiver(eposOrderUpdateReceiver, new IntentFilter(Constants.EPOS_ORDER_UPDATE));
    }

    @Override
    public void onStop() {
        super.onStop();
        if (eposOrderUpdateReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(eposOrderUpdateReceiver);
    }

    BroadcastReceiver eposOrderUpdateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getBooleanExtra("refresh", false)) {
                fetchOffline();
            }
        }
    };

}