package com.example.epos_2021.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.fragment.app.DialogFragment;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.MoneyTextWatcher;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.online.interfaces.DialogDismissListener;
import com.google.android.material.button.MaterialButton;


/**
 * Created by Amrish on 11-11-2020.
 */
public class EnterVoucherAmountDialogFragment extends DialogFragment {
    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private String forWhat, yes_no, title, info;
    private MaterialButton btnConfirm, btnCancel;
    private AppCompatEditText etAmount;
    private TextView tvInfo, tvTitle;
    private float value;
    private DialogDismissListener dialogDismissListener;
    private AlertDialog progressDialog;
    private MyApp myApp = MyApp.getInstance();


    public static EnterVoucherAmountDialogFragment getInstance(String for_what, String title, String info) {
        EnterVoucherAmountDialogFragment rejectReasonDialog = new EnterVoucherAmountDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("for_what", for_what);
        bundle.putString("title", title);
        bundle.putString("info", info);
        rejectReasonDialog.setArguments(bundle);
        return rejectReasonDialog;
    }

    public static EnterVoucherAmountDialogFragment getInstance(String for_what, String title, String info, float value) {
        EnterVoucherAmountDialogFragment rejectReasonDialog = new EnterVoucherAmountDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("for_what", for_what);
        bundle.putString("title", title);
        bundle.putString("info", info);
        bundle.putFloat("value", value);
        rejectReasonDialog.setArguments(bundle);
        return rejectReasonDialog;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.dialog_enter_voucher_amount, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            forWhat = getArguments().getString("for_what");
            yes_no = getArguments().getString("yes_no");
            info = getArguments().getString("info");
            title = getArguments().getString("title");
            value = getArguments().getFloat("value");
        }
        initViews(view);
        setListeners();
    }

    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "");
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnCancel = view.findViewById(R.id.btnCancel);
        etAmount = view.findViewById(R.id.etAmount);
        tvInfo = view.findViewById(R.id.tvInfo);
        tvTitle = view.findViewById(R.id.tvTitle);
        tvInfo.setText(info);
        tvTitle.setText(title);
        etAmount.setText(MyApp.df.format(value));
        etAmount.setSelection(etAmount.getText().toString().length());
    }

    private void setListeners() {
        etAmount.addTextChangedListener(new MoneyTextWatcher(etAmount));
        btnConfirm.setOnClickListener(view -> {
            etAmount.setError(null);
            if (!Validators.isNullOrEmpty(forWhat)) {
                if (forWhat.equalsIgnoreCase("voucher_amount")) {
                    if (etAmount.getText().toString().isEmpty()) {
                        etAmount.setError("Please enter amount");
                    } else {
                        if (dialogDismissListener != null)
                            dialogDismissListener.onDialogDismiss(etAmount.getText().toString().replace(",", ""));
                        CommonFunctions.hideKeyboard(getActivity(), etAmount);
                        dismiss();
                    }
                }
            }
        });
        btnCancel.setOnClickListener(view ->
        {
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss(null);
            dismiss();
        });
    }


}
