package com.example.epos_2021.fragment;

import android.content.Context;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatSpinner;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.sqlite.db.SimpleSQLiteQuery;
import androidx.sqlite.db.SupportSQLiteQuery;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.FullReportAdapter;
import com.example.epos_2021.adapters.SelectedUserAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.comman.printer.SunmiPrinter;
import com.example.epos_2021.comman.printer.WifiPrinter;
import com.example.epos_2021.comman.printer.ZoneRichPrinter;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.models.DaoReportNameTotalValue;
import com.example.epos_2021.models.FullReport;
import com.example.epos_2021.models.Printer;
import com.example.epos_2021.models.ReportOrderPaymentMethod;
import com.example.epos_2021.models.ReportOrderStatus;
import com.example.epos_2021.models.ReportOrderType;
import com.example.epos_2021.models.ReportTextModel;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.models.User;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;
import com.google.android.material.button.MaterialButtonToggleGroup;
import com.google.android.material.card.MaterialCardView;
import com.google.android.material.chip.Chip;
import com.wdullaer.materialdatetimepicker.date.DatePickerDialog;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

public class FullReportFragment extends BaseFragment {

    private Chip chipBack;
    private LinearLayout llToDate, llFromDate;
    private String fromDateString, toDateString;
    private Calendar today = Calendar.getInstance();
    int fromYear, fromMonth, fromDay, toYear, toMonth, toDay;
    private Calendar fromCalender = Calendar.getInstance();
    private Calendar toCalender = Calendar.getInstance();
    private Date fromDate, toDate;
    private TextView tvFromDate, tvToDate;
    private MaterialButton btnToday, btnThisWeek, btnThisMonth, btnCustom;
    private MaterialButtonToggleGroup btnGroup;
    private MaterialCardView cvCustom;
    private RecyclerView rvReports;
    private MaterialButton btnBack, btnPrint;
    private ArrayList<Object> reports = new ArrayList<>();
    private FullReportAdapter reportAdapter;
    private LinearLayout llProgress, llData;
    private TextView tvSelectedDate;

    private Date startDate, endDate;
    Calendar c = Calendar.getInstance();
    int year = c.get(Calendar.YEAR);
    int month = c.get(Calendar.MONTH);
    int day = 1;
    private DatePickerDialog fromPicker, toPicker;
    int selectedButton = 0;
    private ZoneRichPrinter zoneRichPrinter;
    private SunmiPrinter sunmiPrinter;
    private WifiPrinter wifiPrinter;
    private SiteSetting ticketHeaderSetting, ticketHeaderTypeSetting, footerASetting, footerBSetting;
    private int headerAlignment = 0;
    Printer defaultPrinter;
    private TextView tvTitle;
    private LinearLayout llUsers;
    private AppCompatSpinner spinnerUsers;
    private boolean isUserReport = false, isOnlineReport, isArchived;
    private ArrayList<User> users = new ArrayList<>();
    private SelectedUserAdapter userArrayAdapter;
    private User selectedUser;

    private String title;

    private LinearLayout llToday, llThisWeek, llThisMonth, llCustom;
    private TextView tvToday, tvThisWeek, tvThisMonth, tvCustom;
    private View vToday, vThisWeek, vThisMonth, vCustom;


    public static FullReportFragment getInstance() {
        return new FullReportFragment();
    }

    public static FullReportFragment getInstance(boolean isUserReport) {
        FullReportFragment fullReportFragment = new FullReportFragment();
        Bundle bundle = new Bundle();
        bundle.putBoolean("user_report", isUserReport);
        fullReportFragment.setArguments(bundle);
        return fullReportFragment;
    }

    public static FullReportFragment getInstance(String title, boolean isUserReport, boolean isOnlineReport) {
        FullReportFragment fullReportFragment = new FullReportFragment();
        Bundle bundle = new Bundle();
        bundle.putBoolean("user_report", isUserReport);
        bundle.putBoolean("online_report", isOnlineReport);
        bundle.putString("title", title);
        fullReportFragment.setArguments(bundle);
        return fullReportFragment;
    }

    public static FullReportFragment getInstance(String title, boolean isArchived) {
        FullReportFragment fullReportFragment = new FullReportFragment();
        Bundle bundle = new Bundle();
        bundle.putBoolean("is_archived", isArchived);
        bundle.putString("title", title);
        fullReportFragment.setArguments(bundle);
        return fullReportFragment;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_full_report, container, false);
    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getArguments() != null) {
            isUserReport = getArguments().getBoolean("user_report", false);
            isOnlineReport = getArguments().getBoolean("online_report", false);
            isArchived = getArguments().getBoolean("is_archived", false);
            title = getArguments().getString("title");
        }
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        if (MyApp.userPermission != null) {
            setListeners();
            if (isUserReport) {
                llUsers.setVisibility(View.VISIBLE);
                tvTitle.setText("Selected User Report");
                fetchUsers();
            } else {
                llUsers.setVisibility(View.GONE);
                getToday();
            }
            if (!Validators.isNullOrEmpty(title))
                tvTitle.setText(title);
        } else {
            ToastUtils.makeSnackToast(getActivity(), "No permissions found!");
        }
    }

    private void fetchUsers() {
        new Thread(() -> {
            users.clear();
            users.addAll(appDatabase.userDao().list());
            if (users.size() > 0)
                selectedUser = users.get(0);
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    userArrayAdapter.notifyDataSetChanged();
                    getToday();
                });
            }
        }).start();


    }

    private void initViews(View view) {
        loadSettings();
        zoneRichPrinter = new ZoneRichPrinter(getActivity());
        sunmiPrinter = new SunmiPrinter(getActivity());
        defaultPrinter = myPreferences.getDefaultPrinter();
        zoneRichPrinter = myApp.connectZonerich(zoneRichPrinter, defaultPrinter);
        wifiPrinter = WifiPrinter.getInstance();

        tvTitle = view.findViewById(R.id.tvTitle);
        llUsers = view.findViewById(R.id.llUsers);
        spinnerUsers = view.findViewById(R.id.spinnerUsers);
        userArrayAdapter = new SelectedUserAdapter(getActivity(), android.R.layout.simple_list_item_1, users);
        spinnerUsers.setAdapter(userArrayAdapter);

        chipBack = view.findViewById(R.id.chipBack);
        btnToday = view.findViewById(R.id.btnToday);
        btnThisWeek = view.findViewById(R.id.btnThisWeek);
        btnThisMonth = view.findViewById(R.id.btnThisMonth);
        btnCustom = view.findViewById(R.id.btnCustom);
        btnGroup = view.findViewById(R.id.btnGroup);

        cvCustom = view.findViewById(R.id.cvCustom);
        rvReports = view.findViewById(R.id.rvReports);
        btnBack = view.findViewById(R.id.btnBack);
        btnPrint = view.findViewById(R.id.btnPrint);
        llToDate = view.findViewById(R.id.llToDate);
        llFromDate = view.findViewById(R.id.llFromDate);
        tvToDate = view.findViewById(R.id.tvToDate);
        tvFromDate = view.findViewById(R.id.tvFromDate);
        llProgress = view.findViewById(R.id.llProgress);
        llData = view.findViewById(R.id.llData);
        tvSelectedDate = view.findViewById(R.id.tvSelectedDate);

        llToday = view.findViewById(R.id.llToday);
        llThisWeek = view.findViewById(R.id.llThisWeek);
        llThisMonth = view.findViewById(R.id.llThisMonth);
        llCustom = view.findViewById(R.id.llCustom);

        tvToday = view.findViewById(R.id.tvToday);
        tvThisWeek = view.findViewById(R.id.tvThisWeek);
        tvThisMonth = view.findViewById(R.id.tvThisMonth);
        tvCustom = view.findViewById(R.id.tvCustom);

        vToday = view.findViewById(R.id.vToday);
        vThisWeek = view.findViewById(R.id.vThisWeek);
        vThisMonth = view.findViewById(R.id.vThisMonth);
        vCustom = view.findViewById(R.id.vCustom);

        chekedBtn(llToday);
        cvCustom.setVisibility(View.GONE);
        reportAdapter = new FullReportAdapter(reports, (position, data) -> {

        });
        rvReports.setLayoutManager(new WrapContentLinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvReports.setItemAnimator(null);
        rvReports.setAdapter(reportAdapter);

    }

    public class WrapContentLinearLayoutManager extends LinearLayoutManager {
        public WrapContentLinearLayoutManager(Context context, int orientation, boolean reverseLayout) {
            super(context, orientation, reverseLayout);
        }

        @Override
        public void onLayoutChildren(RecyclerView.Recycler recycler, RecyclerView.State state) {
            try {
                super.onLayoutChildren(recycler, state);
            } catch (IndexOutOfBoundsException e) {
                e.printStackTrace();
            }
        }
    }

    private void loadSettings() {
        ticketHeaderTypeSetting = myApp.findSetting("ticket_header_type");
        if (ticketHeaderTypeSetting != null && ticketHeaderTypeSetting.value.equalsIgnoreCase("right")) {
            headerAlignment = 2;
        }
        if (ticketHeaderTypeSetting != null && ticketHeaderTypeSetting.value.equalsIgnoreCase("center")) {
            headerAlignment = 1;
        }
        ticketHeaderSetting = myApp.findSetting("ticket_header");

        footerASetting = myApp.findSetting("footer_a");
        footerBSetting = myApp.findSetting("footer_b");
    }


    private void setListeners() {
        chipBack.setOnClickListener(v -> getActivity().onBackPressed());
        llToday.setOnClickListener(v -> {
            selectedButton = llToday.getId();
            cvCustom.setVisibility(View.GONE);
            chekedBtn(llToday);
            getToday();
        });
        llThisWeek.setOnClickListener(v -> {
            selectedButton = llThisWeek.getId();
            cvCustom.setVisibility(View.GONE);
            chekedBtn(llThisWeek);
            getWeek();
        });
        llThisMonth.setOnClickListener(v -> {
            selectedButton = llThisMonth.getId();
            cvCustom.setVisibility(View.GONE);
            chekedBtn(llThisMonth);
            getCurrentMonth();
        });
        llCustom.setOnClickListener(v -> {
            selectedButton = llCustom.getId();
            cvCustom.setVisibility(View.VISIBLE);
            chekedBtn(llCustom);
        });

        llFromDate.setOnClickListener(view -> {
            DatePickerDialog datePickerDialog = DatePickerDialog.newInstance((view1, y, m, d) -> {
                fromDay = d;
                fromMonth = m;
                fromYear = y;
                fromDateString = y + "-" + String.format("%02d", (m + 1)) + "-" + String.format("%02d", d);
                fromCalender.set(y, m, d);
                tvFromDate.setText(CommonFunctions.formatMiliToDesireFormat(fromCalender.getTimeInMillis(), "dd/MM/yyyy"));
                setDateText();
                fetchReport();
            }, fromCalender.get(Calendar.YEAR), fromCalender.get(Calendar.MONTH), fromCalender.get(Calendar.DAY_OF_MONTH));
            datePickerDialog.setThemeDark(false);
            datePickerDialog.showYearPickerFirst(false);
            datePickerDialog.show(getActivity().getFragmentManager(), "FromPickerDialog");
            if (!Validators.isNullOrEmpty(toDateString)) {
                datePickerDialog.setMaxDate(toCalender);
            }
        });

        llToDate.setOnClickListener(view -> {
            DatePickerDialog datePickerDialog = DatePickerDialog.newInstance((view12, y, m, d) -> {
                toDay = d;
                toMonth = m;
                toYear = y;
                toDateString = y + "-" + String.format("%02d", (m + 1)) + "-" + String.format("%02d", d);
                toCalender.set(y, m, d);
                tvToDate.setText("To: " + CommonFunctions.formatMiliToDesireFormat(toCalender.getTimeInMillis(), "dd/MM/yyyy"));
                setDateText();
                fetchReport();
            }, toCalender.get(Calendar.YEAR), toCalender.get(Calendar.MONTH), toCalender.get(Calendar.DAY_OF_MONTH));
            datePickerDialog.setThemeDark(false);
            datePickerDialog.showYearPickerFirst(false);
            datePickerDialog.show(getActivity().getFragmentManager(), "ToPickerDialog");
            if (!Validators.isNullOrEmpty(fromDateString)) {
                datePickerDialog.setMinDate(fromCalender);
            }

        });
        btnBack.setOnClickListener(view -> {
            getActivity().onBackPressed();
        });
        btnPrint.setOnClickListener(view -> {
            createPrint();
        });
        spinnerUsers.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
                if (users.size() - 1 >= i) {
                    selectedUser = users.get(i);
                    fetchReport();
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> adapterView) {

            }
        });
    }

    private void chekedBtn(View v) {
        vToday.setBackgroundResource(R.color.light_gray);
        tvToday.setAlpha(0.5f);

        vThisWeek.setBackgroundResource(R.color.light_gray);
        tvThisWeek.setAlpha(0.5f);

        vThisMonth.setBackgroundResource(R.color.light_gray);
        tvThisMonth.setAlpha(0.5f);

        vCustom.setBackgroundResource(R.color.light_gray);
        tvCustom.setAlpha(0.5f);

        if (v == llToday) {
            vToday.setBackgroundResource(R.color.persian_green);
            tvToday.setAlpha(1f);
        }
        if (v == llThisWeek) {
            vThisWeek.setBackgroundResource(R.color.persian_green);
            tvThisWeek.setAlpha(1f);
        }
        if (v == llThisMonth) {
            vThisMonth.setBackgroundResource(R.color.persian_green);
            tvThisMonth.setAlpha(1f);
        }
        if (v == llCustom) {
            vCustom.setBackgroundResource(R.color.persian_green);
            tvCustom.setAlpha(1f);
        }
    }

    private void setDateText() {
        if (fromDateString.equalsIgnoreCase(toDateString)) {
            tvSelectedDate.setText(CommonFunctions.formatUnknownDateTime(fromDateString, "yyyy-MM-dd", "dd MMM yyyy"));
        } else {
            tvSelectedDate.setText(CommonFunctions.formatUnknownDateTime(fromDateString, "yyyy-MM-dd", "dd MMM yyyy") + " to " + CommonFunctions.formatUnknownDateTime(toDateString, "yyyy-MM-dd", "dd MMM yyyy"));
        }

        tvFromDate.setText("From: " + CommonFunctions.formatUnknownDateTime(fromDateString, "yyyy-MM-dd", "dd/MM/yyyy"));
        tvToDate.setText("To: " + CommonFunctions.formatUnknownDateTime(toDateString, "yyyy-MM-dd", "dd/MM/yyyy"));
    }

    private void getCurrentMonth() {
        c.set(year, month, day);
        startDate = c.getTime();


        int numOfDaysInMonth = c.getActualMaximum(Calendar.DAY_OF_MONTH);
        c.add(Calendar.DAY_OF_MONTH, numOfDaysInMonth - 1);

        endDate = c.getTime();

        fromDateString = CommonFunctions.formatMiliToDesireFormat(startDate.getTime(), "yyyy-MM-dd");
        toDateString = CommonFunctions.formatMiliToDesireFormat(endDate.getTime(), "yyyy-MM-dd");

        setDateText();

        fetchReport();
    }

    private void getWeek() {
        Calendar cal = Calendar.getInstance();
        cal.setFirstDayOfWeek(Calendar.MONDAY);
        cal.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        startDate = cal.getTime();

        cal.add(Calendar.DATE, 6);
        endDate = cal.getTime();

        fromDateString = CommonFunctions.formatMiliToDesireFormat(startDate.getTime(), "yyyy-MM-dd");
        toDateString = CommonFunctions.formatMiliToDesireFormat(endDate.getTime(), "yyyy-MM-dd");

        setDateText();
        fetchReport();
    }

    private void getToday() {
        fromDateString = CommonFunctions.formatMiliToDesireFormat(System.currentTimeMillis(), "yyyy-MM-dd");
        toDateString = CommonFunctions.formatMiliToDesireFormat(System.currentTimeMillis(), "yyyy-MM-dd");

        setDateText();
        fetchReport();
    }

    private class ReportAsync extends AsyncTask<String, String, String> {

        @Override
        protected String doInBackground(String... strings) {
            StringBuffer stringTotalOrdersQuery = new StringBuffer();
            StringBuffer stringTotalProductSaleQuery = new StringBuffer();
            StringBuffer stringTotalProductSaleAmountQuery = new StringBuffer();
            StringBuffer stringTotalDiscountQuery = new StringBuffer();
            StringBuffer stringTotalGratuityQuery = new StringBuffer();
            StringBuffer stringTotalServiceChargeQuery = new StringBuffer();
            StringBuffer stringTotalDinersQuery = new StringBuffer();
            StringBuffer stringTotalTablesQuery = new StringBuffer();
            StringBuffer stringOrderTypeQuery = new StringBuffer();
            StringBuffer stringPaymentTypeQuery = new StringBuffer();
            StringBuffer stringOrderStatusQuery = new StringBuffer();

            ArrayList<String> totalOrdersParams = new ArrayList<>();
            ArrayList<String> totalProductsSaleParams = new ArrayList<>();
            ArrayList<String> totalProductsSaleAmountParams = new ArrayList<>();
            ArrayList<String> totalDiscountsParams = new ArrayList<>();
            ArrayList<String> totalGratuityParams = new ArrayList<>();
            ArrayList<String> totalServiceChargeParams = new ArrayList<>();
            ArrayList<String> totalDinersParams = new ArrayList<>();
            ArrayList<String> totalTablesParams = new ArrayList<>();
            ArrayList<String> orderTypeParams = new ArrayList<>();
            ArrayList<String> paymentTypeParams = new ArrayList<>();
            ArrayList<String> orderStatusParams = new ArrayList<>();

            stringTotalOrdersQuery.append("SELECT COUNT(*) from 'Order' WHERE date(created_at) >= ?  AND date(created_at) <= ?");
            stringOrderStatusQuery.append("SELECT COUNT(*) as total, SUM(total) as value, order_status as name from 'Order' WHERE date(created_at) >= ?  AND date(created_at) <= ? ");
            stringOrderTypeQuery.append("SELECT COUNT(*) as total, SUM(total) as value, order_type as name from 'Order' WHERE date(created_at) >= ?  AND date(created_at) <= ? ");
            stringPaymentTypeQuery.append("SELECT COUNT(*) as total, SUM(amount) as value, payment_method_name as name from OrderPayment LEFT JOIN 'Order' ON 'Order'._id = OrderPayment._order_id  WHERE date('Order'.created_at) >= ?  AND date('Order'.created_at) <= ? ");
            stringTotalDinersQuery.append("SELECT SUM(no_guest) from 'Order' WHERE date(created_at) >= ?  AND date(created_at) <= ? AND order_type_id = 1");
            stringTotalTablesQuery.append("SELECT COUNT(*) from 'Order' WHERE date(created_at) >= ?  AND date(created_at) <= ? AND order_type_id = 1");
            stringTotalDiscountQuery.append("SELECT SUM(discount) from 'Order' WHERE date(created_at) >= ?  AND date(created_at) <= ?");
            stringTotalGratuityQuery.append("SELECT SUM(gratuity) from 'Order' WHERE date(created_at) >= ?  AND date(created_at) <= ?");
            stringTotalServiceChargeQuery.append("SELECT SUM(service_charge) from 'Order' WHERE date(created_at) >= ?  AND date(created_at) <= ?");
            stringTotalProductSaleQuery.append("SELECT COUNT(*) from OrderItem LEFT JOIN 'Order' ON 'Order'._id = OrderItem._order_id WHERE date('Order'.created_at) >= ?  AND date('Order'.created_at) <= ? ");
            stringTotalProductSaleAmountQuery.append("SELECT SUM(OrderItem.total) from OrderItem LEFT JOIN 'Order' ON 'Order'._id = OrderItem._order_id WHERE date('Order'.created_at) >= ?  AND date('Order'.created_at) <= ?");

            totalOrdersParams.add(fromDateString);
            totalOrdersParams.add(toDateString);

            orderStatusParams.add(fromDateString);
            orderStatusParams.add(toDateString);

            orderTypeParams.add(fromDateString);
            orderTypeParams.add(toDateString);

            totalProductsSaleParams.add(fromDateString);
            totalProductsSaleParams.add(toDateString);

            totalProductsSaleAmountParams.add(fromDateString);
            totalProductsSaleAmountParams.add(toDateString);

            totalDiscountsParams.add(fromDateString);
            totalDiscountsParams.add(toDateString);

            totalGratuityParams.add(fromDateString);
            totalGratuityParams.add(toDateString);

            totalServiceChargeParams.add(fromDateString);
            totalServiceChargeParams.add(toDateString);

            totalDinersParams.add(fromDateString);
            totalDinersParams.add(toDateString);

            totalTablesParams.add(fromDateString);
            totalTablesParams.add(toDateString);


            paymentTypeParams.add(fromDateString);
            paymentTypeParams.add(toDateString);


            if (isUserReport && selectedUser != null) {
                stringTotalOrdersQuery.append(" AND updater_id = ?");
                stringOrderTypeQuery.append(" AND updater_id = ?");
                stringOrderStatusQuery.append(" AND updater_id = ?");
                stringTotalDiscountQuery.append(" AND updater_id = ?");
                stringTotalDinersQuery.append(" AND updater_id = ?");
                stringTotalTablesQuery.append(" AND updater_id = ?");
                stringTotalGratuityQuery.append(" AND updater_id = ?");
                stringTotalServiceChargeQuery.append(" AND updater_id = ?");
                stringTotalProductSaleQuery.append(" AND OrderItem.updater_id = ?");
                stringTotalProductSaleAmountQuery.append(" AND OrderItem.updater_id = ?");
                stringPaymentTypeQuery.append(" AND OrderPayment.updater_id = ?");

                totalTablesParams.add(selectedUser.id);
                orderTypeParams.add(selectedUser.id);
                orderStatusParams.add(selectedUser.id);
                totalDinersParams.add(selectedUser.id);
                totalOrdersParams.add(selectedUser.id);
                totalDiscountsParams.add(selectedUser.id);
                totalGratuityParams.add(selectedUser.id);
                totalServiceChargeParams.add(selectedUser.id);
                totalProductsSaleParams.add(selectedUser.id);
                totalProductsSaleAmountParams.add(selectedUser.id);
                paymentTypeParams.add(selectedUser.id);
            } else {
                stringTotalOrdersQuery.append(" AND 'Order'.is_archived = ?");
                stringOrderTypeQuery.append(" AND 'Order'.is_archived = ?");
                stringOrderStatusQuery.append(" AND 'Order'.is_archived = ?");
                stringTotalDiscountQuery.append(" AND 'Order'.is_archived = ?");
                stringTotalDinersQuery.append(" AND 'Order'.is_archived = ?");
                stringTotalTablesQuery.append(" AND 'Order'.is_archived = ?");
                stringTotalGratuityQuery.append(" AND 'Order'.is_archived = ?");
                stringTotalServiceChargeQuery.append(" AND 'Order'.is_archived = ?");
                stringTotalProductSaleQuery.append(" AND 'Order'.is_archived = ?");
                stringTotalProductSaleAmountQuery.append(" AND 'Order'.is_archived = ?");
                stringPaymentTypeQuery.append(" AND 'Order'.is_archived = ?");


                totalTablesParams.add(isArchived ? "1" : "0");
                orderTypeParams.add(isArchived ? "1" : "0");
                orderStatusParams.add(isArchived ? "1" : "0");
                totalDinersParams.add(isArchived ? "1" : "0");
                totalOrdersParams.add(isArchived ? "1" : "0");
                totalDiscountsParams.add(isArchived ? "1" : "0");
                totalGratuityParams.add(isArchived ? "1" : "0");
                totalServiceChargeParams.add(isArchived ? "1" : "0");
                totalProductsSaleParams.add(isArchived ? "1" : "0");
                totalProductsSaleAmountParams.add(isArchived ? "1" : "0");
                paymentTypeParams.add(isArchived ? "1" : "0");
            }

            stringOrderStatusQuery.append(" Group By 'Order'.order_status_id");
            stringOrderTypeQuery.append(" Group By 'Order'.order_type_id");
            stringPaymentTypeQuery.append(" Group By OrderPayment.payment_method_id");

            SupportSQLiteQuery totalOrdersQuery = new SimpleSQLiteQuery(stringTotalOrdersQuery.toString(), totalOrdersParams.toArray());
            SupportSQLiteQuery totalProductsSaleQuery = new SimpleSQLiteQuery(stringTotalProductSaleQuery.toString(), totalProductsSaleParams.toArray());
            SupportSQLiteQuery totalProductsSaleAmountQuery = new SimpleSQLiteQuery(stringTotalProductSaleAmountQuery.toString(), totalProductsSaleAmountParams.toArray());
            SupportSQLiteQuery totalDiscountQuery = new SimpleSQLiteQuery(stringTotalDiscountQuery.toString(), totalDiscountsParams.toArray());
            SupportSQLiteQuery totalGratuityQuery = new SimpleSQLiteQuery(stringTotalGratuityQuery.toString(), totalGratuityParams.toArray());
            SupportSQLiteQuery totalServiceChargeQuery = new SimpleSQLiteQuery(stringTotalServiceChargeQuery.toString(), totalServiceChargeParams.toArray());
            SupportSQLiteQuery totalDinersQuery = new SimpleSQLiteQuery(stringTotalDinersQuery.toString(), totalDinersParams.toArray());
            SupportSQLiteQuery totalTablesQuery = new SimpleSQLiteQuery(stringTotalTablesQuery.toString(), totalTablesParams.toArray());
            SupportSQLiteQuery orderStatusQuery = new SimpleSQLiteQuery(stringOrderStatusQuery.toString(), orderStatusParams.toArray());
            SupportSQLiteQuery orderTypeQuery = new SimpleSQLiteQuery(stringOrderTypeQuery.toString(), orderTypeParams.toArray());
            SupportSQLiteQuery paymentTypeQuery = new SimpleSQLiteQuery(stringPaymentTypeQuery.toString(), paymentTypeParams.toArray());

            int totalOrders = appDatabase.reportDao().totalOrders(totalOrdersQuery);
            int totalProducts = appDatabase.reportDao().totalProducts(totalProductsSaleQuery);
            float totalProductSaleAmount = appDatabase.reportDao().totalProductsAmount(totalProductsSaleAmountQuery);
            float totalDiscounts = appDatabase.reportDao().totalDiscount(totalDiscountQuery);
            float totalGratuity = appDatabase.reportDao().totalGratuity(totalGratuityQuery);
            float totalServiceCharge = appDatabase.reportDao().totalServiceCharge(totalServiceChargeQuery);
            int totalDiners = appDatabase.reportDao().totalDiners(totalDinersQuery);
            int totalTables = appDatabase.reportDao().totalTables(totalTablesQuery);
            List<DaoReportNameTotalValue> orderStatus = appDatabase.reportDao().getOrderStatusReport(orderStatusQuery);
            List<DaoReportNameTotalValue> orderTypes = appDatabase.reportDao().getOrderStatusReport(orderTypeQuery);
            List<DaoReportNameTotalValue> orderPayments = appDatabase.reportDao().getOrderStatusReport(paymentTypeQuery);

            reports.clear();
            if (MyApp.userPermission.order_count.actions.list)
                reports.add(new ReportTextModel("Total Orders", String.valueOf(totalOrders), false));
            if (MyApp.userPermission.orders_total_product.actions.list)
                reports.add(new ReportTextModel("Total Product Sale", String.valueOf(totalProducts), false));
            if (MyApp.userPermission.orders_amount_product.actions.list)
                reports.add(new ReportTextModel("Value Product Sale", MyApp.currencySymbol + MyApp.df.format(totalProductSaleAmount), false));
            if (MyApp.userPermission.total_discount.actions.list)
                reports.add(new ReportTextModel("Total Discounts", MyApp.currencySymbol + MyApp.df.format(totalDiscounts), false));
            if (MyApp.userPermission.total_no_guest.actions.list)
                reports.add(new ReportTextModel("Total Dinners", String.valueOf(totalDiners), false));
            if (MyApp.userPermission.total_no_table.actions.list)
                reports.add(new ReportTextModel("Total Tables", String.valueOf(totalTables), false));


            if (totalGratuity > 0) {
                reports.add(new ReportTextModel("Total gratuity", MyApp.df.format(totalGratuity), false));
            }
            if (totalServiceCharge > 0) {
                reports.add(new ReportTextModel("Total service charge", MyApp.df.format(totalServiceCharge), false));
            }
            if (orderTypes != null && orderTypes.size() > 0 && MyApp.userPermission.order_type_wise.actions.list) {
                reports.add(new ReportTextModel("Order type", "", true));
                for (DaoReportNameTotalValue something : orderTypes) {
                    reports.add(new ReportTextModel("Total " + something.name + " orders", String.valueOf(something.total), false));
                    reports.add(new ReportTextModel("Value " + something.name + " orders", MyApp.currencySymbol + MyApp.df.format(Float.parseFloat(something.value)), false));
                }
            }
            if (orderStatus != null && orderStatus.size() > 0 && MyApp.userPermission.order_type_wise.actions.list) {
                reports.add(new ReportTextModel("Order Status", "", true));
                for (DaoReportNameTotalValue something : orderStatus) {
                    reports.add(new ReportTextModel("Total " + something.name + " orders", String.valueOf(something.total), false));
                    reports.add(new ReportTextModel("Value " + something.name + " orders", MyApp.currencySymbol + MyApp.df.format(Float.parseFloat(something.value)), false));
                }
            }
            float cashTotal = 0, cardTotal = 0;
            if (orderPayments != null && orderPayments.size() > 0 && MyApp.userPermission.order_type_wise.actions.list) {
                reports.add(new ReportTextModel("Payment Type", "", true));
                for (DaoReportNameTotalValue something : orderPayments) {
                    reports.add(new ReportTextModel("Total " + something.name + " orders", String.valueOf(something.total), false));
                    reports.add(new ReportTextModel("Value " + something.name + " orders", MyApp.currencySymbol + MyApp.df.format(Float.parseFloat(something.value)), false));
                    if (something.name.toLowerCase().contains("cash")) {
                        cashTotal = cashTotal + Float.parseFloat(something.value);
                    }
                    if (something.name.toLowerCase().contains("card")) {
                        cardTotal = cardTotal + Float.parseFloat(something.value);
                    }

                }
                reports.add(new ReportTextModel("Grand Total", "", true));
                reports.add(new ReportTextModel("Total Cash", MyApp.currencySymbol + MyApp.df.format(cashTotal), false));
                reports.add(new ReportTextModel("Total Card", MyApp.currencySymbol + MyApp.df.format(cardTotal), false));
                reports.add(new ReportTextModel("Total", MyApp.currencySymbol + MyApp.df.format(cardTotal + cashTotal), false));
            }

            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            llProgress.setVisibility(View.GONE);
            llData.setVisibility(View.VISIBLE);
            if (getActivity() != null)
                rvReports.post(() -> {
                    rvReports.stopScroll();
                    reportAdapter.notifyDataSetChanged();
                });

        }
    }

    private void fetchReport() {
        if (!isOnlineReport) {
            new ReportAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
            return;
        }
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                llProgress.setVisibility(View.VISIBLE);
                llData.setVisibility(View.GONE);
            });
        }
        String url = ApiEndPoints.full_report;
        if (isOnlineReport) {
            url = ApiEndPoints.online_report;
        }
        ANRequest.GetRequestBuilder requestBuilder = AndroidNetworking.get(url)
                .addQueryParameter("shift_status", "both")
                .addQueryParameter("from_date", fromDateString)
                .addQueryParameter("restaurant_id", myApp.restaurant_id)
                .addQueryParameter("to_date", toDateString);
        if (isUserReport && selectedUser != null) {
            requestBuilder.addQueryParameter("user_id", selectedUser.id);
        }
        requestBuilder.addQueryParameter("archived", isArchived ? "1" : "0");
        requestBuilder.build()
                .getAsObject(FullReport.class, new ParsedRequestListener<FullReport>() {
                    @Override
                    public void onResponse(FullReport response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                llProgress.setVisibility(View.GONE);
                                llData.setVisibility(View.VISIBLE);
                            });
                        }

                        reports.clear();
                        if (isOnlineReport) {
                            if (MyApp.userPermission.online_order_count.actions.list)
                                reports.add(new ReportTextModel("Total Online Orders", response.online_order_count, false));

                            if (MyApp.userPermission.online_reservation_count.actions.list)
                                reports.add(new ReportTextModel("Total Online Reservations", response.online_reservation_count, false));
                            if (MyApp.userPermission.online_order_total_business.actions.list)
                                reports.add(new ReportTextModel("Total Online Order Businesses", response.online_order_total_business, false));

                            if (MyApp.userPermission.online_order_calculated_commission.actions.list)
                                reports.add(new ReportTextModel("Total Online Order Commission", response.online_order_calculated_commission, false));

                            if (response.online_order_type != null && response.online_order_type.size() > 0 && MyApp.userPermission.online_order_type.actions.list) {
                                reports.add(new ReportTextModel("Order type", "", true));
                                for (ReportOrderType something : response.online_order_type) {
                                    reports.add(new ReportTextModel("Total " + something.order_type + " orders", String.valueOf(something.count), false));
                                    reports.add(new ReportTextModel("Value " + something.order_type + " orders", MyApp.currencySymbol + MyApp.df.format(something.total), false));
                                }
                            }
                            if (response.online_order_payment != null && response.online_order_payment.size() > 0 && MyApp.userPermission.online_order_payment.actions.list) {
                                reports.add(new ReportTextModel("Payment type", "", true));
                                for (ReportOrderPaymentMethod something : response.online_order_payment) {
                                    reports.add(new ReportTextModel("Total " + something.payment_method + " orders", String.valueOf(something.count), false));
                                    reports.add(new ReportTextModel("Value " + something.payment_method + " orders", MyApp.currencySymbol + MyApp.df.format(something.total), false));
                                }
                            }

                        } else {

                            if (MyApp.userPermission.order_count.actions.list)
                                reports.add(new ReportTextModel("Total Orders", response.order_count, false));
                            if (MyApp.userPermission.orders_total_product.actions.list)
                                reports.add(new ReportTextModel("Total Product Sale", response.orders_total_product, false));
                            if (MyApp.userPermission.orders_amount_product.actions.list)
                                reports.add(new ReportTextModel("Value Product Sale", MyApp.currencySymbol + MyApp.df.format(response.orders_amount_product), false));
                            if (MyApp.userPermission.total_discount.actions.list)
                                reports.add(new ReportTextModel("Total Discounts", MyApp.currencySymbol + MyApp.df.format(response.total_discount), false));
                            if (MyApp.userPermission.total_no_guest.actions.list)
                                reports.add(new ReportTextModel("Total Dinners", response.total_no_guest, false));
                            if (MyApp.userPermission.total_no_table.actions.list)
                                reports.add(new ReportTextModel("Total Tables", response.total_no_table, false));

                            if (response.allocated_voucher != 0) {
                                reports.add(new ReportTextModel("Total Voucher", String.valueOf(response.allocated_voucher), false));
                            }
                            if (response.allocated_voucher_value != 0) {
                                reports.add(new ReportTextModel("Total Voucher value", String.valueOf(response.allocated_voucher_value), false));
                            }
                            if (!Validators.isNullOrEmpty(response.total_gratuity)) {
                                reports.add(new ReportTextModel("Total gratuity", response.total_gratuity, false));
                            }
                            if (!Validators.isNullOrEmpty(response.total_service_charge)) {
                                reports.add(new ReportTextModel("Total service charge", String.valueOf(response.total_service_charge), false));
                            }
                            if (response.order_type_wise != null && response.order_type_wise.size() > 0 && MyApp.userPermission.order_type_wise.actions.list) {
                                reports.add(new ReportTextModel("Order type", "", true));
                                for (ReportOrderType something : response.order_type_wise) {
                                    reports.add(new ReportTextModel("Total " + something.type + " orders", String.valueOf(something.count), false));
                                    reports.add(new ReportTextModel("Value " + something.type + " orders", MyApp.currencySymbol + MyApp.df.format(something.total), false));
                                }
                            }
                            if (response.order_status_wise != null && response.order_status_wise.size() > 0 && MyApp.userPermission.order_status_wise.actions.list) {
                                reports.add(new ReportTextModel("Order status", "", true));
                                for (ReportOrderStatus something : response.order_status_wise) {
                                    reports.add(new ReportTextModel("Total " + something.status, String.valueOf(something.count), false));
                                    reports.add(new ReportTextModel("Value " + something.status, MyApp.currencySymbol + MyApp.df.format(something.total), false));
                                }
                            }
                            if (response.order_payment_wise != null && response.order_payment_wise.size() > 0 && MyApp.userPermission.order_payment_wise.actions.list) {
                                reports.add(new ReportTextModel("Order payment", "", true));
                                for (ReportOrderPaymentMethod something : response.order_payment_wise) {
                                    reports.add(new ReportTextModel("Total " + something.name + " Payment", String.valueOf(something.count), false));
                                    reports.add(new ReportTextModel("Value " + something.name + " Payment", MyApp.currencySymbol + MyApp.df.format(something.total), false));
                                }
                            }
                        }
                        if (getActivity() != null)
                            rvReports.post(() -> {
                                rvReports.stopScroll();
                                reportAdapter.notifyDataSetChanged();
                                btnPrint.setVisibility(View.VISIBLE);
                            });
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                llProgress.setVisibility(View.GONE);
                                llData.setVisibility(View.GONE);
                            });
                        }

                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    btnPrint.setVisibility(View.GONE);
                                });
                            }
                        }
                    }
                });
    }

    private void createPrint() {
        if (defaultPrinter != null && defaultPrinter.printer_model_name != null) {
            String title = "Full report";
            if (isUserReport) {
                title = "User report";
            }
            if (isOnlineReport) {
                title = "Online order report";
            }
            if (defaultPrinter.printer_model_name.toLowerCase().contains("88 h")) {
                zoneRichPrinter.connectPrinter(defaultPrinter.ip);
                zoneRichPrinter.printFullReport(myApp.businessLogo, ticketHeaderSetting != null ? ticketHeaderSetting.value : "", headerAlignment, title, tvSelectedDate.getText().toString(), selectedUser != null ? selectedUser.username : null, reports, footerASetting != null ? footerASetting.value : "", footerBSetting != null ? footerBSetting.value : "");
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("sunmi") || defaultPrinter.printer_model_name.toLowerCase().contains("t2")) {
                sunmiPrinter.printFullReport(myApp.businessLogo, ticketHeaderSetting != null ? ticketHeaderSetting.value : "", headerAlignment, title, tvSelectedDate.getText().toString(), selectedUser != null ? selectedUser.username : null, reports, footerASetting != null ? footerASetting.value : "", footerBSetting != null ? footerBSetting.value : "");
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("pos-80") || defaultPrinter.printer_model_name.toLowerCase().contains("pos 80") || defaultPrinter.printer_model_name.toLowerCase().contains("yoke4")) {
                wifiPrinter.connect(defaultPrinter.ip);
                CommonFunctions.functionThatDelay(200);
                wifiPrinter.printFullReport(myApp.businessLogo, ticketHeaderSetting != null ? ticketHeaderSetting.value : "", headerAlignment, title, tvSelectedDate.getText().toString(), selectedUser != null ? selectedUser.username : null, reports, footerASetting != null ? footerASetting.value : "", footerBSetting != null ? footerBSetting.value : "");
            }
            getActivity().onBackPressed();
        } else {
            ToastUtils.makeSnackToast(getActivity(), "No printer model found");
        }
    }

}