package com.example.epos_2021.fragment;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Paint;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.Priority;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.activities.NewOrderActivity;
import com.example.epos_2021.adapters.HomeCategoriesAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.models.CallLogs;
import com.example.epos_2021.models.Customer;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.OrderItemAddon;
import com.example.epos_2021.models.OrderItemIngredient;
import com.example.epos_2021.models.OrderPayment;
import com.example.epos_2021.models.OrderSplit;
import com.example.epos_2021.models.RecyclerMenuItem;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.online.models.Counts;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.LogUtils;
import com.google.android.material.button.MaterialButton;
import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import io.mpos.shared.helper.Log;

public class HomeFragment extends BaseFragment {

    private MaterialButton btnNewOrder;
    private TextView tvEdit;
    private RecyclerView rvItems;
    private HomeCategoriesAdapter itemAdapter;
    private ArrayList<RecyclerMenuItem> items = new ArrayList<>();


    public static HomeFragment getInstance() {
        return new HomeFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_home, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        initViews(view);
        setUpItems();
        setListeners();
    }

    private void setListeners() {
        btnNewOrder.setOnClickListener(v -> {
            OrderTypeSelectionDialogFragment orderTypeSelectionDialog = OrderTypeSelectionDialogFragment.getInstance();
            orderTypeSelectionDialog.show(getActivity().getSupportFragmentManager(), "order_type");
            orderTypeSelectionDialog.setDialogDismissListener(o -> {
                String orderTypeId = o.toString();
                openNewOrderDialog(orderTypeId);
            });
        });
    }

    private void openNewOrderDialog(String orderTypeId) {
        NewOrderDialogFragment newOrderDialogFragment = NewOrderDialogFragment.getInstance(orderTypeId);
        newOrderDialogFragment.show(getActivity().getSupportFragmentManager(), "new_order");
        newOrderDialogFragment.setDialogDismissListener(o -> {
            if (o instanceof Order) {
                startActivity(new Intent(getActivity(), NewOrderActivity.class)
                        .putExtra("order", new Gson().toJson(o))
                        .putExtra("fetch_delivery_charge", ((Order) o).order_type_id.equalsIgnoreCase("3")));
            } else if (o instanceof CallLogs) {
                changeFragment(NewReservationFragment.getInstance((CallLogs) o));
            }
        });
    }

    private void setUpItems() {
        //1 = Dine In, 2= Collection, 3 = Delivery, 4 = Bar, 5= Waiting, 6 Web Order
        SiteSetting collectionSetting = myApp.findSetting("is_collection");
        SiteSetting waitingSetting = myApp.findSetting("is_waiting");
        SiteSetting deliverySetting = myApp.findSetting("is_delivery");
        SiteSetting dineinSetting = myApp.findSetting("is_dinein");
        SiteSetting webOrderSetting = myApp.findSetting("is_weborder");
        SiteSetting reservationSetting = myApp.findSetting("is_reservation");
        items.clear();

        RecyclerMenuItem homeItems = new RecyclerMenuItem();
        if (waitingSetting != null && (waitingSetting.value.equals("true") || waitingSetting.value.equals("1")) && (MyApp.userPermission != null && MyApp.userPermission.waiting.actions.list)) {
            homeItems.id = "5";
            homeItems.title = "Waiting";
            items.add(homeItems);
        }
        if (collectionSetting != null && (collectionSetting.value.equals("true") || collectionSetting.value.equals("1")) && (MyApp.userPermission != null && MyApp.userPermission.collection.actions.list)) {
            homeItems = new RecyclerMenuItem();
            homeItems.id = "2";
            homeItems.title = "Collection";
            items.add(homeItems);
        }
        if (deliverySetting != null && (deliverySetting.value.equals("true") || deliverySetting.value.equals("1")) && (MyApp.userPermission != null && MyApp.userPermission.delivery.actions.list)) {
            homeItems = new RecyclerMenuItem();
            homeItems.id = "3";
            homeItems.title = "Delivery";
            items.add(homeItems);
        }
        if (!Validators.isNullOrEmpty(myApp.restaurant_id) && webOrderSetting != null && (webOrderSetting.value.equalsIgnoreCase("true") || webOrderSetting.value.equalsIgnoreCase("1")) && (MyApp.userPermission != null && MyApp.userPermission.web_orders.actions.list)) {
            homeItems = new RecyclerMenuItem();
            homeItems.id = "6";
            homeItems.title = "Online orders";
            items.add(homeItems);
        }

        if (dineinSetting != null && (dineinSetting.value.equals("true") || dineinSetting.value.equals("1")) && (MyApp.userPermission != null && MyApp.userPermission.dinein.actions.list)) {
            homeItems = new RecyclerMenuItem();
            homeItems.id = "1";
            homeItems.title = "Tables";
            items.add(homeItems);
        }

        if (MyApp.userPermission != null && MyApp.userPermission.order_history.actions.list) {
            homeItems = new RecyclerMenuItem();
            homeItems.id = "9";
            homeItems.title = "Order history";
            items.add(homeItems);
        }
        if (reservationSetting != null && (reservationSetting.value.equals("true") || reservationSetting.value.equals("1")) && (MyApp.userPermission != null && MyApp.userPermission.reservation.actions.list)) {
            homeItems = new RecyclerMenuItem();
            homeItems.id = "7";
            homeItems.title = "Reservation";
            items.add(homeItems);
        }
        if (MyApp.userPermission != null && MyApp.userPermission.vouchers.actions.list) {
            homeItems = new RecyclerMenuItem();
            homeItems.id = "8";
            homeItems.title = "Voucher";
            items.add(homeItems);
        }

        itemAdapter.notifyDataSetChanged();
        fetchEposOrdersCount();
    }

    private void initViews(View view) {
        rvItems = view.findViewById(R.id.rvItems);
        btnNewOrder = view.findViewById(R.id.btnNewOrder);
        tvEdit = view.findViewById(R.id.tvEdit);

        tvEdit.setPaintFlags(Paint.UNDERLINE_TEXT_FLAG);

        itemAdapter = new HomeCategoriesAdapter(items, (position, data) -> {
            RecyclerMenuItem categories = (RecyclerMenuItem) data;

            if (categories.id.equalsIgnoreCase("2") || categories.id.equalsIgnoreCase("3") || categories.id.equalsIgnoreCase("5") || categories.id.equalsIgnoreCase("6")) {
                LocalBroadcastManager.getInstance(getActivity()).sendBroadcast(new Intent("order_fragment_change").putExtra("order_type", categories.id).putExtra("destination", "active_orders"));
            }
            if (categories.title.equalsIgnoreCase("Tables")) {
                LocalBroadcastManager.getInstance(getActivity()).sendBroadcast(new Intent("order_fragment_change").putExtra("destination", "tables"));
                changeFragment(DineInFragment.getInstance());
            }
            if (categories.title.equalsIgnoreCase("Order history")) {
                LocalBroadcastManager.getInstance(getActivity()).sendBroadcast(new Intent("order_fragment_change").putExtra("destination", "order_history"));
                changeFragment(EposOrdersHistoryFragment.getInstance(null, false));
            }
            if (categories.title.equalsIgnoreCase("Reservation")) {
                LocalBroadcastManager.getInstance(getActivity()).sendBroadcast(new Intent("order_fragment_change").putExtra("destination", "reservation_history"));
                changeFragment(ReservationListFragment.getInstance());
            }
            if (categories.title.equalsIgnoreCase("Voucher")) {
                LocalBroadcastManager.getInstance(getActivity()).sendBroadcast(new Intent("order_fragment_change").putExtra("destination", "voucher_history"));
                changeFragment(VoucherHistoryFragment.getInstance());
            }
            if (categories.title.equalsIgnoreCase("Notifications")) {

            }
        });
        rvItems.setLayoutManager(new GridLayoutManager(getActivity(), getActivity().getResources().getInteger(R.integer.home_menu_span)));
        rvItems.setAdapter(itemAdapter);

    }

    BroadcastReceiver eposOrderUpdateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            fetchEposOrdersCount();
        }
    };
    BroadcastReceiver onlineOrdersNotificationReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getBooleanExtra("refresh", false)) {
                Counts allCounts = new Gson().fromJson(intent.getStringExtra("counts"), Counts.class);
                setOnlineOrderCounts(allCounts);
            }
        }
    };
    BroadcastReceiver newOrderReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getBooleanExtra("refresh", false)) {

                LogUtils.e("shallWeRefreshOrders?" + myApp.shallWeRefreshOrders + "");
                if (myApp.shallWeRefreshOrders) {
                    if (intent.hasExtra("notification_section") && intent.getStringExtra("notification_section").equalsIgnoreCase("tables")) {

                    } else {

                        fetchOrders();
                    }

                } else {
                    LogUtils.e("Already refreshing");
                    myApp.shallWeRefreshOrders = false;
                }
            }
        }
    };

    private void fetchOrders() {
        if (getActivity() != null && myApp.isConnected(getActivity()) && "auto".equalsIgnoreCase(myApp.myPreferences.getOrderSyncMode())) {
            fetchOnlineOrders();
        }
    }

    private HashMap<String, String> getOrdersParameters(Order latestOrder, Calendar calendar) {
        HashMap<String, String> params = new HashMap<>();
        Calendar fromCalendar = Calendar.getInstance();
        fromCalendar.add(Calendar.DATE, -1);
        Calendar toCalendar = Calendar.getInstance();

        String updated_at = (latestOrder != null && latestOrder.updated_at != null) ? CommonFunctions.formatUnknownDateTime(latestOrder.updated_at, Constants.PHP_DATE_TIME_FORMAT_ZULU, Constants.PHP_DATE_TIME_FORMAT) : CommonFunctions.convertMsToDesiredFormat(calendar.getTimeInMillis(), Constants.PHP_DATE_TIME_FORMAT);
        Date updatedDate = CommonFunctions.convertToDate(updated_at, Constants.PHP_DATE_TIME_FORMAT);
        Calendar updatedAtCalendar = Calendar.getInstance();
        updatedAtCalendar.setTime(updatedDate);
        updatedAtCalendar.add(Calendar.HOUR_OF_DAY, -4);
        updated_at = CommonFunctions.formatMiliToDesireFormat(updatedAtCalendar.getTimeInMillis(), Constants.PHP_DATE_TIME_FORMAT);

        LogUtils.e("Fetching after updated " + updated_at);
        params.put("archived", "0");
        params.put("nopaginate", "1");
//        params.put("screen", "dashboard");
        params.put("from_date", CommonFunctions.formatMiliToDesireFormat(fromCalendar.getTimeInMillis(), "yyyy-MM-dd"));
        params.put("from_updated_at", updated_at);
        params.put("to_date", CommonFunctions.formatMiliToDesireFormat(toCalendar.getTimeInMillis(), "yyyy-MM-dd"));
        return params;
    }

    private void fetchOnlineOrders() {

        myApp.shallWeRefreshOrders = false;
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE, -2);
        new Thread(() -> {
            Order latestOrder;
            latestOrder = appDatabase.orderDao().lastUpdatedOrder();
            AndroidNetworking.get(ApiEndPoints.orders + "latest")
                    .setPriority(Priority.IMMEDIATE)
                    .addQueryParameter(getOrdersParameters(latestOrder, calendar))
                    .build()
                    .getAsObjectList(Order.class, new ParsedRequestListener<List<Order>>() {
                        @Override
                        public void onResponse(List<Order> response) {
                            LogUtils.e("ORDERS", "API CALL response avyo " + response.size());
                            Collections.reverse(response);
                            new DbSaverAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, response);

                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                            fetchEposOrdersCount();
                            myApp.shallWeRefreshOrders = true;
                            boolean shouldDelete = false;
                            if (anError.getErrorCode() == 400) shouldDelete = true;
                        }
                    });
        }).start();

    }

    private class DbSaverAsync extends AsyncTask<Object, Void, String> {
        private ArrayList<Object> localObjects = new ArrayList<>();

        @Override
        protected void onPreExecute() {
            LogUtils.e("ORDERS", "PRE EXECUTE");
            super.onPreExecute();
        }

        @Override
        protected String doInBackground(Object... lists) {
            LogUtils.e("ORDERS", "IN background start");
            Thread.currentThread().setPriority(Thread.MAX_PRIORITY);

            if (lists[0] instanceof ArrayList<?>) {
                ArrayList<Order> parameterOrders = (ArrayList<Order>) lists[0];
                ArrayList<String> allOrderIds = new ArrayList<>();
                ArrayList<Order> newOnlineOrders = new ArrayList<>();
                for (Order o : parameterOrders) {
                    allOrderIds.add(o.id);
                }

                // list all online orders that are not in local DB
                ArrayList<Order> localOrdersWithOnlineID = new ArrayList<>();
                localOrdersWithOnlineID = (ArrayList<Order>) appDatabase.orderDao().listWithOnlineIds(allOrderIds);
                List<String> shouldNotDeleteThisIds;
                for (Order onlineOrder : parameterOrders) {
                    boolean found = false;
                    for (Order dbOrder : localOrdersWithOnlineID) {
                        if (onlineOrder.id.equalsIgnoreCase(dbOrder.id)) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        newOnlineOrders.add(onlineOrder);
                    }
                }
                // list all orders that are already in local DB
                for (Order dbOrder : localOrdersWithOnlineID) {
                    Order matchedOnlineOrder = null;
                    for (Order onlineOrder : parameterOrders) {
                        if (onlineOrder.id.equalsIgnoreCase(dbOrder.id)) {
                            matchedOnlineOrder = onlineOrder;
                            break;
                        }
                    }
                    if (matchedOnlineOrder != null) {
                        if (dbOrder.updated_at != null &&
                                dbOrder.updated_at.toLowerCase().equalsIgnoreCase(matchedOnlineOrder.updated_at.toLowerCase()) &&
                                dbOrder.total == matchedOnlineOrder.total &&
                                dbOrder.total_paid == matchedOnlineOrder.total_paid
                        ) {
                            LogUtils.e("Continuing... second");
                            continue;
                        } else {
                            Customer dbCustomer = appDatabase.customerDao().view(matchedOnlineOrder.customer_id);
                            int _customer_id;
                            if (dbCustomer == null) {
                                dbCustomer = matchedOnlineOrder.customer;
                                _customer_id = (int) appDatabase.customerDao().insert(matchedOnlineOrder.customer);
                            } else {
                                _customer_id = dbCustomer._id;
                                dbCustomer = matchedOnlineOrder.customer;
                                dbCustomer._id = _customer_id;
                                appDatabase.customerDao().update(dbCustomer);
                                _customer_id = dbCustomer._id;
                            }
                            matchedOnlineOrder._customer_id = _customer_id;
                            matchedOnlineOrder.customer._id = _customer_id;

                            int _order_id;
                            matchedOnlineOrder.customer_name = dbCustomer.name;
                            if (dbOrder == null) {
                                _order_id = (int) appDatabase.orderDao().insert(matchedOnlineOrder);
                                matchedOnlineOrder._id = _order_id;
                            } else {
                                matchedOnlineOrder._id = dbOrder._id;
                                _order_id = dbOrder._id;
                                appDatabase.orderDao().update(matchedOnlineOrder);
                            }


                            shouldNotDeleteThisIds = new ArrayList<>();
                            if (matchedOnlineOrder.order_splits != null) {
                                for (OrderSplit split : matchedOnlineOrder.order_splits) {
                                    shouldNotDeleteThisIds.add(split.id);
                                    split._order_id = _order_id;
                                    OrderSplit dbSplit = appDatabase.orderSplitDao().view(split.id);
                                    if (dbSplit == null) {
                                        split._id = (int) appDatabase.orderSplitDao().insert(split);
                                    } else {
                                        split._id = dbSplit._id;
                                        appDatabase.orderSplitDao().update(split);
                                    }
                                }
                            }
                            appDatabase.orderSplitDao().deleteOtherIdsButNotThis(_order_id, shouldNotDeleteThisIds);

                            shouldNotDeleteThisIds = new ArrayList<>();
                            if (matchedOnlineOrder.order_payments != null) {
                                for (OrderPayment payment : matchedOnlineOrder.order_payments) {
                                    shouldNotDeleteThisIds.add(payment.id);
                                    payment._order_id = _order_id;
                                    OrderPayment dbPayment = appDatabase.orderPaymentDao().view(payment.id);
                                    if (dbPayment == null) {
                                        payment._id = (int) appDatabase.orderPaymentDao().insert(payment);
                                    } else {
                                        payment._id = dbPayment._id;

                                    }
                                    if (!Validators.isNullOrEmpty(payment.order_split_id)) {
                                        OrderSplit dbOrderSplit = appDatabase.orderSplitDao().view(payment.order_split_id);
                                        if (dbOrderSplit != null)
                                            payment._order_split_id = dbOrderSplit._id;
                                    }
                                    appDatabase.orderPaymentDao().insert(payment);
                                }
                            }

                            appDatabase.orderPaymentDao().deleteOtherIdsButNotThis(_order_id, shouldNotDeleteThisIds);

                            if (matchedOnlineOrder.order_items != null) {
                                appDatabase.orderItemDao().deleteAll(_order_id);
                                for (OrderItem orderItem : matchedOnlineOrder.order_items) {
                                    orderItem._order_id = _order_id;
                                    OrderItem dbOrderItem = myApp.appDatabase.orderItemDao().view(orderItem.id);

                                    if (!Validators.isNullOrEmpty(orderItem.order_split_id)) {
                                        OrderSplit orderSplit = appDatabase.orderSplitDao().view(orderItem.order_split_id);
                                        if (orderSplit != null)
                                            orderItem._order_split_id = orderSplit._id;
                                    }

                                    int _order_item_id = 0;
                                    if (dbOrderItem == null) {
                                        try {
                                            _order_item_id = (int) appDatabase.orderItemDao().insert(orderItem);
                                            orderItem._id = _order_item_id;
                                        } catch (Exception e) {

                                        }
                                    } else {
                                        _order_item_id = dbOrderItem._id;
                                        orderItem._id = _order_item_id;
                                        appDatabase.orderItemDao().update(orderItem);
                                    }

                                    float addonTotal = 0;
                                    shouldNotDeleteThisIds = new ArrayList<>();
                                    if (orderItem.order_item_addons != null) {
                                        for (OrderItemAddon itemAddon : orderItem.order_item_addons) {
                                            shouldNotDeleteThisIds.add(itemAddon.id);
                                            itemAddon._order_item_id = _order_item_id;
                                            itemAddon.total = itemAddon.quantity * itemAddon.price;
                                            addonTotal = addonTotal + itemAddon.total;
                                            OrderItemAddon dbAddon = appDatabase.orderItemAddonDao().view(itemAddon.id);
                                            if (dbAddon == null) {
                                                itemAddon._id = (int) appDatabase.orderItemAddonDao().insert(itemAddon);
                                            } else {
                                                itemAddon._id = dbAddon._id;
                                                appDatabase.orderItemAddonDao().update(itemAddon);
                                            }
                                        }
                                    }
                                    appDatabase.orderItemAddonDao().deleteOtherIdsButNotThis(_order_item_id, shouldNotDeleteThisIds);

                                    float ingredientTotal = 0;
                                    if (orderItem.order_item_ingredients != null) {
                                        appDatabase.orderItemIngredientDao().deleteAll(_order_item_id);
                                        for (OrderItemIngredient itemIngredient : orderItem.order_item_ingredients) {
                                            itemIngredient._order_item_id = _order_item_id;
                                            itemIngredient.total = itemIngredient.quantity * itemIngredient.price;
                                            ingredientTotal = ingredientTotal + itemIngredient.total;
                                            OrderItemIngredient dbAddon = appDatabase.orderItemIngredientDao().view(itemIngredient.id);
                                            if (dbAddon == null) {
                                                itemIngredient._id = (int) appDatabase.orderItemIngredientDao().insert(itemIngredient);
                                            } else {
                                                itemIngredient._id = dbAddon._id;
                                                appDatabase.orderItemIngredientDao().update(itemIngredient);
                                            }
                                        }
                                    }

                                    ingredientTotal = ingredientTotal * orderItem.quantity;
                                    addonTotal = addonTotal * orderItem.quantity;
                                    myApp.appDatabase.orderItemDao().updateOrderItemTotal(_order_item_id, addonTotal, ingredientTotal);
                                }
                            }
                        }
                    }
                }

                Iterator<Order> iterator = newOnlineOrders.iterator();

                while (iterator.hasNext()) {
                    Order order = iterator.next();
                    if (isCancelled()) break;
                    Order dbOrder = appDatabase.orderDao().view(order.id);
                    if (dbOrder != null && dbOrder.updated_at != null &&
                            dbOrder.updated_at.toLowerCase().equalsIgnoreCase(order.updated_at.toLowerCase()) &&
                            dbOrder.total == order.total &&
                            dbOrder.total_paid == order.total_paid
                    ) {
                        LogUtils.e("Continuing...");
                        continue;
                    }
                    Customer dbCustomer = appDatabase.customerDao().view(order.customer_id);
                    int _customer_id;
                    if (dbCustomer == null) {
                        dbCustomer = order.customer;
                        _customer_id = (int) appDatabase.customerDao().insert(order.customer);
                    } else {
                        _customer_id = dbCustomer._id;
                        dbCustomer = order.customer;
                        dbCustomer._id = _customer_id;
                        appDatabase.customerDao().update(dbCustomer);
                        _customer_id = dbCustomer._id;
                    }
                    order._customer_id = _customer_id;
                    order.customer._id = _customer_id;

                    int _order_id;
                    order.customer_name = dbCustomer.name;
                    if (dbOrder == null) {
                        _order_id = (int) appDatabase.orderDao().insert(order);
                        order._id = _order_id;
                    } else {
                        order._id = dbOrder._id;
                        _order_id = dbOrder._id;
                        appDatabase.orderDao().update(order);
                    }


                    shouldNotDeleteThisIds = new ArrayList<>();
                    if (order.order_splits != null) {
                        for (OrderSplit split : order.order_splits) {
                            shouldNotDeleteThisIds.add(split.id);
                            split._order_id = _order_id;
                            OrderSplit dbSplit = appDatabase.orderSplitDao().view(split.id);
                            if (dbSplit == null) {
                                split._id = (int) appDatabase.orderSplitDao().insert(split);
                            } else {
                                split._id = dbSplit._id;
                                appDatabase.orderSplitDao().update(split);
                            }
                        }
                    }
                    appDatabase.orderSplitDao().deleteOtherIdsButNotThis(_order_id, shouldNotDeleteThisIds);

                    shouldNotDeleteThisIds = new ArrayList<>();
                    if (order.order_payments != null) {
                        for (OrderPayment payment : order.order_payments) {
                            shouldNotDeleteThisIds.add(payment.id);
                            payment._order_id = _order_id;
                            OrderPayment dbPayment = appDatabase.orderPaymentDao().view(payment.id);
                            if (dbPayment == null) {
                                payment._id = (int) appDatabase.orderPaymentDao().insert(payment);
                            } else {
                                payment._id = dbPayment._id;

                            }
                            if (!Validators.isNullOrEmpty(payment.order_split_id)) {
                                OrderSplit dbOrderSplit = appDatabase.orderSplitDao().view(payment.order_split_id);
                                if (dbOrderSplit != null)
                                    payment._order_split_id = dbOrderSplit._id;
                            }
                            appDatabase.orderPaymentDao().insert(payment);
                        }
                    }

                    appDatabase.orderPaymentDao().deleteOtherIdsButNotThis(_order_id, shouldNotDeleteThisIds);

                    if (order.order_items != null) {
                        appDatabase.orderItemDao().deleteAll(_order_id);
                        for (OrderItem orderItem : order.order_items) {
                            orderItem._order_id = _order_id;
                            OrderItem dbOrderItem = myApp.appDatabase.orderItemDao().view(orderItem.id);

                            if (!Validators.isNullOrEmpty(orderItem.order_split_id)) {
                                OrderSplit orderSplit = appDatabase.orderSplitDao().view(orderItem.order_split_id);
                                if (orderSplit != null) orderItem._order_split_id = orderSplit._id;
                            }

                            int _order_item_id = 0;
                            if (dbOrderItem == null) {
                                try {
                                    _order_item_id = (int) appDatabase.orderItemDao().insert(orderItem);
                                    orderItem._id = _order_item_id;
                                } catch (Exception e) {

                                }
                            } else {
                                _order_item_id = dbOrderItem._id;
                                orderItem._id = _order_item_id;
                                appDatabase.orderItemDao().update(orderItem);
                            }

                            float addonTotal = 0;
                            shouldNotDeleteThisIds = new ArrayList<>();
                            if (orderItem.order_item_addons != null) {
                                for (OrderItemAddon itemAddon : orderItem.order_item_addons) {
                                    shouldNotDeleteThisIds.add(itemAddon.id);
                                    itemAddon._order_item_id = _order_item_id;
                                    itemAddon.total = itemAddon.quantity * itemAddon.price;
                                    addonTotal = addonTotal + itemAddon.total;
                                    OrderItemAddon dbAddon = appDatabase.orderItemAddonDao().view(itemAddon.id);
                                    if (dbAddon == null) {
                                        itemAddon._id = (int) appDatabase.orderItemAddonDao().insert(itemAddon);
                                    } else {
                                        itemAddon._id = dbAddon._id;
                                        appDatabase.orderItemAddonDao().update(itemAddon);
                                    }
                                }
                            }
                            appDatabase.orderItemAddonDao().deleteOtherIdsButNotThis(_order_item_id, shouldNotDeleteThisIds);

                            float ingredientTotal = 0;
                            if (orderItem.order_item_ingredients != null) {
                                appDatabase.orderItemIngredientDao().deleteAll(_order_item_id);
                                for (OrderItemIngredient itemIngredient : orderItem.order_item_ingredients) {
                                    itemIngredient._order_item_id = _order_item_id;
                                    itemIngredient.total = itemIngredient.quantity * itemIngredient.price;
                                    ingredientTotal = ingredientTotal + itemIngredient.total;
                                    OrderItemIngredient dbAddon = appDatabase.orderItemIngredientDao().view(itemIngredient.id);
                                    if (dbAddon == null) {
                                        itemIngredient._id = (int) appDatabase.orderItemIngredientDao().insert(itemIngredient);
                                    } else {
                                        itemIngredient._id = dbAddon._id;
                                        appDatabase.orderItemIngredientDao().update(itemIngredient);
                                    }
                                }
                            }

                            ingredientTotal = ingredientTotal * orderItem.quantity;
                            addonTotal = addonTotal * orderItem.quantity;
                            myApp.appDatabase.orderItemDao().updateOrderItemTotal(_order_item_id, addonTotal, ingredientTotal);
                        }
                    } else {

                    }
                }
                if (isCancelled()) return null;

            }
            if (isCancelled()) return null;


            LogUtils.e("ORDERS", "IN background finish");
            return "Executed";
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            LogUtils.e("ORDERS", "Post Execute");
            myApp.shallWeRefreshOrders = true;
            if (getActivity() != null) {
                getActivity().runOnUiThread(HomeFragment.this::fetchEposOrdersCount);
            }
        }

        @Override
        protected void onCancelled() {
            super.onCancelled();
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {

                });
            }
        }
    }

    private void fetchEposOrdersCount() {
        new Thread(() -> {
            for (RecyclerMenuItem orderType : items) {
                if (!orderType.id.equalsIgnoreCase("0") && !orderType.id.equalsIgnoreCase("6")) {
                    orderType.count = appDatabase.orderDao().activeOrdersCount(orderType.id);
                } else if (orderType.id.equalsIgnoreCase("0")) {
                    orderType.count = appDatabase.orderDao().activeOrdersCount();
                }
            }
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    itemAdapter.notifyDataSetChanged();
                });
            }
        }).start();
    }

    private void setOnlineOrderCounts(Counts allCounts) {
        if (allCounts == null) return;
        for (RecyclerMenuItem menuItem : items) {
            if (menuItem.id.equalsIgnoreCase("6")) {
                menuItem.count = allCounts.pending_orders;
            }
        }

        itemAdapter.notifyDataSetChanged();
    }

    @Override
    public void onResume() {
        super.onResume();
        if (getActivity() != null) {
            LocalBroadcastManager.getInstance(getActivity()).registerReceiver(onlineOrdersNotificationReceiver, new IntentFilter(Constants.NOTIFICATION));
            LocalBroadcastManager.getInstance(getActivity()).registerReceiver(newOrderReceiver, new IntentFilter(Constants.EPOS_NEW_ORDER));
            LocalBroadcastManager.getInstance(getActivity()).registerReceiver(eposOrderUpdateReceiver, new IntentFilter(Constants.EPOS_ORDER_UPDATE));
            if (itemAdapter != null) {
                myApp.stopOrderSound();
                setOnlineOrderCounts(null);
            }
        }
        myApp.shallWeRefreshOrders = true;
        myApp.isReservationOpen = false;
    }

    @Override
    public void onPause() {
        super.onPause();
        if (onlineOrdersNotificationReceiver != null && getActivity() != null) {
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(onlineOrdersNotificationReceiver);
        }
        if (getActivity() != null && eposOrderUpdateReceiver != null) {
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(eposOrderUpdateReceiver);
        }
        if (getActivity() != null && newOrderReceiver != null) {
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(newOrderReceiver);
        }
        myApp.shallWeRefreshOrders = true;
    }

    private void changeFragment(Fragment fragment) {
        FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
        ft.replace(getId(), fragment);
        ft.addToBackStack("can_go_back");
        ft.commit();
    }

}
