package com.example.epos_2021.fragment;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.R;
import com.example.epos_2021.adapters.CategoryAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.models.Category;
import com.google.gson.Gson;

import java.util.ArrayList;

public class MainCategoryFragment extends BaseFragment {

    private RecyclerView rvCategory;
    private CategoryAdapter categoryAdapter;

    private ArrayList<Category> filteredItems = new ArrayList<>();
    private ArrayList<Category> categories = new ArrayList<>();


    public static MainCategoryFragment getInstance() {
        return new MainCategoryFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_main_category, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        fetchCategories();
    }

    @Override
    public void onResume() {
        super.onResume();
        LocalBroadcastManager.getInstance(getActivity()).registerReceiver(searchReceiver, new IntentFilter("search_receiver"));
    }

    private void initViews(View view) {
        rvCategory = view.findViewById(R.id.rvCategories);

        categoryAdapter = new CategoryAdapter(filteredItems, (position, data) -> {
            Category selectedCategory = (Category) data;
            //send brodcast
            LocalBroadcastManager.getInstance(getActivity()).sendBroadcast(new Intent("search_receiver").putExtra("category_data", new Gson().toJson(selectedCategory)));

        });
        rvCategory.setLayoutManager(new GridLayoutManager(getActivity(), 2, LinearLayoutManager.VERTICAL, false));
        rvCategory.setAdapter(categoryAdapter);

    }

    BroadcastReceiver searchReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String searchedItem = intent.getStringExtra("searched_item");

        }
    };


    private void fetchCategories() {
        categories.clear();
        categories.addAll(myApp.categories);
        filteredItems.clear();
        filteredItems.addAll(categories);
        // Set first floor item selected;
        if (categories.size() > 0) {
            categoryAdapter.selectedCategoryId = categories.get(0).id;
            Category selectedCategory = categories.get(0);
            //send brodcast
            LocalBroadcastManager.getInstance(getActivity()).sendBroadcast(new Intent("search_receiver").putExtra("category_data", new Gson().toJson(selectedCategory)));
        }
        categoryAdapter.notifyDataSetChanged();

    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(searchReceiver);
    }
}
