package com.example.epos_2021.fragment;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.DialogFragment;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.CustomerAutoSuggestAdapter;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.daos.AppDatabase;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.models.Customer;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderType;
import com.example.epos_2021.models.Table;
import com.example.epos_2021.models.TableStatus;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;
import com.google.android.material.chip.Chip;
import com.google.android.material.textfield.MaterialAutoCompleteTextView;
import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.List;


public class NewDineinOrderDialogFragment extends DialogFragment {

    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private DialogDismissListener dialogDismissListener;

    public static NewDineinOrderDialogFragment getInstance(Table table) {
        Bundle bundle = new Bundle();
        NewDineinOrderDialogFragment fragment = new NewDineinOrderDialogFragment();
        bundle.putString("table", new Gson().toJson(table));
        fragment.setArguments(bundle);
        return fragment;
    }

    protected void manageIntents() {
        if (getArguments() != null) {
            table = new Gson().fromJson(getArguments().getString("table"), Table.class);
        }
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    private Chip chipBack;
    private MaterialButton btnConfirm, btnCancel;
    private MaterialAutoCompleteTextView etName;
    private EditText etMobile, etEmail;
    private ArrayList<Customer> customers = new ArrayList<>();
    private CustomerAutoSuggestAdapter customerAutoSuggestAdapter;
    private Customer selectedCustomer;
    private Table table;
    private LinearLayout llMainLayout, llLoading;
    private ImageView ivSecDecrease, ivSecIncrease;
    private TextView tvModifyQty, tvTableNo;
    private AlertDialog progressBarDialog;
    private OrderType selectedOrderType;
    private AppDatabase appDatabase;
    private MyApp myApp = MyApp.getInstance();
    public TableStatus vacantStatus = myApp.findStatus("Vacant");
    public TableStatus takingOrderStatus = myApp.findStatus("Taking Order");

    private CheckBox signupForNewsletter;


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_new_dinein_order_dialog, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        appDatabase = MyApp.getInstance().appDatabase;
        manageIntents();
        initViews(view);
        setListeners();
        fetchCustomers();

        setUpOrderTypes();
    }

    private void initViews(View view) {
        progressBarDialog = CommonFunctions.customProgressDialog(getActivity());
        btnCancel = view.findViewById(R.id.btnCancel);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        etName = view.findViewById(R.id.etName);
        etMobile = view.findViewById(R.id.etMobile);
        etEmail = view.findViewById(R.id.etEmail);
        tvTableNo = view.findViewById(R.id.tvTableNo);
        tvModifyQty = view.findViewById(R.id.tvModifyQty);
        ivSecIncrease = view.findViewById(R.id.ivSecIncrease);
        ivSecDecrease = view.findViewById(R.id.ivSecDecrease);
        llMainLayout = view.findViewById(R.id.llMainLayout);
        llLoading = view.findViewById(R.id.llLoading);
        chipBack = view.findViewById(R.id.chipBack);

        signupForNewsletter = view.findViewById(R.id.signupForNewsletter);

        customerAutoSuggestAdapter = new CustomerAutoSuggestAdapter(getActivity(), customers);
        etName.setAdapter(customerAutoSuggestAdapter);

        tvTableNo.setText("Table No: " + table.number);
        etName.setThreshold(3);
    }

    int guestCount = 1;

    private void setListeners() {
        chipBack.setOnClickListener(view -> {
            dismiss();
        });

        btnConfirm.setOnClickListener(view -> {
            if (isValid()) {
                createOrderOffline();
            }

        });

        etName.setOnItemClickListener((adapterView, view, i, l) -> {
            int indext = customers.indexOf(adapterView.getItemAtPosition(i));
            if (indext != -1) {
                selectedCustomer = customers.get(indext);
                etName.setText(selectedCustomer.name);
                etMobile.setText(selectedCustomer.mobile);
                etEmail.setText(selectedCustomer.email);
                new Thread(() -> {
                    if (selectedCustomer._id == 0 && !Validators.isNullOrEmpty(selectedCustomer.id)) {
                        selectedCustomer._id = (int) appDatabase.customerDao().insert(selectedCustomer);
                    }
                }).start();
            }

        });

        ivSecIncrease.setOnClickListener(view -> {
            guestCount++;
            tvModifyQty.setText("" + guestCount);
        });

        ivSecDecrease.setOnClickListener(view -> {
            if (guestCount > 1) {
                guestCount--;
                tvModifyQty.setText("" + guestCount);
            }
        });
    }

    private boolean isValid() {


        return true;
    }

    Customer customer;


    private void setUpOrderTypes() {

        llLoading.setVisibility(View.GONE);
        llMainLayout.setVisibility(View.VISIBLE);

        selectedOrderType = new OrderType("1", "Dine in");
    }

    private void fetchCustomers() {
        if (MyApp.getInstance().isConnected(getActivity()) && "auto".equalsIgnoreCase(myApp.myPreferences.getOrderSyncMode())) {
            //fetchOnlineCustomers();
            new CustomersDbOperations().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, customers);
        } else {
            new CustomersDbOperations().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, customers);
        }
    }

    private void fetchOnlineCustomers() {
        AndroidNetworking.get(ApiEndPoints.customers)
                .addQueryParameter("nopaginate", "1")
                .build()
                .getAsObjectList(Customer.class, new ParsedRequestListener<List<Customer>>() {
                    @Override
                    public void onResponse(List<Customer> response) {
                        customerAutoSuggestAdapter.updateData(response);
                        new CustomersDbOperations().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, (ArrayList<Customer>) response);
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        }
                        new CustomersDbOperations().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, customers);
                    }
                });

    }

    private void createOrderOffline() {
        Order order = new Order();
        Customer customer;
        if (selectedCustomer != null) {
            customer = selectedCustomer;
        } else {
            customer = new Customer();
        }

        //1->true , 0->false
        if(signupForNewsletter.isChecked())
            customer.news_letter = "1";
        else
            customer.news_letter = "0";

        //Add By SP
        if (MyApp.getInstance().myPreferences.getRemoteDeviceEnableStatus()) {
            order.print_id = 1;
        } else {
            order.print_id = 0;
        }
        //Toast.makeText(getActivity(), "print_id:"+order.print_id, Toast.LENGTH_SHORT).show();

        customer.name = etName.getText().toString();
        customer.mobile = etMobile.getText().toString();
        customer.email = etEmail.getText().toString();

        order.customer_id = customer.id;
        order.table_id = table.id;
        order.table_number = table.number;
        order.no_guest = String.valueOf(guestCount);
        order.customer = customer;
        order.customer_name = customer.name;
        order.updater_id = MyApp.getInstance().myPreferences.getLoggedInUser().id;
        order.order_status_id = "1";
        order.order_status = "Taking Order";
        order.order_type_id = selectedOrderType.id;
        order.order_items = new ArrayList<>();
        order.device_id = MyApp.getInstance().myPreferences.getRegisteredDevice().id;
        order.created_at = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
        order.updated_at = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
        order.delivery_date = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);


        new Thread(() -> {
            // Update Customer
            selectedCustomer = customer;
            if (selectedCustomer._id == 0)
                selectedCustomer._id = (int) appDatabase.customerDao().insert(selectedCustomer);
            else {
                appDatabase.customerDao().insert(selectedCustomer);
            }
            order._customer_id = selectedCustomer._id;
            order.customer_name = selectedCustomer.name;
            // Save Order
            order._id = (int) MyApp.getInstance().appDatabase.orderDao().insert(order);
            table._last_order_id = order._id;
            table.last_order_id = null;
            table.table_status_id = takingOrderStatus.id;
            table.status = takingOrderStatus.status;
            appDatabase.tableDao().insert(table);
            if (MyApp.getInstance().isConnected(getActivity()) && "auto".equalsIgnoreCase(MyApp.getInstance().myPreferences.getOrderSyncMode())) {
                createOrderOnline(order);
            } else {
                changeTableStatus(order, takingOrderStatus.id, takingOrderStatus.status);
                dialogDismissListener.onDialogDismiss(order);
                dismiss();
            }

        }).start();
    }

    private void createOrderOnline(Order order) {

        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                progressBarDialog.show();
            });
        }
        myApp.shallWeRefreshOrders = false;
        AndroidNetworking.post(ApiEndPoints.orders)
                .addApplicationJsonBody(order)
                .build()
                .getAsObject(Order.class, new ParsedRequestListener<Order>() {
                    @Override
                    public void onResponse(Order response) {
                        response.customer._id = selectedCustomer._id;
                        response._id = order._id;
                        response._customer_id = selectedCustomer._id;
                        new Thread(() -> {
                            // Update Customer
                            changeTableStatus(order, takingOrderStatus.id, takingOrderStatus.status);
                        }).start();
                        new UpdateOrderWithOnline(response).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                progressBarDialog.dismiss();
                            });
                        }
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        }
                    }
                });

    }

    private void changeTableStatus(Order order, String table_status_id, String status) {
        if (!order.order_type_id.equals("1")) {
            return;
        }

        new Thread(() -> {
            Table table = appDatabase.tableDao().view(order.table_id);

            table.locked = true;
            table.table_status_id = table_status_id;
            table.status = status;

            appDatabase.tableDao().insert(table);

        }).start();
    }

    private class CustomersDbOperations extends AsyncTask<ArrayList<Customer>, Void, String> {

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
        }

        @Override
        protected String doInBackground(ArrayList<Customer>... lists) {
            for (Customer customer : lists[0]) {
                Customer dbCustomer = appDatabase.customerDao().view(customer.id);
                if (dbCustomer == null) {
                    appDatabase.customerDao().insert(customer);
                } else {
                    appDatabase.customerDao().update(customer);
                }
            }
            customers.clear();
            customers.addAll(appDatabase.customerDao().listNoNull());

            return "Executed";
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    progressBarDialog.dismiss();
                    customerAutoSuggestAdapter.updateData(customers);
                });
            }
        }
    }

    private class UpdateOrderWithOnline extends AsyncTask<String, String, String> {
        Order onlineorder;

        public UpdateOrderWithOnline(Order onlineorder) {
            this.onlineorder = onlineorder;
        }

        @Override
        protected String doInBackground(String... strings) {
            appDatabase.customerDao().update(onlineorder.customer);
            appDatabase.orderDao().insert(onlineorder);
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            myApp.shallWeRefreshOrders = true;
            dialogDismissListener.onDialogDismiss(onlineorder);
            dismiss();
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    progressBarDialog.dismiss();
                });
            }
        }
    }

    // changed by SP
    private void printFunction(){

    }
}