package com.example.epos_2021.fragment;

import android.graphics.Typeface;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatButton;
import androidx.cardview.widget.CardView;
import androidx.fragment.app.DialogFragment;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.google.android.material.button.MaterialButton;

public class NumberpadWithFlatPercetnageDialog extends DialogFragment {

    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private DialogDismissListener dialogDismissListener;

    public static NumberpadWithFlatPercetnageDialog getInstance(float total) {
        NumberpadWithFlatPercetnageDialog fragment = new NumberpadWithFlatPercetnageDialog();
        Bundle bundle = new Bundle();
        bundle.putFloat("total", total);
        fragment.setArguments(bundle);
        return fragment;
    }

    public static NumberpadWithFlatPercetnageDialog getInstance(float total, String title) {
        NumberpadWithFlatPercetnageDialog fragment = new NumberpadWithFlatPercetnageDialog();
        Bundle bundle = new Bundle();
        bundle.putFloat("total", total);
        bundle.putString("title", title);
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    private TextView tvTitle;
    private Button btn1, btn2, btn3, btn4, btn5, btn6, btn7, btn8, btn9, btn0, btnDot;
    private AppCompatButton btnLogin, btnClear;
    private MaterialButton btnConfirm, btnCancel;
    private CardView cvDot;
    private EditText etPercentage, etFlatAmount;
    private boolean flatAmountSelected = true;
    private boolean delPressed = false;
    private float orderTotal = 10.00f;
    private String title;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_gratuity_dialog, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            orderTotal = getArguments().getFloat("total", 0);
            title = getArguments().getString("title");
        }
        initViews(view);
        setListeners();
    }

    private void initViews(View view) {
        tvTitle = view.findViewById(R.id.tvTitle);
        btnCancel = view.findViewById(R.id.btnCancel);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnLogin = view.findViewById(R.id.btn_login);
        btnClear = view.findViewById(R.id.btnClear);
        btn1 = view.findViewById(R.id.btn1);
        btn2 = view.findViewById(R.id.btn2);
        btn3 = view.findViewById(R.id.btn3);
        btn4 = view.findViewById(R.id.btn4);
        btn5 = view.findViewById(R.id.btn5);
        btn6 = view.findViewById(R.id.btn6);
        btn7 = view.findViewById(R.id.btn7);
        btn8 = view.findViewById(R.id.btn8);
        btn9 = view.findViewById(R.id.btn9);
        btn0 = view.findViewById(R.id.btn0);
        btnDot = view.findViewById(R.id.btnDot);
        etFlatAmount = view.findViewById(R.id.etFlatAmount);
        etPercentage = view.findViewById(R.id.etPercentage);
        cvDot = view.findViewById(R.id.cvDot);

        cvDot.setVisibility(View.GONE);
        btnLogin.setVisibility(View.GONE);

        if (!Validators.isNullOrEmpty(title)) {
            tvTitle.setText(title);
        }
        etFlatAmount.setTypeface(Typeface.defaultFromStyle(Typeface.BOLD));
        etPercentage.setTypeface(Typeface.defaultFromStyle(Typeface.NORMAL));
    }

    private void setListeners() {

        etPercentage.setOnClickListener(v -> {
            flatAmountSelected = false;
            etPercentage.setTypeface(Typeface.defaultFromStyle(Typeface.BOLD));
            etFlatAmount.setTypeface(Typeface.defaultFromStyle(Typeface.NORMAL));
        });

        etFlatAmount.setOnClickListener(v -> {
            flatAmountSelected = true;
            etFlatAmount.setTypeface(Typeface.defaultFromStyle(Typeface.BOLD));
            etPercentage.setTypeface(Typeface.defaultFromStyle(Typeface.NORMAL));
        });

        etPercentage.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                delPressed = count == 0;

                if (etPercentage.getText().toString().equals("0") || etPercentage.getText().toString().equals("")) {
                    etPercentage.setText("0.00%");
                    etPercentage.setSelection(etPercentage.getText().length());
                    etPercentage.addTextChangedListener(this);
                    return;
                }
                if (!flatAmountSelected) {
                    double percentageValue = Double.parseDouble(etPercentage.getText().toString());
                    if (!delPressed) {
                        percentageValue = percentageValue * 10.0d;
                    } else if (percentageValue > 0.0d) {
                        percentageValue = percentageValue / 10.0d;
                    }
                    etPercentage.removeTextChangedListener(this);
                    etPercentage.setText(MyApp.df.format(percentageValue));
                    etPercentage.setSelection(etPercentage.getText().length());
                    etPercentage.addTextChangedListener(this);
                    double flatAmount = 0;
                    if (orderTotal > 0) flatAmount = orderTotal / 100;
                    flatAmount = percentageValue * flatAmount;

                    try {
                        etFlatAmount.setText(MyApp.df.format(flatAmount));
                        etFlatAmount.setSelection(etFlatAmount.getText().length());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        etFlatAmount.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                delPressed = count == 0;
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (etFlatAmount.getText().toString().equals("0") || etFlatAmount.getText().toString().equals("")) {
                    etFlatAmount.removeTextChangedListener(this);
                    etFlatAmount.setText("0.00");
                    etPercentage.setText("0.00");
                    etFlatAmount.setSelection(etFlatAmount.getText().length());
                    etFlatAmount.addTextChangedListener(this);
                    return;
                }

                if (flatAmountSelected) {
                    double val = Double.parseDouble(etFlatAmount.getText().toString());
                    if (!delPressed) {
                        val = val * 10.0d;
                    } else if (val > 0.0d) {
                        val = val / 10.0d;
                    }
                    etFlatAmount.removeTextChangedListener(this);
                    etFlatAmount.setText(MyApp.df.format(val));
                    etFlatAmount.setSelection(etFlatAmount.getText().length());
                    etFlatAmount.addTextChangedListener(this);
                    try {
                        float percentage;
                        float flatValue = Float.parseFloat("" + val);
                        if (orderTotal == 0) {
                            percentage = 0;
                        } else {
                            percentage = (flatValue * 100 / orderTotal);
                        }
                        double percentageValue = Double.parseDouble("" + percentage);

                        etPercentage.setText(MyApp.df.format(percentageValue));
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

            }
        });

        btnCancel.setOnClickListener(view -> {
            dismiss();
        });

        btnConfirm.setOnClickListener(view -> {
            dialogDismissListener.onDialogDismiss(Float.parseFloat(etFlatAmount.getText().toString()));
            dismiss();
        });

        btn1.setOnClickListener(v -> {
            setValueToEditText("1");

        });

        btn2.setOnClickListener(v -> {
            setValueToEditText("2");

        });

        btn3.setOnClickListener(v -> {
            setValueToEditText("3");

        });

        btn4.setOnClickListener(v -> {
            setValueToEditText("4");

        });

        btn5.setOnClickListener(v -> {
            setValueToEditText("5");

        });

        btn6.setOnClickListener(v -> {
            setValueToEditText("6");

        });

        btn7.setOnClickListener(v -> {
            setValueToEditText("7");

        });

        btn8.setOnClickListener(v -> {
            setValueToEditText("8");

        });

        btn9.setOnClickListener(v -> {
            setValueToEditText("9");

        });

        btn0.setOnClickListener(v -> {
            setValueToEditText("0");

        });

        btnDot.setOnClickListener(v -> {
            setValueToEditText(".");

        });

        btnClear.setOnClickListener(v -> {
            etPercentage.setText("0.00");
            etFlatAmount.setText("0");
        });

    }

    private void setValueToEditText(String value) {
        if (flatAmountSelected) {
            etFlatAmount.setText(etFlatAmount.getText().toString() + value);
        } else {
            etPercentage.setText(etPercentage.getText().toString() + value);
        }

    }

}