package com.example.epos_2021.fragment;

import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.example.epos_2021.R;
import com.example.epos_2021.adapters.OrderPaymentsAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderPayment;

import java.util.ArrayList;
import java.util.concurrent.Callable;

public class OrderPaymentsFragment extends BaseFragment {

    public static OrderPaymentsFragment getInstance(String order_id) {
        OrderPaymentsFragment fragment = new OrderPaymentsFragment();
        Bundle bundle = new Bundle();
        bundle.putString("order_id", order_id);
        fragment.setArguments(bundle);
        return fragment;
    }

    public static OrderPaymentsFragment getInstance() {
        return new OrderPaymentsFragment();
    }

    private RecyclerView rvOrderPayments;
    private ArrayList<OrderPayment> orderPayments = new ArrayList<>();
    private OrderPaymentsAdapter orderPaymentsAdapter;
    private SwipeRefreshLayout swipeRefreshLayout;
    private String order_id;
    private int _order_id, _order_split_id;
    private TextView tvMessage;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_order_payments, container, false);
    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getArguments() != null) {
            order_id = getArguments().getString("order_id");
            _order_id = getArguments().getInt("_order_id");
            _order_split_id = getArguments().getInt("_order_split_id");
        }
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        initViews(view);
        swipeRefreshLayout.setOnRefreshListener(() -> {
            new PaymentHistoryAsync(() -> null).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
        });
        new PaymentHistoryAsync(() -> null).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private void initViews(View view) {
        tvMessage = view.findViewById(R.id.tvMessage);
        rvOrderPayments = view.findViewById(R.id.rvOrderPayments);
        rvOrderPayments.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        orderPaymentsAdapter = new OrderPaymentsAdapter(orderPayments, false);
        rvOrderPayments.setAdapter(orderPaymentsAdapter);

    }

    private class PaymentHistoryAsync extends AsyncTask<String, String, String> {
        private Callable<Void> nextCall;

        public PaymentHistoryAsync(Callable<Void> nextCall) {
            this.nextCall = nextCall;

        }

        @Override
        protected String doInBackground(String... strings) {
            orderPayments.clear();
            Order order = appDatabase.orderDao().view(_order_id);
            if (order != null && order.discount > 0) {
                OrderPayment orderPayment = new OrderPayment();
                orderPayment.payment_method_name = "Discount";
                orderPayment.amount = order.discount;
                orderPayments.add(orderPayment);
            }

            if (_order_split_id > 0) {
                orderPayments.addAll(appDatabase.orderPaymentDao().listBySplit(_order_split_id));
            } else {
                orderPayments.addAll(appDatabase.orderPaymentDao().list(_order_id));
            }
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            orderPaymentsAdapter.notifyDataSetChanged();
            swipeRefreshLayout.setRefreshing(false);
        }
    }
}