package com.example.epos_2021.fragment;

import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.CallLogs;
import com.example.epos_2021.models.SiteSetting;
import com.google.android.material.card.MaterialCardView;
import com.google.gson.Gson;

public class OrderTypeSelectionDialogFragment extends DialogFragment {

    private MaterialCardView cvWaiting, cvCollection, cvDelivery, cvBar, cvReservation;
    private DialogDismissListener dialogDismissListener;

    private CallLogs callLogs;
    private MyApp myApp = MyApp.getInstance();

    @Override
    public int getTheme() {
        return super.getTheme();
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(android.graphics.Color.TRANSPARENT));
    }


    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    public static OrderTypeSelectionDialogFragment getInstance() {
        return new OrderTypeSelectionDialogFragment();
    }

    public static OrderTypeSelectionDialogFragment getInstance(CallLogs callLogs) {
        OrderTypeSelectionDialogFragment fragment = new OrderTypeSelectionDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("calllogs", new Gson().toJson(callLogs));
        fragment.setArguments(bundle);
        return fragment;
    }


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_order_type_dialog, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (getArguments() != null) {
            callLogs = new Gson().fromJson(getArguments().getString("calllogs"), CallLogs.class);
        }
        initViews(view);
        setListeners();
        setUpOrderTypes();
    }

    private void setUpOrderTypes() {
        //1 = Dine In, 2= Collection, 3 = Delivery, 4 = Bar, 5= Waiting, 6 Web Order
        cvBar.setVisibility(View.GONE);

        SiteSetting collectionSetting = myApp.findSetting("is_collection");
        SiteSetting waitingSetting = myApp.findSetting("is_waiting");
        SiteSetting deliverySetting = myApp.findSetting("is_delivery");
        SiteSetting reservationSetting = myApp.findSetting("is_reservation");

        if (collectionSetting != null && (collectionSetting.value.equals("true") || collectionSetting.value.equals("1")) && (MyApp.userPermission != null && MyApp.userPermission.collection.actions.list)) {
            cvCollection.setVisibility(View.VISIBLE);
        } else {
            cvCollection.setVisibility(View.GONE);
        }
        if (deliverySetting != null && (deliverySetting.value.equals("true") || deliverySetting.value.equals("1")) && (MyApp.userPermission != null && MyApp.userPermission.delivery.actions.list)) {
            cvDelivery.setVisibility(View.VISIBLE);
        } else {
            cvDelivery.setVisibility(View.GONE);
        }
        if (waitingSetting != null && (waitingSetting.value.equals("true") || waitingSetting.value.equals("1")) && (MyApp.userPermission != null && MyApp.userPermission.waiting.actions.list)) {
            cvWaiting.setVisibility(View.VISIBLE);
        } else {
            cvWaiting.setVisibility(View.GONE);
        }

        if (callLogs != null && reservationSetting != null && (reservationSetting.value.equals("true") || reservationSetting.value.equals("1")) && MyApp.userPermission != null && MyApp.userPermission.reservation.actions.list) {
            cvReservation.setVisibility(View.VISIBLE);
        }

    }

    private void setListeners() {
        cvCollection.setOnClickListener(v -> dismissDialog("2"));
        cvDelivery.setOnClickListener(v -> dismissDialog("3"));
        cvWaiting.setOnClickListener(v -> dismissDialog("5"));
        cvReservation.setOnClickListener(v -> {
            if (dialogDismissListener != null) {
                dialogDismissListener.onDialogDismiss(callLogs);
                dismiss();
            }
        });
    }

    private void dismissDialog(String id) {
        if (dialogDismissListener != null) {
            dialogDismissListener.onDialogDismiss(id);
            dismiss();
        }
    }

    private void initViews(View view) {
        cvWaiting = view.findViewById(R.id.cvWaiting);
        cvCollection = view.findViewById(R.id.cvCollection);
        cvDelivery = view.findViewById(R.id.cvDelivery);
        cvBar = view.findViewById(R.id.cvBar);
        cvReservation = view.findViewById(R.id.cvReservation);
    }
}
