package com.example.epos_2021.fragment;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.SystemClock;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.view.ContextThemeWrapper;
import androidx.core.app.ActivityCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.activities.InternetCardReaderPaymentActivity;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.MoneyTextWatcher;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.comman.printer.SunmiPrinter;
import com.example.epos_2021.comman.printer.WifiPrinter;
import com.example.epos_2021.comman.printer.ZoneRichPrinter;
import com.example.epos_2021.merchant.fragments.StripeBluetoothCardReaderPaymentFragment;
import com.example.epos_2021.models.Admin;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.models.Business;
import com.example.epos_2021.models.BusinessCardReader;
import com.example.epos_2021.models.Printer;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.LogUtils;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;
import com.stripe.android.ApiResultCallback;
import com.stripe.android.PaymentConfiguration;
import com.stripe.android.Stripe;
import com.stripe.android.model.PaymentIntent;
import com.stripe.exception.StripeException;
import com.stripe.model.PaymentMethod;

import org.jetbrains.annotations.NotNull;
import org.json.JSONException;
import org.json.JSONObject;

import java.math.BigDecimal;
import java.util.HashMap;

import io.mpos.accessories.AccessoryFamily;
import io.mpos.accessories.parameters.AccessoryParameters;
import io.mpos.provider.ProviderMode;
import io.mpos.transactions.Currency;
import io.mpos.transactions.Transaction;
import io.mpos.transactions.parameters.TransactionParameters;
import io.mpos.ui.shared.MposUi;

public class PaymentCardReaderFragment extends BaseFragment {

    private MposUi mposUi;
    private EditText etAmount, etTip;
    private MaterialButton btnCancel, btnPaynow;
    private float tipAmount = 0, amount = 0;
    private ZoneRichPrinter zoneRichPrinter;
    private SunmiPrinter sunmiPrinter;
    private WifiPrinter wifiPrinter;
    private Admin loggedInAdmin = myPreferences.getLoggedInAdmin();
    private Business selectedBusiness = loggedInAdmin.selected_business;

    private String header = "", resAddress = "";
    long lastClickTime = 0;
    final long MIN_CLICK_INTERVAL = 25000; //in millis
    private AlertDialog progressBarDialog;
    private Printer defaultPrinter;
    private TextView tvTip;
    private int headerAlignment = 0;
    private SiteSetting headerAlignmentSetting;
    SiteSetting footerASetting = myApp.findSetting("footer_a");
    SiteSetting footerBSetting = myApp.findSetting("footer_b");
    SiteSetting ticketHeader = myApp.findSetting("ticket_header");
    SiteSetting address = myApp.findSetting("address");

    private BusinessCardReader selectedCardReader;
    private String stripe_mode, stripe_private_key, stripe_public_key;
    private Stripe stripe;

    public static PaymentCardReaderFragment getInstance() {
        return new PaymentCardReaderFragment();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_payment_card_reader, container, false);
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();
        setCardReader();

    }

    private void changeFragment(Fragment fragment) {
        FragmentManager manager = getChildFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.flCardReader, fragment);
        transaction.commit();
        getActivity().invalidateOptionsMenu();
    }

    private void initViews(View view) {
        progressBarDialog = CommonFunctions.customProgressDialog(getActivity());
        defaultPrinter = myPreferences.getDefaultPrinter();
        zoneRichPrinter = new ZoneRichPrinter(getActivity());
        zoneRichPrinter = myApp.connectZonerich(zoneRichPrinter, defaultPrinter);
        wifiPrinter = WifiPrinter.getInstance();
        sunmiPrinter = new SunmiPrinter(getActivity());
        btnPaynow = view.findViewById(R.id.btnPaynow);
        btnCancel = view.findViewById(R.id.btnCancel);
        etTip = view.findViewById(R.id.etTip);
        tvTip = view.findViewById(R.id.tvTip);
        etAmount = view.findViewById(R.id.etAmount);

        selectedCardReader = MyApp.getInstance().myPreferences.getDefaultCardReader();
        headerAlignmentSetting = myApp.findSetting("ticket_header_type");
        if (headerAlignmentSetting != null && headerAlignmentSetting.value.equalsIgnoreCase("right")) {
            headerAlignment = 2;
        }
        if (headerAlignmentSetting != null && headerAlignmentSetting.value.equalsIgnoreCase("center")) {
            headerAlignment = 1;
        }
        if (!selectedBusiness.tip) {
            // Hide tip layout
            tvTip.setVisibility(View.GONE);
            etTip.setVisibility(View.GONE);
        }

        btnPaynow.setEnabled(isValid());
    }

    private void initMpos() {
        if (selectedBusiness.stripe_mode.equalsIgnoreCase("live")) {
            mposUi = MposUi.initialize(getActivity(), ProviderMode.LIVE,
                    selectedCardReader.merchant_key, selectedCardReader.merchant_secret);
        } else {
            mposUi = MposUi.initialize(getActivity(), ProviderMode.MOCK,
                    selectedCardReader.merchant_key, selectedCardReader.merchant_secret);
        }
    }

    float totalAmount = 0;

    private void setUpMposTerminal() {
        if (selectedBusiness.card_readers != null && selectedBusiness.card_readers.size() == 1) {
            selectedCardReader = selectedBusiness.card_readers.get(0);
        }
        if (selectedCardReader != null) {

            if (selectedCardReader.connectivity.toLowerCase().equalsIgnoreCase("internet") || selectedCardReader.connectivity.toLowerCase().equalsIgnoreCase("stripebluetooth") || selectedCardReader.connectivity.toLowerCase().equalsIgnoreCase("posbluetooth")) {
                stripe_mode = selectedBusiness.stripe_mode;

                if (!Validators.isNullOrEmpty(selectedBusiness.s_account_id) && selectedBusiness.connect_service && !Validators.isNullOrEmpty(selectedBusiness.connect_stripe_public_key)) {
                    stripe_private_key = selectedBusiness.connect_stripe_private_key;
                    stripe_public_key = selectedBusiness.connect_stripe_public_key;
                } else {
                    if (stripe_mode.equalsIgnoreCase("live")) {
                        stripe_private_key = selectedBusiness.stripe_private_key_live;
                        stripe_public_key = selectedBusiness.stripe_public_key_live;
                    } else {
                        stripe_private_key = selectedBusiness.stripe_private_key_test;
                        stripe_public_key = selectedBusiness.stripe_public_key_test;
                    }
                }
                PaymentConfiguration.init(getActivity(), stripe_public_key);
                stripe = new Stripe(getActivity(), stripe_public_key);
                openStripeReaders(selectedCardReader);
            } else {
                initMpos();
                AccessoryParameters accessoryParameters;
                if (!Validators.isNullOrEmpty(selectedCardReader.ip)) {
                    accessoryParameters = new AccessoryParameters.Builder(AccessoryFamily.MIURA_MPI)
                            .tcp(selectedCardReader.ip, Integer.parseInt(selectedCardReader.port))
                            .build();
                } else {
                    accessoryParameters = new AccessoryParameters.Builder(AccessoryFamily.MIURA_MPI)
                            .bluetooth()
                            .build();

                }
                mposUi.getConfiguration().setTerminalParameters(accessoryParameters);
                if (selectedBusiness != null) {
                    String descriptor = selectedBusiness.statement_descriptor;
                    if (!Validators.isNullOrEmpty(selectedBusiness.s_account_id) && selectedBusiness.connect_service) {
                        descriptor = selectedBusiness.statement_descriptor;
                    }
                    if (!Validators.isNullOrEmpty(selectedBusiness.s_account_id) && selectedBusiness.connect_service) {
                        calculateApplicationFee();
                    } else {
                        TransactionParameters transactionParameters = new TransactionParameters.Builder()
                                .charge(new BigDecimal("" + totalAmount), Currency.GBP)
                                .statementDescriptor(descriptor)
                                .subject(selectedBusiness.statement_description)
                                .autoCapture(true)
                                .build();

                        Intent intent = mposUi.createTransactionIntent(transactionParameters);
                        startActivityForResult(intent, Constants.PAYMENT_MPOS_CARDREADER);
                    }
                } else {
                    ToastUtils.makeSnackToast(getActivity(), "Merchant credentials not found");
                }
            }

        } else {
            setCardReader();
        }
    }

    private void setCardReader() {
        SelectCardReaderFragment selectCardReaderDialogFragment = SelectCardReaderFragment.getInstance();
        changeFragment(selectCardReaderDialogFragment);
        selectCardReaderDialogFragment.setDialogDismissListener(o -> {
            if (o instanceof BusinessCardReader) {
                selectedCardReader = (BusinessCardReader) o;
                btnPaynow.setEnabled(isValid());
                myPreferences.saveDefaultCardReader(selectedCardReader);
            } else {
                getActivity().onBackPressed();
            }
        });
    }

    private void openStripeReaders(BusinessCardReader selectedCardReader) {
        if (selectedCardReader.s_location_id == null) {
            selectedCardReader.s_location_id = selectedBusiness.s_location_id;
        }
        if (selectedCardReader.connectivity.toLowerCase().equalsIgnoreCase("internet")) {
            String url = ApiEndPoints.CARD_READER_PAYMENT + "?s_location_id=" + selectedCardReader.s_location_id + "&s_terminal_id=" + selectedCardReader.s_terminal_id + "&amount=" + (totalAmount * 100) + "&business_id=" + selectedBusiness.id;
            startActivityForResult(new Intent(getActivity(), InternetCardReaderPaymentActivity.class).putExtra("url", url), Constants.PAYMENT_WEBVIEW_CARDREADER);
        } else if (selectedCardReader.connectivity.toLowerCase().toLowerCase().equalsIgnoreCase("stripebluetooth") || selectedCardReader.connectivity.toLowerCase().toLowerCase().equalsIgnoreCase("posbluetooth")) {
            askForPermission();
        }
    }

    private void askForPermission() {
        final int locationPermission = ActivityCompat.checkSelfPermission(getActivity(), Manifest.permission.ACCESS_FINE_LOCATION);

        if (locationPermission == PackageManager.PERMISSION_GRANTED) {
            if (verifyGpsEnabled()) {
                showStripeBluetoothReaderDialog();
            }
        } else {
            // If we don't have them yet, request them before doing anything else
            final String[] permissions = {Manifest.permission.ACCESS_FINE_LOCATION};
            requestPermissions(permissions, Constants.PERMISSION_CODE_LOCATION);
        }
    }

    private void showStripeBluetoothReaderDialog() {
        StripeBluetoothCardReaderPaymentFragment stripeBluetoothCardReaderPaymentFragment = StripeBluetoothCardReaderPaymentFragment.getInstance(selectedCardReader, totalAmount);
        stripeBluetoothCardReaderPaymentFragment.show(getChildFragmentManager(), "card_reader");
        stripeBluetoothCardReaderPaymentFragment.setCancelable(false);
        stripeBluetoothCardReaderPaymentFragment.setDialogDismissListener(o -> {
            etAmount.addTextChangedListener(new MoneyTextWatcher(etAmount));
            etTip.addTextChangedListener(new MoneyTextWatcher(etTip));
            resetBox();
            if (o instanceof com.stripe.stripeterminal.external.models.PaymentIntent) {
                retrivePaymentIntentAndPrint(((com.stripe.stripeterminal.external.models.PaymentIntent) o).getClientSecret());
            } else if (o instanceof String) {
                myApp.startPaymentFailedSound();
                ToastUtils.makeSnackToast(getActivity(), (String) o);
            } else {
                myApp.startPaymentFailedSound();
                ToastUtils.makeSnackToast(getActivity(), "Transaction was declined, aborted, or failed");

            }
        });
    }

    public static boolean isGpsEnabled(Context context) {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) {
            String providers = Settings.Secure.getString(context.getContentResolver(),
                    Settings.Secure.LOCATION_PROVIDERS_ALLOWED);
            if (TextUtils.isEmpty(providers)) {
                return false;
            }
            return providers.contains(LocationManager.GPS_PROVIDER);
        } else {
            final int locationMode;
            try {
                locationMode = Settings.Secure.getInt(context.getContentResolver(),
                        Settings.Secure.LOCATION_MODE);
            } catch (Settings.SettingNotFoundException e) {
                e.printStackTrace();
                return false;
            }
            switch (locationMode) {

                case Settings.Secure.LOCATION_MODE_HIGH_ACCURACY:
                case Settings.Secure.LOCATION_MODE_SENSORS_ONLY:
                    return true;
                case Settings.Secure.LOCATION_MODE_BATTERY_SAVING:
                case Settings.Secure.LOCATION_MODE_OFF:
                default:
                    return false;
            }
        }
    }

    private boolean verifyGpsEnabled() {
        if (isGpsEnabled(getActivity())) {
            LogUtils.e("RETURNING TRUE");
            return true;
        }
        final LocationManager locationManager = (LocationManager) getActivity().getSystemService(Context.LOCATION_SERVICE);

        boolean gpsEnabled = false;
        if (locationManager == null) {
            ToastUtils.makeToast(getActivity(), "Location manager null");
        }
        try {
            gpsEnabled = locationManager != null &&
                    locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
        } catch (Exception exception) {
            ToastUtils.makeLongToast(getActivity(), exception.getMessage());
            exception.printStackTrace();
        }

        if (!gpsEnabled) {
            // notify user
            new AlertDialog.Builder(new ContextThemeWrapper(getActivity(), R.style.Theme_MaterialComponents_DayNight_DarkActionBar))
                    .setMessage("Please enable location services")
                    .setCancelable(false)
                    .setPositiveButton("Open location settings", (dialog, which) -> {
                        startActivity(new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS));
//                        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) {
//                            turnGPSOn();
//                        } else {
//                            startActivity(new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS));
//                        }
                    })
                    .create()
                    .show();
        }

        return gpsEnabled;
    }

    private void turnGPSOn() {
        String provider = Settings.Secure.getString(getActivity().getContentResolver(), Settings.Secure.LOCATION_PROVIDERS_ALLOWED);
        if (!provider.contains("gps")) {
            final Intent poke = new Intent();
            poke.setClassName("com.android.settings", "com.android.settings.widget.SettingsAppWidgetProvider");
            poke.addCategory(Intent.CATEGORY_ALTERNATIVE);
            poke.setData(Uri.parse("3"));
            getActivity().sendBroadcast(poke);
        }
    }

    public static boolean checkGPSStatus(Context context) {
        LocationManager manager = (LocationManager) context.getSystemService(Context.LOCATION_SERVICE);
        return manager.isProviderEnabled(LocationManager.GPS_PROVIDER);
    }


    private HashMap<String, String> generateFeeParams() {
        HashMap<String, String> params = new HashMap<>();

        params.put("amount", MyApp.df.format(totalAmount * 100));
        params.put("currency", "GBP");
        params.put("transaction_type", !selectedBusiness.connect_service ? "merchant" : "connect");
        params.put("business_id", selectedBusiness.id);
        return params;
    }

    private void calculateApplicationFee() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                progressBarDialog.show();
                etAmount.setText(null);

            });
        }
        AndroidNetworking.post(ApiEndPoints.APPLICATION_FEE)
                .addBodyParameter(generateFeeParams())
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressBarDialog.dismiss());
                        }
                        try {
                            if (response.has("payment_intent_id")) {
                                String paymentSecret = response.getString("payment_intent_id");
                                Intent intent = mposUi.createTransactionIntent(paymentSecret);
                                startActivityForResult(intent, Constants.PAYMENT_MPOS_CARDREADER);
                            } else if (response.has("fee")) {
                                float fee = Float.parseFloat(response.getString("fee"));
                                TransactionParameters transactionParameters = new TransactionParameters.Builder()
                                        .charge(new BigDecimal("" + totalAmount), Currency.GBP)
                                        .statementDescriptor(selectedBusiness.connect_service ? selectedBusiness.statement_descriptor : selectedBusiness.statement_descriptor)
                                        .applicationFee(new BigDecimal(fee))
                                        .subject(selectedBusiness.statement_description)
                                        .autoCapture(true)
                                        .build();

                                Intent intent = mposUi.createTransactionIntent(transactionParameters);
                                startActivityForResult(intent, Constants.PAYMENT_MPOS_CARDREADER);
                            }
                        } catch (JSONException e) {
                            e.printStackTrace();
                            ToastUtils.makeSnackToast(getActivity(), "Something went wrong!");
                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("MOTO PAYMENT ERROR");
                        LogUtils.e(anError.getErrorBody());
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressBarDialog.dismiss());
                        }
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        } else {
                            ToastUtils.makeSnackToast(getActivity(), "Unknown Error");
                        }


                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void setListeners() {
        etAmount.addTextChangedListener(new MoneyTextWatcher(etAmount));
        etAmount.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                btnPaynow.setEnabled(isValid());
            }
        });
        etTip.addTextChangedListener(new MoneyTextWatcher(etTip));

        btnPaynow.setOnClickListener(view -> {

            if (etTip.getText().toString().equalsIgnoreCase("")) {
                amount = Float.parseFloat(etAmount.getText().toString().replace(",", ""));
                totalAmount = amount;
                //totalAmount = Float.parseFloat(etAmount.getText().toString().replace(",", ""));
            } else {
                amount = Float.parseFloat(etAmount.getText().toString().replace(",", ""));
                tipAmount = Float.parseFloat(etTip.getText().toString().replace(",", ""));
                totalAmount = (amount + tipAmount);
                //totalAmount = (Float.parseFloat(etAmount.getText().toString().replace(",", "")) + Float.parseFloat(etTip.getText().toString().replace(",", "")));
            }
            if (selectedCardReader == null) {
                ToastUtils.makeSnackToast(getActivity(), "Please select reader");
                return;
            }
            if (totalAmount > 0) {
                long currentTime = SystemClock.elapsedRealtime();
                if (currentTime - lastClickTime > MIN_CLICK_INTERVAL) {
                    lastClickTime = currentTime;
                    setUpMposTerminal();
                }
            } else {
                ToastUtils.makeSnackToast(getActivity(), "Amount should be greater than zero");
            }
        });
        btnCancel.setOnClickListener(view -> {
            etAmount.setText(null);
        });

    }

    private boolean isValid() {
        return (!Validators.isNullOrEmpty(etAmount.getText().toString())) && selectedCardReader != null;
    }

    /**
     * Receive the result of our permissions check, and initialize if we can
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        // If we receive a response to our permission check, initialize
        if (requestCode == Constants.PERMISSION_CODE_LOCATION && verifyGpsEnabled()) {
            showStripeBluetoothReaderDialog();
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == Constants.PAYMENT_MPOS_CARDREADER) {
            boolean paymentSuccess = false;
            if (resultCode == MposUi.RESULT_CODE_APPROVED) {
                paymentSuccess = true;
                myApp.startPaymentSuccessSound();
                ToastUtils.makeSnackToast(getActivity(), "Payment Approved");
            } else {
                myApp.startPaymentFailedSound();
                ToastUtils.makeSnackToast(getActivity(), "Transaction was declined, aborted, or failed");
            }
            etAmount.addTextChangedListener(new MoneyTextWatcher(etAmount));

            try {
                Transaction transaction = MposUi.getInitializedInstance().getTransaction();
                if (transaction != null && paymentSuccess) {
                    createCardReaderPrint(transaction, paymentSuccess);
                } else {
                    ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Failed response", "Transaction Cancelled", 2, "OKAY", null);
                    confirmationDialogFragment.show(getChildFragmentManager(), "dialog");
                }

            } catch (Exception e) {
                ToastUtils.makeLongToast(getActivity(), e.toString());
                ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Failed response", "Transaction Cancelled", 2, "OKAY", null);
                confirmationDialogFragment.show(getChildFragmentManager(), "dialog");
                e.printStackTrace();
            }
            resetBox();
        }
        if (requestCode == Constants.PAYMENT_WEBVIEW_CARDREADER) {
            if (data != null) {
                etAmount.addTextChangedListener(new MoneyTextWatcher(etAmount));
                etTip.addTextChangedListener(new MoneyTextWatcher(etTip));

                String status = data.getStringExtra("status");
                if (Validators.isNullOrEmpty(status) || status.equalsIgnoreCase("0")) {
                    myApp.startPaymentFailedSound();
                    ToastUtils.makeSnackToast(getActivity(), "Transaction was declined, aborted, or failed");
                    return;
                }

                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        if (progressBarDialog == null)
                            progressBarDialog = CommonFunctions.customProgressDialog(getActivity());
                        progressBarDialog.show();
                    });
                }
                String paymentIntentId = data.getStringExtra("payment_intent_id");
                String client_secret = data.getStringExtra("client_secret");
                retrivePaymentIntentAndPrint(client_secret);
                resetBox();
            } else {

                myApp.startPaymentFailedSound();
                ToastUtils.makeSnackToast(getActivity(), "Transaction was declined, aborted, or failed");

                resetBox();
            }
        }
    }

    private void resetBox() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                etAmount.setText("0.00");
                etTip.setText("0.00");
                etAmount.addTextChangedListener(new MoneyTextWatcher(etAmount));
                etTip.addTextChangedListener(new MoneyTextWatcher(etTip));
            });
        }
    }

    private void retrivePaymentIntentAndPrint(String clientSecret) {
        if (getActivity() != null) getActivity().runOnUiThread(() -> {
            if (progressBarDialog == null) {
                progressBarDialog = CommonFunctions.customProgressDialog(getActivity());
            }
            progressBarDialog.show();
        });
        stripe.retrievePaymentIntent(clientSecret, new ApiResultCallback<PaymentIntent>() {
            @Override
            public void onSuccess(@NotNull PaymentIntent paymentIntent) {
                com.stripe.Stripe.apiKey = stripe_private_key;
                new Thread(() -> {
                    try {
                        PaymentMethod paymentMethod = PaymentMethod.retrieve(paymentIntent.getPaymentMethodId());
                        boolean paymentSuccess = paymentIntent.getStatus().toString().contains("succeeded");
                        ToastUtils.makeSnackToast(getActivity(), paymentSuccess ? "Payment Approved" : "Payment Declined");
                        if (paymentSuccess) myApp.startPaymentSuccessSound();
                        else myApp.startPaymentFailedSound();
                        if (getActivity() != null) getActivity().runOnUiThread(() -> {
                            printUsingPaymentIntent("Customer Copy", paymentIntent, paymentMethod.getCard() != null ? paymentMethod.getCard().getBrand() : "", paymentMethod.getCard() != null ? paymentMethod.getCard().getLast4() : "", paymentSuccess);
                        });

                    } catch (StripeException e) {
                        e.printStackTrace();
                        boolean paymentSuccess = paymentIntent.getStatus().toString().contains("succeeded");
                        ToastUtils.makeSnackToast(getActivity(), paymentSuccess ? "Payment Approved" : "Payment Declined");
                        if (paymentSuccess) myApp.startPaymentSuccessSound();
                        else myApp.startPaymentFailedSound();
                        if (getActivity() != null) getActivity().runOnUiThread(() -> {
                            printUsingPaymentIntent("Customer Copy", paymentIntent, "", "", paymentSuccess);
                        });
                    }
                    if (getActivity() != null) getActivity().runOnUiThread(() -> {
                        if (progressBarDialog == null) {
                            progressBarDialog = CommonFunctions.customProgressDialog(getActivity());
                        }
                        progressBarDialog.dismiss();
                    });
                }).start();

            }

            @Override
            public void onError(@NotNull Exception e) {
                e.printStackTrace();
            }
        });
    }

    String payment_type;
    String actualAmount;
    String tip_amount;
    String printTotal;
    String card;
    String account;
    String stripe_id;
    String entry_mode;
    String status;
    String please;
    String date_final;
    String main_content;
    String footerA;
    String footerB;

    /**
     * Print for Bluetooth BBPOS Card reader & Internet Cardreader
     */
    private void printUsingPaymentIntent(String title, PaymentIntent paymentIntent, String cardBrand, String cardNumber, boolean paymentSuccess) {

        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                if (progressBarDialog == null) {
                    progressBarDialog = CommonFunctions.customProgressDialog(getActivity());
                }
                progressBarDialog.dismiss();
            });
        }

        String payment_type = "PIN";
        if (ticketHeader != null) header = ticketHeader.value;
        if (address != null) resAddress = address.value;

        actualAmount = "£" + MyApp.df.format(amount);
        tip_amount = "Tip £" + MyApp.df.format(tipAmount);

        printTotal = "£" + MyApp.df.format(totalAmount);

        card = "Card                    " + cardBrand;
        account = "Account      " + cardNumber;
        stripe_id = "TID  " + paymentIntent.getId();
        entry_mode = "Entry Mode         " + payment_type;
        status = "Status            " + (paymentSuccess ? "Approved" : "Declined");
        please = "Please Retain Receipt\nFor Your Record";

        date_final = CommonFunctions.getCurrentTimeFormatted("dd-MM-yyyy hh:mm:ss a");

        main_content = card + "\n" + account + "\n" + stripe_id + "\n" + entry_mode + "\n" + status + "\n";

        footerA = "";
        footerB = "";

        if (footerASetting != null) {
            footerA = footerASetting.value;
        }
        if (footerBSetting != null)
            footerB = footerBSetting.value;

        if (defaultPrinter != null && defaultPrinter.printer_model_name != null) {
            if (defaultPrinter.printer_model_name.toLowerCase().contains("88 h")) {
                printZonrich("Merchant Copy");
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("t2") || defaultPrinter.printer_model_name.toLowerCase().contains("sunmi")) {
                printSunmi("Merchant Copy");
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("pos-80") || defaultPrinter.printer_model_name.toLowerCase().contains("pos 80") || defaultPrinter.printer_model_name.toLowerCase().contains("yoke4")) {
                printWifi("Merchant Copy");
            }
        } else {
            ToastUtils.makeLongToast(getActivity(), "No printer model found");
        }
    }

    /**
     * MUIRA Cardreader payment print
     *
     * @param transaction
     * @param paymentSuccess
     */
    private void createCardReaderPrint(Transaction transaction, boolean paymentSuccess) {
        payment_type = "" + transaction.getPaymentDetails().getCustomerVerification().name();
        if (payment_type.equals("PIN")) {
            payment_type = "EMV";
        } else {
            payment_type = "Contactless";
        }

        if (ticketHeader != null) header = ticketHeader.value;
        if (address != null) resAddress = address.value;

        actualAmount = "£" + MyApp.df.format(amount);
        tip_amount = "Tip £" + MyApp.df.format(tipAmount);

        printTotal = "£" + MyApp.df.format(totalAmount);

        card = "Card                    " + transaction.getPaymentDetails().getScheme().name();
        account = "Account      " + transaction.getPaymentDetails().getMaskedAccountNumber();
        stripe_id = "TID  " + transaction.getIdentifier();
        entry_mode = "Entry Mode         " + payment_type;
        status = "Status            " + (paymentSuccess ? "Approved" : "Declined");
        please = "Please Retain Receipt\nFor Your Record";

        date_final = CommonFunctions.getCurrentTimeFormatted("dd-MM-yyyy hh:mm:ss a");

        main_content = card + "\n" + account + "\n" + stripe_id + "\n" + entry_mode + "\n" + status + "\n";
        footerA = "";
        footerB = "";

        if (footerASetting != null) footerA = footerASetting.value;

        if (footerBSetting != null) footerB = footerBSetting.value;

        if (defaultPrinter != null && defaultPrinter.printer_model_name != null) {
            if (defaultPrinter.printer_model_name.toLowerCase().contains("88 h")) {
                printZonrich("Merchant Copy");
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("t2") || defaultPrinter.printer_model_name.toLowerCase().contains("sunmi")) {
                printSunmi("Merchant Copy");
            } else if (defaultPrinter.printer_model_name.toLowerCase().contains("pos-80") || defaultPrinter.printer_model_name.toLowerCase().contains("pos 80") || defaultPrinter.printer_model_name.toLowerCase().contains("yoke4")) {
                printWifi("Merchant Copy");
            }
        } else {
            ToastUtils.makeLongToast(getActivity(), "No printer model found");
        }
    }

    private void printZonrich(String title) {
        zoneRichPrinter.connectPrinter(defaultPrinter.ip);
        CommonFunctions.functionThatDelay(200);
        zoneRichPrinter.printCardReaderReceipt(myApp.businessLogo, title, header, headerAlignment, actualAmount, tipAmount > 0 ? tip_amount : null, printTotal, main_content, please, date_final, footerA, footerB);

        if (title.toLowerCase().contains("merchant")) {
            ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Is Customer Copy required?", null, 0, "Confirm", "Cancel");
            confirmationDialogFragment.show(getChildFragmentManager(), "printer");
            confirmationDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String) {
                    if (((String) o).equalsIgnoreCase("confirm")) {
                        printZonrich("Customer Copy");
                    }
                }
            });
        }

    }

    private void printSunmi(String title) {
        sunmiPrinter.printCardReaderReceipt(myApp.businessLogo, title, header, headerAlignment, actualAmount, tipAmount > 0 ? tip_amount : null, printTotal, main_content, please, date_final, footerA, footerB);

        if (title.toLowerCase().contains("merchant")) {
            ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Is Customer Copy required?", null, 0, "Confirm", "Cancel");
            confirmationDialogFragment.show(getChildFragmentManager(), "printer");
            confirmationDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String) {
                    if (((String) o).equalsIgnoreCase("confirm")) {
                        printSunmi("Customer Copy");
                    }
                }
            });
        }

    }

    private void printWifi(String title) {
        wifiPrinter.connect(defaultPrinter.ip);
        CommonFunctions.functionThatDelay(200);
        wifiPrinter.printCardReaderReceipt(myApp.businessLogo, title, header, headerAlignment, actualAmount, tipAmount > 0 ? tip_amount : null, printTotal, main_content, please, date_final, footerA, footerB, true);

        if (title.toLowerCase().contains("merchant")) {
            ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Is Customer Copy required?", null, 0, "Confirm", "Cancel");
            confirmationDialogFragment.show(getChildFragmentManager(), "printer");
            confirmationDialogFragment.setDialogDismissListener(o -> {
                if (o instanceof String) {
                    if (((String) o).equalsIgnoreCase("confirm")) {
                        printWifi("Customer Copy");
                    }
                }
            });
        }

    }

    @Override
    public void onStop() {
        CommonFunctions.hideKeyboard(getActivity(), etAmount);
        super.onStop();
    }


}