package com.example.epos_2021.fragment;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.activities.UploadOrdersActivity;
import com.example.epos_2021.adapters.MenuAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.merchant.fragments.MerchantDashboardFragment;
import com.example.epos_2021.merchant.fragments.MerchantTransactionsFragment;
import com.example.epos_2021.merchant.fragments.PaymentLinkFragment;
import com.example.epos_2021.models.Business;
import com.example.epos_2021.models.ExclusionModel;
import com.example.epos_2021.models.MerchantUserPermission;
import com.example.epos_2021.online.fragments.EnterPasswordDialogFragment;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;
import com.google.android.material.chip.Chip;

import java.util.ArrayList;


public class PaymentFragment extends BaseFragment {

    public static PaymentFragment getInstance() {
        return new PaymentFragment();
    }

    private Chip chipBack;
    private RecyclerView rvPayments;
    private MaterialButton btnPayment;
    private ArrayList<ExclusionModel> paymentList = new ArrayList<>();
    private MenuAdapter menuAdapter;
    private int merchantRole = 0;
    private Business selectedBusiness = myPreferences.getLoggedInAdmin().selected_business;
    private MerchantUserPermission permission;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_payment, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView(view);
        setUpPayments();
        btnPayment.setOnClickListener(view1 -> {
            FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
            ft.replace(R.id.nav_host_fragment, PaymentLinkFragment.getInstance(), "payment_sms");
            ft.addToBackStack("can_go_back");
            ft.commit();
        });
        chipBack.setOnClickListener(v -> getActivity().onBackPressed());
    }

    private void initView(View view) {
        merchantRole = myPreferences.getMerchantRole();
        btnPayment = view.findViewById(R.id.btnPayment);
        rvPayments = view.findViewById(R.id.rvPayments);
        chipBack = view.findViewById(R.id.chipBack);

        menuAdapter = new MenuAdapter(paymentList, true, true, (position, data) -> {
            ExclusionModel exclusionModel = (ExclusionModel) data;

            if (exclusionModel.id.equalsIgnoreCase("overview")) {
                changeFragment(MerchantDashboardFragment.getInstance(id -> {
                    if (id.equalsIgnoreCase("transactions")) {
                        menuAdapter.checkedId = "transactions";
                        menuAdapter.notifyDataSetChanged();
                        changeFragment(MerchantTransactionsFragment.getInstance("normal"));
                    }
                    if (id.equalsIgnoreCase("statements")) {
                        menuAdapter.checkedId = "statements";
                        menuAdapter.notifyDataSetChanged();
                        changeFragment(MerchantStatementFragment.getInstance());
                    }

                }));
            } else if (exclusionModel.id.equalsIgnoreCase("payment_link")) {
                changeFragment(PaymentLinkFragment.getInstance());
            } else if (exclusionModel.id.equalsIgnoreCase("moto_payment")) {
                changeFragment(PaymentMotoFragment.getInstance());
            } else if (exclusionModel.id.equalsIgnoreCase("card_reader_payment")) {
                changeFragment(PaymentCardReaderFragment.getInstance());
            } else if (exclusionModel.id.equalsIgnoreCase("report")) {
                changeFragment(ReportListFragment.getInstance());
            } else if (exclusionModel.id.equalsIgnoreCase("transactions")) {
                changeFragment(MerchantTransactionsFragment.getInstance("normal"));
            } else if (exclusionModel.id.equalsIgnoreCase("refunded_transactions")) {
                changeFragment(MerchantTransactionsFragment.getInstance("refunded"));
            } else if (exclusionModel.id.equalsIgnoreCase("archived_transactions")) {
                changeFragment(MerchantTransactionsFragment.getInstance("archived"));
            } else if (exclusionModel.id.equalsIgnoreCase("statements")) {
                changeFragment(MerchantStatementFragment.getInstance());
            } else if (exclusionModel.id.equalsIgnoreCase("logout")) {
                askForLogout();
            } else if (exclusionModel.id.equalsIgnoreCase("login")) {
                passwordFragment();
            }

        });

        rvPayments.setLayoutManager(new LinearLayoutManager(getActivity(), LinearLayoutManager.VERTICAL, false));
        rvPayments.setAdapter(menuAdapter);

    }

    boolean ordersUploaded = false;

    private void uploadOrders() {
        ToastUtils.makeSnackToast(getActivity(), "Please wait, Report will generate after orders uploading");
        startActivityForResult(new Intent(getActivity(), UploadOrdersActivity.class).putExtra("finish", true), Constants.CODE_REFRESH);
    }

    private void askForLogout() {
        ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Logout", "Are you sure you want to logout?", 0, "Confirm", "Cancel");
        confirmationDialogFragment.show(getChildFragmentManager(), "logout");
        confirmationDialogFragment.setDialogDismissListener(o -> {
            if (o instanceof String) {
                if (((String) o).equalsIgnoreCase("confirm")) {
                    myPreferences.saveMerchantRole(0);
                    setUpPayments();
                }
            }
        });
    }

    private void passwordFragment() {
        EnterPasswordDialogFragment enterPasswordDialogFragment = EnterPasswordDialogFragment.getInstance(null, "Enter password", "Please enter password to login");
        enterPasswordDialogFragment.show(getChildFragmentManager(), "password");
        enterPasswordDialogFragment.setDialogDismissListener(o -> {
            if (o instanceof String) {
                String password = (String) o;
                if (password.equals(selectedBusiness.admin_password)) {
                    myPreferences.saveMerchantRole(2);
                    merchantRole = 2;
                    setUpPayments();
                    ToastUtils.makeSnackToast(getActivity(), "Login successful");
                } else if (password.equals(selectedBusiness.supervisor_password)) {
                    myPreferences.saveMerchantRole(1);
                    merchantRole = 1;
                    setUpPayments();
                    ToastUtils.makeSnackToast(getActivity(), "Login successful");
                } else {
                    ToastUtils.makeSnackToast(getActivity(), "Invalid password");
                }
            }
        });

    }

    private void setUpPayments() {
        merchantRole = myPreferences.getMerchantRole();
        if (merchantRole == 1) {
            permission = selectedBusiness.supervisor_permissions;
        } else if (merchantRole == 2) {
            permission = selectedBusiness.admin_permissions;
        } else {
            permission = null;
        }
        paymentList.clear();

        ExclusionModel exclusionModel = new ExclusionModel();
        if (selectedBusiness.merchant_service) {
            if (merchantRole > 0 || MyApp.userPermission.payment_overview.actions.list) {
                exclusionModel.id = "overview";
                exclusionModel.name = "Overview";
                exclusionModel.isCheckable = true;
                exclusionModel.isChecked = true;
                paymentList.add(exclusionModel);
            }

            exclusionModel = new ExclusionModel();
            exclusionModel.id = "payment_link";
            exclusionModel.name = "Payment SMS";
            exclusionModel.isCheckable = true;
            exclusionModel.isChecked = true;
            paymentList.add(exclusionModel);

            if (merchantRole == 0 || (permission != null && permission.moto_service)) {
                exclusionModel = new ExclusionModel();
                exclusionModel.id = "moto_payment";
                exclusionModel.name = "Moto payment";
                exclusionModel.isCheckable = true;
                paymentList.add(exclusionModel);
            }
            if (merchantRole == 0 || (permission != null && permission.payworks)) {
                exclusionModel = new ExclusionModel();
                exclusionModel.id = "card_reader_payment";
                exclusionModel.name = "Card reader payment";
                exclusionModel.isCheckable = true;
                paymentList.add(exclusionModel);
            }

            if ((permission != null && permission.archive_list)) {
                exclusionModel = new ExclusionModel();
                exclusionModel.id = "archived_transactions";
                exclusionModel.name = "Archived Transactions";
                exclusionModel.isCheckable = true;
                paymentList.add(exclusionModel);
            }

            if ((permission != null && permission.refund_list)) {
                exclusionModel = new ExclusionModel();
                exclusionModel.id = "refunded_transactions";
                exclusionModel.name = "Refunded transactions";
                exclusionModel.isCheckable = true;
                paymentList.add(exclusionModel);
            }
            if (merchantRole == 0 || (permission != null && permission.transaction_list)) {
                exclusionModel = new ExclusionModel();
                exclusionModel.id = "transactions";
                exclusionModel.name = "Transactions";
                exclusionModel.isCheckable = true;
                paymentList.add(exclusionModel);
            }

            if (permission != null && permission.statement) {
                exclusionModel = new ExclusionModel();
                exclusionModel.id = "statements";
                exclusionModel.name = "Statements";
                exclusionModel.isCheckable = true;
                paymentList.add(exclusionModel);
            }
        }

        exclusionModel = new ExclusionModel();
        exclusionModel.id = "report";
        exclusionModel.name = "Report";
        exclusionModel.isCheckable = true;
        paymentList.add(exclusionModel);

        if (selectedBusiness.merchant_service) {
            if (myPreferences.getMerchantRole() == 0) {
                exclusionModel = new ExclusionModel();
                exclusionModel.id = "login";
                exclusionModel.name = "Login";
                exclusionModel.isCheckable = false;
                paymentList.add(exclusionModel);
            } else {
                exclusionModel = new ExclusionModel();
                exclusionModel.id = "logout";
                exclusionModel.name = "Logout";
                exclusionModel.isCheckable = false;
                paymentList.add(exclusionModel);
            }
        }
        if (selectedBusiness.merchant_service) {
            if (merchantRole > 0 || MyApp.userPermission.payment_overview.actions.list) {
                menuAdapter.checkedId = "overview";
                menuAdapter.notifyDataSetChanged();

                changeFragment(MerchantDashboardFragment.getInstance(id -> {
                    if (id.equalsIgnoreCase("transactions")) {
                        menuAdapter.checkedId = "transactions";
                        menuAdapter.notifyDataSetChanged();
                        changeFragment(MerchantTransactionsFragment.getInstance("normal"));
                    }
                    if (id.equalsIgnoreCase("statements")) {
                        menuAdapter.checkedId = "statements";
                        menuAdapter.notifyDataSetChanged();
                        changeFragment(MerchantStatementFragment.getInstance());
                    }
                }));
            } else {
                menuAdapter.checkedId = paymentList.get(0).id;
                menuAdapter.notifyDataSetChanged();

                if (menuAdapter.checkedId.equalsIgnoreCase("moto_payment")) {
                    changeFragment(new PaymentMotoFragment());
                } else if (menuAdapter.checkedId.equalsIgnoreCase("card_reader_payment")) {
                    changeFragment(new PaymentCardReaderFragment());
                } else if (menuAdapter.checkedId.equalsIgnoreCase("payment_link")) {
                    changeFragment(new PaymentLinkFragment());
                }
            }
        } else {
            menuAdapter.checkedId = "report";
            changeFragment(ReportListFragment.getInstance());
        }
        menuAdapter.notifyDataSetChanged();

    }

    private void changeFragment(Fragment fragment) {
        FragmentManager manager = getChildFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.nav_payment_fragment, fragment);
        transaction.addToBackStack("can_go_back");
        transaction.commit();
        getActivity().invalidateOptionsMenu();
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable @org.jetbrains.annotations.Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == Constants.CODE_REFRESH) {
            if (resultCode == Activity.RESULT_OK) {
                ordersUploaded = true;
                changeFragment(ReportListFragment.getInstance());
            }
        }
    }
}