package com.example.epos_2021.fragment;

import android.content.Intent;
import android.os.Bundle;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.viewpager.widget.ViewPager;

import com.example.epos_2021.R;
import com.example.epos_2021.activities.ProductAddonDialogFragment;
import com.example.epos_2021.adapters.ViewpagerTabAdapter;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.Product;
import com.google.android.material.chip.Chip;
import com.google.gson.Gson;

public class ProductAddonAndIngrediantFragment extends DialogFragment {

    private TextView tvOption, tvIngredient;
    private View vOption, vIngrediant;
    private LinearLayout llOption, llIngredient;
    private Chip chipBack;
    private ViewPager viewPager;
    private ViewpagerTabAdapter viewpagerAdapter;
    private boolean isOptionSelected = true;
    private DialogDismissListener dialogDismissListener;
    private String order_id;
    private Product product;
    private OrderItem orderItem;
    private int total_ingredients;

    public static ProductAddonAndIngrediantFragment getInstance(String order_id, OrderItem orderItem, Product product, int total_ingredients) {
        ProductAddonAndIngrediantFragment fragment = new ProductAddonAndIngrediantFragment();
        Bundle bundle = new Bundle();
        bundle.putString("order_type_id", order_id);
        bundle.putString("order_item", new Gson().toJson(orderItem));
        bundle.putString("product", new Gson().toJson(product));
        bundle.putInt("total_ingredients", total_ingredients);
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    public int getTheme() {
        return R.style.MyRightDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.MATCH_PARENT);
        getDialog().getWindow().setGravity(Gravity.END);
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            product = new Gson().fromJson(getArguments().getString("product"), Product.class);
            order_id = getArguments().getString("order_type_id");
            orderItem = new Gson().fromJson(getArguments().getString("order_item"), OrderItem.class);
            total_ingredients = getArguments().getInt("total_ingredients");

        }
        return inflater.inflate(R.layout.fragment_product_addon_ingrediant, container);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();
    }

    private void setListeners() {
        chipBack.setOnClickListener(v -> dismiss());
        llOption.setOnClickListener(v -> {
            isOptionSelected = true;
            selectFragment();
        });

        llIngredient.setOnClickListener(v -> {
            isOptionSelected = false;
            selectFragment();
        });

        viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

            }

            @Override
            public void onPageSelected(int position) {
                if (position == 0) {
                    changeTextColor(llOption);
                } else
                    changeTextColor(llIngredient);
            }

            @Override
            public void onPageScrollStateChanged(int state) {

            }
        });
    }

    private void changeTextColor(View v) {
        vOption.setBackgroundResource(R.color.light_gray);
        tvOption.setAlpha(0.5f);

        vIngrediant.setBackgroundResource(R.color.light_gray);
        tvIngredient.setAlpha(0.5f);

        if (v == llIngredient) {
            vIngrediant.setBackgroundResource(R.color.persian_green);
            tvIngredient.setAlpha(1f);
        }
        if (v == llOption) {
            vOption.setBackgroundResource(R.color.persian_green);
            tvOption.setAlpha(1f);
        }
    }

    private void initViews(View view) {
        chipBack = view.findViewById(R.id.chipBack);

        llOption = view.findViewById(R.id.llOption);
        llIngredient = view.findViewById(R.id.llIngredient);
        tvOption = view.findViewById(R.id.tvOption);
        tvIngredient = view.findViewById(R.id.tvIngredient);
        vOption = view.findViewById(R.id.vOptionSelector);
        vIngrediant = view.findViewById(R.id.vIngrediantSelector);
        viewPager = view.findViewById(R.id.vpProducts);


        llOption.setVisibility(View.GONE);
        llIngredient.setVisibility(View.GONE);
        setAdapters();
    }

    private void setAdapters() {
        CartProductEditDialogFragment cartEditFragment = CartProductEditDialogFragment.getInstance("1", orderItem);
        cartEditFragment.setDialogDismissListener(o -> {
            if (o.equals("open_addon")) {
                isOptionSelected = true;
                changeTextColor(llOption);
                viewPager.setCurrentItem(0);
            } else {
                if (dialogDismissListener != null)
                    dialogDismissListener.onDialogDismiss("dismiss");
                dismiss();
            }
        });


        ProductAddonDialogFragment addonDialogFragment = ProductAddonDialogFragment.getInstance(order_id, product, total_ingredients);
        addonDialogFragment.setDialogDismissListener(o -> {
            orderItem = (OrderItem) o;
            LocalBroadcastManager.getInstance(getActivity()).sendBroadcast(new Intent("addon_update").putExtra("order_addon", new Gson().toJson(orderItem)));
            if (total_ingredients > 0) {
                isOptionSelected = false;
                changeTextColor(llIngredient);
                viewPager.setCurrentItem(1);
            } else {
                if (dialogDismissListener != null)
                    dialogDismissListener.onDialogDismiss("dismiss");
                dismiss();
            }
        });

        viewpagerAdapter = new ViewpagerTabAdapter(getChildFragmentManager(), FragmentPagerAdapter.BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);

        if (product != null) {
            if (product.total_addons > 0 || total_ingredients > 0) {
                if (product.total_addons > 0) {
                    viewpagerAdapter.addFragment(addonDialogFragment, null);
                    llOption.setVisibility(View.VISIBLE);
                    changeTextColor(llOption);
                }
                if (total_ingredients > 0) {
                    viewpagerAdapter.addFragment(cartEditFragment, null);
                    llIngredient.setVisibility(View.VISIBLE);
                    changeTextColor(llIngredient);
                }
            } else {
                viewpagerAdapter.addFragment(cartEditFragment, null);
                llIngredient.setVisibility(View.VISIBLE);
                isOptionSelected = false;
                changeTextColor(llIngredient);
            }
        }
        viewPager.setAdapter(viewpagerAdapter);
        selectFragment();
    }

    private void selectFragment() {
        if (isOptionSelected) {
            changeTextColor(llOption);
            viewPager.setCurrentItem(0);
        } else {
            changeTextColor(llIngredient);
            viewPager.setCurrentItem(1);
        }
    }
}
