package com.example.epos_2021.fragment;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.R;
import com.example.epos_2021.adapters.CategoryAdapter;
import com.example.epos_2021.adapters.SubCategoryAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.models.Category;
import com.example.epos_2021.models.Order;
import com.google.gson.Gson;

import java.util.ArrayList;

public class ProductAreaFragment extends BaseFragment {

    private RecyclerView rvCategories, rvSubCategory;

    private ProductsFragment productsFragment;
    private CategoryAdapter categoryAdapter;
    private SubCategoryAdapter subCategoryAdapter;

    private ArrayList<Category> filteredItems = new ArrayList<>();
    private ArrayList<Category> categories = new ArrayList<>();
    private ArrayList<Category> subCategories = new ArrayList<>();

    private Order order;
    private int _order_id;
    private String order_id;
    private String selectedCategoryId, selectedSubCategoryId;
    private int itemWidth = 0;
    private Category saveSelectedCategory;

    public static ProductAreaFragment getInstance(String order_id, int _order_id) {
        Bundle bundle = new Bundle();
        bundle.putString("order_id", order_id);
        bundle.putInt("_order_id", _order_id);
        ProductAreaFragment productAreaFragment = new ProductAreaFragment();
        productAreaFragment.setArguments(bundle);
        return productAreaFragment;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_product_area, container, false);
    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getArguments() != null) {
            order = new Gson().fromJson(getArguments().getString("order"), Order.class);
            _order_id = getArguments().getInt("_order_id");
            order_id = getArguments().getString("order_id");
        }
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        new FetchOfflineAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }


    private void initViews(View view) {
        rvCategories = view.findViewById(R.id.rvCategories);
        rvSubCategory = view.findViewById(R.id.rvSubCategory);

        setAdapters();
    }


    private void setAdapters() {
        //top category

        categoryAdapter = new CategoryAdapter(filteredItems, (position, data) -> {
            Category selectedCategory = (Category) data;
            subCategories.clear();
            if (selectedCategory.children != null && selectedCategory.children.size() > 0) {
                subCategories.addAll(selectedCategory.children);
            }
            subCategoryAdapter.selectedSubCategoryId = null;
            int rvWidth = rvSubCategory.getWidth();
            if (subCategories.size() == 0) {
                rvSubCategory.setLayoutManager(new GridLayoutManager(getActivity(), 1));
            } else if (subCategories.size() < 7) {
                rvSubCategory.setLayoutManager(new GridLayoutManager(getActivity(), subCategories.size()));
            } else {
                rvSubCategory.setLayoutManager(new GridLayoutManager(getActivity(), 2, RecyclerView.HORIZONTAL, false));
                itemWidth = rvWidth / 6;
            }
            subCategoryAdapter.itemWidth = itemWidth;
            subCategoryAdapter.notifyDataSetChanged();

            productsFragment = ProductsFragment.getInstance(_order_id, order_id, selectedCategory.id, order.order_type_id);
            changeFragment(productsFragment);
            productsFragment.setNoDataFoundListener(o -> {
                if (o instanceof Integer) {
                    if ((int) o == 0) {
                        if (subCategories.size() > 0) {
                            Category sc = subCategories.get(0);
                            subCategoryAdapter.selectedSubCategoryId = sc.id;
                            subCategoryAdapter.notifyDataSetChanged();
                            changeFragment(ProductsFragment.getInstance(_order_id, order_id, sc.id, order.order_type_id));
                        }
                    }
                }
            });

        });

        subCategoryAdapter = new SubCategoryAdapter(subCategories, (position, data) -> {
            Category selectedCategory = (Category) data;
            selectedSubCategoryId = selectedCategory.id;
            subCategoryAdapter.notifyDataSetChanged();
            changeFragment(ProductsFragment.getInstance(_order_id, order_id, selectedCategory.id, order.order_type_id));
        });

        rvCategories.setLayoutManager(new GridLayoutManager(getActivity(), 2, LinearLayoutManager.HORIZONTAL, false));
        rvCategories.setAdapter(categoryAdapter);
        int rvWidth = rvSubCategory.getWidth();
        if (subCategories.size() == 0) {
            rvSubCategory.setLayoutManager(new GridLayoutManager(getActivity(), 1));
        } else if (subCategories.size() < 7) {
            rvSubCategory.setLayoutManager(new GridLayoutManager(getActivity(), subCategories.size()));
        } else {
            rvSubCategory.setLayoutManager(new GridLayoutManager(getActivity(), 2, RecyclerView.HORIZONTAL, false));
            itemWidth = rvWidth / 6;
        }
        subCategoryAdapter.itemWidth = itemWidth;
        rvSubCategory.setAdapter(subCategoryAdapter);


    }

    private void fetchCategories() {
        categories.clear();
        if (saveSelectedCategory == null) {
            categories.addAll(myApp.categories);
        } else {
            categories.add(saveSelectedCategory);
        }

        filteredItems.clear();
        filteredItems.addAll(categories);
        subCategories.clear();
        // Set first floor item selected;
        if (categories.size() > 0) {

            Category selectedCategory = categories.get(0);
            if (selectedCategory.children != null && selectedCategory.children.size() > 0) {
                subCategories.addAll(selectedCategory.children);
            }
            subCategoryAdapter.notifyDataSetChanged();
            if (Validators.isNullOrEmpty(selectedCategoryId)) {
                selectedCategoryId = selectedCategory.id;
            }
            productsFragment = ProductsFragment.getInstance(_order_id, order_id, selectedCategoryId, order.order_type_id);
            changeFragment(productsFragment);
            productsFragment.setNoDataFoundListener(o -> {
                if (o instanceof Integer) {
                    if ((int) o == 0) {
                        if (subCategories.size() > 0) {
                            Category sc = subCategories.get(0);
                            subCategoryAdapter.selectedSubCategoryId = sc.id;
                            subCategoryAdapter.notifyDataSetChanged();
                            changeFragment(ProductsFragment.getInstance(_order_id, order_id, sc.id, order.order_type_id));
                        }
                    }
                }
            });
        }
        categoryAdapter.notifyDataSetChanged();

    }

    private void changeFragment(Fragment fragment) {
        FragmentManager manager = getChildFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.fragment_products, fragment);
        transaction.commit();
    }

    private class FetchOfflineAsync extends AsyncTask<String, String, String> {

        @Override
        protected String doInBackground(String... strings) {
            order = appDatabase.orderDao().view(_order_id);
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            fetchCategories();
        }
    }

    BroadcastReceiver selectedCategoryReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String categoryItem = intent.getStringExtra("category_data");
            saveSelectedCategory = new Gson().fromJson(categoryItem, Category.class);
            subCategories.clear();
            if (saveSelectedCategory.children != null && saveSelectedCategory.children.size() > 0) {
                subCategories.addAll(saveSelectedCategory.children);
            }
            subCategoryAdapter.selectedSubCategoryId = null;
            int rvWidth = rvSubCategory.getWidth();
            itemWidth = 0;
            if (subCategories.size() == 0) {
                rvSubCategory.setLayoutManager(new GridLayoutManager(getActivity(), 1));
            } else if (subCategories.size() < 7) {
                rvSubCategory.setLayoutManager(new GridLayoutManager(getActivity(), subCategories.size()));
            } else {
                rvSubCategory.setLayoutManager(new GridLayoutManager(getActivity(), 2, RecyclerView.HORIZONTAL, false));
                itemWidth = rvWidth / 6;
            }
            subCategoryAdapter.itemWidth = itemWidth;
            subCategoryAdapter.notifyDataSetChanged();
            selectedCategoryId = saveSelectedCategory.id;
            productsFragment = ProductsFragment.getInstance(_order_id, order_id, saveSelectedCategory.id, order.order_type_id);
            changeFragment(productsFragment);
            productsFragment.setNoDataFoundListener(o -> {
                if (o instanceof Integer) {
                    if ((int) o == 0) {
                        if (subCategories.size() > 0) {
                            Category sc = subCategories.get(0);
                            subCategoryAdapter.selectedSubCategoryId = sc.id;
                            subCategoryAdapter.notifyDataSetChanged();
                            changeFragment(ProductsFragment.getInstance(_order_id, order_id, sc.id, order.order_type_id));
                        }
                    }
                }
            });
        }
    };

    @Override
    public void onResume() {
        super.onResume();
        LocalBroadcastManager.getInstance(getActivity()).registerReceiver(selectedCategoryReceiver, new IntentFilter("search_receiver"));
    }


    @Override
    public void onDestroy() {
        super.onDestroy();
        LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(selectedCategoryReceiver);
    }
}
