package com.example.epos_2021.fragment;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.activities.NewOrderActivity;
import com.example.epos_2021.adapters.AddonAdapter;
import com.example.epos_2021.adapters.ProductsAdapter;
import com.example.epos_2021.adapters.SubAddonAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.Addon;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.OrderType;
import com.example.epos_2021.models.Product;
import com.example.epos_2021.models.ProductAddon;
import com.example.epos_2021.utils.ToastUtils;
import com.google.gson.Gson;

import java.util.ArrayList;

public class ProductsFragment extends BaseFragment {
    private RecyclerView rvItems;
    private RelativeLayout rlMainLayout;
    private ArrayList<Product> products = new ArrayList<>();
    private ArrayList<Product> allProducts = new ArrayList<>();
    private ArrayList<Product> filteredItems = new ArrayList<>();
    private ArrayList<ProductAddon> productAddons = new ArrayList<>();
    private ArrayList<Addon> subaddons = new ArrayList<>();
    public ProductsAdapter productsAdapter;
    private AddonAdapter addonAdapter;
    private SubAddonAdapter subAddonAdapter;
    public EditText etSearchItem;
    private TextView tvSearch, tvErrorMessage;
    private String categoryId;
    private OrderType orderType;
    private int _order_id;
    private String order_id;


    public static ProductsFragment getInstance(int _order_id, String order_id, String categoryId, String orderTypeId) {
        Bundle bundle = new Bundle();
        ProductsFragment fragment = new ProductsFragment();
        bundle.putString("category_id", categoryId);
        bundle.putString("order_type_id", orderTypeId);
        bundle.putString("order_id", order_id);
        bundle.putInt("_order_id", _order_id);
        fragment.setArguments(bundle);
        return fragment;
    }

    DialogDismissListener errorListener;

    public void setNoDataFoundListener(DialogDismissListener errorListener) {
        this.errorListener = errorListener;
    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getArguments() != null) {
            categoryId = getArguments().getString("category_id");
            String orderTypeId = getArguments().getString("order_type_id");
            _order_id = getArguments().getInt("_order_id");
            order_id = getArguments().getString("order_id");
            orderType = new Gson().fromJson(getArguments().getString("order_type"), OrderType.class);
            if (!Validators.isNullOrEmpty(orderTypeId) && orderType == null) {
                orderType = new OrderType(orderTypeId, "");
            }
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_products, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        iniViews(view);
        setListeners();
        fetchProducts();
    }

    private void iniViews(View view) {

        rvItems = view.findViewById(R.id.rvItems);
        etSearchItem = view.findViewById(R.id.etSearch);
        tvSearch = view.findViewById(R.id.tvSearch);
        tvErrorMessage = view.findViewById(R.id.tvErrorMessage);
        rlMainLayout = view.findViewById(R.id.rlMainLayout);
        productsAdapter = new ProductsAdapter(filteredItems, (position, data) -> {
            Product product = (Product) data;
            getOrderItemFromProduct(product);

        });
        int spanCount = getResources().getInteger(R.integer.order_products_span);
        rvItems.setLayoutManager(new GridLayoutManager(getActivity(), spanCount));
        rvItems.setAdapter(productsAdapter);

    }

    private void getOrderItemFromProduct(Product product) {
        OrderItem orderItem = new OrderItem();
        orderItem._order_id = _order_id;
        orderItem.order_id = order_id;
        orderItem.product_name = product.name;
        orderItem.product_short_name = product.short_name;
        orderItem.product_id = product.id;
        orderItem.quantity = 1;
        orderItem.price = product.selected_price;
        orderItem.sub_total = (orderItem.quantity * orderItem.price);
        orderItem.total = orderItem.sub_total + orderItem.addons_price + orderItem.ingredients_price;
        orderItem.updater_id = myPreferences.getLoggedInUser().id;
        openProductEdit(product, orderItem);

    }

    int total_ingredient;


    private void setListeners() {
        NewOrderActivity orderActivity = (NewOrderActivity) getActivity();
        orderActivity.setSearchListener(o -> {
            String searchText = (String) o;
            etSearchItem.setText(searchText);
        });
        etSearchItem.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                filteredItems.clear();
                filteredItems.addAll(searchItems());
                productsAdapter.notifyDataSetChanged();
            }
        });

        etSearchItem.setOnKeyListener((textView, i, keyEvent) -> {
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                getActivity().onBackPressed();
            } else if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                filteredItems.clear();
                filteredItems.addAll(searchItems());
                productsAdapter.notifyDataSetChanged();
                if (getActivity() != null) {
                    CommonFunctions.hideKeyboard(getActivity(), etSearchItem);
                }
            }
            return false;
        });

        tvSearch.setOnClickListener(v -> {
            if (!Validators.isNullOrEmpty(etSearchItem.getText().toString())) {
                filteredItems.clear();
                filteredItems.addAll(searchItems());
                productsAdapter.notifyDataSetChanged();
            } else {
                ToastUtils.makeToast(getActivity(), "please enter item name or number");
            }

        });

        //Listen Cart item
        LocalBroadcastManager.getInstance(getActivity()).registerReceiver(new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                boolean notifyAdapter = intent.getBooleanExtra("notify_products", false);
                if (notifyAdapter) productsAdapter.notifyDataSetChanged();
            }
        }, new IntentFilter("cart_update"));
    }

    private void fetchProducts() {
        tvErrorMessage.setVisibility(View.GONE);
        new ProductAsyncTask().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, categoryId);
    }


    private void openProductEdit(Product product, OrderItem orderItem) {
        if ((product.auto_addon && product.total_addons > 0) || product.auto_modify) {
            CartProductEditDialogFragment cartProductEditDialogFragment = CartProductEditDialogFragment.getInstance(orderType.id, orderItem);
            cartProductEditDialogFragment.show(getChildFragmentManager(), "product_view");
            cartProductEditDialogFragment.setDialogDismissListener((data) -> {
            });
        } else {
            myApp.notifyCart(getContext(), new Intent("cart_update").putExtra("order_item", new Gson().toJson(orderItem)));
        }
    }

    private ArrayList<Product> searchItems() {
        if (etSearchItem.getText().toString().isEmpty()) {
            return products;
        }

        ArrayList<Product> searchedRest = new ArrayList<>();

        for (Product s : allProducts) {
            if (s.name != null && s.name.toLowerCase().contains(etSearchItem.getText().toString().toLowerCase())) {
                searchedRest.add(s);
            }
        }
        getActivity().runOnUiThread(() -> {
            if (searchedRest.size() == 0) {
                tvErrorMessage.setVisibility(View.VISIBLE);
            } else {
                tvErrorMessage.setVisibility(View.GONE);
            }
        });
        return searchedRest;
    }

    private class ProductAsyncTask extends AsyncTask<String, String, String> {
        @Override
        protected String doInBackground(String... strings) {
            Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
            String categoryId = strings[0];
            products.clear();
            allProducts.clear();
            if (myApp.showBanquetPorducts) {
                switch (orderType.id) {
                    case "1":
                        products.addAll(appDatabase.productDao().listDineinProductByCategory(categoryId, 1));
                        allProducts.addAll(appDatabase.productDao().listDineinProductByCategory(1));
                        break;
                    case "2": //Collection & Waiting
                    case "4":
                        products.addAll(appDatabase.productDao().listCollectionProductByCategory(categoryId, 1));
                        allProducts.addAll(appDatabase.productDao().listCollectionProductByCategory(1));
                        break;
                    case "3":
                        products.addAll(appDatabase.productDao().listDeliveryProductByCategory(categoryId, 1));
                        allProducts.addAll(appDatabase.productDao().listDeliveryProductByCategory(1));
                        break;
                    default:
                        products.addAll(appDatabase.productDao().listByCategory(categoryId, 1));
                        allProducts.addAll(appDatabase.productDao().listBanquetProductByCategory(1));
                        break;
                }
            } else {
                switch (orderType.id) {
                    case "1":
                        products.addAll(appDatabase.productDao().listDineinProductByCategory(categoryId));
                        allProducts.addAll(appDatabase.productDao().listDineinProductByCategory());
                        break;
                    case "2": //Collection & Waiting
                    case "4":
                        products.addAll(appDatabase.productDao().listCollectionProductByCategory(categoryId));
                        allProducts.addAll(appDatabase.productDao().listCollectionProductByCategory());
                        break;
                    case "3":
                        products.addAll(appDatabase.productDao().listDeliveryProductByCategory(categoryId));
                        allProducts.addAll(appDatabase.productDao().listDeliveryProductByCategory());
                        break;
                    default:
                        products.addAll(appDatabase.productDao().listByCategory(categoryId));
                        allProducts.addAll(appDatabase.productDao().list());
                        break;
                }
            }
            for (Product p : products) {
                p.selected_price = MyApp.calculatedProductPrice(orderType.id, p);
            }
            for (Product p : allProducts) {
                p.selected_price = MyApp.calculatedProductPrice(orderType.id, p);
            }

            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            filteredItems.clear();
            filteredItems.addAll(products);

            if (getActivity() != null) {
                productsAdapter.notifyDataSetChanged();
                if (products.size() == 0) {
                    tvErrorMessage.setVisibility(View.VISIBLE);
                    tvErrorMessage.setText("No products found");
                    if (errorListener != null)
                        errorListener.onDialogDismiss(0);
                } else {
                    if (errorListener != null)
                        errorListener.onDialogDismiss(1);
                }
            }
        }
    }
}
