package com.example.epos_2021.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.cardview.widget.CardView;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.DialogFragment;

import com.example.epos_2021.R;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.interfaces.DialogDismissListener;

public class PushDialogFragment extends DialogFragment {
    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private AppCompatImageView ivClose;
    private TextView tvTitle, tvSubTitle;
    private CardView cvCard;
    private DialogDismissListener dialogDismissListener;
    String title, subTitle;
    String orderId, paymentStatus, orderType;

    public static PushDialogFragment getInstance(String title, String subTitle, String orderId, String paymentStatus, String orderType) {
        PushDialogFragment rejectReasonDialog = new PushDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("title", title);
        bundle.putString("sub_title", subTitle);
        bundle.putString("order_id", orderId);
        bundle.putString("payment_status", paymentStatus);
        bundle.putString("order_type", orderType);

        rejectReasonDialog.setArguments(bundle);
        return rejectReasonDialog;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.dialog_push_fragment, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            title = getArguments().getString("title");
            subTitle = getArguments().getString("sub_title");
            orderId = getArguments().getString("order_type");
            orderType = getArguments().getString("order_type");
            paymentStatus = getArguments().getString("payment_status");
        }
        initViews(view);
        setListeners();
    }

    private void initViews(View view) {
        ivClose = view.findViewById(R.id.ivClose);
        tvSubTitle = view.findViewById(R.id.tvSubTitle);
        tvTitle = view.findViewById(R.id.tvTitle);
        cvCard = view.findViewById(R.id.cvCardBg);
        tvTitle.setText(title);
        tvSubTitle.setText(subTitle);
        if (!Validators.isNullOrEmpty(orderType)) {
            String title = orderType + " - " + paymentStatus;
            String subTitle = "New online order";

            tvTitle.setText(title);
            tvSubTitle.setText(subTitle);
            if (orderType.equalsIgnoreCase("delivery")) {
                cvCard.setCardBackgroundColor(ContextCompat.getColor(getActivity(), R.color.orders_card_bg_green));
            } else if (orderType.equalsIgnoreCase("collection") || orderType.equalsIgnoreCase("pickup")) {
                cvCard.setCardBackgroundColor(ContextCompat.getColor(getActivity(), R.color.orders_card_bg_indigo));
            } else if (orderType.equalsIgnoreCase("dinein")) {
                cvCard.setCardBackgroundColor(ContextCompat.getColor(getActivity(), R.color.orders_card_bg_amber));
            } else {
                cvCard.setCardBackgroundColor(ContextCompat.getColor(getActivity(), R.color.orders_card_bg_orange));
            }
        } else {
            String title = "Dine in" + " - " + paymentStatus;
            String subTitle = "New online order";
            cvCard.setCardBackgroundColor(ContextCompat.getColor(getActivity(), R.color.orders_card_bg_orange));
            tvTitle.setText(title);
            tvSubTitle.setText(subTitle);
        }
    }

    private void setListeners() {
        cvCard.setOnClickListener(view -> {
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss(orderId);
            dismiss();
        });
        ivClose.setOnClickListener(view -> {
            dismiss();
        });
    }


}