package com.example.epos_2021.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.MenuAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.models.ExclusionModel;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;

public class ReportListFragment extends BaseFragment {

    public static ReportListFragment getInstance() {
        return new ReportListFragment();
    }

    private RecyclerView rvReports;
    private ArrayList<ExclusionModel> paymentList = new ArrayList<>();
    private MenuAdapter menuAdapter;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_report_list, container, false);
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView(view);
        setUpPayments();
    }

    private void initView(View view) {

        rvReports = view.findViewById(R.id.rvReports);

        menuAdapter = new MenuAdapter(paymentList, (position, data) -> {
            ExclusionModel exclusionModel = (ExclusionModel) data;
            if (exclusionModel.id.equalsIgnoreCase("full_report")) {
                changeFragment(FullReportFragment.getInstance("Full Report", false));
            }
            if (exclusionModel.id.equalsIgnoreCase("full_archived_report")) {
                changeFragment(FullReportFragment.getInstance("Full Archived Report", true));
            }
            if (exclusionModel.id.equalsIgnoreCase("online_order_report")) {
                changeFragment(FullReportFragment.getInstance("Online Order Report", false, true));
            }
            if (exclusionModel.id.equalsIgnoreCase("selected_user_report")) {
                changeFragment(FullReportFragment.getInstance(true));
            }
            if (exclusionModel.id.equalsIgnoreCase("product_sales_report")) {
                changeFragment(ProductSalesReportFragment.getInstance());
            }

            if (exclusionModel.id.equalsIgnoreCase("product_category_sales_report")) {
                changeFragment(CategorySalesReportFragment.getInstance());
            }
        });

        rvReports.setLayoutManager(new LinearLayoutManager(getActivity(), LinearLayoutManager.VERTICAL, false));
        rvReports.setAdapter(menuAdapter);

    }

    private void setUpPayments() {
        paymentList.clear();

        ExclusionModel exclusionModel = new ExclusionModel();
        exclusionModel.id = "full_report";
        exclusionModel.name = "Full Report";
        paymentList.add(exclusionModel);
        if (MyApp.userPermission != null && MyApp.userPermission.archive_order.actions.list) {
            exclusionModel = new ExclusionModel();
            exclusionModel.id = "full_archived_report";
            exclusionModel.name = "Full Archived Report";
            paymentList.add(exclusionModel);
        }
        if (MyApp.userPermission != null && MyApp.userPermission.product_wise.actions.list) {
            exclusionModel = new ExclusionModel();
            exclusionModel.id = "product_sales_report";
            exclusionModel.name = "Product Sales Report";
            paymentList.add(exclusionModel);
        }
        if (MyApp.userPermission != null && MyApp.userPermission.product_category_wise.actions.list) {
            exclusionModel = new ExclusionModel();
            exclusionModel.id = "product_category_sales_report";
            exclusionModel.name = "Product Category Report";
            paymentList.add(exclusionModel);
        }


        exclusionModel = new ExclusionModel();
        exclusionModel.id = "selected_user_report";
        exclusionModel.name = "Selected User Report";
        paymentList.add(exclusionModel);

        exclusionModel = new ExclusionModel();
        exclusionModel.id = "online_order_report";
        exclusionModel.name = "Online Order Report";
        paymentList.add(exclusionModel);


    }

    private void changeFragment(Fragment fragment) {
        FragmentManager manager = getActivity().getSupportFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.nav_payment_fragment, fragment);
        transaction.addToBackStack("can_go_back");
        transaction.commit();
        getActivity().invalidateOptionsMenu();
    }
}