package com.example.epos_2021.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.CardReaderAdapter;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.BusinessCardReader;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.stripe.stripeterminal.external.models.Reader;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 07-12-2020.
 */
public class SelectCardReaderDialogFragment extends DialogFragment {
    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private String title, info;
    private DialogDismissListener dialogDismissListener;
    private MaterialButton btnConfirm, btnCancel;
    private BusinessCardReader selectedCardReader;
    private RecyclerView rvReaders;
    private CardReaderAdapter cardReaderAdapter;
    private ArrayList<Object> cardReaders = new ArrayList<>();
    private boolean stripe = false;
    private Reader stripeReader;

    public static SelectCardReaderDialogFragment getInstance() {
        SelectCardReaderDialogFragment rejectReasonDialog = new SelectCardReaderDialogFragment();
        Bundle bundle = new Bundle();
        rejectReasonDialog.setArguments(bundle);
        return rejectReasonDialog;
    }

    public static SelectCardReaderDialogFragment getInstance(ArrayList<Reader> terminalReaders, boolean stripe) {
        SelectCardReaderDialogFragment rejectReasonDialog = new SelectCardReaderDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("readers", new Gson().toJson(terminalReaders));
        bundle.putBoolean("stripe", true);
        rejectReasonDialog.setArguments(bundle);
        return rejectReasonDialog;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.dialog_select_cardreader, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        selectedCardReader = MyApp.getInstance().myPreferences.getDefaultCardReader();
        if (getArguments() != null) {
            ArrayList<Object> cr = new Gson().fromJson(getArguments().getString("readers"), new TypeToken<List<Reader>>() {
            }.getType());
            if (cr != null && cr.size() > 0) cardReaders.addAll(cr);
            stripe = getArguments().getBoolean("stripe");
        }
        initViews(view);
        setListeners();
    }

    private void initViews(View view) {
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnCancel = view.findViewById(R.id.btnCancel);
        rvReaders = view.findViewById(R.id.rvCardReaders);

        cardReaders.addAll(MyApp.getInstance().myPreferences.getLoggedInAdmin().selected_business.card_readers);
        cardReaderAdapter = new CardReaderAdapter(cardReaders, (position, data) -> {
            if (data instanceof BusinessCardReader) {
                selectedCardReader = (BusinessCardReader) data;
                btnConfirm.setEnabled(true);
            } else if (data instanceof Reader) {
                stripeReader = (Reader) data;
                btnConfirm.setEnabled(true);
            }
        });
        rvReaders.setAdapter(cardReaderAdapter);
        rvReaders.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));

        if (selectedCardReader != null) {
            cardReaderAdapter.selectedId = selectedCardReader.id;
            cardReaderAdapter.notifyDataSetChanged();
            btnConfirm.setEnabled(true);
        }
    }

    private void setListeners() {
        btnConfirm.setOnClickListener(view -> {
            if (stripe && stripeReader == null) {
                ToastUtils.makeSnackToast(getActivity(), "Please select card reader");
                return;
            }
            if (!stripe && selectedCardReader == null) {
                ToastUtils.makeSnackToast(getActivity(), "Please select card reader");
                return;
            }
            if (dialogDismissListener != null) {
                if (stripe) dialogDismissListener.onDialogDismiss(stripeReader);
                else dialogDismissListener.onDialogDismiss(selectedCardReader);
            }
            dismiss();
        });
        btnCancel.setOnClickListener(view -> {
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss(null);
            dismiss();
        });
    }


}
