package com.example.epos_2021.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.CardReaderAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.BusinessCardReader;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;

import java.util.ArrayList;

public class SelectCardReaderFragment extends BaseFragment {

    private String title, info;
    private DialogDismissListener dialogDismissListener;
    private MaterialButton btnConfirm, btnCancel;
    private BusinessCardReader selectedCardReader;
    private RecyclerView rvReaders;
    private CardReaderAdapter cardReaderAdapter;
    private ArrayList<Object> cardReaders = new ArrayList<>();

    public static SelectCardReaderFragment getInstance() {
        SelectCardReaderFragment rejectReasonDialog = new SelectCardReaderFragment();
        Bundle bundle = new Bundle();
        rejectReasonDialog.setArguments(bundle);
        return rejectReasonDialog;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_select_cardreader, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        selectedCardReader = MyApp.getInstance().myPreferences.getDefaultCardReader();
        initViews(view);
        setListeners();
    }

    private void initViews(View view) {
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnCancel = view.findViewById(R.id.btnCancel);
        rvReaders = view.findViewById(R.id.rvCardReaders);

        btnConfirm.setVisibility(View.GONE);
        btnCancel.setVisibility(View.GONE);
        cardReaders.addAll(MyApp.getInstance().myPreferences.getLoggedInAdmin().selected_business.card_readers);
        cardReaderAdapter = new CardReaderAdapter(cardReaders, (position, data) -> {
            if (data instanceof BusinessCardReader) {
                selectedCardReader = (BusinessCardReader) data;
                if (dialogDismissListener != null)
                    dialogDismissListener.onDialogDismiss(selectedCardReader);

            }
        });
        rvReaders.setAdapter(cardReaderAdapter);
        rvReaders.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));

        if (selectedCardReader != null) {
            cardReaderAdapter.selectedId = selectedCardReader.id;
            cardReaderAdapter.notifyDataSetChanged();
            btnConfirm.setEnabled(true);
        }
    }

    private void setListeners() {
        btnConfirm.setOnClickListener(view -> {
            if (selectedCardReader == null) {
                ToastUtils.makeSnackToast(getActivity(), "Please select card reader");
            } else {
                if (dialogDismissListener != null)
                    dialogDismissListener.onDialogDismiss(selectedCardReader);


            }
        });
        btnCancel.setOnClickListener(view -> {
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss(null);

        });
    }


}
