package com.example.epos_2021.fragment;

import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.AppCompatButton;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.viewpager.widget.ViewPager;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.FloorAdapter;
import com.example.epos_2021.adapters.ViewpagerTabAdapter;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.daos.AppDatabase;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.example.epos_2021.models.Floor;
import com.example.epos_2021.models.Order;
import com.google.gson.Gson;

import java.util.ArrayList;

public class SwipeTableDialogFragment extends DialogFragment {

    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private DialogDismissListener dialogDismissListener;

    public static SwipeTableDialogFragment getInstance() {
        return new SwipeTableDialogFragment();
    }

    public static SwipeTableDialogFragment getInstance(Order order) {
        Bundle bundle = new Bundle();
        SwipeTableDialogFragment fragment = new SwipeTableDialogFragment();
        bundle.putString("order", new Gson().toJson(order));
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    private Order order;
    private TextView tvTitle;
    private ViewPager vpTables;
    private ViewpagerTabAdapter viewpagerAdapter;
    private ArrayList<Floor> floors = new ArrayList<>();
    private RecyclerView rvFloors;
    private FloorAdapter floorAdapter;
    private AlertDialog progressDialog;
    private AppCompatButton btnConfirm, btnCancel;
    private AppDatabase appDatabase = MyApp.getInstance().appDatabase;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_swipe_table_dialog, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (getArguments() != null) {
            order = new Gson().fromJson(getArguments().getString("order"), Order.class);
        }

        initViews(view);
        setListeners();
        new FloorsAsyncTask().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity());
        rvFloors = view.findViewById(R.id.rvFloors);
        vpTables = view.findViewById(R.id.vpTables);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnCancel = view.findViewById(R.id.btnCancel);
        tvTitle = view.findViewById(R.id.tvTitle);

        floorAdapter = new FloorAdapter(floors, (position, data) -> {
            Floor selectedFloor = (Floor) data;
            vpTables.setCurrentItem(position);
            floorAdapter.selectedFloorId = selectedFloor.id;
            floorAdapter.notifyDataSetChanged();
        });

        rvFloors.setAdapter(floorAdapter);
        rvFloors.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false));
        viewpagerAdapter = new ViewpagerTabAdapter(getChildFragmentManager(), FragmentPagerAdapter.BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
        vpTables.setAdapter(viewpagerAdapter);

        if (order == null) {
            tvTitle.setText("Assign Table");
        } else {
            tvTitle.setText("Swipe Table");
        }
    }

    private void setListeners() {

        btnCancel.setOnClickListener(v -> dismiss());
        btnConfirm.setOnClickListener(v -> dismiss());

        vpTables.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

            }

            @Override
            public void onPageSelected(int position) {
                // on page scroll change floor tables
                Floor selectedFloor = floors.get(position);
                floorAdapter.selectedFloorId = selectedFloor.id;
                floorAdapter.notifyDataSetChanged();
            }

            @Override
            public void onPageScrollStateChanged(int state) {

            }
        });
    }

    private class FloorsAsyncTask extends AsyncTask<String, String, String> {

        @Override
        protected String doInBackground(String... strings) {
            floors.clear();
            floors.addAll(appDatabase.floorDao().list());

            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            progressDialog.dismiss();
            floorAdapter.selectedFloorId = floors.get(0).id;
            floorAdapter.notifyDataSetChanged();
            for (int i = 0; i < floors.size(); i++) {
                if (order == null)
                    viewpagerAdapter.addFragment(TablesFragment.getInstance(floors.get(i).id, "assignTable"), floors.get(i).name);
                else
                    viewpagerAdapter.addFragment(TablesFragment.getInstance(floors.get(i).id, order), floors.get(i).name);
            }
            viewpagerAdapter.notifyDataSetChanged();
        }
    }


}