package com.example.epos_2021.fragment;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.TableLockingAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.Table;
import com.example.epos_2021.models.TableStatus;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.services.SingleOrderUploaderService;
import com.example.epos_2021.utils.Constants;
import com.google.android.material.chip.Chip;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 18-06-2021.
 */
public class TablesLockingStatusFragment extends BaseFragment {
    public static TablesLockingStatusFragment getInstance() {
        return new TablesLockingStatusFragment();
    }

    private SwipeRefreshLayout swipeRefreshLayout;
    private TableLockingAdapter tableLockingAdapter;
    private RecyclerView rvTables;
    private ArrayList<Object> objects = new ArrayList<>();
    private ArrayList<Table> tables = new ArrayList<>();
    private Chip chipBack;
    TableStatus vacantStatus = myApp.findStatus("Vacant");
    TableStatus takingOrderStatus = myApp.findStatus("Taking Order");
    TableStatus orderTakenStatus = myApp.findStatus("Order Taken");
    TableStatus servedStatus = myApp.findStatus("Served");
    TableStatus servedPaidStatus = myApp.findStatus("Served and paid");
    private AlertDialog progressBarDialog;

    @Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_tables_locking_status, container, false);
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        fetchTables();
        swipeRefreshLayout.setOnRefreshListener(this::fetchTables);
    }

    private void initViews(View view) {
        progressBarDialog = CommonFunctions.customProgressDialog(getActivity(), "");
        chipBack = view.findViewById(R.id.chipBack);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        rvTables = view.findViewById(R.id.rvTables);
        int spanCount = 4;
        if (MyApp.screenWidth > 2000) {
            spanCount = 6;
        } else if (MyApp.screenWidth > 1600) {
            spanCount = 5;
        }
        GridLayoutManager gridLayoutManager = new GridLayoutManager(getActivity(), spanCount);
        rvTables.setLayoutManager(gridLayoutManager);
        tableLockingAdapter = new TableLockingAdapter(objects, (position, data) -> {
            changeTableStatus((Table) data);
        });
        rvTables.setAdapter(tableLockingAdapter);

        chipBack.setOnClickListener(view1 -> getActivity().onBackPressed());
    }

    private void changeTableStatus(Table table) {
        ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Successful", "Table status changed successfully", 1, "Okay", null);
        confirmationDialogFragment.show(getChildFragmentManager(), "status");
        confirmationDialogFragment.setDialogDismissListener(o -> {

            uploadOrder(table._last_order_id);
            unlockableTable = table;
        });

    }

    Table unlockableTable;

    private void uploadOrder(int orderId) {
        progressBarDialog.show();
        Intent intent = new Intent(getActivity(), SingleOrderUploaderService.class);
        intent.putExtra("_order_id", orderId);
        getActivity().startService(intent);
    }

    private void unlockTable(Table table) {
        new Thread(() -> {
            table.locked = false;
            appDatabase.tableDao().insert(table);
            if (getActivity() != null) getActivity().runOnUiThread(() -> {
                tableLockingAdapter.notifyDataSetChanged();
            });
            AndroidNetworking.post(ApiEndPoints.tables + table.id)
                    .addApplicationJsonBody(table)
                    .build()
                    .getAsObject(Table.class, new ParsedRequestListener() {
                        @Override
                        public void onResponse(Object response) {
                            unlockableTable = null;
                            fetchTables();
                        }

                        @Override
                        public void onError(ANError anError) {

                        }
                    });
        }).start();
    }

    private void fetchTables() {
        if (!myApp.isConnected(getActivity())) {
            new TablesAsyncTask().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
            return;
        }
        swipeRefreshLayout.setRefreshing(true);
        AndroidNetworking.get(ApiEndPoints.tables)
                .addQueryParameter("nopaginate", "1")
                .build()
                .getAsObjectList(Table.class, new ParsedRequestListener<List<Table>>() {
                    @Override
                    public void onResponse(List<Table> response) {
                        new TablesAsyncTask().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, response);

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        new TablesAsyncTask().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                    }
                });
    }

    private class TablesAsyncTask extends AsyncTask<List<Table>, String, String> {

        @Override
        protected String doInBackground(List<Table>... tablesParam) {
            if (tablesParam.length > 0) {
                List<Table> tabs = tablesParam[0];

                for (Table table : tabs) {
                    if (!Validators.isNullOrEmpty(table.table_status_id))
                        table.table_status = appDatabase.tableStatusDao().view(table.table_status_id);
                    Order dbOrder = appDatabase.orderDao().findLastOrderOfTable(table.id);
                    if (dbOrder != null) {
                        table._last_order_id = dbOrder._id;
                        table.last_order_total = dbOrder.total;
                        table.last_order_created_at = dbOrder.created_at;
                    }
                }
                tables.clear();
                tables.addAll(tabs);
                objects.clear();
                objects.addAll(tables);
            } else {
                tables.clear();
                tables.addAll(appDatabase.tableDao().list());
                objects.clear();
                objects.addAll(tables);
            }


            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            swipeRefreshLayout.setRefreshing(false);
            tableLockingAdapter.notifyDataSetChanged();
        }
    }

    @Override
    public void onResume() {
        super.onResume();

        if (getActivity() != null && orderUpdateReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).registerReceiver(orderUpdateReceiver, new IntentFilter(
                    Constants.EPOS_ORDER_UPDATE));
    }

    @Override
    public void onPause() {
        super.onPause();
        if (getActivity() != null && orderUpdateReceiver != null) {
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(orderUpdateReceiver);
        }
    }

    BroadcastReceiver orderUpdateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getBooleanExtra("result", false)) {
                progressBarDialog.dismiss();
                if (unlockableTable != null)
                    unlockTable(unlockableTable);
            }
        }
    };
}
