package com.example.epos_2021.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import com.bumptech.glide.Glide;
import com.example.epos_2021.R;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.interfaces.DialogDismissListener;
import com.google.android.material.chip.Chip;
import com.mikhaellopez.circularimageview.CircularImageView;

public class UserDetailsDialogFragment extends DialogFragment {
    public static UserDetailsDialogFragment getInstance(String logo, String name, String phone) {
        UserDetailsDialogFragment detailsDialogFragment = new UserDetailsDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("customer_logo", logo);
        bundle.putString("customer_name", name);
        bundle.putString("customer_phone", phone);
        detailsDialogFragment.setArguments(bundle);
        return detailsDialogFragment;
    }

    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private DialogDismissListener dialogDismissListener;

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    private Chip chipBack;
    private CircularImageView ivLogo;
    private TextView tvCustomerName;
    private ImageView ivCall, ivMessage, ivDelete;
    private TextView tvMobile, tvEmail, tvAddress;
    private RelativeLayout rlCall, rlChat, rlDelete;
    private LinearLayout llContact, llEmail, llAddress;

    private String customer_logo, customer_name, customer_phone;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            customer_name = getArguments().getString("customer_name");
            customer_phone = getArguments().getString("customer_phone");
            customer_logo = getArguments().getString("customer_logo");
        }
        return inflater.inflate(R.layout.dialog_user_info, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();
    }

    private void setListeners() {
        chipBack.setOnClickListener(v -> dismiss());

        ivCall.setOnClickListener(v -> {
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss(customer_phone);
            dismiss();
        });
        ivMessage.setOnClickListener(v -> {
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss("compose");
            dismiss();
        });
    }

    protected void initViews(View view) {
        chipBack = view.findViewById(R.id.chipBack);
        ivLogo = view.findViewById(R.id.ivLogo);
        tvCustomerName = view.findViewById(R.id.tvCustomerName);
        ivCall = view.findViewById(R.id.ivCall);
        ivMessage = view.findViewById(R.id.ivChat);
        ivDelete = view.findViewById(R.id.ivDelete);
        rlCall = view.findViewById(R.id.rlCall);
        rlDelete = view.findViewById(R.id.rlDelete);
        rlChat = view.findViewById(R.id.rlChat);
        llContact = view.findViewById(R.id.llContact);
        llEmail = view.findViewById(R.id.llEmail);
        llAddress = view.findViewById(R.id.llAddress);
        tvMobile = view.findViewById(R.id.tvContactNumber);
        tvEmail = view.findViewById(R.id.tvCustomerEmail);
        tvAddress = view.findViewById(R.id.tvAddress);

        rlDelete.setVisibility(View.GONE);
        llEmail.setVisibility(View.GONE);
        llAddress.setVisibility(View.GONE);

        if (!Validators.isNullOrEmpty(customer_logo)) {
            Glide.with(getActivity())
                    .load(customer_logo)
                    .placeholder(R.drawable.avatar_logo_placeholder)
                    .error(R.drawable.avatar_logo_placeholder)
                    .into(ivLogo);
        } else {
            ivLogo.setImageResource(R.drawable.avatar_logo_placeholder);
        }

        if (!Validators.isNullOrEmpty(customer_phone)) {
            tvMobile.setText(customer_phone);
            llContact.setVisibility(View.VISIBLE);
            rlCall.setVisibility(View.VISIBLE);
        } else {
            llContact.setVisibility(View.GONE);
            rlCall.setVisibility(View.GONE);
        }

        if (!Validators.isNullOrEmpty(customer_name)) {
            tvCustomerName.setText(customer_name);
            tvCustomerName.setVisibility(View.VISIBLE);
        } else {
            tvCustomerName.setVisibility(View.GONE);
        }
    }


}
