package com.example.epos_2021.merchant.fragments;

import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatEditText;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.models.Restaurant;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.online.base.BaseBottomSheet;
import com.example.epos_2021.online.interfaces.DialogDismissListener;
import com.example.epos_2021.storageutils.MyPreferences;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;

import org.json.JSONObject;

public class EditRestaurantDescriptionBottomsheet extends BaseBottomSheet {
    private AppCompatEditText etDecription;
    private TextView tvWordCounter;
    private MaterialButton btnCancel, btnUpdate;
    private Restaurant loggedInRestaurant = MyApp.getInstance().myPreferences.getRestaurant();
    private MyPreferences myPreferences = myApp.myPreferences;

    private String description;
    private DialogDismissListener dialogDismissListener;

    public static EditRestaurantDescriptionBottomsheet getInstance(String description) {
        EditRestaurantDescriptionBottomsheet descriptionBottomsheet = new EditRestaurantDescriptionBottomsheet();
        Bundle bundle = new Bundle();
        bundle.putString("res_description", description);
        descriptionBottomsheet.setArguments(bundle);
        return descriptionBottomsheet;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_edit_description_bottomsheet, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        updateViews();
    }


    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (intentExtrasData.containsKey("res_description")) {
            description = String.valueOf(intentExtrasData.get("res_description"));
        }
    }

    @Override
    protected void setListeners() {
        super.setListeners();
        etDecription.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                tvWordCounter.setText(etDecription.length() + "/300");
                if (etDecription.length() > 300) {
                    tvWordCounter.setTextColor(getResources().getColor(R.color.red));
                } else {
                    tvWordCounter.setTextColor(getResources().getColor(R.color.black));
                }
            }
        });
        btnCancel.setOnClickListener(v -> dismiss());
        btnUpdate.setOnClickListener(view -> {
            if (etDecription.length() > 300) {
                ToastUtils.makeToast(getActivity(), "Description length should not more than 300 characters");
            } else {
                updateRestaurantDescription();
            }
        });
    }

    private void updateRestaurantDescription() {
        if (getActivity() != null) getActivity().runOnUiThread(() -> {
            progressDialog.show();
        });
        AndroidNetworking.post(ApiEndPoints.restaurant_details + loggedInRestaurant.id)
                .addBodyParameter("restaurant_about", etDecription.getText().toString())
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) getActivity().runOnUiThread(() -> {
                            progressDialog.dismiss();
                        });
                        ToastUtils.makeToast(getActivity(), "Restaurant description updated!");
                        loggedInRestaurant.restaurant_about = etDecription.getText().toString();
                        myPreferences.saveRestaurant((loggedInRestaurant));
                        if (dialogDismissListener != null) {
                            dialogDismissListener.onDialogDismiss(null);
                        }
                        dismiss();

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) getActivity().runOnUiThread(() -> {
                            progressDialog.dismiss();
                        });
                        if (anError.getErrorCode() == 400) {
                            ToastUtils.makeToast(getActivity(), anError.getErrorAsObject(ApiError.class).getMessage());
                        }
                    }
                });
    }

    private void updateViews() {
        etDecription.setText(description);
        etDecription.setSelection(etDecription.getText().length());
        tvWordCounter.setText(etDecription.length() + "/300");
        tvWordCounter.setTextColor(getResources().getColor(R.color.black));
        if (etDecription.length() > 300) {
            tvWordCounter.setTextColor(getResources().getColor(R.color.red));
        } else {
            tvWordCounter.setTextColor(getResources().getColor(R.color.black));
        }
    }


    @Override
    protected void initViews(View view) {
        super.initViews(view);
        ivClose = view.findViewById(R.id.ivClose);
        etDecription = view.findViewById(R.id.etDecription);
        tvWordCounter = view.findViewById(R.id.tvWordCounter);
        btnCancel = view.findViewById(R.id.btnCancel);
        btnUpdate = view.findViewById(R.id.btnUpdate);
    }
}
