package com.example.epos_2021.merchant.fragments;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.R;
import com.example.epos_2021.adapters.MenuAdapter;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.fragment.SettingsUserManaementFragment;
import com.example.epos_2021.models.ExclusionModel;
import com.example.epos_2021.models.Restaurant;
import com.example.epos_2021.network.ApiEndPoints;
import com.google.android.material.chip.Chip;

import org.json.JSONObject;

import java.util.ArrayList;

public class MerchantSettingsFragment extends BaseFragment {

    private Chip chipBack;
    private LinearLayout llProgress, llData, llNoRecord;
    private RecyclerView rvSettings;
    private ArrayList<ExclusionModel> settingList = new ArrayList<>();
    private MenuAdapter menuAdapter;


    public static MerchantSettingsFragment getInstance() {
        return new MerchantSettingsFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_merchant_settings, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        if (!Validators.isNullOrEmpty(myApp.restaurant_id))
            fetchRestaurantDetails();
        else
            setUpSettings();
    }


    private void fetchRestaurantDetails() {
        if (!myApp.isConnected(getActivity())) {
            setDataManagementUI();
            return;
        }
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                llProgress.setVisibility(View.VISIBLE);
                llData.setVisibility(View.GONE);
            });
        }
        if (Validators.isNullOrEmpty(myApp.restaurant_id)) {
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    llProgress.setVisibility(View.GONE);
                    llData.setVisibility(View.VISIBLE);
                });
            }

            setUpSettings();
            return;
        }
        AndroidNetworking.get(ApiEndPoints.restaurant_details + myApp.restaurant_id)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        myPreferences.saveRestaurant(response);
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                llProgress.setVisibility(View.GONE);
                                llData.setVisibility(View.VISIBLE);
                            });
                        }

                        setUpSettings();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                llProgress.setVisibility(View.GONE);
                                llData.setVisibility(View.VISIBLE);
                            });
                        }
                        setUpSettings();
                    }
                });
    }

    private void setDataManagementUI() {
        llData.setVisibility(View.VISIBLE);
        settingList.clear();
        if (MyApp.userPermission != null && MyApp.userPermission.clear_database.actions.list) {
            ExclusionModel model = new ExclusionModel();
            model.id = "data_management";
            model.name = "Data Management";
            model.isCheckable = true;
            settingList.add(model);
            selectDefault();
        }
    }

    private void setUpSettings() {
        settingList.clear();
        ExclusionModel model = new ExclusionModel();
        if (!Validators.isNullOrEmpty(myApp.restaurant_id)) {

            model = new ExclusionModel();
            model.id = "restaurant_details";
            model.name = "Restaurant Details";
            model.isCheckable = true;
            settingList.add(model);

            model = new ExclusionModel();
            model.id = "restaurant_management";
            model.name = "Restaurant Management";
            model.isCheckable = true;
            settingList.add(model);

            model = new ExclusionModel();
            model.id = "profile";
            model.name = "Profile";
            model.isCheckable = true;
            settingList.add(model);

            model = new ExclusionModel();
            model.id = "bank_details";
            model.name = "Bank Account Details";
            model.isCheckable = true;
            settingList.add(model);

            model = new ExclusionModel();
            model.id = "feedback";
            model.name = "Feedback";
            model.isCheckable = true;
            settingList.add(model);

            model = new ExclusionModel();
            model.id = "contact_tiffintom";
            model.name = "Contact TiffinTom";
            model.isCheckable = false;
            settingList.add(model);
        }

        if (MyApp.userPermission != null && MyApp.userPermission.clear_database.actions.list) {
            model = new ExclusionModel();
            model.id = "data_management";
            model.name = "Data Management";
            model.isCheckable = true;
            settingList.add(model);
        }
        if (false) {
            model = new ExclusionModel();
            model.id = "user_management";
            model.name = "User Management";
            model.isCheckable = true;
            settingList.add(model);
        }
        selectDefault();
    }

    private void selectDefault() {
        if (settingList.size() > 0) {
            settingList.get(0).isChecked = true;
            menuAdapter.checkedId = settingList.get(0).id;
            handleMenuClick(settingList.get(0));
        }
        menuAdapter.notifyDataSetChanged();
    }

    int Year, Month, Day;

    private void initViews(View view) {
        rvSettings = view.findViewById(R.id.rvSettings);
        llProgress = view.findViewById(R.id.llProgress);
        llData = view.findViewById(R.id.llData);
        llNoRecord = view.findViewById(R.id.llNoRecord);
        chipBack = view.findViewById(R.id.chipBack);

        menuAdapter = new MenuAdapter(settingList, true, true, (position, data) -> {
            ExclusionModel model = (ExclusionModel) data;
            handleMenuClick(model);

        });
        rvSettings.setLayoutManager(new LinearLayoutManager(getActivity(), LinearLayoutManager.VERTICAL, false));
        rvSettings.setAdapter(menuAdapter);

        chipBack.setOnClickListener(v -> getActivity().onBackPressed());
    }

    private void handleMenuClick(ExclusionModel model) {
        if (model.id.equalsIgnoreCase("restaurant_details")) {
            changeFragment(SettingsRestaurantDetailsFragment.getInstance());
        } else if (model.id.equalsIgnoreCase("restaurant_management")) {
            changeFragment(SettingsRestaurantManagementFragment.getInstance());
        } else if (model.id.equalsIgnoreCase("profile")) {
            changeFragment(SettingsProfileFragment.getInstance());
        } else if (model.id.equalsIgnoreCase("bank_details")) {
            changeFragment(SettingsBankDetailsFragment.getInstance());
        } else if (model.id.equalsIgnoreCase("feedback")) {
            changeFragment(SettingsFeedbackFragment.getInstance());
        } else if (model.id.equalsIgnoreCase("contact_tiffintom")) {
            openContactDialog();
        } else if (model.id.equalsIgnoreCase("data_management")) {
            changeFragment(SettingsDataManagementFragment.getInstance());
        } else if (model.id.equalsIgnoreCase("user_management")) {
            changeFragment(SettingsUserManaementFragment.getInstance());
        }
    }

    private void openContactDialog() {
        Restaurant restaurant = myPreferences.getRestaurant();
        SettingsContactTiffintomDialogFragment contactDialogFragment = SettingsContactTiffintomDialogFragment.getInstance();
        contactDialogFragment.show(getChildFragmentManager(), "contact_us");
        contactDialogFragment.setDialogDismissListener(o -> {
            if (((String) o).equalsIgnoreCase("call")) {
                // Do Call
                Intent intent = new Intent();
                intent.setAction(Intent.ACTION_DIAL);
                intent.setData(Uri.parse("tel: " + restaurant.contact_phone));
                startActivity(intent);
            } else {
                //Do Email
                Intent intent = new Intent(Intent.ACTION_SENDTO);
                intent.setData(Uri.parse("mailto:")); // only email apps should handle this
                intent.putExtra(Intent.EXTRA_EMAIL, new String[]{restaurant.contact_email});
                startActivity(intent);
            }
        });
    }

    private void changeFragment(Fragment fragment) {
        try {
            FragmentManager manager = getChildFragmentManager();
            FragmentTransaction transaction = manager.beginTransaction();
            transaction.replace(R.id.nav_setting_fragment, fragment);
            transaction.addToBackStack("can_go_back");
            transaction.commitAllowingStateLoss();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
