package com.example.epos_2021.merchant.fragments;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.example.epos_2021.R;
import com.example.epos_2021.base.BaseFragment;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.MoneyTextWatcher;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.merchant.adapters.MerchantTransactionsAdapter;
import com.example.epos_2021.merchant.models.PaymentLink;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.models.Business;
import com.example.epos_2021.models.MerchantUserPermission;
import com.example.epos_2021.models.Restaurant;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.utils.LogUtils;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class PaymentLinkFragment extends BaseFragment {
    public static PaymentLinkFragment getInstance() {
        return new PaymentLinkFragment();
    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
    }

    private EditText etAmount, etPhone, etTip;
    private MaterialButton btnSend, btnCancel;
    private TextView tvViewAll, tvTransactionTitle;
    private RecyclerView rvPaymentLinks;
    private ArrayList<Object> paymentLinks = new ArrayList<>();
    private MerchantTransactionsAdapter transactionsAndStatementsAdapter;
    private LinearLayout llMoto;
    private float amount = 0;
    int merchantRole = myPreferences.getMerchantRole();
    private LinearLayout llMainLayout;
    private Business loggedInBusiness;
    private Restaurant restaurant = myPreferences.getRestaurant();
    String stripe_private_key, stripe_public_key, stripe_mode;
    private AlertDialog progressDialog;
    private MerchantUserPermission permission;
    private ImageView ivBack;

    protected void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity());
        loggedInBusiness = myPreferences.getLoggedInAdmin().selected_business;
        llMainLayout = view.findViewById(R.id.llMainLayout);
        merchantRole = myPreferences.getMerchantRole();
        if (merchantRole == 1) {
            permission = loggedInBusiness.supervisor_permissions;
        } else if (merchantRole == 2) {
            permission = loggedInBusiness.admin_permissions;
        }
        stripe_mode = loggedInBusiness.stripe_mode;
        if (stripe_mode.equalsIgnoreCase("live")) {
            stripe_private_key = loggedInBusiness.stripe_private_key_live;
            stripe_public_key = loggedInBusiness.stripe_public_key_live;
        } else {
            stripe_private_key = loggedInBusiness.stripe_private_key_test;
            stripe_public_key = loggedInBusiness.stripe_public_key_test;
        }
        ivBack = view.findViewById(R.id.ivBack);
        etAmount = view.findViewById(R.id.etAmount);
        etPhone = view.findViewById(R.id.etPhone);
        etTip = view.findViewById(R.id.etTip);
        btnCancel = view.findViewById(R.id.btnCancel);
        btnSend = view.findViewById(R.id.btnSend);
        rvPaymentLinks = view.findViewById(R.id.rvPaymentLinks);
        tvViewAll = view.findViewById(R.id.tvViewAll);
        tvTransactionTitle = view.findViewById(R.id.tvRecentTransactions);
        llMoto = view.findViewById(R.id.llMoto);

        rvPaymentLinks.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));

        transactionsAndStatementsAdapter = new MerchantTransactionsAdapter(paymentLinks, false, (position, data) -> {
            resendPaymentLink((PaymentLink) data);
        }, (position, data) -> {
        }, ((position, data) -> {
        }));
        rvPaymentLinks.setAdapter(transactionsAndStatementsAdapter);


    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_payment_link, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        getRecentPaymentLinks();
        startTimer();
    }

    private void setListeners() {
        ivBack.setOnClickListener(view -> {
            if (getActivity() != null) {
                getActivity().onBackPressed();
            }
        });
        etAmount.addTextChangedListener(new MoneyTextWatcher(etAmount));
        etTip.addTextChangedListener(new MoneyTextWatcher(etTip));
        btnSend.setOnClickListener(view -> {
            if (Validators.isNullOrEmpty(etPhone.getText().toString())) {
                etPhone.setError("Please enter phone");
                etPhone.requestFocus();
            } else if (Validators.isNullOrEmpty(etAmount.getText().toString())) {
                etAmount.setError("Please enter amount");
                etAmount.requestFocus();
            } else if (Float.parseFloat(etAmount.getText().toString().replace(",", "")) == 0) {
                etPhone.setError("Amount should be greater than zero");
                etPhone.requestFocus();
            } else {
                new Thread(this::generatePaymentLink).start();

            }
        });
        btnCancel.setOnClickListener(view -> {
            CommonFunctions.hideKeyboard(getActivity(), etAmount);
            CommonFunctions.hideKeyboard(getActivity(), etPhone);
            getActivity().onBackPressed();
        });
        tvViewAll.setOnClickListener(view -> {
            CommonFunctions.hideKeyboard(getActivity(), etAmount);
            CommonFunctions.hideKeyboard(getActivity(), etPhone);
            FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
            ft.replace(R.id.nav_payment_fragment, PaymentLinksTransactionFragment.getInstance(), "Payment link transactions");
            ft.addToBackStack("can_go_back");
            ft.commit();
        });

    }

    private void resendPaymentLink(PaymentLink data) {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        AndroidNetworking.post(ApiEndPoints.online_payment_links_resend)
                .addPathParameter("id", data.id)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                            });
                        }
                        ToastUtils.showSnackBar(getActivity(), llMainLayout, "Payment link resent!");

                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        } else {
                            ToastUtils.showSnackBar(getActivity(), llMainLayout, "Something went wrong!");
                        }
                    }
                });
    }

    private void generatePaymentLink() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        float total_amount = Float.parseFloat(etAmount.getText().toString().replace(",", ""));
        if (!Validators.isNullOrEmpty(etTip.getText().toString()) && Float.parseFloat(etTip.getText().toString().replace(",", "")) >= 0) {
            total_amount = total_amount + Float.parseFloat(etTip.getText().toString().replace(",", ""));
        }
        AndroidNetworking.post(ApiEndPoints.online_payment_links)
                .addBodyParameter("phone", etPhone.getText().toString())
                .addBodyParameter("secretkey", stripe_private_key)
                .addBodyParameter("publishkey", stripe_public_key)
                .addBodyParameter("amount", String.valueOf(total_amount))
                .addBodyParameter("name", loggedInBusiness.name)
                .addBodyParameter("business_id", loggedInBusiness.id)
                .addBodyParameter("device_id", myPreferences.getUserFCMToken())
                .addBodyParameter("description", loggedInBusiness.statement_description)
                .addBodyParameter("descriptor", loggedInBusiness.statement_descriptor)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                                etAmount.setText(null);
                                etTip.setText(null);
                                etPhone.setText(null);
                                CommonFunctions.hideKeyboard(getActivity(), etAmount);
                                CommonFunctions.hideKeyboard(getActivity(), etPhone);
                                CommonFunctions.hideKeyboard(getActivity(), etTip);
                            });
                        }
                        ToastUtils.showSnackBar(getActivity(), llMainLayout, "Payment link sent successfully");

                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        } else {
                            ToastUtils.showSnackBar(getActivity(), llMainLayout, "Something went wrong!");
                        }
                    }
                });
    }

    private void getRecentPaymentLinks() {

        AndroidNetworking.get(ApiEndPoints.online_payment_links)
                .addQueryParameter("business_id", loggedInBusiness.id)
                .addQueryParameter("name", loggedInBusiness.name)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {

                            });
                        }
                        try {
                            Type type = new TypeToken<List<PaymentLink>>() {
                            }.getType();
                            paymentLinks.clear();
                            paymentLinks.addAll(new Gson().fromJson(response.getJSONArray("data").toString(), type));
                            transactionsAndStatementsAdapter.notifyDataSetChanged();

                            if (paymentLinks.size() > 0) {
                                tvTransactionTitle.setVisibility(View.VISIBLE);
                            } else {
                                tvTransactionTitle.setVisibility(View.GONE);
                            }

                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        if (paymentLinks.size() > 0) {
                            tvTransactionTitle.setVisibility(View.VISIBLE);
                        } else {
                            tvTransactionTitle.setVisibility(View.GONE);
                        }
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            LogUtils.e(apiError.getMessage());
                        }
                        anError.printStackTrace();
                    }
                });
    }

    TimerTask timerTask;
    Timer timer;

    public void startTimer() {
        //set a new Timer
        timer = new Timer();
        initTimerTask();
        //schedule the timer, to wake up every 2 second
        try {
            if (timerTask != null) timer.schedule(timerTask, 1000, 15000); //
        } catch (Exception e) {
        }
    }

    private void initTimerTask() {
        timerTask = new TimerTask() {
            public void run() {
                try {
                    getRecentPaymentLinks();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        };
    }

    @Override
    public void onResume() {
        super.onResume();
        setListeners();
    }

    @Override
    public void onStop() {
        CommonFunctions.hideKeyboard(getActivity(), etAmount);
        super.onStop();
    }

    @Override
    public void onDestroy() {
        if (timerTask != null) timerTask.cancel();
        super.onDestroy();
    }
}