package com.example.epos_2021.online.fragments;


import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import com.example.epos_2021.R;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.online.interfaces.FilterDialogDismissListener;
import com.example.epos_2021.utils.ToastUtils;
import com.google.android.material.button.MaterialButton;
import com.wdullaer.materialdatetimepicker.date.DatePickerDialog;

import java.util.Calendar;

/**
 * Created by Amrish on 11-11-2020.
 */
public class DateRangePickerDialogFragment extends DialogFragment {
    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().setCancelable(false);
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private String title;
    private MaterialButton btnConfirm, btnCancel;
    private LinearLayout llToDate, llFromDate;
    private TextView tvInfo, tvTitle, tvFromDate, tvToDate;
    private FilterDialogDismissListener dialogDismissListener;
    private String fromDateString, toDateString;
    private Calendar today = Calendar.getInstance();
    int fromYear, fromMonth, fromDay, toYear, toMonth, toDay;
    private Calendar fromCalender = Calendar.getInstance();
    private Calendar toCalender = Calendar.getInstance();

    public static DateRangePickerDialogFragment getInstance(String title, String fromDate, String toDate) {
        DateRangePickerDialogFragment rejectReasonDialog = new DateRangePickerDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("title", title);
        bundle.putString("fromDate", fromDate);
        bundle.putString("toDate", toDate);
        rejectReasonDialog.setArguments(bundle);
        return rejectReasonDialog;
    }

    public void setDialogDismissListener(FilterDialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.dialog_online_orders_date_picker, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        title = getArguments().getString("title");
        fromDateString = getArguments().getString("fromDate");
        toDateString = getArguments().getString("toDate");
        initViews(view);
        setListeners();
    }

    private void initViews(View view) {
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnCancel = view.findViewById(R.id.btnCancel);
        tvTitle = view.findViewById(R.id.tvTitle);
        tvFromDate = view.findViewById(R.id.tvFromDate);
        tvToDate = view.findViewById(R.id.tvToDate);
        llFromDate = view.findViewById(R.id.llFromDate);
        llToDate = view.findViewById(R.id.llToDate);
        tvTitle.setText(title);
    }

    private void setListeners() {
        llFromDate.setOnClickListener(view -> {
            DatePickerDialog datePickerDialog = DatePickerDialog.newInstance((view12, y, m, d) -> {
                fromDay = d;
                fromMonth = m;
                fromYear = y;
                fromDateString = String.format("%02d", d) + "/" + String.format("%02d", (m + 1)) + "/" + String.format("%02d", y);
                tvFromDate.setText(fromDateString);
                fromCalender.set(y, m, d);
            }, today.get(Calendar.YEAR), today.get(Calendar.MONTH), today.get(Calendar.DAY_OF_MONTH));
            datePickerDialog.setThemeDark(false);
            datePickerDialog.showYearPickerFirst(false);
            datePickerDialog.show(getActivity().getFragmentManager(), "FromPickerDialog");
            if (!Validators.isNullOrEmpty(toDateString)) {
                datePickerDialog.setMaxDate(toCalender);
            }

        });
        llToDate.setOnClickListener(view -> {
            if (!Validators.isNullOrEmpty(fromDateString)) {
                DatePickerDialog datePickerDialog = DatePickerDialog.newInstance((view1, y, m, d) -> {
                    toDateString = String.format("%02d", d) + "/" + String.format("%02d", (m + 1)) + "/" + String.format("%02d", y);
                    toDay = d;
                    toMonth = m;
                    toYear = y;
                    tvToDate.setText(toDateString);
                    toCalender.set(y, m, d);
                }, toYear > 0 ? toYear : today.get(Calendar.YEAR), toMonth > 0 ? toMonth : today.get(Calendar.MONTH), toDay > 0 ? toDay : today.get(Calendar.DAY_OF_MONTH));
                datePickerDialog.setMinDate(fromCalender);
                datePickerDialog.setThemeDark(false);
                datePickerDialog.showYearPickerFirst(false);
                datePickerDialog.show(getActivity().getFragmentManager(), "ToPickerDialog");
            }
        });
        btnConfirm.setOnClickListener(view -> {
            if (Validators.isNullOrEmpty(fromDateString) || Validators.isNullOrEmpty(toDateString)) {
                ToastUtils.makeToast(getActivity(), "Please select date range");
            } else {
                if (dialogDismissListener != null)
                    dialogDismissListener.onDialogDismiss(fromDateString, toDateString);
                dismiss();
            }
        });
        btnCancel.setOnClickListener(view -> {
            dismiss();
        });
    }

}
