package com.example.epos_2021.online.notification;

import android.content.Context;
import android.content.Intent;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.example.epos_2021.MyApp;
import com.example.epos_2021.activities.HomeActivity;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.LogUtils;
import com.google.firebase.messaging.FirebaseMessagingService;
import com.google.firebase.messaging.RemoteMessage;

import org.json.JSONException;
import org.json.JSONObject;


public class MyOnlineFirebaseMessagingService extends FirebaseMessagingService {

    private static final String TAG = MyOnlineFirebaseMessagingService.class.getSimpleName();
    private NotificationUtils notificationUtils;
    String message = "", title = "";

    @Override
    public void onMessageReceived(RemoteMessage remoteMessage) {
        if (remoteMessage.getNotification() != null)
            LogUtils.e("FCM: " + remoteMessage.getNotification().getTitle());
        LogUtils.e("FCM MESSAGE");
        // Check if message contains a notification payload.

        if (remoteMessage.getData().size() > 0) {

            try {
                JSONObject json = new JSONObject(remoteMessage.getData().toString());
                JSONObject data = json.getJSONObject("data");
                Log.e("Notification","data:"+data.toString());
                if (data.has("notification_for")) {
                    LogUtils.e("Notification for:" + data.getString("notification_for"));

                    if (data.getString("notification_for").equalsIgnoreCase("epos")) {
                        Intent pushNotification = new Intent(Constants.EPOS_NEW_ORDER);
                        pushNotification.putExtra("refresh", true);
                        pushNotification.putExtra("notification_for", data.getString("notification_for"));
                        if (data.has("notification_section")) {
                            LogUtils.e("Notification section:" + data.getString("notification_section"));
                            pushNotification.putExtra("notification_section", data.getString("notification_section"));
                            if(data.getString("notification_section").equalsIgnoreCase("orders")){
                                pushNotification.putExtra("print", data.getBoolean("print"));
                                pushNotification.putExtra("order_id_", data.getInt("order_id"));
                            }
                        }
                        if (data.has("order_id")) {
                            LogUtils.e("new Order :" + data.get("order_id"));
                            pushNotification.putExtra("order_id", data.get("order_id").toString());
                        }
                        LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(pushNotification);
                    }
                } else {
                    handleDataMessage(data);
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }

        }
    }


    private void handleDataMessage(JSONObject data) {
        LogUtils.e("Notification Message");
        LogUtils.e(data);
        try {

            title = data.getString("title");
            message = data.getString("message");
            String timestamp = data.getString("timestamp");
            String order_id = null;
            String payment_status = null;
            String order_status = null;
            String order_type = null;
            if (data.has("order_id"))
                order_id = data.getString("order_id");
            if (data.has("payment_status"))
                payment_status = data.getString("payment_status");
            if (data.has("order_status"))
                order_status = data.getString("order_status");
            if (data.has("order_type"))
                order_type = data.getString("order_type");
            LogUtils.e("OrderId=>" + order_id);
            LogUtils.e("PaymentStatus=>" + payment_status);
            LogUtils.e("OrderStatus=>" + order_status);
            LogUtils.e("OrderType=>" + order_type);
            sendPushNotificationBroadCast(title, message, order_id, payment_status, order_status, order_type);
            Intent resultIntent = new Intent(getApplicationContext(), HomeActivity.class);
            if (title.equalsIgnoreCase("newtable")) {
                resultIntent.putExtra("destination", "reservations");
            } else {
                resultIntent.putExtra("destination", "orders");
            }
            if (!NotificationUtils.isAppIsInBackground(getApplicationContext())) {
                // app is in foreground, broadcast the push message
                Intent pushNotification = new Intent(Constants.NOTIFICATION);
                pushNotification.putExtra("refresh", true);

                LocalBroadcastManager.getInstance(this).sendBroadcast(pushNotification);

                // play notification sound
                NotificationUtils notificationUtils = new NotificationUtils(getApplicationContext());
                notificationUtils.playNotificationSound();

            }
//            showNotificationMessage(getApplicationContext(), title, message, timestamp, resultIntent);
        } catch (JSONException e) {
            LogUtils.e(TAG, "Json Exception: " + e.getMessage());
        } catch (Exception e) {
            LogUtils.e(TAG, "Exception: " + e.getMessage());
        }
    }

    private void sendPushNotificationBroadCast(String title, String message, String order_id, String payment_status, String order_status, String order_type) {
        Intent pushNotification = new Intent(Constants.PUSH_NOTIFICATION);
        pushNotification.putExtra("push", true);
        pushNotification.putExtra("title", title);
        pushNotification.putExtra("message", message);
        pushNotification.putExtra("order_id", order_id);
        pushNotification.putExtra("order_status", order_status);
        pushNotification.putExtra("order_type", order_type);
        pushNotification.putExtra("payment_status", payment_status);
        LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(pushNotification);
    }

    /**
     * Showing notification with text only
     */
    private void showNotificationMessage(Context context, String title, String message, String timeStamp, Intent intent) {
        notificationUtils = new NotificationUtils(context);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
        notificationUtils.showNotificationMessage("TiffinTom Restaurant", message, timeStamp, intent);
    }

    @Override
    public void onNewToken(@NonNull String s) {
        super.onNewToken(s);
        MyApp.getInstance().myPreferences.setUserFCMToken(s);
    }
}
