package com.example.epos_2021.services;

import android.annotation.SuppressLint;
import android.app.IntentService;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.util.Log;

import androidx.annotation.Nullable;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.example.epos_2021.MyApp;
import com.example.epos_2021.comman.CommonFunctions;
import com.example.epos_2021.comman.Validators;
import com.example.epos_2021.daos.AppDatabase;
import com.example.epos_2021.daos.relations.OrderItemWithAddonsIngredients;
import com.example.epos_2021.daos.relations.OrderWithItems;
import com.example.epos_2021.models.ApiError;
import com.example.epos_2021.models.Customer;
import com.example.epos_2021.models.Order;
import com.example.epos_2021.models.OrderItem;
import com.example.epos_2021.models.OrderItemAddon;
import com.example.epos_2021.models.OrderItemIngredient;
import com.example.epos_2021.models.OrderPayment;
import com.example.epos_2021.models.OrderSplit;
import com.example.epos_2021.models.SiteSetting;
import com.example.epos_2021.models.Table;
import com.example.epos_2021.models.TableStatus;
import com.example.epos_2021.models.Voucher;
import com.example.epos_2021.network.ApiEndPoints;
import com.example.epos_2021.utils.Constants;
import com.example.epos_2021.utils.LogUtils;
import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;

/**
 * Created by Amrish on 01-06-2021.
 */


public class SingleOrderUploaderService extends IntentService {

    int _order_id = 0;

    private Order order;
    private MyApp myApp;
    private int position = -1;
    private AppDatabase appDatabase;
    private Order onlineOrderObject;
    private boolean forceFully;
    Table table = null;
    Voucher voucher;
    String oldStatus = null;
    String TAG = "::SERVICE::";

    public SingleOrderUploaderService(Context context) {
        super("Upload Service");
    }

    public SingleOrderUploaderService() {
        super("Upload Service");
    }

    boolean orderCompleteAuto = false;

    @Override
    protected void onHandleIntent(@Nullable Intent intent) {
        myApp = MyApp.getInstance();
        appDatabase = myApp.appDatabase;
        myApp.shallWeRefreshOrders = false;
        if (intent != null) {
            _order_id = intent.getIntExtra("_order_id", 0);
            position = intent.getIntExtra("position", -1);
            forceFully = intent.getBooleanExtra("forcefully", false);
        }
        SiteSetting orderCompleteMode = myApp.findSetting("order_complete_mode");
        if (orderCompleteMode != null && !Validators.isNullOrEmpty(orderCompleteMode.value) && (orderCompleteMode.value.toLowerCase().equalsIgnoreCase("auto") || orderCompleteMode.value.toLowerCase().equalsIgnoreCase("1") || orderCompleteMode.value.toLowerCase().equalsIgnoreCase("yes"))) {
            orderCompleteAuto = true;
        }

        //FetchOrderOfflineAsyncTask();
        new FetchOrderOfflineAsyncTask(() -> {
            if (myApp.isConnected(this) && (forceFully || "auto".equalsIgnoreCase(myApp.myPreferences.getOrderSyncMode()))) {
                int sendAtLevel = 0;
                if (order.order_items.size() > 0 && (order.order_splits.size() == 0 && order.order_payments.size() == 0)) {
                    sendAtLevel = 0;
                } else if (order.order_items.size() > 0 && (order.order_splits.size() >= 0 || order.order_payments.size() >= 0)) {
                    sendAtLevel = 1;
                } else if (order.order_items.size() == 0) {
                    sendAtLevel = 1;
                }
                createUpdateOrderOnline(() -> {
                    if (order.order_splits.size() == 0) {
                        if (order.order_payments.size() == 0) {
                            new DeleteOldAndSaveNewAsync(() -> {
                                uploadTableStatusOnline();
                                return null;
                            }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                        } else {
                            createBulkPayment(() -> {
                                createUpdateOrderOnline(() -> {
                                    new DeleteOldAndSaveNewAsync(() -> {
                                        uploadTableStatusOnline();
                                        return null;
                                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                                    return null;
                                }, true);
                                return null;
                            });
                        }
                    } else {
                        createBulkSplits(() -> {
                            if (order.order_payments.size() == 0) {
                                createUpdateOrderOnline(() -> {
                                    new DeleteOldAndSaveNewAsync(() -> {
                                        uploadTableStatusOnline();
                                        return null;
                                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                                    return null;
                                }, true);
                            } else {
                                createBulkPayment(() -> {
                                    createUpdateOrderOnline(() -> {
                                        new DeleteOldAndSaveNewAsync(() -> {
                                            uploadTableStatusOnline();
                                            return null;
                                        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                                        return null;
                                    }, true);
                                    return null;
                                });
                            }
                            return null;
                        });
                    }
                    return null;
                }, sendAtLevel == 0);
            } else {
                uploadTableStatusOnline();

            }
            return null;
        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);

    }

    private void FetchOrderOfflineAsyncTask(){
        new FetchOrderOfflineAsyncTask(() -> {
            if (myApp.isConnected(this) && (forceFully || "auto".equalsIgnoreCase(myApp.myPreferences.getOrderSyncMode()))) {
                int sendAtLevel = 0;
                if (order.order_items.size() > 0 && (order.order_splits.size() == 0 && order.order_payments.size() == 0)) {
                    sendAtLevel = 0;
                } else if (order.order_items.size() > 0 && (order.order_splits.size() >= 0 || order.order_payments.size() >= 0)) {
                    sendAtLevel = 1;
                } else if (order.order_items.size() == 0) {
                    sendAtLevel = 1;
                }
                createUpdateOrderOnline(() -> {
                    if (order.order_splits.size() == 0) {
                        if (order.order_payments.size() == 0) {
                            new DeleteOldAndSaveNewAsync(() -> {
                                uploadTableStatusOnline();
                                return null;
                            }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                        } else {
                            createBulkPayment(() -> {
                                createUpdateOrderOnline(() -> {
                                    new DeleteOldAndSaveNewAsync(() -> {
                                        uploadTableStatusOnline();
                                        return null;
                                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                                    return null;
                                }, true);
                                return null;
                            });
                        }
                    } else {
                        createBulkSplits(() -> {
                            if (order.order_payments.size() == 0) {
                                createUpdateOrderOnline(() -> {
                                    new DeleteOldAndSaveNewAsync(() -> {
                                        uploadTableStatusOnline();
                                        return null;
                                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                                    return null;
                                }, true);
                            } else {
                                createBulkPayment(() -> {
                                    createUpdateOrderOnline(() -> {
                                        new DeleteOldAndSaveNewAsync(() -> {
                                            uploadTableStatusOnline();
                                            return null;
                                        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                                        return null;
                                    }, true);
                                    return null;
                                });
                            }
                            return null;
                        });
                    }
                    return null;
                }, sendAtLevel == 0);
            } else {
                uploadTableStatusOnline();

            }
            return null;
        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private void fetchAndUploadVoucher(Callable<Void> nextMethod) {
        new FetchVoucherAsync(() -> {
            if (myApp.isConnected(this)) {
                uploadVoucher(() -> {
                    new UpdateVoucherAsync(() -> {
                        nextMethod.call();
                        return null;
                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                    return null;
                });
            }
            return null;
        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private class FetchOrderOfflineAsync extends AsyncTask<Object, Object, Object> {
        Callable<Void> nextMethod;

        public FetchOrderOfflineAsync(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected Object doInBackground(Object... objects) {
            if (myApp.isConnected(SingleOrderUploaderService.this) && (forceFully || "auto".equalsIgnoreCase(myApp.myPreferences.getOrderSyncMode()))) {
                int sendAtLevel = 0;
                if (order.order_items.size() > 0 && (order.order_splits.size() == 0 && order.order_payments.size() == 0)) {
                    sendAtLevel = 0;
                } else if (order.order_items.size() > 0 && (order.order_splits.size() >= 0 || order.order_payments.size() >= 0)) {
                    sendAtLevel = 1;
                } else if (order.order_items.size() == 0) {
                    sendAtLevel = 1;
                }
                createUpdateOrderOnline(() -> {
                    if (order.order_splits.size() == 0) {
                        if (order.order_payments.size() == 0) {
                            new DeleteOldAndSaveNewAsync(() -> {
                                uploadTableStatusOnline();
                                return null;
                            }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                        } else {
                            createBulkPayment(() -> {
                                createUpdateOrderOnline(() -> {
                                    new DeleteOldAndSaveNewAsync(() -> {
                                        uploadTableStatusOnline();
                                        return null;
                                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                                    return null;
                                }, true);
                                return null;
                            });
                        }
                    } else {
                        createBulkSplits(() -> {
                            if (order.order_payments.size() == 0) {
                                createUpdateOrderOnline(() -> {
                                    new DeleteOldAndSaveNewAsync(() -> {
                                        uploadTableStatusOnline();
                                        return null;
                                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                                    return null;
                                }, true);
                            } else {
                                createBulkPayment(() -> {
                                    createUpdateOrderOnline(() -> {
                                        new DeleteOldAndSaveNewAsync(() -> {
                                            uploadTableStatusOnline();
                                            return null;
                                        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                                        return null;
                                    }, true);
                                    return null;
                                });
                            }
                            return null;
                        });
                    }
                    return null;
                }, sendAtLevel == 0);
            } else {
                uploadTableStatusOnline();

            }
            return null;
        }

        @Override
        protected void onPostExecute(Object s) {
            super.onPostExecute(s);

        }
    }

    private class FetchVoucherAsync extends AsyncTask<Object, Object, Object> {
        Callable<Void> nextMethod;

        public FetchVoucherAsync(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected Object doInBackground(Object... objects) {
            voucher = appDatabase.voucherDao().view(voucher._id);
            voucher.customer = appDatabase.customerDao().view(voucher._customer_id);
            return null;
        }

        @Override
        protected void onPostExecute(Object s) {
            super.onPostExecute(s);
            if (nextMethod != null) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                publishResult(true);
            }
        }
    }

    private void uploadVoucher(Callable<Void> nextMethod) {
        String url = ApiEndPoints.voucher_add;
        if (!Validators.isNullOrEmpty(voucher.id)) {
            url = url + voucher.id;
        }
        AndroidNetworking.post(url)
                .addApplicationJsonBody(voucher)
                .build()
                .getAsObject(Voucher.class, new ParsedRequestListener<Voucher>() {
                    @Override
                    public void onResponse(Voucher response) {
                        response._id = voucher._id;
                        response._customer_id = voucher._customer_id;
                        response.customer._id = voucher.customer._id;
                        response.uploadable = false;
                        voucher = response;

                        if (nextMethod != null) {
                            try {
                                nextMethod.call();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        publishResult(true, "\n[Voucher ID=> " + voucher._id + "]  " + anError.getErrorAsObject(ApiError.class).getMessage());
                    }
                });
    }

    private class UpdateVoucherAsync extends AsyncTask<String, String, String> {
        Callable<Void> nextMethod;

        public UpdateVoucherAsync(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected String doInBackground(String... strings) {
            myApp.appDatabase.customerDao().insert(voucher.customer);
            myApp.appDatabase.voucherDao().insert(voucher);

            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            if (nextMethod != null) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                publishResult(true);
            }
        }
    }

    private void publishResult(boolean b) {
        MyApp.getInstance().shallWeRefreshOrders = true;
        Intent intent = new Intent(Constants.EPOS_ORDER_UPDATE);
        intent.putExtra("_order_id", _order_id);
        intent.putExtra("result", b);
        intent.putExtra("position", position);
        myApp.notifyCart(this, intent);
    }

    private void publishResult(boolean b, String message) {
        MyApp.getInstance().shallWeRefreshOrders = true;
        Intent intent = new Intent(Constants.EPOS_ORDER_UPDATE);
        intent.putExtra("_order_id", _order_id);
        intent.putExtra("result", b);
        intent.putExtra("message", message);
        intent.putExtra("position", position);
        myApp.notifyCart(this, intent);
    }

    private class FetchOrderOfflineAsyncTask extends AsyncTask<Object, Object, Object> {
        Callable<Void> nextMethod;

        public FetchOrderOfflineAsyncTask(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected Object doInBackground(Object... objects) {
            OrderWithItems orderWithItems = myApp.appDatabase.orderDao().orderWithItems(_order_id);
            if (orderWithItems != null) {

                order = orderWithItems.order;
                float total_paid = appDatabase.orderPaymentDao().getTotalPaidAmount(_order_id);
                if (order.total_paid != total_paid) {
                    order.total_paid = total_paid;
                    order.updated_at = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
                } else {
                    LogUtils.e(TAG, "Order will not updated at");
                }
                order.order_payments = (ArrayList<OrderPayment>) orderWithItems.orderPayments;
                order.order_splits = (ArrayList<OrderSplit>) orderWithItems.orderSplits;
                order.order_items = new ArrayList<>();
                for (OrderItemWithAddonsIngredients item : orderWithItems.orderItems) {
                    OrderItem orderItem = item.orderItem;
                    orderItem.order_item_addons = (ArrayList<OrderItemAddon>) item.orderItemAddons;
                    orderItem.order_item_ingredients = (ArrayList<OrderItemIngredient>) item.orderItemIngredients;
                    order.order_items.add(orderItem);

                }
                order.customer = myApp.appDatabase.customerDao().view(order._customer_id);
                if (Validators.isNullOrEmpty(order.customer_id)) {
                    order.customer_id = order.customer.id;
                }
                if (order.customer == null) {
                    order.customer = appDatabase.customerDao().view(order.customer_id);
                }
                String currentStatusId = order.order_status_id;

                if (order.order_type_id.equalsIgnoreCase("1")) {
                    table = appDatabase.tableDao().view(order.table_id);
                    oldStatus = table.status;
                }
                TableStatus vacantStatus = myApp.findStatus("Vacant");
                TableStatus orderTakenStatus = myApp.findStatus("Order Taken");
                TableStatus servedStatus = myApp.findStatus("Served");
                TableStatus takingOrderStatus = myApp.findStatus("Taking Order");
                TableStatus servedPaidStatus = myApp.findStatus("Served and paid");

                if (!order.order_status_id.equalsIgnoreCase("5") && !order.order_status_id.equalsIgnoreCase("10")) {
                    if (order.total > 0) {
                        LogUtils.e(TAG, "SERVICE_TABLE AUto Complete? " + orderCompleteAuto);
                        if (CommonFunctions.orderPaymentStatus(order.total, order.total_paid) == 1) {
                            if (orderCompleteAuto) {
                                order.order_status_id = "5";
                                order.order_status = "Order Completed";
                                LogUtils.e(TAG, "SERVICE_TABLE Order status " + order.order_status);
                            } else {
                                if (!order.order_status_id.equalsIgnoreCase("6")) {
                                    order.order_status_id = "4";
                                    order.order_status = "Order Paid";
                                }
                            }
                        } else if (CommonFunctions.orderPaymentStatus(order.total, order.total_paid) == 0) {
                            order.order_status_id = "7";
                            order.order_status = "Part Payment Received";
                        } else if (CommonFunctions.orderPaymentStatus(order.total, order.total_paid) == -1) {
                            if (!order.order_status_id.equalsIgnoreCase("3")) {
                                order.order_status_id = "2";
                                order.order_status = "Order Taken";
                            }

                        }
                    } else {
                        order.order_status_id = "2";
                        order.order_status = "Order Taken";

                    }
                    if (!currentStatusId.equalsIgnoreCase(order.order_status_id)) {
                        order.updated_at = CommonFunctions.convertMsToDesiredFormat(System.currentTimeMillis(), Constants.PHP_DATE_TIME_FORMAT_ZULU);
                    }
                    LogUtils.e(TAG, "Updating SERVICE_TABLE order status " + order.order_status);
                    appDatabase.orderDao().update(order);
                    LogUtils.e(TAG, "Updating after update : SERVICE_TABLE order status " + order.order_status);
                    if (table != null) {
                        if (order.total > 0) {
                            if (order.total_paid > 0) {
                                if (!order.order_status_id.equalsIgnoreCase("5") && !order.order_status_id.equalsIgnoreCase("10")) {
                                    if (CommonFunctions.orderPaymentStatus(order.total, order.total_paid) == 1) {
                                        table.status = servedPaidStatus.status;
                                        table.table_status_id = servedPaidStatus.id;
                                    }
                                } else {
                                    table.status = vacantStatus.status;
                                    table.table_status_id = vacantStatus.id;
                                }
                            } else {
                                if (!order.order_status_id.equalsIgnoreCase("5") && !order.order_status_id.equalsIgnoreCase("10")) {
                                    if (order.order_status_id.equalsIgnoreCase("3")) {
                                        table.status = servedStatus.status;
                                        table.table_status_id = servedStatus.id;
                                    } else {
                                        if (Validators.isNullOrEmpty(order.order_action_id)) {
                                            table.table_status_id = takingOrderStatus.id;
                                            table.status = takingOrderStatus.status;
                                        } else {
                                            table.table_status_id = orderTakenStatus.id;
                                            table.status = orderTakenStatus.status;
                                        }
                                    }

                                } else {
                                    table.status = vacantStatus.status;
                                    table.table_status_id = vacantStatus.id;
                                }
                            }

                        } else {
                            if (!order.order_status_id.equalsIgnoreCase("5") && !order.order_status_id.equalsIgnoreCase("10")) {
                                table.status = takingOrderStatus.status;
                                table.table_status_id = takingOrderStatus.id;
                            } else {
                                table.status = vacantStatus.status;
                                table.table_status_id = vacantStatus.id;
                                table.locked = false;
                            }
                        }
                        LogUtils.e(TAG, "Updating SERVICE_TABLE status:::" + table.status);
                        table.locked = false;
                        appDatabase.tableDao().insert(table);
//                        table= appDatabase.tableDao().view(table.id);
                        LogUtils.e(TAG, "Updating after update ::: SERVICE_TABLE status:::" + table.status);
                        OrderPayment op = appDatabase.orderPaymentDao().findByMethod(_order_id, "5");

                    }
                } else {
// Do not write here anything otherwise completed orders may be changed while pushing orders
                }
            } else {
                nextMethod = null;
            }

            return null;
        }

        @Override
        protected void onPostExecute(Object s) {
            super.onPostExecute(s);
            if (nextMethod != null) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                publishResult(true);
            }
        }


    }

    private void uploadTableStatusOnline() {
        if (table == null || (order.table_id == null || !order.order_type_id.equalsIgnoreCase("1"))) {
            LogUtils.e(TAG, "SERVICE_TABLE not dinein order");
            publishResult(true);
            return;
        }
        new Thread(() -> {
            table.locked = false;
            appDatabase.tableDao().insert(table);
            if (false && !Validators.isNullOrEmpty(oldStatus) && oldStatus.equalsIgnoreCase(table.status) && !table.locked) {
                LogUtils.e(TAG, "SERVICE_TABLE same status so we are returning", oldStatus);
                publishResult(true);
                return;
            }
            LogUtils.e(TAG, "Uploading SERVICE_TABLE Status online::" + new Gson().toJson(table));

            LogUtils.e(TAG, "SERVICE_TABLE STATUS:" + table.id + " & status_id: " + table.table_status_id + " as " + table.status + " Loked? " + table.locked);
            AndroidNetworking.post(ApiEndPoints.tables + table.id)
                    .addApplicationJsonBody(table)
                    .build()
                    .getAsObject(Table.class, new ParsedRequestListener() {
                        @Override
                        public void onResponse(Object response) {
                            LogUtils.e(TAG, "SERVICE SERVICE_TABLE STATUS CHANGED RESPONSE:" + new Gson().toJson(response));
                            publishResult(true);
                        }

                        @Override
                        public void onError(ANError anError) {
                            publishResult(true);
                        }
                    });
        }).start();

    }

    private void createUpdateOrderOnline(Callable<Void> nextMethod, boolean shouldSendPush) {
        String url = ApiEndPoints.orders;
        if (!Validators.isNullOrEmpty(order.id)) {
            url = url + order.id;
        }
        LogUtils.e(TAG, "shouldSendPush?" + shouldSendPush);
        AndroidNetworking.post(url)
                .addApplicationJsonBody(order)
                .addQueryParameter("send_push", table == null && shouldSendPush ? "1" : "0")
                .build()
                .getAsObject(Order.class, new ParsedRequestListener<Order>() {
                    @Override
                    public void onResponse(Order response) {
                        order.id = response.id;
                        response.customer._id = order._customer_id;
                        order.customer = response.customer;
                        order.customer_id = response.customer.id;
                        order.customer_name = response.customer.name;
                        onlineOrderObject = response;
                        onlineOrderObject.customer_name = response.customer.name;
                        new UpdateSplitAndPaymentOrderIdsWithOnlineOrderIds(response, () -> {
                            nextMethod.call();
                            return null;
                        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                    }

                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e(TAG, "Order error createUpdateOrderOnline()" + Log.getStackTraceString(anError.fillInStackTrace()));
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            publishResult(false, "\n[ID=> " + order._id + "]  " + anError.getErrorAsObject(ApiError.class).getMessage());
                        }

                    }
                });
    }

    private void createBulkSplits(Callable<Void> nextMethod) {
        AndroidNetworking.post(ApiEndPoints.order_splits + "split-bulk")
                .addBodyParameter("splits", new Gson().toJson(order.order_splits))
                .build()
                .getAsObjectList(OrderSplit.class, new ParsedRequestListener<List<OrderSplit>>() {
                    @Override
                    public void onResponse(List<OrderSplit> response) {
                        new LinkSplitItemWithNameAndLocalId(() -> {
                            new FetchOrderOfflineAsyncTask(() -> {
                                if (nextMethod != null) {
                                    nextMethod.call();
                                }
                                return null;
                            }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                            return null;
                        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, response);

                    }

                    @Override
                    public void onError(ANError anError) {
                        if (anError.getErrorCode() == 400) {
                            publishResult(false, "\n[ID=> " + order._id + "]  " + anError.getErrorAsObject(ApiError.class).getMessage());
                        }
                    }
                });
    }

    private void createBulkPayment(Callable<Void> nextMethod) {
        AndroidNetworking.post(ApiEndPoints.order_payments + "payment-bulk")
                .addBodyParameter("payments", new Gson().toJson(order.order_payments))
                .build()
                .getAsObjectList(OrderPayment.class, new ParsedRequestListener<List<OrderPayment>>() {
                    @Override
                    public void onResponse(List<OrderPayment> response) {
                        new FetchOrderOfflineAsyncTask(() -> {
                            if (nextMethod != null) {
                                nextMethod.call();
                            }
                            return null;
                        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);

                    }

                    @Override
                    public void onError(ANError anError) {
                        if (anError.getErrorCode() == 400) {
                            publishResult(false, "\n[ID=> " + order._id + "]  " + anError.getErrorAsObject(ApiError.class).getMessage());
                        }
                    }
                });
    }

    private class LinkSplitItemWithNameAndLocalId extends AsyncTask<List<OrderSplit>, String, String> {
        Callable<Void> nextMethod;

        public LinkSplitItemWithNameAndLocalId(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected String doInBackground(List<OrderSplit>... arrayLists) {
            List<OrderSplit> splits = arrayLists[0];
            for (OrderSplit split : splits) {
                for (OrderSplit dbSplit : order.order_splits) {
                    if (dbSplit.group_name.equalsIgnoreCase(split.group_name)) {
                        split._id = dbSplit._id;
                        split._order_id = _order_id;
                        break;
                    }
                }
            }

            appDatabase.orderSplitDao().insertMultiple(splits);
            for (OrderSplit s : splits) {
                appDatabase.orderItemDao().updateSplitIds(s._id, s.id);
                appDatabase.orderPaymentDao().updateSplitIds(_order_id, s._id, s.id);
            }
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            if (nextMethod != null) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private class DeleteOldAndSaveNewAsync extends AsyncTask<String, String, String> {
        Callable<Void> nextMethod;

        public DeleteOldAndSaveNewAsync(Callable<Void> nextMethod) {
            this.nextMethod = nextMethod;
        }

        @Override
        protected String doInBackground(String... strings) {
//            appDatabase.orderDao().delete(order);
            onlineOrderObject._id = order._id;

            Customer dbCustomer = appDatabase.customerDao().view(onlineOrderObject.customer_id);
            int _customer_id;
            if (dbCustomer == null) {
                _customer_id = (int) appDatabase.customerDao().insert(onlineOrderObject.customer);
            } else {
                appDatabase.customerDao().update(dbCustomer);
                _customer_id = dbCustomer._id;
            }
            onlineOrderObject._customer_id = _customer_id;

            _order_id = (int) appDatabase.orderDao().insert(onlineOrderObject);
            onlineOrderObject._id = _order_id;

            appDatabase.orderPaymentDao().deleteAll(_order_id);
            appDatabase.orderSplitDao().deleteAll(_order_id);
            appDatabase.orderItemAddonDao().deleteAll(_order_id);
            appDatabase.orderItemIngredientDao().deleteAll(_order_id);
            appDatabase.orderItemIngredientDao().deleteAll(_order_id);

            if (onlineOrderObject.order_splits != null) {
                for (OrderSplit split : onlineOrderObject.order_splits) {
                    split._order_id = _order_id;
                }
                appDatabase.orderSplitDao().insertMultiple(onlineOrderObject.order_splits);
            }

            if (onlineOrderObject.order_payments != null) {
                for (OrderPayment payment : onlineOrderObject.order_payments) {
                    payment._order_id = _order_id;
                    if (!Validators.isNullOrEmpty(payment.order_split_id)) {
                        OrderSplit dbOrderSplit = appDatabase.orderSplitDao().view(payment.order_split_id);
                        if (dbOrderSplit != null)
                            payment._order_split_id = dbOrderSplit._id;
                    }
                }
                appDatabase.orderPaymentDao().insertMultiple(onlineOrderObject.order_payments);
            }


            for (OrderItem orderItem : onlineOrderObject.order_items) {
                orderItem._order_id = _order_id;

                if (!Validators.isNullOrEmpty(orderItem.order_split_id)) {
                    OrderSplit orderSplit = appDatabase.orderSplitDao().view(orderItem.order_split_id);
                    if (orderSplit != null) orderItem._order_split_id = orderSplit._id;
                }

                int _order_item_id = (int) appDatabase.orderItemDao().insert(orderItem);

                if (orderItem.order_item_addons != null) {
                    for (OrderItemAddon itemAddon : orderItem.order_item_addons) {
                        itemAddon._order_item_id = _order_item_id;
                    }
                    appDatabase.orderItemAddonDao().insertAll(orderItem.order_item_addons);
                }

                if (orderItem.order_item_ingredients != null) {
                    appDatabase.orderItemIngredientDao().deleteAll(_order_item_id);
                    for (OrderItemIngredient itemIngredient : orderItem.order_item_ingredients) {
                        itemIngredient._order_item_id = _order_item_id;
                    }
                    appDatabase.orderItemIngredientDao().insertAll(orderItem.order_item_ingredients);
                }

            }
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            if (nextMethod != null) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private class UpdateSplitAndPaymentOrderIdsWithOnlineOrderIds extends AsyncTask<String, String, String> {
        Order response;
        Callable<Void> nextMethod;

        public UpdateSplitAndPaymentOrderIdsWithOnlineOrderIds(Order response, Callable<Void> nextMethod) {
            this.response = response;
            this.nextMethod = nextMethod;
        }

        @Override
        protected String doInBackground(String... strings) {
            appDatabase.customerDao().insert(order.customer);
            appDatabase.orderDao().update(order);
            appDatabase.orderPaymentDao().updateOrderId(_order_id, response.id);
            appDatabase.orderSplitDao().updateOrderId(_order_id, response.id);
            for (OrderPayment orderPayment : order.order_payments) {
                orderPayment.order_id = response.id;
            }
            for (OrderSplit orderSplit : order.order_splits) {
                orderSplit.order_id = response.id;
            }
//            order.order_payments = (ArrayList<OrderPayment>) appDatabase.orderPaymentDao().list(response.id);
//            order.order_splits = (ArrayList<OrderSplit>) appDatabase.orderSplitDao().list(response.id);
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            if (nextMethod != null) {
                try {
                    nextMethod.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
