package com.example.epos_2021.utils;

import android.app.Activity;
import android.content.Context;
import android.os.Build;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.core.content.ContextCompat;

import com.example.epos_2021.R;
import com.google.android.material.snackbar.Snackbar;

public class ToastUtils {
    public static void makeToast(Activity context, String message) {
        if (context != null)
            context.runOnUiThread(() -> Toast.makeText(context, message, Toast.LENGTH_SHORT).show());
    }

    public static void makeLongToast(Activity context, String message) {
        if (context != null)
            context.runOnUiThread(() -> Toast.makeText(context, message, Toast.LENGTH_LONG).show());
    }

    public static void makeToast(Context context, String message) {
        if (context != null)
            Toast.makeText(context, message, Toast.LENGTH_SHORT).show();
    }

    public static void makeLongToast(Context context, String message) {
        if (context != null)
            Toast.makeText(context, message, Toast.LENGTH_LONG).show();
    }

    public static void makeSnackToast(Activity activity, String message) {
        if (activity != null)
            activity.runOnUiThread(() -> Toast.makeText(activity, message, Toast.LENGTH_LONG).show());
    }

    public static void makeSnackToast(Context activity, String message) {
        try {
            if (activity != null)
                Toast.makeText(activity, message, Toast.LENGTH_LONG).show();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void showSnackBar(Context context, View coordinatorLayout, String message, String type) {
        Snackbar snackbar = Snackbar.make(coordinatorLayout, message, Snackbar.LENGTH_LONG);
        View mView = snackbar.getView();
        if (type.equalsIgnoreCase("success"))
            snackbar.setBackgroundTint(ContextCompat.getColor(context, R.color.chip_green_color));
        else if ((type.equalsIgnoreCase("remove") || type.equalsIgnoreCase("reject") || type.equalsIgnoreCase("failed")))
            snackbar.setBackgroundTint(ContextCompat.getColor(context, R.color.reject_color));
        else snackbar.setBackgroundTint(ContextCompat.getColor(context, R.color.eclipse_color));
        snackbar.setActionTextColor(ContextCompat.getColor(context, R.color.white));
        (mView).getLayoutParams().width = context.getResources().getDimensionPixelSize(R.dimen.width_400);

        TextView sbTextView = (TextView) mView.findViewById(com.google.android.material.R.id.snackbar_text);
        sbTextView.setTextSize(TypedValue.COMPLEX_UNIT_PX, context.getResources().getDimension(R.dimen.text_size_16));
        sbTextView.setGravity(Gravity.CENTER_HORIZONTAL);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
            sbTextView.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);
        } else {
            sbTextView.setGravity(Gravity.CENTER_HORIZONTAL);
        }
        snackbar.show();
    }

    public static void showSnackBarFull(Context context, View coordinatorLayout, String message, String type) {
        Snackbar snackbar = Snackbar.make(coordinatorLayout, message, Snackbar.LENGTH_LONG);
        View mView = snackbar.getView();
        if (type.equalsIgnoreCase("success"))
            snackbar.setBackgroundTint(ContextCompat.getColor(context, R.color.chip_green_color));
        else if ((type.equalsIgnoreCase("remove") || type.equalsIgnoreCase("reject")))
            snackbar.setBackgroundTint(ContextCompat.getColor(context, R.color.reject_color));
        else snackbar.setBackgroundTint(ContextCompat.getColor(context, R.color.eclipse_color));
        snackbar.setActionTextColor(ContextCompat.getColor(context, R.color.white));
        (mView).getLayoutParams().width = ViewGroup.LayoutParams.MATCH_PARENT;
        TextView sbTextView = (TextView) mView.findViewById(com.google.android.material.R.id.snackbar_text);
        sbTextView.setTextSize(TypedValue.COMPLEX_UNIT_PX, context.getResources().getDimension(R.dimen.text_size_16));
        sbTextView.setGravity(Gravity.CENTER_HORIZONTAL);
        snackbar.show();
    }

    public static void showSnackBar(Context context, View coordinatorLayout, String message, int length, int color) {
        Snackbar greySnackBar = Snackbar.make(coordinatorLayout, message, length);
        greySnackBar.setBackgroundTint(ContextCompat.getColor(context, color));
        greySnackBar.show();
    }

    public static void showSnackBar(Context context, View coordinatorLayout, String message, int length) {
        Snackbar greySnackBar = Snackbar.make(coordinatorLayout, message, length);
        greySnackBar.setBackgroundTint(ContextCompat.getColor(context, R.color.eclipse_color));
        greySnackBar.show();
    }

    public static void showSnackBar(Context context, View coordinatorLayout, String message) {
        Snackbar greySnackBar = Snackbar.make(coordinatorLayout, message, Snackbar.LENGTH_LONG);
        greySnackBar.setBackgroundTint(ContextCompat.getColor(context, R.color.eclipse_color));
        (greySnackBar.getView()).getLayoutParams().width = ViewGroup.LayoutParams.MATCH_PARENT;
        greySnackBar.show();
    }

}
